# /* Copyright (C) 2001
#  * Housemarque Oy
#  * http://www.housemarque.com
#  *
#  * Distributed under the Boost Software License, Version 1.0. (See
#  * accompanying file LICENSE_1_0.txt or copy at
#  * http://www.boost.org/LICENSE_1_0.txt)
#  */
#
# /* Revised by Paul Mensonides (2002) */
# /* Revised by Edward Diener (2020) */
#
# /* See http://www.boost.org for most recent version. */
#
# ifndef BOOST_PREPROCESSOR_ARITHMETIC_DEC_1024_HPP
# define BOOST_PREPROCESSOR_ARITHMETIC_DEC_1024_HPP
#
# define BOOST_PP_DEC_514 513
# define BOOST_PP_DEC_515 514
# define BOOST_PP_DEC_516 515
# define BOOST_PP_DEC_517 516
# define BOOST_PP_DEC_518 517
# define BOOST_PP_DEC_519 518
# define BOOST_PP_DEC_520 519
# define BOOST_PP_DEC_521 520
# define BOOST_PP_DEC_522 521
# define BOOST_PP_DEC_523 522
# define BOOST_PP_DEC_524 523
# define BOOST_PP_DEC_525 524
# define BOOST_PP_DEC_526 525
# define BOOST_PP_DEC_527 526
# define BOOST_PP_DEC_528 527
# define BOOST_PP_DEC_529 528
# define BOOST_PP_DEC_530 529
# define BOOST_PP_DEC_531 530
# define BOOST_PP_DEC_532 531
# define BOOST_PP_DEC_533 532
# define BOOST_PP_DEC_534 533
# define BOOST_PP_DEC_535 534
# define BOOST_PP_DEC_536 535
# define BOOST_PP_DEC_537 536
# define BOOST_PP_DEC_538 537
# define BOOST_PP_DEC_539 538
# define BOOST_PP_DEC_540 539
# define BOOST_PP_DEC_541 540
# define BOOST_PP_DEC_542 541
# define BOOST_PP_DEC_543 542
# define BOOST_PP_DEC_544 543
# define BOOST_PP_DEC_545 544
# define BOOST_PP_DEC_546 545
# define BOOST_PP_DEC_547 546
# define BOOST_PP_DEC_548 547
# define BOOST_PP_DEC_549 548
# define BOOST_PP_DEC_550 549
# define BOOST_PP_DEC_551 550
# define BOOST_PP_DEC_552 551
# define BOOST_PP_DEC_553 552
# define BOOST_PP_DEC_554 553
# define BOOST_PP_DEC_555 554
# define BOOST_PP_DEC_556 555
# define BOOST_PP_DEC_557 556
# define BOOST_PP_DEC_558 557
# define BOOST_PP_DEC_559 558
# define BOOST_PP_DEC_560 559
# define BOOST_PP_DEC_561 560
# define BOOST_PP_DEC_562 561
# define BOOST_PP_DEC_563 562
# define BOOST_PP_DEC_564 563
# define BOOST_PP_DEC_565 564
# define BOOST_PP_DEC_566 565
# define BOOST_PP_DEC_567 566
# define BOOST_PP_DEC_568 567
# define BOOST_PP_DEC_569 568
# define BOOST_PP_DEC_570 569
# define BOOST_PP_DEC_571 570
# define BOOST_PP_DEC_572 571
# define BOOST_PP_DEC_573 572
# define BOOST_PP_DEC_574 573
# define BOOST_PP_DEC_575 574
# define BOOST_PP_DEC_576 575
# define BOOST_PP_DEC_577 576
# define BOOST_PP_DEC_578 577
# define BOOST_PP_DEC_579 578
# define BOOST_PP_DEC_580 579
# define BOOST_PP_DEC_581 580
# define BOOST_PP_DEC_582 581
# define BOOST_PP_DEC_583 582
# define BOOST_PP_DEC_584 583
# define BOOST_PP_DEC_585 584
# define BOOST_PP_DEC_586 585
# define BOOST_PP_DEC_587 586
# define BOOST_PP_DEC_588 587
# define BOOST_PP_DEC_589 588
# define BOOST_PP_DEC_590 589
# define BOOST_PP_DEC_591 590
# define BOOST_PP_DEC_592 591
# define BOOST_PP_DEC_593 592
# define BOOST_PP_DEC_594 593
# define BOOST_PP_DEC_595 594
# define BOOST_PP_DEC_596 595
# define BOOST_PP_DEC_597 596
# define BOOST_PP_DEC_598 597
# define BOOST_PP_DEC_599 598
# define BOOST_PP_DEC_600 599
# define BOOST_PP_DEC_601 600
# define BOOST_PP_DEC_602 601
# define BOOST_PP_DEC_603 602
# define BOOST_PP_DEC_604 603
# define BOOST_PP_DEC_605 604
# define BOOST_PP_DEC_606 605
# define BOOST_PP_DEC_607 606
# define BOOST_PP_DEC_608 607
# define BOOST_PP_DEC_609 608
# define BOOST_PP_DEC_610 609
# define BOOST_PP_DEC_611 610
# define BOOST_PP_DEC_612 611
# define BOOST_PP_DEC_613 612
# define BOOST_PP_DEC_614 613
# define BOOST_PP_DEC_615 614
# define BOOST_PP_DEC_616 615
# define BOOST_PP_DEC_617 616
# define BOOST_PP_DEC_618 617
# define BOOST_PP_DEC_619 618
# define BOOST_PP_DEC_620 619
# define BOOST_PP_DEC_621 620
# define BOOST_PP_DEC_622 621
# define BOOST_PP_DEC_623 622
# define BOOST_PP_DEC_624 623
# define BOOST_PP_DEC_625 624
# define BOOST_PP_DEC_626 625
# define BOOST_PP_DEC_627 626
# define BOOST_PP_DEC_628 627
# define BOOST_PP_DEC_629 628
# define BOOST_PP_DEC_630 629
# define BOOST_PP_DEC_631 630
# define BOOST_PP_DEC_632 631
# define BOOST_PP_DEC_633 632
# define BOOST_PP_DEC_634 633
# define BOOST_PP_DEC_635 634
# define BOOST_PP_DEC_636 635
# define BOOST_PP_DEC_637 636
# define BOOST_PP_DEC_638 637
# define BOOST_PP_DEC_639 638
# define BOOST_PP_DEC_640 639
# define BOOST_PP_DEC_641 640
# define BOOST_PP_DEC_642 641
# define BOOST_PP_DEC_643 642
# define BOOST_PP_DEC_644 643
# define BOOST_PP_DEC_645 644
# define BOOST_PP_DEC_646 645
# define BOOST_PP_DEC_647 646
# define BOOST_PP_DEC_648 647
# define BOOST_PP_DEC_649 648
# define BOOST_PP_DEC_650 649
# define BOOST_PP_DEC_651 650
# define BOOST_PP_DEC_652 651
# define BOOST_PP_DEC_653 652
# define BOOST_PP_DEC_654 653
# define BOOST_PP_DEC_655 654
# define BOOST_PP_DEC_656 655
# define BOOST_PP_DEC_657 656
# define BOOST_PP_DEC_658 657
# define BOOST_PP_DEC_659 658
# define BOOST_PP_DEC_660 659
# define BOOST_PP_DEC_661 660
# define BOOST_PP_DEC_662 661
# define BOOST_PP_DEC_663 662
# define BOOST_PP_DEC_664 663
# define BOOST_PP_DEC_665 664
# define BOOST_PP_DEC_666 665
# define BOOST_PP_DEC_667 666
# define BOOST_PP_DEC_668 667
# define BOOST_PP_DEC_669 668
# define BOOST_PP_DEC_670 669
# define BOOST_PP_DEC_671 670
# define BOOST_PP_DEC_672 671
# define BOOST_PP_DEC_673 672
# define BOOST_PP_DEC_674 673
# define BOOST_PP_DEC_675 674
# define BOOST_PP_DEC_676 675
# define BOOST_PP_DEC_677 676
# define BOOST_PP_DEC_678 677
# define BOOST_PP_DEC_679 678
# define BOOST_PP_DEC_680 679
# define BOOST_PP_DEC_681 680
# define BOOST_PP_DEC_682 681
# define BOOST_PP_DEC_683 682
# define BOOST_PP_DEC_684 683
# define BOOST_PP_DEC_685 684
# define BOOST_PP_DEC_686 685
# define BOOST_PP_DEC_687 686
# define BOOST_PP_DEC_688 687
# define BOOST_PP_DEC_689 688
# define BOOST_PP_DEC_690 689
# define BOOST_PP_DEC_691 690
# define BOOST_PP_DEC_692 691
# define BOOST_PP_DEC_693 692
# define BOOST_PP_DEC_694 693
# define BOOST_PP_DEC_695 694
# define BOOST_PP_DEC_696 695
# define BOOST_PP_DEC_697 696
# define BOOST_PP_DEC_698 697
# define BOOST_PP_DEC_699 698
# define BOOST_PP_DEC_700 699
# define BOOST_PP_DEC_701 700
# define BOOST_PP_DEC_702 701
# define BOOST_PP_DEC_703 702
# define BOOST_PP_DEC_704 703
# define BOOST_PP_DEC_705 704
# define BOOST_PP_DEC_706 705
# define BOOST_PP_DEC_707 706
# define BOOST_PP_DEC_708 707
# define BOOST_PP_DEC_709 708
# define BOOST_PP_DEC_710 709
# define BOOST_PP_DEC_711 710
# define BOOST_PP_DEC_712 711
# define BOOST_PP_DEC_713 712
# define BOOST_PP_DEC_714 713
# define BOOST_PP_DEC_715 714
# define BOOST_PP_DEC_716 715
# define BOOST_PP_DEC_717 716
# define BOOST_PP_DEC_718 717
# define BOOST_PP_DEC_719 718
# define BOOST_PP_DEC_720 719
# define BOOST_PP_DEC_721 720
# define BOOST_PP_DEC_722 721
# define BOOST_PP_DEC_723 722
# define BOOST_PP_DEC_724 723
# define BOOST_PP_DEC_725 724
# define BOOST_PP_DEC_726 725
# define BOOST_PP_DEC_727 726
# define BOOST_PP_DEC_728 727
# define BOOST_PP_DEC_729 728
# define BOOST_PP_DEC_730 729
# define BOOST_PP_DEC_731 730
# define BOOST_PP_DEC_732 731
# define BOOST_PP_DEC_733 732
# define BOOST_PP_DEC_734 733
# define BOOST_PP_DEC_735 734
# define BOOST_PP_DEC_736 735
# define BOOST_PP_DEC_737 736
# define BOOST_PP_DEC_738 737
# define BOOST_PP_DEC_739 738
# define BOOST_PP_DEC_740 739
# define BOOST_PP_DEC_741 740
# define BOOST_PP_DEC_742 741
# define BOOST_PP_DEC_743 742
# define BOOST_PP_DEC_744 743
# define BOOST_PP_DEC_745 744
# define BOOST_PP_DEC_746 745
# define BOOST_PP_DEC_747 746
# define BOOST_PP_DEC_748 747
# define BOOST_PP_DEC_749 748
# define BOOST_PP_DEC_750 749
# define BOOST_PP_DEC_751 750
# define BOOST_PP_DEC_752 751
# define BOOST_PP_DEC_753 752
# define BOOST_PP_DEC_754 753
# define BOOST_PP_DEC_755 754
# define BOOST_PP_DEC_756 755
# define BOOST_PP_DEC_757 756
# define BOOST_PP_DEC_758 757
# define BOOST_PP_DEC_759 758
# define BOOST_PP_DEC_760 759
# define BOOST_PP_DEC_761 760
# define BOOST_PP_DEC_762 761
# define BOOST_PP_DEC_763 762
# define BOOST_PP_DEC_764 763
# define BOOST_PP_DEC_765 764
# define BOOST_PP_DEC_766 765
# define BOOST_PP_DEC_767 766
# define BOOST_PP_DEC_768 767
# define BOOST_PP_DEC_769 768
# define BOOST_PP_DEC_770 769
# define BOOST_PP_DEC_771 770
# define BOOST_PP_DEC_772 771
# define BOOST_PP_DEC_773 772
# define BOOST_PP_DEC_774 773
# define BOOST_PP_DEC_775 774
# define BOOST_PP_DEC_776 775
# define BOOST_PP_DEC_777 776
# define BOOST_PP_DEC_778 777
# define BOOST_PP_DEC_779 778
# define BOOST_PP_DEC_780 779
# define BOOST_PP_DEC_781 780
# define BOOST_PP_DEC_782 781
# define BOOST_PP_DEC_783 782
# define BOOST_PP_DEC_784 783
# define BOOST_PP_DEC_785 784
# define BOOST_PP_DEC_786 785
# define BOOST_PP_DEC_787 786
# define BOOST_PP_DEC_788 787
# define BOOST_PP_DEC_789 788
# define BOOST_PP_DEC_790 789
# define BOOST_PP_DEC_791 790
# define BOOST_PP_DEC_792 791
# define BOOST_PP_DEC_793 792
# define BOOST_PP_DEC_794 793
# define BOOST_PP_DEC_795 794
# define BOOST_PP_DEC_796 795
# define BOOST_PP_DEC_797 796
# define BOOST_PP_DEC_798 797
# define BOOST_PP_DEC_799 798
# define BOOST_PP_DEC_800 799
# define BOOST_PP_DEC_801 800
# define BOOST_PP_DEC_802 801
# define BOOST_PP_DEC_803 802
# define BOOST_PP_DEC_804 803
# define BOOST_PP_DEC_805 804
# define BOOST_PP_DEC_806 805
# define BOOST_PP_DEC_807 806
# define BOOST_PP_DEC_808 807
# define BOOST_PP_DEC_809 808
# define BOOST_PP_DEC_810 809
# define BOOST_PP_DEC_811 810
# define BOOST_PP_DEC_812 811
# define BOOST_PP_DEC_813 812
# define BOOST_PP_DEC_814 813
# define BOOST_PP_DEC_815 814
# define BOOST_PP_DEC_816 815
# define BOOST_PP_DEC_817 816
# define BOOST_PP_DEC_818 817
# define BOOST_PP_DEC_819 818
# define BOOST_PP_DEC_820 819
# define BOOST_PP_DEC_821 820
# define BOOST_PP_DEC_822 821
# define BOOST_PP_DEC_823 822
# define BOOST_PP_DEC_824 823
# define BOOST_PP_DEC_825 824
# define BOOST_PP_DEC_826 825
# define BOOST_PP_DEC_827 826
# define BOOST_PP_DEC_828 827
# define BOOST_PP_DEC_829 828
# define BOOST_PP_DEC_830 829
# define BOOST_PP_DEC_831 830
# define BOOST_PP_DEC_832 831
# define BOOST_PP_DEC_833 832
# define BOOST_PP_DEC_834 833
# define BOOST_PP_DEC_835 834
# define BOOST_PP_DEC_836 835
# define BOOST_PP_DEC_837 836
# define BOOST_PP_DEC_838 837
# define BOOST_PP_DEC_839 838
# define BOOST_PP_DEC_840 839
# define BOOST_PP_DEC_841 840
# define BOOST_PP_DEC_842 841
# define BOOST_PP_DEC_843 842
# define BOOST_PP_DEC_844 843
# define BOOST_PP_DEC_845 844
# define BOOST_PP_DEC_846 845
# define BOOST_PP_DEC_847 846
# define BOOST_PP_DEC_848 847
# define BOOST_PP_DEC_849 848
# define BOOST_PP_DEC_850 849
# define BOOST_PP_DEC_851 850
# define BOOST_PP_DEC_852 851
# define BOOST_PP_DEC_853 852
# define BOOST_PP_DEC_854 853
# define BOOST_PP_DEC_855 854
# define BOOST_PP_DEC_856 855
# define BOOST_PP_DEC_857 856
# define BOOST_PP_DEC_858 857
# define BOOST_PP_DEC_859 858
# define BOOST_PP_DEC_860 859
# define BOOST_PP_DEC_861 860
# define BOOST_PP_DEC_862 861
# define BOOST_PP_DEC_863 862
# define BOOST_PP_DEC_864 863
# define BOOST_PP_DEC_865 864
# define BOOST_PP_DEC_866 865
# define BOOST_PP_DEC_867 866
# define BOOST_PP_DEC_868 867
# define BOOST_PP_DEC_869 868
# define BOOST_PP_DEC_870 869
# define BOOST_PP_DEC_871 870
# define BOOST_PP_DEC_872 871
# define BOOST_PP_DEC_873 872
# define BOOST_PP_DEC_874 873
# define BOOST_PP_DEC_875 874
# define BOOST_PP_DEC_876 875
# define BOOST_PP_DEC_877 876
# define BOOST_PP_DEC_878 877
# define BOOST_PP_DEC_879 878
# define BOOST_PP_DEC_880 879
# define BOOST_PP_DEC_881 880
# define BOOST_PP_DEC_882 881
# define BOOST_PP_DEC_883 882
# define BOOST_PP_DEC_884 883
# define BOOST_PP_DEC_885 884
# define BOOST_PP_DEC_886 885
# define BOOST_PP_DEC_887 886
# define BOOST_PP_DEC_888 887
# define BOOST_PP_DEC_889 888
# define BOOST_PP_DEC_890 889
# define BOOST_PP_DEC_891 890
# define BOOST_PP_DEC_892 891
# define BOOST_PP_DEC_893 892
# define BOOST_PP_DEC_894 893
# define BOOST_PP_DEC_895 894
# define BOOST_PP_DEC_896 895
# define BOOST_PP_DEC_897 896
# define BOOST_PP_DEC_898 897
# define BOOST_PP_DEC_899 898
# define BOOST_PP_DEC_900 899
# define BOOST_PP_DEC_901 900
# define BOOST_PP_DEC_902 901
# define BOOST_PP_DEC_903 902
# define BOOST_PP_DEC_904 903
# define BOOST_PP_DEC_905 904
# define BOOST_PP_DEC_906 905
# define BOOST_PP_DEC_907 906
# define BOOST_PP_DEC_908 907
# define BOOST_PP_DEC_909 908
# define BOOST_PP_DEC_910 909
# define BOOST_PP_DEC_911 910
# define BOOST_PP_DEC_912 911
# define BOOST_PP_DEC_913 912
# define BOOST_PP_DEC_914 913
# define BOOST_PP_DEC_915 914
# define BOOST_PP_DEC_916 915
# define BOOST_PP_DEC_917 916
# define BOOST_PP_DEC_918 917
# define BOOST_PP_DEC_919 918
# define BOOST_PP_DEC_920 919
# define BOOST_PP_DEC_921 920
# define BOOST_PP_DEC_922 921
# define BOOST_PP_DEC_923 922
# define BOOST_PP_DEC_924 923
# define BOOST_PP_DEC_925 924
# define BOOST_PP_DEC_926 925
# define BOOST_PP_DEC_927 926
# define BOOST_PP_DEC_928 927
# define BOOST_PP_DEC_929 928
# define BOOST_PP_DEC_930 929
# define BOOST_PP_DEC_931 930
# define BOOST_PP_DEC_932 931
# define BOOST_PP_DEC_933 932
# define BOOST_PP_DEC_934 933
# define BOOST_PP_DEC_935 934
# define BOOST_PP_DEC_936 935
# define BOOST_PP_DEC_937 936
# define BOOST_PP_DEC_938 937
# define BOOST_PP_DEC_939 938
# define BOOST_PP_DEC_940 939
# define BOOST_PP_DEC_941 940
# define BOOST_PP_DEC_942 941
# define BOOST_PP_DEC_943 942
# define BOOST_PP_DEC_944 943
# define BOOST_PP_DEC_945 944
# define BOOST_PP_DEC_946 945
# define BOOST_PP_DEC_947 946
# define BOOST_PP_DEC_948 947
# define BOOST_PP_DEC_949 948
# define BOOST_PP_DEC_950 949
# define BOOST_PP_DEC_951 950
# define BOOST_PP_DEC_952 951
# define BOOST_PP_DEC_953 952
# define BOOST_PP_DEC_954 953
# define BOOST_PP_DEC_955 954
# define BOOST_PP_DEC_956 955
# define BOOST_PP_DEC_957 956
# define BOOST_PP_DEC_958 957
# define BOOST_PP_DEC_959 958
# define BOOST_PP_DEC_960 959
# define BOOST_PP_DEC_961 960
# define BOOST_PP_DEC_962 961
# define BOOST_PP_DEC_963 962
# define BOOST_PP_DEC_964 963
# define BOOST_PP_DEC_965 964
# define BOOST_PP_DEC_966 965
# define BOOST_PP_DEC_967 966
# define BOOST_PP_DEC_968 967
# define BOOST_PP_DEC_969 968
# define BOOST_PP_DEC_970 969
# define BOOST_PP_DEC_971 970
# define BOOST_PP_DEC_972 971
# define BOOST_PP_DEC_973 972
# define BOOST_PP_DEC_974 973
# define BOOST_PP_DEC_975 974
# define BOOST_PP_DEC_976 975
# define BOOST_PP_DEC_977 976
# define BOOST_PP_DEC_978 977
# define BOOST_PP_DEC_979 978
# define BOOST_PP_DEC_980 979
# define BOOST_PP_DEC_981 980
# define BOOST_PP_DEC_982 981
# define BOOST_PP_DEC_983 982
# define BOOST_PP_DEC_984 983
# define BOOST_PP_DEC_985 984
# define BOOST_PP_DEC_986 985
# define BOOST_PP_DEC_987 986
# define BOOST_PP_DEC_988 987
# define BOOST_PP_DEC_989 988
# define BOOST_PP_DEC_990 989
# define BOOST_PP_DEC_991 990
# define BOOST_PP_DEC_992 991
# define BOOST_PP_DEC_993 992
# define BOOST_PP_DEC_994 993
# define BOOST_PP_DEC_995 994
# define BOOST_PP_DEC_996 995
# define BOOST_PP_DEC_997 996
# define BOOST_PP_DEC_998 997
# define BOOST_PP_DEC_999 998
# define BOOST_PP_DEC_1000 999
# define BOOST_PP_DEC_1001 1000
# define BOOST_PP_DEC_1002 1001
# define BOOST_PP_DEC_1003 1002
# define BOOST_PP_DEC_1004 1003
# define BOOST_PP_DEC_1005 1004
# define BOOST_PP_DEC_1006 1005
# define BOOST_PP_DEC_1007 1006
# define BOOST_PP_DEC_1008 1007
# define BOOST_PP_DEC_1009 1008
# define BOOST_PP_DEC_1010 1009
# define BOOST_PP_DEC_1011 1010
# define BOOST_PP_DEC_1012 1011
# define BOOST_PP_DEC_1013 1012
# define BOOST_PP_DEC_1014 1013
# define BOOST_PP_DEC_1015 1014
# define BOOST_PP_DEC_1016 1015
# define BOOST_PP_DEC_1017 1016
# define BOOST_PP_DEC_1018 1017
# define BOOST_PP_DEC_1019 1018
# define BOOST_PP_DEC_1020 1019
# define BOOST_PP_DEC_1021 1020
# define BOOST_PP_DEC_1022 1021
# define BOOST_PP_DEC_1023 1022
# define BOOST_PP_DEC_1024 1023
# define BOOST_PP_DEC_1025 1024
#
# endif
