from .clap import clap
from .default import default
from .kick import kick
from .multiband_compressor import multiband_compressor
from .simple_sine import simple_sine
from .sweep_filter import sweep_filter
from .system_synthdefs import (
    system_link_audio_1,
    system_link_audio_2,
    system_link_audio_3,
    system_link_audio_4,
    system_link_audio_5,
    system_link_audio_6,
    system_link_audio_7,
    system_link_audio_8,
    system_link_audio_9,
    system_link_audio_10,
    system_link_audio_11,
    system_link_audio_12,
    system_link_audio_13,
    system_link_audio_14,
    system_link_audio_15,
    system_link_audio_16,
    system_link_control_1,
    system_link_control_2,
    system_link_control_3,
    system_link_control_4,
    system_link_control_5,
    system_link_control_6,
    system_link_control_7,
    system_link_control_8,
    system_link_control_9,
    system_link_control_10,
    system_link_control_11,
    system_link_control_12,
    system_link_control_13,
    system_link_control_14,
    system_link_control_15,
    system_link_control_16,
)
from .test import test, test_two_voice

__all__ = [
    "clap",
    "default",
    "kick",
    "multiband_compressor",
    "simple_sine",
    "sweep_filter",
    "system_link_audio_1",
    "system_link_audio_10",
    "system_link_audio_11",
    "system_link_audio_12",
    "system_link_audio_13",
    "system_link_audio_14",
    "system_link_audio_15",
    "system_link_audio_16",
    "system_link_audio_2",
    "system_link_audio_3",
    "system_link_audio_4",
    "system_link_audio_5",
    "system_link_audio_6",
    "system_link_audio_7",
    "system_link_audio_8",
    "system_link_audio_9",
    "system_link_control_1",
    "system_link_control_10",
    "system_link_control_11",
    "system_link_control_12",
    "system_link_control_13",
    "system_link_control_14",
    "system_link_control_15",
    "system_link_control_16",
    "system_link_control_2",
    "system_link_control_3",
    "system_link_control_4",
    "system_link_control_5",
    "system_link_control_6",
    "system_link_control_7",
    "system_link_control_8",
    "system_link_control_9",
    "test",
    "test_two_voice",
]
