# /* **************************************************************************
#  *                                                                          *
#  *     (C) Copyright Paul Mensonides 2002.
#  *     Distributed under the Boost Software License, Version 1.0. (See
#  *     accompanying file LICENSE_1_0.txt or copy at
#  *     http://www.boost.org/LICENSE_1_0.txt)
#  *                                                                          *
#  ************************************************************************** */
#
# /* Revised by Edward Diener (2020) */
#
# /* See http://www.boost.org for most recent version. */
#
# if !defined(BOOST_PP_LOCAL_LIMITS)
#    error BOOST_PP_ERROR:  local iteration boundaries are not defined
# elif !defined(BOOST_PP_LOCAL_MACRO)
#    error BOOST_PP_ERROR:  local iteration target macro is not defined
# else
#    if ~BOOST_PP_CONFIG_FLAGS() & BOOST_PP_CONFIG_EDG()
#        define BOOST_PP_LOCAL_S BOOST_PP_TUPLE_ELEM(2, 0, BOOST_PP_LOCAL_LIMITS)
#        define BOOST_PP_LOCAL_F BOOST_PP_TUPLE_ELEM(2, 1, BOOST_PP_LOCAL_LIMITS)
#    else
#        define BOOST_PP_VALUE BOOST_PP_TUPLE_ELEM(2, 0, BOOST_PP_LOCAL_LIMITS)
#        include <boost/preprocessor/iteration/detail/start.hpp>
#        define BOOST_PP_VALUE BOOST_PP_TUPLE_ELEM(2, 1, BOOST_PP_LOCAL_LIMITS)
#        include <boost/preprocessor/iteration/detail/finish.hpp>
#        define BOOST_PP_LOCAL_S BOOST_PP_LOCAL_SE()
#        define BOOST_PP_LOCAL_F BOOST_PP_LOCAL_FE()
#    endif
# endif
#
# if (BOOST_PP_LOCAL_S) > (BOOST_PP_LOCAL_F)
#    include <boost/preprocessor/iteration/detail/rlocal.hpp>
# else
#
# include <boost/preprocessor/config/config.hpp>
#
# if ~BOOST_PP_CONFIG_FLAGS() & BOOST_PP_CONFIG_STRICT()
#
#    if BOOST_PP_LOCAL_C(0)
        BOOST_PP_LOCAL_MACRO(0)
#    endif
#    if BOOST_PP_LOCAL_C(1)
        BOOST_PP_LOCAL_MACRO(1)
#    endif
#    if BOOST_PP_LOCAL_C(2)
        BOOST_PP_LOCAL_MACRO(2)
#    endif
#    if BOOST_PP_LOCAL_C(3)
        BOOST_PP_LOCAL_MACRO(3)
#    endif
#    if BOOST_PP_LOCAL_C(4)
        BOOST_PP_LOCAL_MACRO(4)
#    endif
#    if BOOST_PP_LOCAL_C(5)
        BOOST_PP_LOCAL_MACRO(5)
#    endif
#    if BOOST_PP_LOCAL_C(6)
        BOOST_PP_LOCAL_MACRO(6)
#    endif
#    if BOOST_PP_LOCAL_C(7)
        BOOST_PP_LOCAL_MACRO(7)
#    endif
#    if BOOST_PP_LOCAL_C(8)
        BOOST_PP_LOCAL_MACRO(8)
#    endif
#    if BOOST_PP_LOCAL_C(9)
        BOOST_PP_LOCAL_MACRO(9)
#    endif
#    if BOOST_PP_LOCAL_C(10)
        BOOST_PP_LOCAL_MACRO(10)
#    endif
#    if BOOST_PP_LOCAL_C(11)
        BOOST_PP_LOCAL_MACRO(11)
#    endif
#    if BOOST_PP_LOCAL_C(12)
        BOOST_PP_LOCAL_MACRO(12)
#    endif
#    if BOOST_PP_LOCAL_C(13)
        BOOST_PP_LOCAL_MACRO(13)
#    endif
#    if BOOST_PP_LOCAL_C(14)
        BOOST_PP_LOCAL_MACRO(14)
#    endif
#    if BOOST_PP_LOCAL_C(15)
        BOOST_PP_LOCAL_MACRO(15)
#    endif
#    if BOOST_PP_LOCAL_C(16)
        BOOST_PP_LOCAL_MACRO(16)
#    endif
#    if BOOST_PP_LOCAL_C(17)
        BOOST_PP_LOCAL_MACRO(17)
#    endif
#    if BOOST_PP_LOCAL_C(18)
        BOOST_PP_LOCAL_MACRO(18)
#    endif
#    if BOOST_PP_LOCAL_C(19)
        BOOST_PP_LOCAL_MACRO(19)
#    endif
#    if BOOST_PP_LOCAL_C(20)
        BOOST_PP_LOCAL_MACRO(20)
#    endif
#    if BOOST_PP_LOCAL_C(21)
        BOOST_PP_LOCAL_MACRO(21)
#    endif
#    if BOOST_PP_LOCAL_C(22)
        BOOST_PP_LOCAL_MACRO(22)
#    endif
#    if BOOST_PP_LOCAL_C(23)
        BOOST_PP_LOCAL_MACRO(23)
#    endif
#    if BOOST_PP_LOCAL_C(24)
        BOOST_PP_LOCAL_MACRO(24)
#    endif
#    if BOOST_PP_LOCAL_C(25)
        BOOST_PP_LOCAL_MACRO(25)
#    endif
#    if BOOST_PP_LOCAL_C(26)
        BOOST_PP_LOCAL_MACRO(26)
#    endif
#    if BOOST_PP_LOCAL_C(27)
        BOOST_PP_LOCAL_MACRO(27)
#    endif
#    if BOOST_PP_LOCAL_C(28)
        BOOST_PP_LOCAL_MACRO(28)
#    endif
#    if BOOST_PP_LOCAL_C(29)
        BOOST_PP_LOCAL_MACRO(29)
#    endif
#    if BOOST_PP_LOCAL_C(30)
        BOOST_PP_LOCAL_MACRO(30)
#    endif
#    if BOOST_PP_LOCAL_C(31)
        BOOST_PP_LOCAL_MACRO(31)
#    endif
#    if BOOST_PP_LOCAL_C(32)
        BOOST_PP_LOCAL_MACRO(32)
#    endif
#    if BOOST_PP_LOCAL_C(33)
        BOOST_PP_LOCAL_MACRO(33)
#    endif
#    if BOOST_PP_LOCAL_C(34)
        BOOST_PP_LOCAL_MACRO(34)
#    endif
#    if BOOST_PP_LOCAL_C(35)
        BOOST_PP_LOCAL_MACRO(35)
#    endif
#    if BOOST_PP_LOCAL_C(36)
        BOOST_PP_LOCAL_MACRO(36)
#    endif
#    if BOOST_PP_LOCAL_C(37)
        BOOST_PP_LOCAL_MACRO(37)
#    endif
#    if BOOST_PP_LOCAL_C(38)
        BOOST_PP_LOCAL_MACRO(38)
#    endif
#    if BOOST_PP_LOCAL_C(39)
        BOOST_PP_LOCAL_MACRO(39)
#    endif
#    if BOOST_PP_LOCAL_C(40)
        BOOST_PP_LOCAL_MACRO(40)
#    endif
#    if BOOST_PP_LOCAL_C(41)
        BOOST_PP_LOCAL_MACRO(41)
#    endif
#    if BOOST_PP_LOCAL_C(42)
        BOOST_PP_LOCAL_MACRO(42)
#    endif
#    if BOOST_PP_LOCAL_C(43)
        BOOST_PP_LOCAL_MACRO(43)
#    endif
#    if BOOST_PP_LOCAL_C(44)
        BOOST_PP_LOCAL_MACRO(44)
#    endif
#    if BOOST_PP_LOCAL_C(45)
        BOOST_PP_LOCAL_MACRO(45)
#    endif
#    if BOOST_PP_LOCAL_C(46)
        BOOST_PP_LOCAL_MACRO(46)
#    endif
#    if BOOST_PP_LOCAL_C(47)
        BOOST_PP_LOCAL_MACRO(47)
#    endif
#    if BOOST_PP_LOCAL_C(48)
        BOOST_PP_LOCAL_MACRO(48)
#    endif
#    if BOOST_PP_LOCAL_C(49)
        BOOST_PP_LOCAL_MACRO(49)
#    endif
#    if BOOST_PP_LOCAL_C(50)
        BOOST_PP_LOCAL_MACRO(50)
#    endif
#    if BOOST_PP_LOCAL_C(51)
        BOOST_PP_LOCAL_MACRO(51)
#    endif
#    if BOOST_PP_LOCAL_C(52)
        BOOST_PP_LOCAL_MACRO(52)
#    endif
#    if BOOST_PP_LOCAL_C(53)
        BOOST_PP_LOCAL_MACRO(53)
#    endif
#    if BOOST_PP_LOCAL_C(54)
        BOOST_PP_LOCAL_MACRO(54)
#    endif
#    if BOOST_PP_LOCAL_C(55)
        BOOST_PP_LOCAL_MACRO(55)
#    endif
#    if BOOST_PP_LOCAL_C(56)
        BOOST_PP_LOCAL_MACRO(56)
#    endif
#    if BOOST_PP_LOCAL_C(57)
        BOOST_PP_LOCAL_MACRO(57)
#    endif
#    if BOOST_PP_LOCAL_C(58)
        BOOST_PP_LOCAL_MACRO(58)
#    endif
#    if BOOST_PP_LOCAL_C(59)
        BOOST_PP_LOCAL_MACRO(59)
#    endif
#    if BOOST_PP_LOCAL_C(60)
        BOOST_PP_LOCAL_MACRO(60)
#    endif
#    if BOOST_PP_LOCAL_C(61)
        BOOST_PP_LOCAL_MACRO(61)
#    endif
#    if BOOST_PP_LOCAL_C(62)
        BOOST_PP_LOCAL_MACRO(62)
#    endif
#    if BOOST_PP_LOCAL_C(63)
        BOOST_PP_LOCAL_MACRO(63)
#    endif
#    if BOOST_PP_LOCAL_C(64)
        BOOST_PP_LOCAL_MACRO(64)
#    endif
#    if BOOST_PP_LOCAL_C(65)
        BOOST_PP_LOCAL_MACRO(65)
#    endif
#    if BOOST_PP_LOCAL_C(66)
        BOOST_PP_LOCAL_MACRO(66)
#    endif
#    if BOOST_PP_LOCAL_C(67)
        BOOST_PP_LOCAL_MACRO(67)
#    endif
#    if BOOST_PP_LOCAL_C(68)
        BOOST_PP_LOCAL_MACRO(68)
#    endif
#    if BOOST_PP_LOCAL_C(69)
        BOOST_PP_LOCAL_MACRO(69)
#    endif
#    if BOOST_PP_LOCAL_C(70)
        BOOST_PP_LOCAL_MACRO(70)
#    endif
#    if BOOST_PP_LOCAL_C(71)
        BOOST_PP_LOCAL_MACRO(71)
#    endif
#    if BOOST_PP_LOCAL_C(72)
        BOOST_PP_LOCAL_MACRO(72)
#    endif
#    if BOOST_PP_LOCAL_C(73)
        BOOST_PP_LOCAL_MACRO(73)
#    endif
#    if BOOST_PP_LOCAL_C(74)
        BOOST_PP_LOCAL_MACRO(74)
#    endif
#    if BOOST_PP_LOCAL_C(75)
        BOOST_PP_LOCAL_MACRO(75)
#    endif
#    if BOOST_PP_LOCAL_C(76)
        BOOST_PP_LOCAL_MACRO(76)
#    endif
#    if BOOST_PP_LOCAL_C(77)
        BOOST_PP_LOCAL_MACRO(77)
#    endif
#    if BOOST_PP_LOCAL_C(78)
        BOOST_PP_LOCAL_MACRO(78)
#    endif
#    if BOOST_PP_LOCAL_C(79)
        BOOST_PP_LOCAL_MACRO(79)
#    endif
#    if BOOST_PP_LOCAL_C(80)
        BOOST_PP_LOCAL_MACRO(80)
#    endif
#    if BOOST_PP_LOCAL_C(81)
        BOOST_PP_LOCAL_MACRO(81)
#    endif
#    if BOOST_PP_LOCAL_C(82)
        BOOST_PP_LOCAL_MACRO(82)
#    endif
#    if BOOST_PP_LOCAL_C(83)
        BOOST_PP_LOCAL_MACRO(83)
#    endif
#    if BOOST_PP_LOCAL_C(84)
        BOOST_PP_LOCAL_MACRO(84)
#    endif
#    if BOOST_PP_LOCAL_C(85)
        BOOST_PP_LOCAL_MACRO(85)
#    endif
#    if BOOST_PP_LOCAL_C(86)
        BOOST_PP_LOCAL_MACRO(86)
#    endif
#    if BOOST_PP_LOCAL_C(87)
        BOOST_PP_LOCAL_MACRO(87)
#    endif
#    if BOOST_PP_LOCAL_C(88)
        BOOST_PP_LOCAL_MACRO(88)
#    endif
#    if BOOST_PP_LOCAL_C(89)
        BOOST_PP_LOCAL_MACRO(89)
#    endif
#    if BOOST_PP_LOCAL_C(90)
        BOOST_PP_LOCAL_MACRO(90)
#    endif
#    if BOOST_PP_LOCAL_C(91)
        BOOST_PP_LOCAL_MACRO(91)
#    endif
#    if BOOST_PP_LOCAL_C(92)
        BOOST_PP_LOCAL_MACRO(92)
#    endif
#    if BOOST_PP_LOCAL_C(93)
        BOOST_PP_LOCAL_MACRO(93)
#    endif
#    if BOOST_PP_LOCAL_C(94)
        BOOST_PP_LOCAL_MACRO(94)
#    endif
#    if BOOST_PP_LOCAL_C(95)
        BOOST_PP_LOCAL_MACRO(95)
#    endif
#    if BOOST_PP_LOCAL_C(96)
        BOOST_PP_LOCAL_MACRO(96)
#    endif
#    if BOOST_PP_LOCAL_C(97)
        BOOST_PP_LOCAL_MACRO(97)
#    endif
#    if BOOST_PP_LOCAL_C(98)
        BOOST_PP_LOCAL_MACRO(98)
#    endif
#    if BOOST_PP_LOCAL_C(99)
        BOOST_PP_LOCAL_MACRO(99)
#    endif
#    if BOOST_PP_LOCAL_C(100)
        BOOST_PP_LOCAL_MACRO(100)
#    endif
#    if BOOST_PP_LOCAL_C(101)
        BOOST_PP_LOCAL_MACRO(101)
#    endif
#    if BOOST_PP_LOCAL_C(102)
        BOOST_PP_LOCAL_MACRO(102)
#    endif
#    if BOOST_PP_LOCAL_C(103)
        BOOST_PP_LOCAL_MACRO(103)
#    endif
#    if BOOST_PP_LOCAL_C(104)
        BOOST_PP_LOCAL_MACRO(104)
#    endif
#    if BOOST_PP_LOCAL_C(105)
        BOOST_PP_LOCAL_MACRO(105)
#    endif
#    if BOOST_PP_LOCAL_C(106)
        BOOST_PP_LOCAL_MACRO(106)
#    endif
#    if BOOST_PP_LOCAL_C(107)
        BOOST_PP_LOCAL_MACRO(107)
#    endif
#    if BOOST_PP_LOCAL_C(108)
        BOOST_PP_LOCAL_MACRO(108)
#    endif
#    if BOOST_PP_LOCAL_C(109)
        BOOST_PP_LOCAL_MACRO(109)
#    endif
#    if BOOST_PP_LOCAL_C(110)
        BOOST_PP_LOCAL_MACRO(110)
#    endif
#    if BOOST_PP_LOCAL_C(111)
        BOOST_PP_LOCAL_MACRO(111)
#    endif
#    if BOOST_PP_LOCAL_C(112)
        BOOST_PP_LOCAL_MACRO(112)
#    endif
#    if BOOST_PP_LOCAL_C(113)
        BOOST_PP_LOCAL_MACRO(113)
#    endif
#    if BOOST_PP_LOCAL_C(114)
        BOOST_PP_LOCAL_MACRO(114)
#    endif
#    if BOOST_PP_LOCAL_C(115)
        BOOST_PP_LOCAL_MACRO(115)
#    endif
#    if BOOST_PP_LOCAL_C(116)
        BOOST_PP_LOCAL_MACRO(116)
#    endif
#    if BOOST_PP_LOCAL_C(117)
        BOOST_PP_LOCAL_MACRO(117)
#    endif
#    if BOOST_PP_LOCAL_C(118)
        BOOST_PP_LOCAL_MACRO(118)
#    endif
#    if BOOST_PP_LOCAL_C(119)
        BOOST_PP_LOCAL_MACRO(119)
#    endif
#    if BOOST_PP_LOCAL_C(120)
        BOOST_PP_LOCAL_MACRO(120)
#    endif
#    if BOOST_PP_LOCAL_C(121)
        BOOST_PP_LOCAL_MACRO(121)
#    endif
#    if BOOST_PP_LOCAL_C(122)
        BOOST_PP_LOCAL_MACRO(122)
#    endif
#    if BOOST_PP_LOCAL_C(123)
        BOOST_PP_LOCAL_MACRO(123)
#    endif
#    if BOOST_PP_LOCAL_C(124)
        BOOST_PP_LOCAL_MACRO(124)
#    endif
#    if BOOST_PP_LOCAL_C(125)
        BOOST_PP_LOCAL_MACRO(125)
#    endif
#    if BOOST_PP_LOCAL_C(126)
        BOOST_PP_LOCAL_MACRO(126)
#    endif
#    if BOOST_PP_LOCAL_C(127)
        BOOST_PP_LOCAL_MACRO(127)
#    endif
#    if BOOST_PP_LOCAL_C(128)
        BOOST_PP_LOCAL_MACRO(128)
#    endif
#    if BOOST_PP_LOCAL_C(129)
        BOOST_PP_LOCAL_MACRO(129)
#    endif
#    if BOOST_PP_LOCAL_C(130)
        BOOST_PP_LOCAL_MACRO(130)
#    endif
#    if BOOST_PP_LOCAL_C(131)
        BOOST_PP_LOCAL_MACRO(131)
#    endif
#    if BOOST_PP_LOCAL_C(132)
        BOOST_PP_LOCAL_MACRO(132)
#    endif
#    if BOOST_PP_LOCAL_C(133)
        BOOST_PP_LOCAL_MACRO(133)
#    endif
#    if BOOST_PP_LOCAL_C(134)
        BOOST_PP_LOCAL_MACRO(134)
#    endif
#    if BOOST_PP_LOCAL_C(135)
        BOOST_PP_LOCAL_MACRO(135)
#    endif
#    if BOOST_PP_LOCAL_C(136)
        BOOST_PP_LOCAL_MACRO(136)
#    endif
#    if BOOST_PP_LOCAL_C(137)
        BOOST_PP_LOCAL_MACRO(137)
#    endif
#    if BOOST_PP_LOCAL_C(138)
        BOOST_PP_LOCAL_MACRO(138)
#    endif
#    if BOOST_PP_LOCAL_C(139)
        BOOST_PP_LOCAL_MACRO(139)
#    endif
#    if BOOST_PP_LOCAL_C(140)
        BOOST_PP_LOCAL_MACRO(140)
#    endif
#    if BOOST_PP_LOCAL_C(141)
        BOOST_PP_LOCAL_MACRO(141)
#    endif
#    if BOOST_PP_LOCAL_C(142)
        BOOST_PP_LOCAL_MACRO(142)
#    endif
#    if BOOST_PP_LOCAL_C(143)
        BOOST_PP_LOCAL_MACRO(143)
#    endif
#    if BOOST_PP_LOCAL_C(144)
        BOOST_PP_LOCAL_MACRO(144)
#    endif
#    if BOOST_PP_LOCAL_C(145)
        BOOST_PP_LOCAL_MACRO(145)
#    endif
#    if BOOST_PP_LOCAL_C(146)
        BOOST_PP_LOCAL_MACRO(146)
#    endif
#    if BOOST_PP_LOCAL_C(147)
        BOOST_PP_LOCAL_MACRO(147)
#    endif
#    if BOOST_PP_LOCAL_C(148)
        BOOST_PP_LOCAL_MACRO(148)
#    endif
#    if BOOST_PP_LOCAL_C(149)
        BOOST_PP_LOCAL_MACRO(149)
#    endif
#    if BOOST_PP_LOCAL_C(150)
        BOOST_PP_LOCAL_MACRO(150)
#    endif
#    if BOOST_PP_LOCAL_C(151)
        BOOST_PP_LOCAL_MACRO(151)
#    endif
#    if BOOST_PP_LOCAL_C(152)
        BOOST_PP_LOCAL_MACRO(152)
#    endif
#    if BOOST_PP_LOCAL_C(153)
        BOOST_PP_LOCAL_MACRO(153)
#    endif
#    if BOOST_PP_LOCAL_C(154)
        BOOST_PP_LOCAL_MACRO(154)
#    endif
#    if BOOST_PP_LOCAL_C(155)
        BOOST_PP_LOCAL_MACRO(155)
#    endif
#    if BOOST_PP_LOCAL_C(156)
        BOOST_PP_LOCAL_MACRO(156)
#    endif
#    if BOOST_PP_LOCAL_C(157)
        BOOST_PP_LOCAL_MACRO(157)
#    endif
#    if BOOST_PP_LOCAL_C(158)
        BOOST_PP_LOCAL_MACRO(158)
#    endif
#    if BOOST_PP_LOCAL_C(159)
        BOOST_PP_LOCAL_MACRO(159)
#    endif
#    if BOOST_PP_LOCAL_C(160)
        BOOST_PP_LOCAL_MACRO(160)
#    endif
#    if BOOST_PP_LOCAL_C(161)
        BOOST_PP_LOCAL_MACRO(161)
#    endif
#    if BOOST_PP_LOCAL_C(162)
        BOOST_PP_LOCAL_MACRO(162)
#    endif
#    if BOOST_PP_LOCAL_C(163)
        BOOST_PP_LOCAL_MACRO(163)
#    endif
#    if BOOST_PP_LOCAL_C(164)
        BOOST_PP_LOCAL_MACRO(164)
#    endif
#    if BOOST_PP_LOCAL_C(165)
        BOOST_PP_LOCAL_MACRO(165)
#    endif
#    if BOOST_PP_LOCAL_C(166)
        BOOST_PP_LOCAL_MACRO(166)
#    endif
#    if BOOST_PP_LOCAL_C(167)
        BOOST_PP_LOCAL_MACRO(167)
#    endif
#    if BOOST_PP_LOCAL_C(168)
        BOOST_PP_LOCAL_MACRO(168)
#    endif
#    if BOOST_PP_LOCAL_C(169)
        BOOST_PP_LOCAL_MACRO(169)
#    endif
#    if BOOST_PP_LOCAL_C(170)
        BOOST_PP_LOCAL_MACRO(170)
#    endif
#    if BOOST_PP_LOCAL_C(171)
        BOOST_PP_LOCAL_MACRO(171)
#    endif
#    if BOOST_PP_LOCAL_C(172)
        BOOST_PP_LOCAL_MACRO(172)
#    endif
#    if BOOST_PP_LOCAL_C(173)
        BOOST_PP_LOCAL_MACRO(173)
#    endif
#    if BOOST_PP_LOCAL_C(174)
        BOOST_PP_LOCAL_MACRO(174)
#    endif
#    if BOOST_PP_LOCAL_C(175)
        BOOST_PP_LOCAL_MACRO(175)
#    endif
#    if BOOST_PP_LOCAL_C(176)
        BOOST_PP_LOCAL_MACRO(176)
#    endif
#    if BOOST_PP_LOCAL_C(177)
        BOOST_PP_LOCAL_MACRO(177)
#    endif
#    if BOOST_PP_LOCAL_C(178)
        BOOST_PP_LOCAL_MACRO(178)
#    endif
#    if BOOST_PP_LOCAL_C(179)
        BOOST_PP_LOCAL_MACRO(179)
#    endif
#    if BOOST_PP_LOCAL_C(180)
        BOOST_PP_LOCAL_MACRO(180)
#    endif
#    if BOOST_PP_LOCAL_C(181)
        BOOST_PP_LOCAL_MACRO(181)
#    endif
#    if BOOST_PP_LOCAL_C(182)
        BOOST_PP_LOCAL_MACRO(182)
#    endif
#    if BOOST_PP_LOCAL_C(183)
        BOOST_PP_LOCAL_MACRO(183)
#    endif
#    if BOOST_PP_LOCAL_C(184)
        BOOST_PP_LOCAL_MACRO(184)
#    endif
#    if BOOST_PP_LOCAL_C(185)
        BOOST_PP_LOCAL_MACRO(185)
#    endif
#    if BOOST_PP_LOCAL_C(186)
        BOOST_PP_LOCAL_MACRO(186)
#    endif
#    if BOOST_PP_LOCAL_C(187)
        BOOST_PP_LOCAL_MACRO(187)
#    endif
#    if BOOST_PP_LOCAL_C(188)
        BOOST_PP_LOCAL_MACRO(188)
#    endif
#    if BOOST_PP_LOCAL_C(189)
        BOOST_PP_LOCAL_MACRO(189)
#    endif
#    if BOOST_PP_LOCAL_C(190)
        BOOST_PP_LOCAL_MACRO(190)
#    endif
#    if BOOST_PP_LOCAL_C(191)
        BOOST_PP_LOCAL_MACRO(191)
#    endif
#    if BOOST_PP_LOCAL_C(192)
        BOOST_PP_LOCAL_MACRO(192)
#    endif
#    if BOOST_PP_LOCAL_C(193)
        BOOST_PP_LOCAL_MACRO(193)
#    endif
#    if BOOST_PP_LOCAL_C(194)
        BOOST_PP_LOCAL_MACRO(194)
#    endif
#    if BOOST_PP_LOCAL_C(195)
        BOOST_PP_LOCAL_MACRO(195)
#    endif
#    if BOOST_PP_LOCAL_C(196)
        BOOST_PP_LOCAL_MACRO(196)
#    endif
#    if BOOST_PP_LOCAL_C(197)
        BOOST_PP_LOCAL_MACRO(197)
#    endif
#    if BOOST_PP_LOCAL_C(198)
        BOOST_PP_LOCAL_MACRO(198)
#    endif
#    if BOOST_PP_LOCAL_C(199)
        BOOST_PP_LOCAL_MACRO(199)
#    endif
#    if BOOST_PP_LOCAL_C(200)
        BOOST_PP_LOCAL_MACRO(200)
#    endif
#    if BOOST_PP_LOCAL_C(201)
        BOOST_PP_LOCAL_MACRO(201)
#    endif
#    if BOOST_PP_LOCAL_C(202)
        BOOST_PP_LOCAL_MACRO(202)
#    endif
#    if BOOST_PP_LOCAL_C(203)
        BOOST_PP_LOCAL_MACRO(203)
#    endif
#    if BOOST_PP_LOCAL_C(204)
        BOOST_PP_LOCAL_MACRO(204)
#    endif
#    if BOOST_PP_LOCAL_C(205)
        BOOST_PP_LOCAL_MACRO(205)
#    endif
#    if BOOST_PP_LOCAL_C(206)
        BOOST_PP_LOCAL_MACRO(206)
#    endif
#    if BOOST_PP_LOCAL_C(207)
        BOOST_PP_LOCAL_MACRO(207)
#    endif
#    if BOOST_PP_LOCAL_C(208)
        BOOST_PP_LOCAL_MACRO(208)
#    endif
#    if BOOST_PP_LOCAL_C(209)
        BOOST_PP_LOCAL_MACRO(209)
#    endif
#    if BOOST_PP_LOCAL_C(210)
        BOOST_PP_LOCAL_MACRO(210)
#    endif
#    if BOOST_PP_LOCAL_C(211)
        BOOST_PP_LOCAL_MACRO(211)
#    endif
#    if BOOST_PP_LOCAL_C(212)
        BOOST_PP_LOCAL_MACRO(212)
#    endif
#    if BOOST_PP_LOCAL_C(213)
        BOOST_PP_LOCAL_MACRO(213)
#    endif
#    if BOOST_PP_LOCAL_C(214)
        BOOST_PP_LOCAL_MACRO(214)
#    endif
#    if BOOST_PP_LOCAL_C(215)
        BOOST_PP_LOCAL_MACRO(215)
#    endif
#    if BOOST_PP_LOCAL_C(216)
        BOOST_PP_LOCAL_MACRO(216)
#    endif
#    if BOOST_PP_LOCAL_C(217)
        BOOST_PP_LOCAL_MACRO(217)
#    endif
#    if BOOST_PP_LOCAL_C(218)
        BOOST_PP_LOCAL_MACRO(218)
#    endif
#    if BOOST_PP_LOCAL_C(219)
        BOOST_PP_LOCAL_MACRO(219)
#    endif
#    if BOOST_PP_LOCAL_C(220)
        BOOST_PP_LOCAL_MACRO(220)
#    endif
#    if BOOST_PP_LOCAL_C(221)
        BOOST_PP_LOCAL_MACRO(221)
#    endif
#    if BOOST_PP_LOCAL_C(222)
        BOOST_PP_LOCAL_MACRO(222)
#    endif
#    if BOOST_PP_LOCAL_C(223)
        BOOST_PP_LOCAL_MACRO(223)
#    endif
#    if BOOST_PP_LOCAL_C(224)
        BOOST_PP_LOCAL_MACRO(224)
#    endif
#    if BOOST_PP_LOCAL_C(225)
        BOOST_PP_LOCAL_MACRO(225)
#    endif
#    if BOOST_PP_LOCAL_C(226)
        BOOST_PP_LOCAL_MACRO(226)
#    endif
#    if BOOST_PP_LOCAL_C(227)
        BOOST_PP_LOCAL_MACRO(227)
#    endif
#    if BOOST_PP_LOCAL_C(228)
        BOOST_PP_LOCAL_MACRO(228)
#    endif
#    if BOOST_PP_LOCAL_C(229)
        BOOST_PP_LOCAL_MACRO(229)
#    endif
#    if BOOST_PP_LOCAL_C(230)
        BOOST_PP_LOCAL_MACRO(230)
#    endif
#    if BOOST_PP_LOCAL_C(231)
        BOOST_PP_LOCAL_MACRO(231)
#    endif
#    if BOOST_PP_LOCAL_C(232)
        BOOST_PP_LOCAL_MACRO(232)
#    endif
#    if BOOST_PP_LOCAL_C(233)
        BOOST_PP_LOCAL_MACRO(233)
#    endif
#    if BOOST_PP_LOCAL_C(234)
        BOOST_PP_LOCAL_MACRO(234)
#    endif
#    if BOOST_PP_LOCAL_C(235)
        BOOST_PP_LOCAL_MACRO(235)
#    endif
#    if BOOST_PP_LOCAL_C(236)
        BOOST_PP_LOCAL_MACRO(236)
#    endif

#    if BOOST_PP_LOCAL_C(237)
        BOOST_PP_LOCAL_MACRO(237)
#    endif
#    if BOOST_PP_LOCAL_C(238)
        BOOST_PP_LOCAL_MACRO(238)
#    endif
#    if BOOST_PP_LOCAL_C(239)
        BOOST_PP_LOCAL_MACRO(239)
#    endif
#    if BOOST_PP_LOCAL_C(240)
        BOOST_PP_LOCAL_MACRO(240)
#    endif
#    if BOOST_PP_LOCAL_C(241)
        BOOST_PP_LOCAL_MACRO(241)
#    endif
#    if BOOST_PP_LOCAL_C(242)
        BOOST_PP_LOCAL_MACRO(242)
#    endif
#    if BOOST_PP_LOCAL_C(243)
        BOOST_PP_LOCAL_MACRO(243)
#    endif
#    if BOOST_PP_LOCAL_C(244)
        BOOST_PP_LOCAL_MACRO(244)
#    endif
#    if BOOST_PP_LOCAL_C(245)
        BOOST_PP_LOCAL_MACRO(245)
#    endif
#    if BOOST_PP_LOCAL_C(246)
        BOOST_PP_LOCAL_MACRO(246)
#    endif
#    if BOOST_PP_LOCAL_C(247)
        BOOST_PP_LOCAL_MACRO(247)
#    endif
#    if BOOST_PP_LOCAL_C(248)
        BOOST_PP_LOCAL_MACRO(248)
#    endif
#    if BOOST_PP_LOCAL_C(249)
        BOOST_PP_LOCAL_MACRO(249)
#    endif
#    if BOOST_PP_LOCAL_C(250)
        BOOST_PP_LOCAL_MACRO(250)
#    endif
#    if BOOST_PP_LOCAL_C(251)
        BOOST_PP_LOCAL_MACRO(251)
#    endif
#    if BOOST_PP_LOCAL_C(252)
        BOOST_PP_LOCAL_MACRO(252)
#    endif
#    if BOOST_PP_LOCAL_C(253)
        BOOST_PP_LOCAL_MACRO(253)
#    endif
#    if BOOST_PP_LOCAL_C(254)
        BOOST_PP_LOCAL_MACRO(254)
#    endif
#    if BOOST_PP_LOCAL_C(255)
        BOOST_PP_LOCAL_MACRO(255)
#    endif
#    if BOOST_PP_LOCAL_C(256)
        BOOST_PP_LOCAL_MACRO(256)
#    endif
#
# else
#
#    include <boost/preprocessor/config/limits.hpp>
#    
#    if BOOST_PP_LIMIT_ITERATION == 256
#    include <boost/preprocessor/iteration/detail/limits/local_256.hpp>
#    elif BOOST_PP_LIMIT_ITERATION == 512
#    include <boost/preprocessor/iteration/detail/limits/local_256.hpp>
#    include <boost/preprocessor/iteration/detail/limits/local_512.hpp>
#    elif BOOST_PP_LIMIT_ITERATION == 1024
#    include <boost/preprocessor/iteration/detail/limits/local_256.hpp>
#    include <boost/preprocessor/iteration/detail/limits/local_512.hpp>
#    include <boost/preprocessor/iteration/detail/limits/local_1024.hpp>
#    else
#    error Incorrect value for the BOOST_PP_LIMIT_ITERATION limit
#    endif
#
# endif
#
# endif
#
# undef BOOST_PP_LOCAL_LIMITS
#
# undef BOOST_PP_LOCAL_S
# undef BOOST_PP_LOCAL_F
#
# undef BOOST_PP_LOCAL_MACRO
