/* Generated by Cython 3.0.11 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "language": "c",
        "name": "supriya.utils._intervals",
        "sources": [
            "supriya/utils/_intervals.pyx"
        ]
    },
    "module_name": "supriya.utils._intervals"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#if defined(CYTHON_LIMITED_API) && CYTHON_LIMITED_API
#define __PYX_EXTRA_ABI_MODULE_NAME "limited"
#else
#define __PYX_EXTRA_ABI_MODULE_NAME ""
#endif
#define CYTHON_ABI "3_0_11" __PYX_EXTRA_ABI_MODULE_NAME
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x03000BF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(Py_GIL_DISABLED) || defined(Py_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000 && PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    static CYTHON_INLINE PyObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030B0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030B0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
    #ifndef CO_OPTIMIZED
    #define CO_OPTIMIZED 0x0001
    #endif
    #ifndef CO_NEWLOCALS
    #define CO_NEWLOCALS 0x0002
    #endif
    #ifndef CO_VARARGS
    #define CO_VARARGS 0x0004
    #endif
    #ifndef CO_VARKEYWORDS
    #define CO_VARKEYWORDS 0x0008
    #endif
    #ifndef CO_ASYNC_GENERATOR
    #define CO_ASYNC_GENERATOR 0x0200
    #endif
    #ifndef CO_GENERATOR
    #define CO_GENERATOR 0x0020
    #endif
    #ifndef CO_COROUTINE
    #define CO_COROUTINE 0x0080
    #endif
#elif PY_VERSION_HEX >= 0x030B0000
  static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    PyObject *empty_bytes = PyBytes_FromStringAndSize("", 0);
    if (!empty_bytes) return NULL;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, empty_bytes);
    Py_DECREF(empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef CO_COROUTINE
  #define CO_COROUTINE 0x80
#endif
#ifndef CO_ASYNC_GENERATOR
  #define CO_ASYNC_GENERATOR 0x200
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_MAJOR_VERSION >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void *cfunc) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE(obj), name, func_ctype)
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if PY_MAJOR_VERSION < 3
    #if CYTHON_COMPILING_IN_PYPY
        #if PYPY_VERSION_NUM < 0x07030600
            #if defined(__cplusplus) && __cplusplus >= 201402L
                [[deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")]]
            #elif defined(__GNUC__) || defined(__clang__)
                __attribute__ ((__deprecated__("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")))
            #elif defined(_MSC_VER)
                __declspec(deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6"))
            #endif
            static CYTHON_INLINE int PyGILState_Check(void) {
                return 0;
            }
        #else  // PYPY_VERSION_NUM < 0x07030600
        #endif  // PYPY_VERSION_NUM < 0x07030600
    #else
        static CYTHON_INLINE int PyGILState_Check(void) {
            PyThreadState * tstate = _PyThreadState_Current;
            return tstate && (tstate == PyGILState_GetThisThreadState());
        }
    #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000 || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && PY_VERSION_HEX < 0x030d0000 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyObject_GenericSetAttr((PyObject*)tp, k, v)
#else
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyDict_SetItem(tp->tp_dict, k, v)
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535U : 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = (Py_UNICODE) ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define __Pyx_Py3Int_Check(op)       PyLong_Check(op)
  #define __Pyx_Py3Int_CheckExact(op)  PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#else
  #define __Pyx_Py3Int_Check(op)       (PyLong_Check(op) || PyInt_Check(op))
  #define __Pyx_Py3Int_CheckExact(op)  (PyLong_CheckExact(op) || PyInt_CheckExact(op))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__;  (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__supriya__utils___intervals
#define __PYX_HAVE_API__supriya__utils___intervals
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
#include <string.h>
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = (char) c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#include <string.h>
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "supriya/utils/_intervals.pyx",
  "<stringsource>",
};
/* #### Code section: utility_code_proto_before_types ### */
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_7supriya_5utils_10_intervals__CInterval;
struct __pyx_obj_7supriya_5utils_10_intervals__CNode;
struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx;
struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__;
struct __pyx_opt_args_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after;

/* "supriya/utils/_intervals.pyx":290
 *         return result
 * 
 *     cdef float _recurse_get_offset_after(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */
struct __pyx_opt_args_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after {
  int __pyx_n;
  int depth;
};

/* "supriya/utils/_intervals.pyx":4
 * 
 * 
 * cdef class _CInterval:             # <<<<<<<<<<<<<<
 * 
 *     cdef readonly float start_offset
 */
struct __pyx_obj_7supriya_5utils_10_intervals__CInterval {
  PyObject_HEAD
  struct __pyx_vtabstruct_7supriya_5utils_10_intervals__CInterval *__pyx_vtab;
  float start_offset;
  float stop_offset;
  PyObject *original_interval;
};


/* "supriya/utils/_intervals.pyx":55
 * 
 * 
 * cdef class _CNode:             # <<<<<<<<<<<<<<
 * 
 *     cdef public int balance
 */
struct __pyx_obj_7supriya_5utils_10_intervals__CNode {
  PyObject_HEAD
  int balance;
  int height;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *left_child;
  int node_start_index;
  int node_stop_index;
  PyObject *payload;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *right_child;
  float start_offset;
  float stop_offset_high;
  float stop_offset_low;
  int subtree_start_index;
  int subtree_stop_index;
};


/* "supriya/utils/_intervals.pyx":85
 * 
 * 
 * cdef class IntervalTreeDriverEx:             # <<<<<<<<<<<<<<
 *     """
 *     A mutable always-sorted collection of intervals.
 */
struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx {
  PyObject_HEAD
  struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_vtab;
  PyObject *_root_node;
};


/* "supriya/utils/_intervals.pyx":128
 *         raise TypeError('Indices must be integers or slices, got {}'.format(i))
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef _CNode current
 *         cdef _CInterval cinterval
 */
struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ {
  PyObject_HEAD
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_current;
  Py_ssize_t __pyx_v_i;
  struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self;
  PyObject *__pyx_v_stack;
  Py_ssize_t __pyx_t_0;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
};



/* "supriya/utils/_intervals.pyx":4
 * 
 * 
 * cdef class _CInterval:             # <<<<<<<<<<<<<<
 * 
 *     cdef readonly float start_offset
 */

struct __pyx_vtabstruct_7supriya_5utils_10_intervals__CInterval {
  int (*intersects_interval)(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_7supriya_5utils_10_intervals__CInterval *__pyx_vtabptr_7supriya_5utils_10_intervals__CInterval;


/* "supriya/utils/_intervals.pyx":85
 * 
 * 
 * cdef class IntervalTreeDriverEx:             # <<<<<<<<<<<<<<
 *     """
 *     A mutable always-sorted collection of intervals.
 */

struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_insert_node)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float);
  void (*_insert_interval)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_rebalance)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  PyObject *(*_recurse_find_intervals_intersecting_offset)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float);
  PyObject *(*_recurse_find_intervals_intersecting_interval)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *);
  PyObject *(*_recurse_find_intervals_stopping_at)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float);
  float (*_recurse_get_offset_after)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float, struct __pyx_opt_args_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after *__pyx_optional_args);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_recurse_get_start_offset_after)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_recurse_get_start_offset_before)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float);
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *(*_recurse_getitem_by_index)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, int);
  PyObject *(*_recurse_getitem_by_slice)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, int, int);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_remove_node)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float);
  void (*_remove_interval)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_rotate_left_left)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_rotate_left_right)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_rotate_right_left)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_rotate_right_right)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_search)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float);
  void (*_set_node_left_child)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  void (*_set_node_right_child)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  PyObject *(*_unbox_cintervals)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, PyObject *);
  void (*_update_node_height)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
  void (*_update_indices)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, int);
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*_update_offsets)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *);
};
static struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_vtabptr_7supriya_5utils_10_intervals_IntervalTreeDriverEx;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_VARARGS(args, i) PySequence_GetItem(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#else
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GetItem(args, i)
#endif
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_NewRef_VARARGS(arg) __Pyx_NewRef(arg)
    #define __Pyx_Arg_XDECREF_VARARGS(arg) Py_XDECREF(arg)
#else
    #define __Pyx_Arg_NewRef_VARARGS(arg) arg
    #define __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
    #define __Pyx_Arg_NewRef_FASTCALL(arg) arg  /* no-op, __Pyx_Arg_FASTCALL is direct and this needs
                                                   to have the same reference counting */
    #define __Pyx_Arg_XDECREF_FASTCALL(arg)
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
    #define __Pyx_Arg_NewRef_FASTCALL(arg) __Pyx_Arg_NewRef_VARARGS(arg)
    #define __Pyx_Arg_XDECREF_FASTCALL(arg) __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* pybytes_as_double.proto */
static double __Pyx_SlowPyString_AsDouble(PyObject *obj);
static double __Pyx__PyBytes_AsDouble(PyObject *obj, const char* start, Py_ssize_t length);
static CYTHON_INLINE double __Pyx_PyBytes_AsDouble(PyObject *obj) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
    as_c_string = PyBytes_AS_STRING(obj);
    size = PyBytes_GET_SIZE(obj);
#else
    if (PyBytes_AsStringAndSize(obj, &as_c_string, &size) < 0) {
        return (double)-1;
    }
#endif
    return __Pyx__PyBytes_AsDouble(obj, as_c_string, size);
}
static CYTHON_INLINE double __Pyx_PyByteArray_AsDouble(PyObject *obj) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
    as_c_string = PyByteArray_AS_STRING(obj);
    size = PyByteArray_GET_SIZE(obj);
#else
    as_c_string = PyByteArray_AsString(obj);
    if (as_c_string == NULL) {
        return (double)-1;
    }
    size = PyByteArray_Size(obj);
#endif
    return __Pyx__PyBytes_AsDouble(obj, as_c_string, size);
}

/* pyunicode_as_double.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY && CYTHON_ASSUME_SAFE_MACROS
static const char* __Pyx__PyUnicode_AsDouble_Copy(const void* data, const int kind, char* buffer, Py_ssize_t start, Py_ssize_t end) {
    int last_was_punctuation;
    Py_ssize_t i;
    last_was_punctuation = 1;
    for (i=start; i <= end; i++) {
        Py_UCS4 chr = PyUnicode_READ(kind, data, i);
        int is_punctuation = (chr == '_') | (chr == '.');
        *buffer = (char)chr;
        buffer += (chr != '_');
        if (unlikely(chr > 127)) goto parse_failure;
        if (unlikely(last_was_punctuation & is_punctuation)) goto parse_failure;
        last_was_punctuation = is_punctuation;
    }
    if (unlikely(last_was_punctuation)) goto parse_failure;
    *buffer = '\0';
    return buffer;
parse_failure:
    return NULL;
}
static double __Pyx__PyUnicode_AsDouble_inf_nan(const void* data, int kind, Py_ssize_t start, Py_ssize_t length) {
    int matches = 1;
    Py_UCS4 chr;
    Py_UCS4 sign = PyUnicode_READ(kind, data, start);
    int is_signed = (sign == '-') | (sign == '+');
    start += is_signed;
    length -= is_signed;
    switch (PyUnicode_READ(kind, data, start)) {
        #ifdef Py_NAN
        case 'n':
        case 'N':
            if (unlikely(length != 3)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+1);
            matches &= (chr == 'a') | (chr == 'A');
            chr = PyUnicode_READ(kind, data, start+2);
            matches &= (chr == 'n') | (chr == 'N');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_NAN : Py_NAN;
        #endif
        case 'i':
        case 'I':
            if (unlikely(length < 3)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+1);
            matches &= (chr == 'n') | (chr == 'N');
            chr = PyUnicode_READ(kind, data, start+2);
            matches &= (chr == 'f') | (chr == 'F');
            if (likely(length == 3 && matches))
                return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
            if (unlikely(length != 8)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+3);
            matches &= (chr == 'i') | (chr == 'I');
            chr = PyUnicode_READ(kind, data, start+4);
            matches &= (chr == 'n') | (chr == 'N');
            chr = PyUnicode_READ(kind, data, start+5);
            matches &= (chr == 'i') | (chr == 'I');
            chr = PyUnicode_READ(kind, data, start+6);
            matches &= (chr == 't') | (chr == 'T');
            chr = PyUnicode_READ(kind, data, start+7);
            matches &= (chr == 'y') | (chr == 'Y');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
        case '.': case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
            break;
        default:
            goto parse_failure;
    }
    return 0.0;
parse_failure:
    return -1.0;
}
static double __Pyx_PyUnicode_AsDouble_WithSpaces(PyObject *obj) {
    double value;
    const char *last;
    char *end;
    Py_ssize_t start, length = PyUnicode_GET_LENGTH(obj);
    const int kind = PyUnicode_KIND(obj);
    const void* data = PyUnicode_DATA(obj);
    start = 0;
    while (Py_UNICODE_ISSPACE(PyUnicode_READ(kind, data, start)))
        start++;
    while (start < length - 1 && Py_UNICODE_ISSPACE(PyUnicode_READ(kind, data, length - 1)))
        length--;
    length -= start;
    if (unlikely(length <= 0)) goto fallback;
    value = __Pyx__PyUnicode_AsDouble_inf_nan(data, kind, start, length);
    if (unlikely(value == -1.0)) goto fallback;
    if (value != 0.0) return value;
    if (length < 40) {
        char number[40];
        last = __Pyx__PyUnicode_AsDouble_Copy(data, kind, number, start, start + length);
        if (unlikely(!last)) goto fallback;
        value = PyOS_string_to_double(number, &end, NULL);
    } else {
        char *number = (char*) PyMem_Malloc((length + 1) * sizeof(char));
        if (unlikely(!number)) goto fallback;
        last = __Pyx__PyUnicode_AsDouble_Copy(data, kind, number, start, start + length);
        if (unlikely(!last)) {
            PyMem_Free(number);
            goto fallback;
        }
        value = PyOS_string_to_double(number, &end, NULL);
        PyMem_Free(number);
    }
    if (likely(end == last) || (value == (double)-1 && PyErr_Occurred())) {
        return value;
    }
fallback:
    return __Pyx_SlowPyString_AsDouble(obj);
}
#endif
static CYTHON_INLINE double __Pyx_PyUnicode_AsDouble(PyObject *obj) {
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY && CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(__Pyx_PyUnicode_READY(obj) == -1))
        return (double)-1;
    if (likely(PyUnicode_IS_ASCII(obj))) {
        const char *s;
        Py_ssize_t length;
        s = PyUnicode_AsUTF8AndSize(obj, &length);
        return __Pyx__PyBytes_AsDouble(obj, s, length);
    }
    return __Pyx_PyUnicode_AsDouble_WithSpaces(obj);
#else
    return __Pyx_SlowPyString_AsDouble(obj);
#endif
}

/* pyobject_as_double.proto */
static double __Pyx__PyObject_AsDouble(PyObject* obj);
#if CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyObject_AsDouble(obj)\
(likely(PyFloat_CheckExact(obj)) ? PyFloat_AS_DOUBLE(obj) :\
 likely(PyInt_CheckExact(obj)) ?\
 PyFloat_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#else
#define __Pyx_PyObject_AsDouble(obj)\
((likely(PyFloat_CheckExact(obj))) ?  PyFloat_AS_DOUBLE(obj) :\
 likely(PyLong_CheckExact(obj)) ?\
 PyLong_AsDouble(obj) : __Pyx__PyObject_AsDouble(obj))
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kw, const char* function_name, int kw_allowed);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* pop.proto */
static CYTHON_INLINE PyObject* __Pyx__PyObject_Pop(PyObject* L);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE PyObject* __Pyx_PyList_Pop(PyObject* L);
#define __Pyx_PyObject_Pop(L) (likely(PyList_CheckExact(L)) ?\
    __Pyx_PyList_Pop(L) : __Pyx__PyObject_Pop(L))
#else
#define __Pyx_PyList_Pop(L)  __Pyx__PyObject_Pop(L)
#define __Pyx_PyObject_Pop(L)  __Pyx__PyObject_Pop(L)
#endif

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* PyObjectCall2Args.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto */
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);
#else
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);
#endif

/* PyMethodNew.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *typesModule=NULL, *methodType=NULL, *result=NULL;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    typesModule = PyImport_ImportModule("types");
    if (!typesModule) return NULL;
    methodType = PyObject_GetAttrString(typesModule, "MethodType");
    Py_DECREF(typesModule);
    if (!methodType) return NULL;
    result = PyObject_CallFunctionObjArgs(methodType, func, self, NULL);
    Py_DECREF(methodType);
    return result;
}
#elif PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#else
    #define __Pyx_PyMethod_New PyMethod_New
#endif

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc);
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* pynumber_float.proto */
static CYTHON_INLINE PyObject* __Pyx__PyNumber_Float(PyObject* obj);
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : __Pyx__PyNumber_Float(x))

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (1)
#elif CYTHON_COMPILING_IN_LIMITED_API  ||  (CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030C0000)
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* ValidateBasesTuple.proto */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type);
#endif

/* SetupReduce.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce(PyObject* type_obj);
#endif

/* ClassMethod.proto */
#include "descrobject.h"
CYTHON_UNUSED static PyObject* __Pyx_Method_ClassMethod(PyObject *method);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CoroutineBase.proto */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);

/* CheckBinaryVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

/* #### Code section: module_declarations ### */
static int __pyx_f_7supriya_5utils_10_intervals_10_CInterval_intersects_interval(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__insert_node(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_start_offset); /* proto*/
static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__insert_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rebalance(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node); /* proto*/
static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_intersecting_offset(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset); /* proto*/
static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_intersecting_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval); /* proto*/
static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_stopping_at(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset); /* proto*/
static float __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset, struct __pyx_opt_args_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after *__pyx_optional_args); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_start_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_start_offset_before(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_getitem_by_index(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, int __pyx_v_index); /* proto*/
static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_getitem_by_slice(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, int __pyx_v_start, int __pyx_v_stop); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__remove_node(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_start_offset); /* proto*/
static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__remove_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_left_left(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_left_right(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_right_left(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_right_right(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__search(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_start_offset); /* proto*/
static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__set_node_left_child(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_left_child); /* proto*/
static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__set_node_right_child(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_right_child); /* proto*/
static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__unbox_cintervals(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_cintervals); /* proto*/
static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_node_height(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node); /* proto*/
static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_indices(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, int __pyx_v_parent_stop_index); /* proto*/
static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_offsets(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node); /* proto*/

/* Module declarations from "supriya.utils._intervals" */
static PyObject *__pyx_f_7supriya_5utils_10_intervals___pyx_unpickle__CNode__set_state(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, PyObject *); /*proto*/
static PyObject *__pyx_f_7supriya_5utils_10_intervals___pyx_unpickle_IntervalTreeDriverEx__set_state(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "supriya.utils._intervals"
extern int __pyx_module_is_main_supriya__utils___intervals;
int __pyx_module_is_main_supriya__utils___intervals = 0;

/* Implementation of "supriya.utils._intervals" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_staticmethod;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_AssertionError;
/* #### Code section: string_decls ### */
static const char __pyx_k_x[] = "x";
static const char __pyx_k__2[] = ".";
static const char __pyx_k_gc[] = "gc";
static const char __pyx_k__41[] = "?";
static const char __pyx_k_add[] = "add";
static const char __pyx_k_cls[] = "cls";
static const char __pyx_k_inf[] = "inf";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_expr[] = "expr";
static const char __pyx_k_iter[] = "__iter__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_node[] = "node";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_sort[] = "sort";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_CNode[] = "_CNode";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_inf_2[] = "-inf";
static const char __pyx_k_other[] = "other";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_enable[] = "enable";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_remove[] = "remove";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_disable[] = "disable";
static const char __pyx_k_indices[] = "indices";
static const char __pyx_k_results[] = "results";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_interval[] = "interval";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_CInterval[] = "_CInterval";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_cinterval[] = "cinterval";
static const char __pyx_k_intervals[] = "intervals";
static const char __pyx_k_isenabled[] = "isenabled";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_cintervals[] = "cintervals";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_is_interval[] = "_is_interval";
static const char __pyx_k_stop_offset[] = "stop_offset";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_start_offset[] = "start_offset";
static const char __pyx_k_staticmethod[] = "staticmethod";
static const char __pyx_k_stringsource[] = "<stringsource>";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_from_interval[] = "from_interval";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_AssertionError[] = "AssertionError";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_get_offset_after[] = "get_offset_after";
static const char __pyx_k_original_interval[] = "original_interval";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_subtree_stop_index[] = "subtree_stop_index";
static const char __pyx_k_intersects_interval[] = "intersects_interval";
static const char __pyx_k_pyx_unpickle__CNode[] = "__pyx_unpickle__CNode";
static const char __pyx_k_IntervalTreeDriverEx[] = "IntervalTreeDriverEx";
static const char __pyx_k_not_in_interval_tree[] = "{} not in interval tree.";
static const char __pyx_k_CNode___reduce_cython[] = "_CNode.__reduce_cython__";
static const char __pyx_k_get_start_offset_after[] = "get_start_offset_after";
static const char __pyx_k_CInterval_from_interval[] = "_CInterval.from_interval";
static const char __pyx_k_CNode___setstate_cython[] = "_CNode.__setstate_cython__";
static const char __pyx_k_get_start_offset_before[] = "get_start_offset_before";
static const char __pyx_k_IntervalTreeDriverEx_add[] = "IntervalTreeDriverEx.add";
static const char __pyx_k_supriya_utils__intervals[] = "supriya.utils._intervals";
static const char __pyx_k_CInterval___reduce_cython[] = "_CInterval.__reduce_cython__";
static const char __pyx_k_IntervalTreeDriverEx_index[] = "IntervalTreeDriverEx.index";
static const char __pyx_k_find_intervals_starting_at[] = "find_intervals_starting_at";
static const char __pyx_k_find_intervals_stopping_at[] = "find_intervals_stopping_at";
static const char __pyx_k_CInterval___setstate_cython[] = "_CInterval.__setstate_cython__";
static const char __pyx_k_IntervalTreeDriverEx___iter[] = "IntervalTreeDriverEx.__iter__";
static const char __pyx_k_IntervalTreeDriverEx_remove[] = "IntervalTreeDriverEx.remove";
static const char __pyx_k_IntervalTreeDriverEx_update[] = "IntervalTreeDriverEx.update";
static const char __pyx_k_supriya_utils__intervals_pyx[] = "supriya/utils/_intervals.pyx";
static const char __pyx_k_CInterval_intersects_interval[] = "_CInterval.intersects_interval";
static const char __pyx_k_pyx_unpickle_IntervalTreeDrive[] = "__pyx_unpickle_IntervalTreeDriverEx";
static const char __pyx_k_IntervalTreeDriverEx___setstate[] = "IntervalTreeDriverEx.__setstate_cython__";
static const char __pyx_k_IntervalTreeDriverEx_get_offset[] = "IntervalTreeDriverEx.get_offset_after";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))";
static const char __pyx_k_Indices_must_be_integers_or_slic[] = "Indices must be integers or slices, got {}";
static const char __pyx_k_IntervalTreeDriverEx___reduce_cy[] = "IntervalTreeDriverEx.__reduce_cython__";
static const char __pyx_k_IntervalTreeDriverEx__insert_int[] = "IntervalTreeDriverEx._insert_interval.<locals>.<lambda>";
static const char __pyx_k_IntervalTreeDriverEx__is_interva[] = "IntervalTreeDriverEx._is_interval";
static const char __pyx_k_IntervalTreeDriverEx__unbox_cint[] = "IntervalTreeDriverEx._unbox_cintervals.<locals>.<lambda>";
static const char __pyx_k_IntervalTreeDriverEx_find_interv[] = "IntervalTreeDriverEx.find_intervals_starting_at";
static const char __pyx_k_IntervalTreeDriverEx_get_start_o[] = "IntervalTreeDriverEx.get_start_offset_after";
static const char __pyx_k_find_intervals_intersecting_inte[] = "find_intervals_intersecting_interval";
static const char __pyx_k_find_intervals_intersecting_offs[] = "find_intervals_intersecting_offset";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_2[] = "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))";
static const char __pyx_k_IntervalTreeDriverEx_find_interv_2[] = "IntervalTreeDriverEx.find_intervals_stopping_at";
static const char __pyx_k_IntervalTreeDriverEx_find_interv_3[] = "IntervalTreeDriverEx.find_intervals_intersecting_offset";
static const char __pyx_k_IntervalTreeDriverEx_find_interv_4[] = "IntervalTreeDriverEx.find_intervals_intersecting_interval";
static const char __pyx_k_IntervalTreeDriverEx_get_start_o_2[] = "IntervalTreeDriverEx.get_start_offset_before";
/* #### Code section: decls ### */
static int __pyx_pf_7supriya_5utils_10_intervals_10_CInterval___cinit__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, PyObject *__pyx_v_start_offset, PyObject *__pyx_v_stop_offset, PyObject *__pyx_v_original_interval); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_2__richcmp__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_other, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_4from_interval(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_interval); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_6intersects_interval(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_12start_offset___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_11stop_offset___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_17original_interval___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode___init__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_start_offset); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_7balance___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7balance_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_6height___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_6height_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_16node_start_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16node_start_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_15node_stop_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_12start_offset___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_12start_offset_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_2__reduce_cython__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_4__setstate_cython__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx___init__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_intervals); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_2__contains__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_4__getitem__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_i); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_6__iter__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_9__len__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_11_is_interval(PyObject *__pyx_v_expr); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda1(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_13add(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_15find_intervals_starting_at(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17find_intervals_stopping_at(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_19find_intervals_intersecting_offset(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_21find_intervals_intersecting_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_23get_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_25get_start_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_27get_start_offset_before(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_29index(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_31remove(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_33update(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_intervals); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node___get__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_35__reduce_cython__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_37__setstate_cython__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals___pyx_unpickle__CNode(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_7supriya_5utils_10_intervals_2__pyx_unpickle_IntervalTreeDriverEx(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals__CInterval(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals__CNode(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals_IntervalTreeDriverEx(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals___pyx_scope_struct____iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyList_Type_pop = {0, 0, 0, 0, 0};
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  #if CYTHON_USE_MODULE_STATE
  PyObject *__pyx_type_7supriya_5utils_10_intervals__CInterval;
  PyObject *__pyx_type_7supriya_5utils_10_intervals__CNode;
  PyObject *__pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx;
  PyObject *__pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter__;
  #endif
  PyTypeObject *__pyx_ptype_7supriya_5utils_10_intervals__CInterval;
  PyTypeObject *__pyx_ptype_7supriya_5utils_10_intervals__CNode;
  PyTypeObject *__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx;
  PyTypeObject *__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__;
  PyObject *__pyx_n_s_AssertionError;
  PyObject *__pyx_n_s_CInterval;
  PyObject *__pyx_n_s_CInterval___reduce_cython;
  PyObject *__pyx_n_s_CInterval___setstate_cython;
  PyObject *__pyx_n_s_CInterval_from_interval;
  PyObject *__pyx_n_s_CInterval_intersects_interval;
  PyObject *__pyx_n_s_CNode;
  PyObject *__pyx_n_s_CNode___reduce_cython;
  PyObject *__pyx_n_s_CNode___setstate_cython;
  PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0;
  PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2;
  PyObject *__pyx_n_s_IndexError;
  PyObject *__pyx_kp_u_Indices_must_be_integers_or_slic;
  PyObject *__pyx_n_s_IntervalTreeDriverEx;
  PyObject *__pyx_n_s_IntervalTreeDriverEx___iter;
  PyObject *__pyx_n_s_IntervalTreeDriverEx___reduce_cy;
  PyObject *__pyx_n_s_IntervalTreeDriverEx___setstate;
  PyObject *__pyx_n_s_IntervalTreeDriverEx__insert_int;
  PyObject *__pyx_n_s_IntervalTreeDriverEx__is_interva;
  PyObject *__pyx_n_s_IntervalTreeDriverEx__unbox_cint;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_add;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_find_interv;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_find_interv_2;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_find_interv_3;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_find_interv_4;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_get_offset;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_get_start_o;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_get_start_o_2;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_index;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_remove;
  PyObject *__pyx_n_s_IntervalTreeDriverEx_update;
  PyObject *__pyx_n_s_PickleError;
  PyObject *__pyx_n_s_TypeError;
  PyObject *__pyx_n_s_ValueError;
  PyObject *__pyx_kp_u__2;
  PyObject *__pyx_n_s__41;
  PyObject *__pyx_n_s_add;
  PyObject *__pyx_n_s_append;
  PyObject *__pyx_n_s_args;
  PyObject *__pyx_n_s_asyncio_coroutines;
  PyObject *__pyx_n_s_cinterval;
  PyObject *__pyx_n_s_cintervals;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_close;
  PyObject *__pyx_n_s_cls;
  PyObject *__pyx_n_s_dict;
  PyObject *__pyx_n_s_dict_2;
  PyObject *__pyx_kp_u_disable;
  PyObject *__pyx_kp_u_enable;
  PyObject *__pyx_n_s_expr;
  PyObject *__pyx_n_s_find_intervals_intersecting_inte;
  PyObject *__pyx_n_s_find_intervals_intersecting_offs;
  PyObject *__pyx_n_s_find_intervals_starting_at;
  PyObject *__pyx_n_s_find_intervals_stopping_at;
  PyObject *__pyx_n_s_format;
  PyObject *__pyx_n_s_from_interval;
  PyObject *__pyx_kp_u_gc;
  PyObject *__pyx_n_s_get_offset_after;
  PyObject *__pyx_n_s_get_start_offset_after;
  PyObject *__pyx_n_s_get_start_offset_before;
  PyObject *__pyx_n_s_getstate;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_n_s_index;
  PyObject *__pyx_n_s_indices;
  PyObject *__pyx_n_u_inf;
  PyObject *__pyx_kp_u_inf_2;
  PyObject *__pyx_n_s_intersects_interval;
  PyObject *__pyx_n_s_interval;
  PyObject *__pyx_n_s_intervals;
  PyObject *__pyx_n_s_is_coroutine;
  PyObject *__pyx_n_s_is_interval;
  PyObject *__pyx_kp_u_isenabled;
  PyObject *__pyx_n_s_iter;
  PyObject *__pyx_n_s_key;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_s_new;
  PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
  PyObject *__pyx_n_s_node;
  PyObject *__pyx_kp_u_not_in_interval_tree;
  PyObject *__pyx_n_s_offset;
  PyObject *__pyx_n_s_original_interval;
  PyObject *__pyx_n_s_other;
  PyObject *__pyx_n_s_pickle;
  PyObject *__pyx_n_s_pop;
  PyObject *__pyx_n_s_pyx_PickleError;
  PyObject *__pyx_n_s_pyx_checksum;
  PyObject *__pyx_n_s_pyx_result;
  PyObject *__pyx_n_s_pyx_state;
  PyObject *__pyx_n_s_pyx_type;
  PyObject *__pyx_n_s_pyx_unpickle_IntervalTreeDrive;
  PyObject *__pyx_n_s_pyx_unpickle__CNode;
  PyObject *__pyx_n_s_pyx_vtable;
  PyObject *__pyx_n_s_range;
  PyObject *__pyx_n_s_reduce;
  PyObject *__pyx_n_s_reduce_cython;
  PyObject *__pyx_n_s_reduce_ex;
  PyObject *__pyx_n_s_remove;
  PyObject *__pyx_n_s_result;
  PyObject *__pyx_n_s_results;
  PyObject *__pyx_n_s_self;
  PyObject *__pyx_n_s_send;
  PyObject *__pyx_n_s_setstate;
  PyObject *__pyx_n_s_setstate_cython;
  PyObject *__pyx_n_s_sort;
  PyObject *__pyx_n_s_start_offset;
  PyObject *__pyx_n_u_start_offset;
  PyObject *__pyx_n_s_state;
  PyObject *__pyx_n_s_staticmethod;
  PyObject *__pyx_n_s_stop_offset;
  PyObject *__pyx_n_u_stop_offset;
  PyObject *__pyx_kp_s_stringsource;
  PyObject *__pyx_n_s_subtree_stop_index;
  PyObject *__pyx_n_s_supriya_utils__intervals;
  PyObject *__pyx_kp_s_supriya_utils__intervals_pyx;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_n_s_throw;
  PyObject *__pyx_n_s_update;
  PyObject *__pyx_n_s_use_setstate;
  PyObject *__pyx_n_s_x;
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_8917134;
  PyObject *__pyx_int_149934309;
  PyObject *__pyx_int_163126392;
  PyObject *__pyx_int_168529197;
  PyObject *__pyx_int_244102488;
  PyObject *__pyx_int_249427599;
  PyObject *__pyx_tuple_;
  PyObject *__pyx_tuple__3;
  PyObject *__pyx_tuple__4;
  PyObject *__pyx_tuple__6;
  PyObject *__pyx_tuple__8;
  PyObject *__pyx_tuple__10;
  PyObject *__pyx_tuple__12;
  PyObject *__pyx_tuple__15;
  PyObject *__pyx_tuple__17;
  PyObject *__pyx_tuple__19;
  PyObject *__pyx_tuple__21;
  PyObject *__pyx_tuple__24;
  PyObject *__pyx_tuple__26;
  PyObject *__pyx_tuple__28;
  PyObject *__pyx_tuple__31;
  PyObject *__pyx_tuple__34;
  PyObject *__pyx_tuple__38;
  PyObject *__pyx_codeobj__5;
  PyObject *__pyx_codeobj__7;
  PyObject *__pyx_codeobj__9;
  PyObject *__pyx_codeobj__11;
  PyObject *__pyx_codeobj__13;
  PyObject *__pyx_codeobj__14;
  PyObject *__pyx_codeobj__16;
  PyObject *__pyx_codeobj__18;
  PyObject *__pyx_codeobj__20;
  PyObject *__pyx_codeobj__22;
  PyObject *__pyx_codeobj__23;
  PyObject *__pyx_codeobj__25;
  PyObject *__pyx_codeobj__27;
  PyObject *__pyx_codeobj__29;
  PyObject *__pyx_codeobj__30;
  PyObject *__pyx_codeobj__32;
  PyObject *__pyx_codeobj__33;
  PyObject *__pyx_codeobj__35;
  PyObject *__pyx_codeobj__36;
  PyObject *__pyx_codeobj__37;
  PyObject *__pyx_codeobj__39;
  PyObject *__pyx_codeobj__40;
} __pyx_mstate;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#else
static __pyx_mstate __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstate *__pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7supriya_5utils_10_intervals__CInterval);
  Py_CLEAR(clear_module_state->__pyx_type_7supriya_5utils_10_intervals__CInterval);
  Py_CLEAR(clear_module_state->__pyx_ptype_7supriya_5utils_10_intervals__CNode);
  Py_CLEAR(clear_module_state->__pyx_type_7supriya_5utils_10_intervals__CNode);
  Py_CLEAR(clear_module_state->__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);
  Py_CLEAR(clear_module_state->__pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx);
  Py_CLEAR(clear_module_state->__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__);
  Py_CLEAR(clear_module_state->__pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter__);
  Py_CLEAR(clear_module_state->__pyx_n_s_AssertionError);
  Py_CLEAR(clear_module_state->__pyx_n_s_CInterval);
  Py_CLEAR(clear_module_state->__pyx_n_s_CInterval___reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_CInterval___setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_CInterval_from_interval);
  Py_CLEAR(clear_module_state->__pyx_n_s_CInterval_intersects_interval);
  Py_CLEAR(clear_module_state->__pyx_n_s_CNode);
  Py_CLEAR(clear_module_state->__pyx_n_s_CNode___reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_CNode___setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Incompatible_checksums_0x_x_vs_0);
  Py_CLEAR(clear_module_state->__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_IndexError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Indices_must_be_integers_or_slic);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx___iter);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx___reduce_cy);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx___setstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx__insert_int);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx__is_interva);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx__unbox_cint);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_add);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv_3);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv_4);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_get_offset);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_get_start_o);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_get_start_o_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_index);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_remove);
  Py_CLEAR(clear_module_state->__pyx_n_s_IntervalTreeDriverEx_update);
  Py_CLEAR(clear_module_state->__pyx_n_s_PickleError);
  Py_CLEAR(clear_module_state->__pyx_n_s_TypeError);
  Py_CLEAR(clear_module_state->__pyx_n_s_ValueError);
  Py_CLEAR(clear_module_state->__pyx_kp_u__2);
  Py_CLEAR(clear_module_state->__pyx_n_s__41);
  Py_CLEAR(clear_module_state->__pyx_n_s_add);
  Py_CLEAR(clear_module_state->__pyx_n_s_append);
  Py_CLEAR(clear_module_state->__pyx_n_s_args);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_coroutines);
  Py_CLEAR(clear_module_state->__pyx_n_s_cinterval);
  Py_CLEAR(clear_module_state->__pyx_n_s_cintervals);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_close);
  Py_CLEAR(clear_module_state->__pyx_n_s_cls);
  Py_CLEAR(clear_module_state->__pyx_n_s_dict);
  Py_CLEAR(clear_module_state->__pyx_n_s_dict_2);
  Py_CLEAR(clear_module_state->__pyx_kp_u_disable);
  Py_CLEAR(clear_module_state->__pyx_kp_u_enable);
  Py_CLEAR(clear_module_state->__pyx_n_s_expr);
  Py_CLEAR(clear_module_state->__pyx_n_s_find_intervals_intersecting_inte);
  Py_CLEAR(clear_module_state->__pyx_n_s_find_intervals_intersecting_offs);
  Py_CLEAR(clear_module_state->__pyx_n_s_find_intervals_starting_at);
  Py_CLEAR(clear_module_state->__pyx_n_s_find_intervals_stopping_at);
  Py_CLEAR(clear_module_state->__pyx_n_s_format);
  Py_CLEAR(clear_module_state->__pyx_n_s_from_interval);
  Py_CLEAR(clear_module_state->__pyx_kp_u_gc);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_offset_after);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_start_offset_after);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_start_offset_before);
  Py_CLEAR(clear_module_state->__pyx_n_s_getstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_n_s_index);
  Py_CLEAR(clear_module_state->__pyx_n_s_indices);
  Py_CLEAR(clear_module_state->__pyx_n_u_inf);
  Py_CLEAR(clear_module_state->__pyx_kp_u_inf_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_intersects_interval);
  Py_CLEAR(clear_module_state->__pyx_n_s_interval);
  Py_CLEAR(clear_module_state->__pyx_n_s_intervals);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_coroutine);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_interval);
  Py_CLEAR(clear_module_state->__pyx_kp_u_isenabled);
  Py_CLEAR(clear_module_state->__pyx_n_s_iter);
  Py_CLEAR(clear_module_state->__pyx_n_s_key);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_new);
  Py_CLEAR(clear_module_state->__pyx_kp_s_no_default___reduce___due_to_non);
  Py_CLEAR(clear_module_state->__pyx_n_s_node);
  Py_CLEAR(clear_module_state->__pyx_kp_u_not_in_interval_tree);
  Py_CLEAR(clear_module_state->__pyx_n_s_offset);
  Py_CLEAR(clear_module_state->__pyx_n_s_original_interval);
  Py_CLEAR(clear_module_state->__pyx_n_s_other);
  Py_CLEAR(clear_module_state->__pyx_n_s_pickle);
  Py_CLEAR(clear_module_state->__pyx_n_s_pop);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_PickleError);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_checksum);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_result);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_state);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_type);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_unpickle_IntervalTreeDrive);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_unpickle__CNode);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_vtable);
  Py_CLEAR(clear_module_state->__pyx_n_s_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_reduce_ex);
  Py_CLEAR(clear_module_state->__pyx_n_s_remove);
  Py_CLEAR(clear_module_state->__pyx_n_s_result);
  Py_CLEAR(clear_module_state->__pyx_n_s_results);
  Py_CLEAR(clear_module_state->__pyx_n_s_self);
  Py_CLEAR(clear_module_state->__pyx_n_s_send);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_setstate_cython);
  Py_CLEAR(clear_module_state->__pyx_n_s_sort);
  Py_CLEAR(clear_module_state->__pyx_n_s_start_offset);
  Py_CLEAR(clear_module_state->__pyx_n_u_start_offset);
  Py_CLEAR(clear_module_state->__pyx_n_s_state);
  Py_CLEAR(clear_module_state->__pyx_n_s_staticmethod);
  Py_CLEAR(clear_module_state->__pyx_n_s_stop_offset);
  Py_CLEAR(clear_module_state->__pyx_n_u_stop_offset);
  Py_CLEAR(clear_module_state->__pyx_kp_s_stringsource);
  Py_CLEAR(clear_module_state->__pyx_n_s_subtree_stop_index);
  Py_CLEAR(clear_module_state->__pyx_n_s_supriya_utils__intervals);
  Py_CLEAR(clear_module_state->__pyx_kp_s_supriya_utils__intervals_pyx);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_n_s_throw);
  Py_CLEAR(clear_module_state->__pyx_n_s_update);
  Py_CLEAR(clear_module_state->__pyx_n_s_use_setstate);
  Py_CLEAR(clear_module_state->__pyx_n_s_x);
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_8917134);
  Py_CLEAR(clear_module_state->__pyx_int_149934309);
  Py_CLEAR(clear_module_state->__pyx_int_163126392);
  Py_CLEAR(clear_module_state->__pyx_int_168529197);
  Py_CLEAR(clear_module_state->__pyx_int_244102488);
  Py_CLEAR(clear_module_state->__pyx_int_249427599);
  Py_CLEAR(clear_module_state->__pyx_tuple_);
  Py_CLEAR(clear_module_state->__pyx_tuple__3);
  Py_CLEAR(clear_module_state->__pyx_tuple__4);
  Py_CLEAR(clear_module_state->__pyx_tuple__6);
  Py_CLEAR(clear_module_state->__pyx_tuple__8);
  Py_CLEAR(clear_module_state->__pyx_tuple__10);
  Py_CLEAR(clear_module_state->__pyx_tuple__12);
  Py_CLEAR(clear_module_state->__pyx_tuple__15);
  Py_CLEAR(clear_module_state->__pyx_tuple__17);
  Py_CLEAR(clear_module_state->__pyx_tuple__19);
  Py_CLEAR(clear_module_state->__pyx_tuple__21);
  Py_CLEAR(clear_module_state->__pyx_tuple__24);
  Py_CLEAR(clear_module_state->__pyx_tuple__26);
  Py_CLEAR(clear_module_state->__pyx_tuple__28);
  Py_CLEAR(clear_module_state->__pyx_tuple__31);
  Py_CLEAR(clear_module_state->__pyx_tuple__34);
  Py_CLEAR(clear_module_state->__pyx_tuple__38);
  Py_CLEAR(clear_module_state->__pyx_codeobj__5);
  Py_CLEAR(clear_module_state->__pyx_codeobj__7);
  Py_CLEAR(clear_module_state->__pyx_codeobj__9);
  Py_CLEAR(clear_module_state->__pyx_codeobj__11);
  Py_CLEAR(clear_module_state->__pyx_codeobj__13);
  Py_CLEAR(clear_module_state->__pyx_codeobj__14);
  Py_CLEAR(clear_module_state->__pyx_codeobj__16);
  Py_CLEAR(clear_module_state->__pyx_codeobj__18);
  Py_CLEAR(clear_module_state->__pyx_codeobj__20);
  Py_CLEAR(clear_module_state->__pyx_codeobj__22);
  Py_CLEAR(clear_module_state->__pyx_codeobj__23);
  Py_CLEAR(clear_module_state->__pyx_codeobj__25);
  Py_CLEAR(clear_module_state->__pyx_codeobj__27);
  Py_CLEAR(clear_module_state->__pyx_codeobj__29);
  Py_CLEAR(clear_module_state->__pyx_codeobj__30);
  Py_CLEAR(clear_module_state->__pyx_codeobj__32);
  Py_CLEAR(clear_module_state->__pyx_codeobj__33);
  Py_CLEAR(clear_module_state->__pyx_codeobj__35);
  Py_CLEAR(clear_module_state->__pyx_codeobj__36);
  Py_CLEAR(clear_module_state->__pyx_codeobj__37);
  Py_CLEAR(clear_module_state->__pyx_codeobj__39);
  Py_CLEAR(clear_module_state->__pyx_codeobj__40);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7supriya_5utils_10_intervals__CInterval);
  Py_VISIT(traverse_module_state->__pyx_type_7supriya_5utils_10_intervals__CInterval);
  Py_VISIT(traverse_module_state->__pyx_ptype_7supriya_5utils_10_intervals__CNode);
  Py_VISIT(traverse_module_state->__pyx_type_7supriya_5utils_10_intervals__CNode);
  Py_VISIT(traverse_module_state->__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);
  Py_VISIT(traverse_module_state->__pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx);
  Py_VISIT(traverse_module_state->__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__);
  Py_VISIT(traverse_module_state->__pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter__);
  Py_VISIT(traverse_module_state->__pyx_n_s_AssertionError);
  Py_VISIT(traverse_module_state->__pyx_n_s_CInterval);
  Py_VISIT(traverse_module_state->__pyx_n_s_CInterval___reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_CInterval___setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_CInterval_from_interval);
  Py_VISIT(traverse_module_state->__pyx_n_s_CInterval_intersects_interval);
  Py_VISIT(traverse_module_state->__pyx_n_s_CNode);
  Py_VISIT(traverse_module_state->__pyx_n_s_CNode___reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_CNode___setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Incompatible_checksums_0x_x_vs_0);
  Py_VISIT(traverse_module_state->__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_IndexError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Indices_must_be_integers_or_slic);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx___iter);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx___reduce_cy);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx___setstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx__insert_int);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx__is_interva);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx__unbox_cint);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_add);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv_3);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_find_interv_4);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_get_offset);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_get_start_o);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_get_start_o_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_index);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_remove);
  Py_VISIT(traverse_module_state->__pyx_n_s_IntervalTreeDriverEx_update);
  Py_VISIT(traverse_module_state->__pyx_n_s_PickleError);
  Py_VISIT(traverse_module_state->__pyx_n_s_TypeError);
  Py_VISIT(traverse_module_state->__pyx_n_s_ValueError);
  Py_VISIT(traverse_module_state->__pyx_kp_u__2);
  Py_VISIT(traverse_module_state->__pyx_n_s__41);
  Py_VISIT(traverse_module_state->__pyx_n_s_add);
  Py_VISIT(traverse_module_state->__pyx_n_s_append);
  Py_VISIT(traverse_module_state->__pyx_n_s_args);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_coroutines);
  Py_VISIT(traverse_module_state->__pyx_n_s_cinterval);
  Py_VISIT(traverse_module_state->__pyx_n_s_cintervals);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_close);
  Py_VISIT(traverse_module_state->__pyx_n_s_cls);
  Py_VISIT(traverse_module_state->__pyx_n_s_dict);
  Py_VISIT(traverse_module_state->__pyx_n_s_dict_2);
  Py_VISIT(traverse_module_state->__pyx_kp_u_disable);
  Py_VISIT(traverse_module_state->__pyx_kp_u_enable);
  Py_VISIT(traverse_module_state->__pyx_n_s_expr);
  Py_VISIT(traverse_module_state->__pyx_n_s_find_intervals_intersecting_inte);
  Py_VISIT(traverse_module_state->__pyx_n_s_find_intervals_intersecting_offs);
  Py_VISIT(traverse_module_state->__pyx_n_s_find_intervals_starting_at);
  Py_VISIT(traverse_module_state->__pyx_n_s_find_intervals_stopping_at);
  Py_VISIT(traverse_module_state->__pyx_n_s_format);
  Py_VISIT(traverse_module_state->__pyx_n_s_from_interval);
  Py_VISIT(traverse_module_state->__pyx_kp_u_gc);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_offset_after);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_start_offset_after);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_start_offset_before);
  Py_VISIT(traverse_module_state->__pyx_n_s_getstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_n_s_index);
  Py_VISIT(traverse_module_state->__pyx_n_s_indices);
  Py_VISIT(traverse_module_state->__pyx_n_u_inf);
  Py_VISIT(traverse_module_state->__pyx_kp_u_inf_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_intersects_interval);
  Py_VISIT(traverse_module_state->__pyx_n_s_interval);
  Py_VISIT(traverse_module_state->__pyx_n_s_intervals);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_coroutine);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_interval);
  Py_VISIT(traverse_module_state->__pyx_kp_u_isenabled);
  Py_VISIT(traverse_module_state->__pyx_n_s_iter);
  Py_VISIT(traverse_module_state->__pyx_n_s_key);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_new);
  Py_VISIT(traverse_module_state->__pyx_kp_s_no_default___reduce___due_to_non);
  Py_VISIT(traverse_module_state->__pyx_n_s_node);
  Py_VISIT(traverse_module_state->__pyx_kp_u_not_in_interval_tree);
  Py_VISIT(traverse_module_state->__pyx_n_s_offset);
  Py_VISIT(traverse_module_state->__pyx_n_s_original_interval);
  Py_VISIT(traverse_module_state->__pyx_n_s_other);
  Py_VISIT(traverse_module_state->__pyx_n_s_pickle);
  Py_VISIT(traverse_module_state->__pyx_n_s_pop);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_PickleError);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_checksum);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_result);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_state);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_type);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_unpickle_IntervalTreeDrive);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_unpickle__CNode);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_vtable);
  Py_VISIT(traverse_module_state->__pyx_n_s_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_reduce_ex);
  Py_VISIT(traverse_module_state->__pyx_n_s_remove);
  Py_VISIT(traverse_module_state->__pyx_n_s_result);
  Py_VISIT(traverse_module_state->__pyx_n_s_results);
  Py_VISIT(traverse_module_state->__pyx_n_s_self);
  Py_VISIT(traverse_module_state->__pyx_n_s_send);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_setstate_cython);
  Py_VISIT(traverse_module_state->__pyx_n_s_sort);
  Py_VISIT(traverse_module_state->__pyx_n_s_start_offset);
  Py_VISIT(traverse_module_state->__pyx_n_u_start_offset);
  Py_VISIT(traverse_module_state->__pyx_n_s_state);
  Py_VISIT(traverse_module_state->__pyx_n_s_staticmethod);
  Py_VISIT(traverse_module_state->__pyx_n_s_stop_offset);
  Py_VISIT(traverse_module_state->__pyx_n_u_stop_offset);
  Py_VISIT(traverse_module_state->__pyx_kp_s_stringsource);
  Py_VISIT(traverse_module_state->__pyx_n_s_subtree_stop_index);
  Py_VISIT(traverse_module_state->__pyx_n_s_supriya_utils__intervals);
  Py_VISIT(traverse_module_state->__pyx_kp_s_supriya_utils__intervals_pyx);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_n_s_throw);
  Py_VISIT(traverse_module_state->__pyx_n_s_update);
  Py_VISIT(traverse_module_state->__pyx_n_s_use_setstate);
  Py_VISIT(traverse_module_state->__pyx_n_s_x);
  Py_VISIT(traverse_module_state->__pyx_int_0);
  Py_VISIT(traverse_module_state->__pyx_int_8917134);
  Py_VISIT(traverse_module_state->__pyx_int_149934309);
  Py_VISIT(traverse_module_state->__pyx_int_163126392);
  Py_VISIT(traverse_module_state->__pyx_int_168529197);
  Py_VISIT(traverse_module_state->__pyx_int_244102488);
  Py_VISIT(traverse_module_state->__pyx_int_249427599);
  Py_VISIT(traverse_module_state->__pyx_tuple_);
  Py_VISIT(traverse_module_state->__pyx_tuple__3);
  Py_VISIT(traverse_module_state->__pyx_tuple__4);
  Py_VISIT(traverse_module_state->__pyx_tuple__6);
  Py_VISIT(traverse_module_state->__pyx_tuple__8);
  Py_VISIT(traverse_module_state->__pyx_tuple__10);
  Py_VISIT(traverse_module_state->__pyx_tuple__12);
  Py_VISIT(traverse_module_state->__pyx_tuple__15);
  Py_VISIT(traverse_module_state->__pyx_tuple__17);
  Py_VISIT(traverse_module_state->__pyx_tuple__19);
  Py_VISIT(traverse_module_state->__pyx_tuple__21);
  Py_VISIT(traverse_module_state->__pyx_tuple__24);
  Py_VISIT(traverse_module_state->__pyx_tuple__26);
  Py_VISIT(traverse_module_state->__pyx_tuple__28);
  Py_VISIT(traverse_module_state->__pyx_tuple__31);
  Py_VISIT(traverse_module_state->__pyx_tuple__34);
  Py_VISIT(traverse_module_state->__pyx_tuple__38);
  Py_VISIT(traverse_module_state->__pyx_codeobj__5);
  Py_VISIT(traverse_module_state->__pyx_codeobj__7);
  Py_VISIT(traverse_module_state->__pyx_codeobj__9);
  Py_VISIT(traverse_module_state->__pyx_codeobj__11);
  Py_VISIT(traverse_module_state->__pyx_codeobj__13);
  Py_VISIT(traverse_module_state->__pyx_codeobj__14);
  Py_VISIT(traverse_module_state->__pyx_codeobj__16);
  Py_VISIT(traverse_module_state->__pyx_codeobj__18);
  Py_VISIT(traverse_module_state->__pyx_codeobj__20);
  Py_VISIT(traverse_module_state->__pyx_codeobj__22);
  Py_VISIT(traverse_module_state->__pyx_codeobj__23);
  Py_VISIT(traverse_module_state->__pyx_codeobj__25);
  Py_VISIT(traverse_module_state->__pyx_codeobj__27);
  Py_VISIT(traverse_module_state->__pyx_codeobj__29);
  Py_VISIT(traverse_module_state->__pyx_codeobj__30);
  Py_VISIT(traverse_module_state->__pyx_codeobj__32);
  Py_VISIT(traverse_module_state->__pyx_codeobj__33);
  Py_VISIT(traverse_module_state->__pyx_codeobj__35);
  Py_VISIT(traverse_module_state->__pyx_codeobj__36);
  Py_VISIT(traverse_module_state->__pyx_codeobj__37);
  Py_VISIT(traverse_module_state->__pyx_codeobj__39);
  Py_VISIT(traverse_module_state->__pyx_codeobj__40);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#ifdef __Pyx_Generator_USED
#define __pyx_GeneratorType __pyx_mstate_global->__pyx_GeneratorType
#endif
#ifdef __Pyx_IterableCoroutine_USED
#define __pyx_IterableCoroutineType __pyx_mstate_global->__pyx_IterableCoroutineType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineAwaitType __pyx_mstate_global->__pyx_CoroutineAwaitType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineType __pyx_mstate_global->__pyx_CoroutineType
#endif
#if CYTHON_USE_MODULE_STATE
#define __pyx_type_7supriya_5utils_10_intervals__CInterval __pyx_mstate_global->__pyx_type_7supriya_5utils_10_intervals__CInterval
#define __pyx_type_7supriya_5utils_10_intervals__CNode __pyx_mstate_global->__pyx_type_7supriya_5utils_10_intervals__CNode
#define __pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx __pyx_mstate_global->__pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx
#define __pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ __pyx_mstate_global->__pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter__
#endif
#define __pyx_ptype_7supriya_5utils_10_intervals__CInterval __pyx_mstate_global->__pyx_ptype_7supriya_5utils_10_intervals__CInterval
#define __pyx_ptype_7supriya_5utils_10_intervals__CNode __pyx_mstate_global->__pyx_ptype_7supriya_5utils_10_intervals__CNode
#define __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx __pyx_mstate_global->__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx
#define __pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ __pyx_mstate_global->__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__
#define __pyx_n_s_AssertionError __pyx_mstate_global->__pyx_n_s_AssertionError
#define __pyx_n_s_CInterval __pyx_mstate_global->__pyx_n_s_CInterval
#define __pyx_n_s_CInterval___reduce_cython __pyx_mstate_global->__pyx_n_s_CInterval___reduce_cython
#define __pyx_n_s_CInterval___setstate_cython __pyx_mstate_global->__pyx_n_s_CInterval___setstate_cython
#define __pyx_n_s_CInterval_from_interval __pyx_mstate_global->__pyx_n_s_CInterval_from_interval
#define __pyx_n_s_CInterval_intersects_interval __pyx_mstate_global->__pyx_n_s_CInterval_intersects_interval
#define __pyx_n_s_CNode __pyx_mstate_global->__pyx_n_s_CNode
#define __pyx_n_s_CNode___reduce_cython __pyx_mstate_global->__pyx_n_s_CNode___reduce_cython
#define __pyx_n_s_CNode___setstate_cython __pyx_mstate_global->__pyx_n_s_CNode___setstate_cython
#define __pyx_kp_s_Incompatible_checksums_0x_x_vs_0 __pyx_mstate_global->__pyx_kp_s_Incompatible_checksums_0x_x_vs_0
#define __pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2 __pyx_mstate_global->__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2
#define __pyx_n_s_IndexError __pyx_mstate_global->__pyx_n_s_IndexError
#define __pyx_kp_u_Indices_must_be_integers_or_slic __pyx_mstate_global->__pyx_kp_u_Indices_must_be_integers_or_slic
#define __pyx_n_s_IntervalTreeDriverEx __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx
#define __pyx_n_s_IntervalTreeDriverEx___iter __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx___iter
#define __pyx_n_s_IntervalTreeDriverEx___reduce_cy __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx___reduce_cy
#define __pyx_n_s_IntervalTreeDriverEx___setstate __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx___setstate
#define __pyx_n_s_IntervalTreeDriverEx__insert_int __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx__insert_int
#define __pyx_n_s_IntervalTreeDriverEx__is_interva __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx__is_interva
#define __pyx_n_s_IntervalTreeDriverEx__unbox_cint __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx__unbox_cint
#define __pyx_n_s_IntervalTreeDriverEx_add __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_add
#define __pyx_n_s_IntervalTreeDriverEx_find_interv __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_find_interv
#define __pyx_n_s_IntervalTreeDriverEx_find_interv_2 __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_find_interv_2
#define __pyx_n_s_IntervalTreeDriverEx_find_interv_3 __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_find_interv_3
#define __pyx_n_s_IntervalTreeDriverEx_find_interv_4 __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_find_interv_4
#define __pyx_n_s_IntervalTreeDriverEx_get_offset __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_get_offset
#define __pyx_n_s_IntervalTreeDriverEx_get_start_o __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_get_start_o
#define __pyx_n_s_IntervalTreeDriverEx_get_start_o_2 __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_get_start_o_2
#define __pyx_n_s_IntervalTreeDriverEx_index __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_index
#define __pyx_n_s_IntervalTreeDriverEx_remove __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_remove
#define __pyx_n_s_IntervalTreeDriverEx_update __pyx_mstate_global->__pyx_n_s_IntervalTreeDriverEx_update
#define __pyx_n_s_PickleError __pyx_mstate_global->__pyx_n_s_PickleError
#define __pyx_n_s_TypeError __pyx_mstate_global->__pyx_n_s_TypeError
#define __pyx_n_s_ValueError __pyx_mstate_global->__pyx_n_s_ValueError
#define __pyx_kp_u__2 __pyx_mstate_global->__pyx_kp_u__2
#define __pyx_n_s__41 __pyx_mstate_global->__pyx_n_s__41
#define __pyx_n_s_add __pyx_mstate_global->__pyx_n_s_add
#define __pyx_n_s_append __pyx_mstate_global->__pyx_n_s_append
#define __pyx_n_s_args __pyx_mstate_global->__pyx_n_s_args
#define __pyx_n_s_asyncio_coroutines __pyx_mstate_global->__pyx_n_s_asyncio_coroutines
#define __pyx_n_s_cinterval __pyx_mstate_global->__pyx_n_s_cinterval
#define __pyx_n_s_cintervals __pyx_mstate_global->__pyx_n_s_cintervals
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_close __pyx_mstate_global->__pyx_n_s_close
#define __pyx_n_s_cls __pyx_mstate_global->__pyx_n_s_cls
#define __pyx_n_s_dict __pyx_mstate_global->__pyx_n_s_dict
#define __pyx_n_s_dict_2 __pyx_mstate_global->__pyx_n_s_dict_2
#define __pyx_kp_u_disable __pyx_mstate_global->__pyx_kp_u_disable
#define __pyx_kp_u_enable __pyx_mstate_global->__pyx_kp_u_enable
#define __pyx_n_s_expr __pyx_mstate_global->__pyx_n_s_expr
#define __pyx_n_s_find_intervals_intersecting_inte __pyx_mstate_global->__pyx_n_s_find_intervals_intersecting_inte
#define __pyx_n_s_find_intervals_intersecting_offs __pyx_mstate_global->__pyx_n_s_find_intervals_intersecting_offs
#define __pyx_n_s_find_intervals_starting_at __pyx_mstate_global->__pyx_n_s_find_intervals_starting_at
#define __pyx_n_s_find_intervals_stopping_at __pyx_mstate_global->__pyx_n_s_find_intervals_stopping_at
#define __pyx_n_s_format __pyx_mstate_global->__pyx_n_s_format
#define __pyx_n_s_from_interval __pyx_mstate_global->__pyx_n_s_from_interval
#define __pyx_kp_u_gc __pyx_mstate_global->__pyx_kp_u_gc
#define __pyx_n_s_get_offset_after __pyx_mstate_global->__pyx_n_s_get_offset_after
#define __pyx_n_s_get_start_offset_after __pyx_mstate_global->__pyx_n_s_get_start_offset_after
#define __pyx_n_s_get_start_offset_before __pyx_mstate_global->__pyx_n_s_get_start_offset_before
#define __pyx_n_s_getstate __pyx_mstate_global->__pyx_n_s_getstate
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_n_s_index __pyx_mstate_global->__pyx_n_s_index
#define __pyx_n_s_indices __pyx_mstate_global->__pyx_n_s_indices
#define __pyx_n_u_inf __pyx_mstate_global->__pyx_n_u_inf
#define __pyx_kp_u_inf_2 __pyx_mstate_global->__pyx_kp_u_inf_2
#define __pyx_n_s_intersects_interval __pyx_mstate_global->__pyx_n_s_intersects_interval
#define __pyx_n_s_interval __pyx_mstate_global->__pyx_n_s_interval
#define __pyx_n_s_intervals __pyx_mstate_global->__pyx_n_s_intervals
#define __pyx_n_s_is_coroutine __pyx_mstate_global->__pyx_n_s_is_coroutine
#define __pyx_n_s_is_interval __pyx_mstate_global->__pyx_n_s_is_interval
#define __pyx_kp_u_isenabled __pyx_mstate_global->__pyx_kp_u_isenabled
#define __pyx_n_s_iter __pyx_mstate_global->__pyx_n_s_iter
#define __pyx_n_s_key __pyx_mstate_global->__pyx_n_s_key
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_s_new __pyx_mstate_global->__pyx_n_s_new
#define __pyx_kp_s_no_default___reduce___due_to_non __pyx_mstate_global->__pyx_kp_s_no_default___reduce___due_to_non
#define __pyx_n_s_node __pyx_mstate_global->__pyx_n_s_node
#define __pyx_kp_u_not_in_interval_tree __pyx_mstate_global->__pyx_kp_u_not_in_interval_tree
#define __pyx_n_s_offset __pyx_mstate_global->__pyx_n_s_offset
#define __pyx_n_s_original_interval __pyx_mstate_global->__pyx_n_s_original_interval
#define __pyx_n_s_other __pyx_mstate_global->__pyx_n_s_other
#define __pyx_n_s_pickle __pyx_mstate_global->__pyx_n_s_pickle
#define __pyx_n_s_pop __pyx_mstate_global->__pyx_n_s_pop
#define __pyx_n_s_pyx_PickleError __pyx_mstate_global->__pyx_n_s_pyx_PickleError
#define __pyx_n_s_pyx_checksum __pyx_mstate_global->__pyx_n_s_pyx_checksum
#define __pyx_n_s_pyx_result __pyx_mstate_global->__pyx_n_s_pyx_result
#define __pyx_n_s_pyx_state __pyx_mstate_global->__pyx_n_s_pyx_state
#define __pyx_n_s_pyx_type __pyx_mstate_global->__pyx_n_s_pyx_type
#define __pyx_n_s_pyx_unpickle_IntervalTreeDrive __pyx_mstate_global->__pyx_n_s_pyx_unpickle_IntervalTreeDrive
#define __pyx_n_s_pyx_unpickle__CNode __pyx_mstate_global->__pyx_n_s_pyx_unpickle__CNode
#define __pyx_n_s_pyx_vtable __pyx_mstate_global->__pyx_n_s_pyx_vtable
#define __pyx_n_s_range __pyx_mstate_global->__pyx_n_s_range
#define __pyx_n_s_reduce __pyx_mstate_global->__pyx_n_s_reduce
#define __pyx_n_s_reduce_cython __pyx_mstate_global->__pyx_n_s_reduce_cython
#define __pyx_n_s_reduce_ex __pyx_mstate_global->__pyx_n_s_reduce_ex
#define __pyx_n_s_remove __pyx_mstate_global->__pyx_n_s_remove
#define __pyx_n_s_result __pyx_mstate_global->__pyx_n_s_result
#define __pyx_n_s_results __pyx_mstate_global->__pyx_n_s_results
#define __pyx_n_s_self __pyx_mstate_global->__pyx_n_s_self
#define __pyx_n_s_send __pyx_mstate_global->__pyx_n_s_send
#define __pyx_n_s_setstate __pyx_mstate_global->__pyx_n_s_setstate
#define __pyx_n_s_setstate_cython __pyx_mstate_global->__pyx_n_s_setstate_cython
#define __pyx_n_s_sort __pyx_mstate_global->__pyx_n_s_sort
#define __pyx_n_s_start_offset __pyx_mstate_global->__pyx_n_s_start_offset
#define __pyx_n_u_start_offset __pyx_mstate_global->__pyx_n_u_start_offset
#define __pyx_n_s_state __pyx_mstate_global->__pyx_n_s_state
#define __pyx_n_s_staticmethod __pyx_mstate_global->__pyx_n_s_staticmethod
#define __pyx_n_s_stop_offset __pyx_mstate_global->__pyx_n_s_stop_offset
#define __pyx_n_u_stop_offset __pyx_mstate_global->__pyx_n_u_stop_offset
#define __pyx_kp_s_stringsource __pyx_mstate_global->__pyx_kp_s_stringsource
#define __pyx_n_s_subtree_stop_index __pyx_mstate_global->__pyx_n_s_subtree_stop_index
#define __pyx_n_s_supriya_utils__intervals __pyx_mstate_global->__pyx_n_s_supriya_utils__intervals
#define __pyx_kp_s_supriya_utils__intervals_pyx __pyx_mstate_global->__pyx_kp_s_supriya_utils__intervals_pyx
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_n_s_throw __pyx_mstate_global->__pyx_n_s_throw
#define __pyx_n_s_update __pyx_mstate_global->__pyx_n_s_update
#define __pyx_n_s_use_setstate __pyx_mstate_global->__pyx_n_s_use_setstate
#define __pyx_n_s_x __pyx_mstate_global->__pyx_n_s_x
#define __pyx_int_0 __pyx_mstate_global->__pyx_int_0
#define __pyx_int_8917134 __pyx_mstate_global->__pyx_int_8917134
#define __pyx_int_149934309 __pyx_mstate_global->__pyx_int_149934309
#define __pyx_int_163126392 __pyx_mstate_global->__pyx_int_163126392
#define __pyx_int_168529197 __pyx_mstate_global->__pyx_int_168529197
#define __pyx_int_244102488 __pyx_mstate_global->__pyx_int_244102488
#define __pyx_int_249427599 __pyx_mstate_global->__pyx_int_249427599
#define __pyx_tuple_ __pyx_mstate_global->__pyx_tuple_
#define __pyx_tuple__3 __pyx_mstate_global->__pyx_tuple__3
#define __pyx_tuple__4 __pyx_mstate_global->__pyx_tuple__4
#define __pyx_tuple__6 __pyx_mstate_global->__pyx_tuple__6
#define __pyx_tuple__8 __pyx_mstate_global->__pyx_tuple__8
#define __pyx_tuple__10 __pyx_mstate_global->__pyx_tuple__10
#define __pyx_tuple__12 __pyx_mstate_global->__pyx_tuple__12
#define __pyx_tuple__15 __pyx_mstate_global->__pyx_tuple__15
#define __pyx_tuple__17 __pyx_mstate_global->__pyx_tuple__17
#define __pyx_tuple__19 __pyx_mstate_global->__pyx_tuple__19
#define __pyx_tuple__21 __pyx_mstate_global->__pyx_tuple__21
#define __pyx_tuple__24 __pyx_mstate_global->__pyx_tuple__24
#define __pyx_tuple__26 __pyx_mstate_global->__pyx_tuple__26
#define __pyx_tuple__28 __pyx_mstate_global->__pyx_tuple__28
#define __pyx_tuple__31 __pyx_mstate_global->__pyx_tuple__31
#define __pyx_tuple__34 __pyx_mstate_global->__pyx_tuple__34
#define __pyx_tuple__38 __pyx_mstate_global->__pyx_tuple__38
#define __pyx_codeobj__5 __pyx_mstate_global->__pyx_codeobj__5
#define __pyx_codeobj__7 __pyx_mstate_global->__pyx_codeobj__7
#define __pyx_codeobj__9 __pyx_mstate_global->__pyx_codeobj__9
#define __pyx_codeobj__11 __pyx_mstate_global->__pyx_codeobj__11
#define __pyx_codeobj__13 __pyx_mstate_global->__pyx_codeobj__13
#define __pyx_codeobj__14 __pyx_mstate_global->__pyx_codeobj__14
#define __pyx_codeobj__16 __pyx_mstate_global->__pyx_codeobj__16
#define __pyx_codeobj__18 __pyx_mstate_global->__pyx_codeobj__18
#define __pyx_codeobj__20 __pyx_mstate_global->__pyx_codeobj__20
#define __pyx_codeobj__22 __pyx_mstate_global->__pyx_codeobj__22
#define __pyx_codeobj__23 __pyx_mstate_global->__pyx_codeobj__23
#define __pyx_codeobj__25 __pyx_mstate_global->__pyx_codeobj__25
#define __pyx_codeobj__27 __pyx_mstate_global->__pyx_codeobj__27
#define __pyx_codeobj__29 __pyx_mstate_global->__pyx_codeobj__29
#define __pyx_codeobj__30 __pyx_mstate_global->__pyx_codeobj__30
#define __pyx_codeobj__32 __pyx_mstate_global->__pyx_codeobj__32
#define __pyx_codeobj__33 __pyx_mstate_global->__pyx_codeobj__33
#define __pyx_codeobj__35 __pyx_mstate_global->__pyx_codeobj__35
#define __pyx_codeobj__36 __pyx_mstate_global->__pyx_codeobj__36
#define __pyx_codeobj__37 __pyx_mstate_global->__pyx_codeobj__37
#define __pyx_codeobj__39 __pyx_mstate_global->__pyx_codeobj__39
#define __pyx_codeobj__40 __pyx_mstate_global->__pyx_codeobj__40
/* #### Code section: module_code ### */

/* "supriya/utils/_intervals.pyx":10
 *     cdef readonly object original_interval
 * 
 *     def __cinit__(self, start_offset, stop_offset, original_interval):             # <<<<<<<<<<<<<<
 *         self.start_offset = start_offset
 *         self.stop_offset = stop_offset
 */

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_10_CInterval_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_10_CInterval_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start_offset = 0;
  PyObject *__pyx_v_stop_offset = 0;
  PyObject *__pyx_v_original_interval = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start_offset,&__pyx_n_s_stop_offset,&__pyx_n_s_original_interval,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_VARARGS(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_start_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 10, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_stop_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 10, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 1); __PYX_ERR(0, 10, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_original_interval)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 10, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, 2); __PYX_ERR(0, 10, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__cinit__") < 0)) __PYX_ERR(0, 10, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
      values[1] = __Pyx_Arg_VARARGS(__pyx_args, 1);
      values[2] = __Pyx_Arg_VARARGS(__pyx_args, 2);
    }
    __pyx_v_start_offset = values[0];
    __pyx_v_stop_offset = values[1];
    __pyx_v_original_interval = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 10, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval___cinit__(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self), __pyx_v_start_offset, __pyx_v_stop_offset, __pyx_v_original_interval);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_10_CInterval___cinit__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, PyObject *__pyx_v_start_offset, PyObject *__pyx_v_stop_offset, PyObject *__pyx_v_original_interval) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 1);

  /* "supriya/utils/_intervals.pyx":11
 * 
 *     def __cinit__(self, start_offset, stop_offset, original_interval):
 *         self.start_offset = start_offset             # <<<<<<<<<<<<<<
 *         self.stop_offset = stop_offset
 *         self.original_interval = original_interval
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_start_offset); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 11, __pyx_L1_error)
  __pyx_v_self->start_offset = __pyx_t_1;

  /* "supriya/utils/_intervals.pyx":12
 *     def __cinit__(self, start_offset, stop_offset, original_interval):
 *         self.start_offset = start_offset
 *         self.stop_offset = stop_offset             # <<<<<<<<<<<<<<
 *         self.original_interval = original_interval
 * 
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_stop_offset); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 12, __pyx_L1_error)
  __pyx_v_self->stop_offset = __pyx_t_1;

  /* "supriya/utils/_intervals.pyx":13
 *         self.start_offset = start_offset
 *         self.stop_offset = stop_offset
 *         self.original_interval = original_interval             # <<<<<<<<<<<<<<
 * 
 *     def __richcmp__(_CInterval self, _CInterval other, int op):
 */
  __Pyx_INCREF(__pyx_v_original_interval);
  __Pyx_GIVEREF(__pyx_v_original_interval);
  __Pyx_GOTREF(__pyx_v_self->original_interval);
  __Pyx_DECREF(__pyx_v_self->original_interval);
  __pyx_v_self->original_interval = __pyx_v_original_interval;

  /* "supriya/utils/_intervals.pyx":10
 *     cdef readonly object original_interval
 * 
 *     def __cinit__(self, start_offset, stop_offset, original_interval):             # <<<<<<<<<<<<<<
 *         self.start_offset = start_offset
 *         self.stop_offset = stop_offset
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":15
 *         self.original_interval = original_interval
 * 
 *     def __richcmp__(_CInterval self, _CInterval other, int op):             # <<<<<<<<<<<<<<
 *         if op == 0:  # <
 *             if self.start_offset < other.start_offset:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_3__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_op); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_3__richcmp__(PyObject *__pyx_v_self, PyObject *__pyx_v_other, int __pyx_v_op) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_7supriya_5utils_10_intervals__CInterval, 1, "other", 0))) __PYX_ERR(0, 15, __pyx_L1_error)
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_2__richcmp__(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self), ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_other), ((int)__pyx_v_op));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_2__richcmp__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_other, int __pyx_v_op) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__richcmp__", 1);

  /* "supriya/utils/_intervals.pyx":16
 * 
 *     def __richcmp__(_CInterval self, _CInterval other, int op):
 *         if op == 0:  # <             # <<<<<<<<<<<<<<
 *             if self.start_offset < other.start_offset:
 *                 return True
 */
  __pyx_t_1 = (__pyx_v_op == 0);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":17
 *     def __richcmp__(_CInterval self, _CInterval other, int op):
 *         if op == 0:  # <
 *             if self.start_offset < other.start_offset:             # <<<<<<<<<<<<<<
 *                 return True
 *             if self.start_offset > other.start_offset:
 */
    __pyx_t_1 = (__pyx_v_self->start_offset < __pyx_v_other->start_offset);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":18
 *         if op == 0:  # <
 *             if self.start_offset < other.start_offset:
 *                 return True             # <<<<<<<<<<<<<<
 *             if self.start_offset > other.start_offset:
 *                 return False
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":17
 *     def __richcmp__(_CInterval self, _CInterval other, int op):
 *         if op == 0:  # <
 *             if self.start_offset < other.start_offset:             # <<<<<<<<<<<<<<
 *                 return True
 *             if self.start_offset > other.start_offset:
 */
    }

    /* "supriya/utils/_intervals.pyx":19
 *             if self.start_offset < other.start_offset:
 *                 return True
 *             if self.start_offset > other.start_offset:             # <<<<<<<<<<<<<<
 *                 return False
 *             if self.stop_offset < other.stop_offset:
 */
    __pyx_t_1 = (__pyx_v_self->start_offset > __pyx_v_other->start_offset);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":20
 *                 return True
 *             if self.start_offset > other.start_offset:
 *                 return False             # <<<<<<<<<<<<<<
 *             if self.stop_offset < other.stop_offset:
 *                 return True
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_False);
      __pyx_r = Py_False;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":19
 *             if self.start_offset < other.start_offset:
 *                 return True
 *             if self.start_offset > other.start_offset:             # <<<<<<<<<<<<<<
 *                 return False
 *             if self.stop_offset < other.stop_offset:
 */
    }

    /* "supriya/utils/_intervals.pyx":21
 *             if self.start_offset > other.start_offset:
 *                 return False
 *             if self.stop_offset < other.stop_offset:             # <<<<<<<<<<<<<<
 *                 return True
 *             return False
 */
    __pyx_t_1 = (__pyx_v_self->stop_offset < __pyx_v_other->stop_offset);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":22
 *                 return False
 *             if self.stop_offset < other.stop_offset:
 *                 return True             # <<<<<<<<<<<<<<
 *             return False
 *         if op == 2:  # ==
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":21
 *             if self.start_offset > other.start_offset:
 *                 return False
 *             if self.stop_offset < other.stop_offset:             # <<<<<<<<<<<<<<
 *                 return True
 *             return False
 */
    }

    /* "supriya/utils/_intervals.pyx":23
 *             if self.stop_offset < other.stop_offset:
 *                 return True
 *             return False             # <<<<<<<<<<<<<<
 *         if op == 2:  # ==
 *             if self.start_offset != other.start_offset:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":16
 * 
 *     def __richcmp__(_CInterval self, _CInterval other, int op):
 *         if op == 0:  # <             # <<<<<<<<<<<<<<
 *             if self.start_offset < other.start_offset:
 *                 return True
 */
  }

  /* "supriya/utils/_intervals.pyx":24
 *                 return True
 *             return False
 *         if op == 2:  # ==             # <<<<<<<<<<<<<<
 *             if self.start_offset != other.start_offset:
 *                 return False
 */
  __pyx_t_1 = (__pyx_v_op == 2);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":25
 *             return False
 *         if op == 2:  # ==
 *             if self.start_offset != other.start_offset:             # <<<<<<<<<<<<<<
 *                 return False
 *             if self.stop_offset != other.stop_offset:
 */
    __pyx_t_1 = (__pyx_v_self->start_offset != __pyx_v_other->start_offset);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":26
 *         if op == 2:  # ==
 *             if self.start_offset != other.start_offset:
 *                 return False             # <<<<<<<<<<<<<<
 *             if self.stop_offset != other.stop_offset:
 *                 return False
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_False);
      __pyx_r = Py_False;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":25
 *             return False
 *         if op == 2:  # ==
 *             if self.start_offset != other.start_offset:             # <<<<<<<<<<<<<<
 *                 return False
 *             if self.stop_offset != other.stop_offset:
 */
    }

    /* "supriya/utils/_intervals.pyx":27
 *             if self.start_offset != other.start_offset:
 *                 return False
 *             if self.stop_offset != other.stop_offset:             # <<<<<<<<<<<<<<
 *                 return False
 *             if self.original_interval != other.original_interval:
 */
    __pyx_t_1 = (__pyx_v_self->stop_offset != __pyx_v_other->stop_offset);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":28
 *                 return False
 *             if self.stop_offset != other.stop_offset:
 *                 return False             # <<<<<<<<<<<<<<
 *             if self.original_interval != other.original_interval:
 *                 return False
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_False);
      __pyx_r = Py_False;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":27
 *             if self.start_offset != other.start_offset:
 *                 return False
 *             if self.stop_offset != other.stop_offset:             # <<<<<<<<<<<<<<
 *                 return False
 *             if self.original_interval != other.original_interval:
 */
    }

    /* "supriya/utils/_intervals.pyx":29
 *             if self.stop_offset != other.stop_offset:
 *                 return False
 *             if self.original_interval != other.original_interval:             # <<<<<<<<<<<<<<
 *                 return False
 *             return True
 */
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->original_interval, __pyx_v_other->original_interval, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 29, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 29, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":30
 *                 return False
 *             if self.original_interval != other.original_interval:
 *                 return False             # <<<<<<<<<<<<<<
 *             return True
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_False);
      __pyx_r = Py_False;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":29
 *             if self.stop_offset != other.stop_offset:
 *                 return False
 *             if self.original_interval != other.original_interval:             # <<<<<<<<<<<<<<
 *                 return False
 *             return True
 */
    }

    /* "supriya/utils/_intervals.pyx":31
 *             if self.original_interval != other.original_interval:
 *                 return False
 *             return True             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":24
 *                 return True
 *             return False
 *         if op == 2:  # ==             # <<<<<<<<<<<<<<
 *             if self.start_offset != other.start_offset:
 *                 return False
 */
  }

  /* "supriya/utils/_intervals.pyx":15
 *         self.original_interval = original_interval
 * 
 *     def __richcmp__(_CInterval self, _CInterval other, int op):             # <<<<<<<<<<<<<<
 *         if op == 0:  # <
 *             if self.start_offset < other.start_offset:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":33
 *             return True
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_interval(cls, interval):
 *         start_offset = float(interval.start_offset)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_5from_interval(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_5from_interval = {"from_interval", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_5from_interval, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_5from_interval(PyObject *__pyx_v_cls, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_interval = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("from_interval (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_interval,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_interval)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 33, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "from_interval") < 0)) __PYX_ERR(0, 33, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_interval = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("from_interval", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 33, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.from_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_4from_interval(((PyTypeObject*)__pyx_v_cls), __pyx_v_interval);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_4from_interval(PyTypeObject *__pyx_v_cls, PyObject *__pyx_v_interval) {
  double __pyx_v_start_offset;
  double __pyx_v_stop_offset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("from_interval", 1);

  /* "supriya/utils/_intervals.pyx":35
 *     @classmethod
 *     def from_interval(cls, interval):
 *         start_offset = float(interval.start_offset)             # <<<<<<<<<<<<<<
 *         stop_offset = float(interval.stop_offset)
 *         return cls(
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_interval, __pyx_n_s_start_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_AsDouble(__pyx_t_1); if (unlikely(__pyx_t_2 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_start_offset = __pyx_t_2;

  /* "supriya/utils/_intervals.pyx":36
 *     def from_interval(cls, interval):
 *         start_offset = float(interval.start_offset)
 *         stop_offset = float(interval.stop_offset)             # <<<<<<<<<<<<<<
 *         return cls(
 *             start_offset,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_interval, __pyx_n_s_stop_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_AsDouble(__pyx_t_1); if (unlikely(__pyx_t_2 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_stop_offset = __pyx_t_2;

  /* "supriya/utils/_intervals.pyx":37
 *         start_offset = float(interval.start_offset)
 *         stop_offset = float(interval.stop_offset)
 *         return cls(             # <<<<<<<<<<<<<<
 *             start_offset,
 *             stop_offset,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "supriya/utils/_intervals.pyx":38
 *         stop_offset = float(interval.stop_offset)
 *         return cls(
 *             start_offset,             # <<<<<<<<<<<<<<
 *             stop_offset,
 *             interval,
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_start_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supriya/utils/_intervals.pyx":39
 *         return cls(
 *             start_offset,
 *             stop_offset,             # <<<<<<<<<<<<<<
 *             interval,
 *             )
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_stop_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "supriya/utils/_intervals.pyx":37
 *         start_offset = float(interval.start_offset)
 *         stop_offset = float(interval.stop_offset)
 *         return cls(             # <<<<<<<<<<<<<<
 *             start_offset,
 *             stop_offset,
 */
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_interval);
  __Pyx_GIVEREF(__pyx_v_interval);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_interval)) __PYX_ERR(0, 37, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_v_cls), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":33
 *             return True
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_interval(cls, interval):
 *         start_offset = float(interval.start_offset)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.from_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":43
 *             )
 * 
 *     cpdef bint intersects_interval(_CInterval self, _CInterval other):             # <<<<<<<<<<<<<<
 *         return (
 *             (
 */

static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_7intersects_interval(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static int __pyx_f_7supriya_5utils_10_intervals_10_CInterval_intersects_interval(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_other, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("intersects_interval", 1);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_intersects_interval); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void*) __pyx_pw_7supriya_5utils_10_intervals_10_CInterval_7intersects_interval)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, ((PyObject *)__pyx_v_other)};
          __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 43, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 43, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "supriya/utils/_intervals.pyx":46
 *         return (
 *             (
 *                 other.start_offset <= self.start_offset and             # <<<<<<<<<<<<<<
 *                 self.start_offset < other.stop_offset
 *             ) or (
 */
  __pyx_t_7 = (__pyx_v_other->start_offset <= __pyx_v_self->start_offset);
  if (!__pyx_t_7) {
    goto __pyx_L4_next_or;
  } else {
  }

  /* "supriya/utils/_intervals.pyx":47
 *             (
 *                 other.start_offset <= self.start_offset and
 *                 self.start_offset < other.stop_offset             # <<<<<<<<<<<<<<
 *             ) or (
 *                 self.start_offset <= other.start_offset and
 */
  __pyx_t_7 = (__pyx_v_self->start_offset < __pyx_v_other->stop_offset);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_6 = __pyx_t_7;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_L4_next_or:;

  /* "supriya/utils/_intervals.pyx":49
 *                 self.start_offset < other.stop_offset
 *             ) or (
 *                 self.start_offset <= other.start_offset and             # <<<<<<<<<<<<<<
 *                 other.start_offset < self.stop_offset
 *                 )
 */
  __pyx_t_7 = (__pyx_v_self->start_offset <= __pyx_v_other->start_offset);
  if (__pyx_t_7) {
  } else {
    __pyx_t_6 = __pyx_t_7;
    goto __pyx_L3_bool_binop_done;
  }

  /* "supriya/utils/_intervals.pyx":50
 *             ) or (
 *                 self.start_offset <= other.start_offset and
 *                 other.start_offset < self.stop_offset             # <<<<<<<<<<<<<<
 *                 )
 *             )
 */
  __pyx_t_7 = (__pyx_v_other->start_offset < __pyx_v_self->stop_offset);
  __pyx_t_6 = __pyx_t_7;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_6;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":43
 *             )
 * 
 *     cpdef bint intersects_interval(_CInterval self, _CInterval other):             # <<<<<<<<<<<<<<
 *         return (
 *             (
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.intersects_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_7intersects_interval(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_7intersects_interval = {"intersects_interval", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_7intersects_interval, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_7intersects_interval(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_other = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("intersects_interval (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_other)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 43, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "intersects_interval") < 0)) __PYX_ERR(0, 43, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_other = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("intersects_interval", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 43, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.intersects_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_7supriya_5utils_10_intervals__CInterval, 1, "other", 0))) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_6intersects_interval(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self), __pyx_v_other);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_6intersects_interval(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("intersects_interval", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7supriya_5utils_10_intervals_10_CInterval_intersects_interval(__pyx_v_self, __pyx_v_other, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.intersects_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":6
 * cdef class _CInterval:
 * 
 *     cdef readonly float start_offset             # <<<<<<<<<<<<<<
 *     cdef readonly float stop_offset
 *     cdef readonly object original_interval
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_12start_offset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_12start_offset_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_12start_offset___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_12start_offset___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->start_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.start_offset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":7
 * 
 *     cdef readonly float start_offset
 *     cdef readonly float stop_offset             # <<<<<<<<<<<<<<
 *     cdef readonly object original_interval
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_11stop_offset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_11stop_offset_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_11stop_offset___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_11stop_offset___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->stop_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.stop_offset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":8
 *     cdef readonly float start_offset
 *     cdef readonly float stop_offset
 *     cdef readonly object original_interval             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self, start_offset, stop_offset, original_interval):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_17original_interval_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_17original_interval_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_17original_interval___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_17original_interval___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->original_interval);
  __pyx_r = __pyx_v_self->original_interval;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_9__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_9__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_9__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_8__reduce_cython__(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_8__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_11__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_11__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_11__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 3, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_10_CInterval_10__setstate_cython__(((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_10_CInterval_10__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 */
  __Pyx_Raise(__pyx_builtin_TypeError, __pyx_kp_s_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CInterval.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":70
 *     cdef public int subtree_stop_index
 * 
 *     def __init__(self, start_offset):             # <<<<<<<<<<<<<<
 *         self.balance = 0
 *         self.height = 0
 */

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start_offset = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start_offset,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_start_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_VARARGS(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 70, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__init__") < 0)) __PYX_ERR(0, 70, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
    }
    __pyx_v_start_offset = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 70, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode___init__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), __pyx_v_start_offset);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode___init__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_start_offset) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 1);

  /* "supriya/utils/_intervals.pyx":71
 * 
 *     def __init__(self, start_offset):
 *         self.balance = 0             # <<<<<<<<<<<<<<
 *         self.height = 0
 *         self.left_child = None
 */
  __pyx_v_self->balance = 0;

  /* "supriya/utils/_intervals.pyx":72
 *     def __init__(self, start_offset):
 *         self.balance = 0
 *         self.height = 0             # <<<<<<<<<<<<<<
 *         self.left_child = None
 *         self.node_start_index = -1
 */
  __pyx_v_self->height = 0;

  /* "supriya/utils/_intervals.pyx":73
 *         self.balance = 0
 *         self.height = 0
 *         self.left_child = None             # <<<<<<<<<<<<<<
 *         self.node_start_index = -1
 *         self.node_stop_index = -1
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->left_child);
  __Pyx_DECREF((PyObject *)__pyx_v_self->left_child);
  __pyx_v_self->left_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None);

  /* "supriya/utils/_intervals.pyx":74
 *         self.height = 0
 *         self.left_child = None
 *         self.node_start_index = -1             # <<<<<<<<<<<<<<
 *         self.node_stop_index = -1
 *         self.payload = []
 */
  __pyx_v_self->node_start_index = -1;

  /* "supriya/utils/_intervals.pyx":75
 *         self.left_child = None
 *         self.node_start_index = -1
 *         self.node_stop_index = -1             # <<<<<<<<<<<<<<
 *         self.payload = []
 *         self.right_child = None
 */
  __pyx_v_self->node_stop_index = -1;

  /* "supriya/utils/_intervals.pyx":76
 *         self.node_start_index = -1
 *         self.node_stop_index = -1
 *         self.payload = []             # <<<<<<<<<<<<<<
 *         self.right_child = None
 *         self.start_offset = start_offset
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->payload);
  __Pyx_DECREF(__pyx_v_self->payload);
  __pyx_v_self->payload = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":77
 *         self.node_stop_index = -1
 *         self.payload = []
 *         self.right_child = None             # <<<<<<<<<<<<<<
 *         self.start_offset = start_offset
 *         self.stop_offset_high = start_offset
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->right_child);
  __Pyx_DECREF((PyObject *)__pyx_v_self->right_child);
  __pyx_v_self->right_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None);

  /* "supriya/utils/_intervals.pyx":78
 *         self.payload = []
 *         self.right_child = None
 *         self.start_offset = start_offset             # <<<<<<<<<<<<<<
 *         self.stop_offset_high = start_offset
 *         self.stop_offset_low = start_offset
 */
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_start_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 78, __pyx_L1_error)
  __pyx_v_self->start_offset = __pyx_t_2;

  /* "supriya/utils/_intervals.pyx":79
 *         self.right_child = None
 *         self.start_offset = start_offset
 *         self.stop_offset_high = start_offset             # <<<<<<<<<<<<<<
 *         self.stop_offset_low = start_offset
 *         self.subtree_start_index = -1
 */
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_start_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 79, __pyx_L1_error)
  __pyx_v_self->stop_offset_high = __pyx_t_2;

  /* "supriya/utils/_intervals.pyx":80
 *         self.start_offset = start_offset
 *         self.stop_offset_high = start_offset
 *         self.stop_offset_low = start_offset             # <<<<<<<<<<<<<<
 *         self.subtree_start_index = -1
 *         self.subtree_stop_index = -1
 */
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_start_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 80, __pyx_L1_error)
  __pyx_v_self->stop_offset_low = __pyx_t_2;

  /* "supriya/utils/_intervals.pyx":81
 *         self.stop_offset_high = start_offset
 *         self.stop_offset_low = start_offset
 *         self.subtree_start_index = -1             # <<<<<<<<<<<<<<
 *         self.subtree_stop_index = -1
 * 
 */
  __pyx_v_self->subtree_start_index = -1;

  /* "supriya/utils/_intervals.pyx":82
 *         self.stop_offset_low = start_offset
 *         self.subtree_start_index = -1
 *         self.subtree_stop_index = -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->subtree_stop_index = -1;

  /* "supriya/utils/_intervals.pyx":70
 *     cdef public int subtree_stop_index
 * 
 *     def __init__(self, start_offset):             # <<<<<<<<<<<<<<
 *         self.balance = 0
 *         self.height = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":57
 * cdef class _CNode:
 * 
 *     cdef public int balance             # <<<<<<<<<<<<<<
 *     cdef public int height
 *     cdef public _CNode left_child
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_7balance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_7balance_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7balance___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_7balance___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->balance); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.balance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7balance_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7balance_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7balance_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7balance_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 57, __pyx_L1_error)
  __pyx_v_self->balance = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.balance.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":58
 * 
 *     cdef public int balance
 *     cdef public int height             # <<<<<<<<<<<<<<
 *     cdef public _CNode left_child
 *     cdef public int node_start_index
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_6height_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_6height_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_6height___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_6height___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.height.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_6height_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_6height_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 58, __pyx_L1_error)
  __pyx_v_self->height = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.height.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":59
 *     cdef public int balance
 *     cdef public int height
 *     cdef public _CNode left_child             # <<<<<<<<<<<<<<
 *     cdef public int node_start_index
 *     cdef public int node_stop_index
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->left_child);
  __pyx_r = ((PyObject *)__pyx_v_self->left_child);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 1);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 59, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->left_child);
  __Pyx_DECREF((PyObject *)__pyx_v_self->left_child);
  __pyx_v_self->left_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.left_child.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child_4__del__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_10left_child_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->left_child);
  __Pyx_DECREF((PyObject *)__pyx_v_self->left_child);
  __pyx_v_self->left_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":60
 *     cdef public int height
 *     cdef public _CNode left_child
 *     cdef public int node_start_index             # <<<<<<<<<<<<<<
 *     cdef public int node_stop_index
 *     cdef public object payload
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_16node_start_index_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_16node_start_index_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16node_start_index___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_16node_start_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->node_start_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.node_start_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16node_start_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16node_start_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16node_start_index_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16node_start_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 60, __pyx_L1_error)
  __pyx_v_self->node_start_index = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.node_start_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":61
 *     cdef public _CNode left_child
 *     cdef public int node_start_index
 *     cdef public int node_stop_index             # <<<<<<<<<<<<<<
 *     cdef public object payload
 *     cdef public _CNode right_child
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15node_stop_index___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_15node_stop_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->node_stop_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.node_stop_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_v_self->node_stop_index = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.node_stop_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":62
 *     cdef public int node_start_index
 *     cdef public int node_stop_index
 *     cdef public object payload             # <<<<<<<<<<<<<<
 *     cdef public _CNode right_child
 *     cdef public float start_offset
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->payload);
  __pyx_r = __pyx_v_self->payload;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 1);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->payload);
  __Pyx_DECREF(__pyx_v_self->payload);
  __pyx_v_self->payload = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload_4__del__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_7payload_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->payload);
  __Pyx_DECREF(__pyx_v_self->payload);
  __pyx_v_self->payload = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":63
 *     cdef public int node_stop_index
 *     cdef public object payload
 *     cdef public _CNode right_child             # <<<<<<<<<<<<<<
 *     cdef public float start_offset
 *     cdef public float stop_offset_high
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->right_child);
  __pyx_r = ((PyObject *)__pyx_v_self->right_child);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 1);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 63, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->right_child);
  __Pyx_DECREF((PyObject *)__pyx_v_self->right_child);
  __pyx_v_self->right_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.right_child.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child_4__del__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_11right_child_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->right_child);
  __Pyx_DECREF((PyObject *)__pyx_v_self->right_child);
  __pyx_v_self->right_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":64
 *     cdef public object payload
 *     cdef public _CNode right_child
 *     cdef public float start_offset             # <<<<<<<<<<<<<<
 *     cdef public float stop_offset_high
 *     cdef public float stop_offset_low
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_12start_offset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_12start_offset_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_12start_offset___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_12start_offset___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->start_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.start_offset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_12start_offset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_12start_offset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_12start_offset_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_12start_offset_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  float __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 64, __pyx_L1_error)
  __pyx_v_self->start_offset = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.start_offset.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":65
 *     cdef public _CNode right_child
 *     cdef public float start_offset
 *     cdef public float stop_offset_high             # <<<<<<<<<<<<<<
 *     cdef public float stop_offset_low
 *     cdef public int subtree_start_index
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->stop_offset_high); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.stop_offset_high.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  float __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 65, __pyx_L1_error)
  __pyx_v_self->stop_offset_high = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.stop_offset_high.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":66
 *     cdef public float start_offset
 *     cdef public float stop_offset_high
 *     cdef public float stop_offset_low             # <<<<<<<<<<<<<<
 *     cdef public int subtree_start_index
 *     cdef public int subtree_stop_index
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->stop_offset_low); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.stop_offset_low.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  float __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 66, __pyx_L1_error)
  __pyx_v_self->stop_offset_low = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.stop_offset_low.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":67
 *     cdef public float stop_offset_high
 *     cdef public float stop_offset_low
 *     cdef public int subtree_start_index             # <<<<<<<<<<<<<<
 *     cdef public int subtree_stop_index
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->subtree_start_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.subtree_start_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 67, __pyx_L1_error)
  __pyx_v_self->subtree_start_index = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.subtree_start_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":68
 *     cdef public float stop_offset_low
 *     cdef public int subtree_start_index
 *     cdef public int subtree_stop_index             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, start_offset):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index___get__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index___get__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->subtree_stop_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.subtree_stop_index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 68, __pyx_L1_error)
  __pyx_v_self->subtree_stop_index = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.subtree_stop_index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_6_CNode_3__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_6_CNode_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_2__reduce_cython__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_2__reduce_cython__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.balance, self.height, self.left_child, self.node_start_index, self.node_stop_index, self.payload, self.right_child, self.start_offset, self.stop_offset_high, self.stop_offset_low, self.subtree_start_index, self.subtree_stop_index)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->balance); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->node_start_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->node_stop_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->start_offset); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->stop_offset_high); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->stop_offset_low); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_self->subtree_start_index); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_self->subtree_stop_index); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = PyTuple_New(12); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->left_child);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->left_child);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 2, ((PyObject *)__pyx_v_self->left_child))) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 3, __pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 4, __pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->payload);
  __Pyx_GIVEREF(__pyx_v_self->payload);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 5, __pyx_v_self->payload)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->right_child);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->right_child);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 6, ((PyObject *)__pyx_v_self->right_child))) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 7, __pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_6);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 8, __pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 9, __pyx_t_7)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_8);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 10, __pyx_t_8)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_9);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 11, __pyx_t_9)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_10);
  __pyx_t_10 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.balance, self.height, self.left_child, self.node_start_index, self.node_stop_index, self.payload, self.right_child, self.start_offset, self.stop_offset_high, self.stop_offset_low, self.subtree_start_index, self.subtree_stop_index)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_10 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_v__dict = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "(tree fragment)":7
 *     state = (self.balance, self.height, self.left_child, self.node_start_index, self.node_stop_index, self.payload, self.right_child, self.start_offset, self.stop_offset_high, self.stop_offset_low, self.subtree_start_index, self.subtree_stop_index)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_11 = (__pyx_v__dict != Py_None);
  if (__pyx_t_11) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v__dict)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_9 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_10); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_9));
    __pyx_t_9 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.left_child is not None or self.payload is not None or self.right_child is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.balance, self.height, self.left_child, self.node_start_index, self.node_stop_index, self.payload, self.right_child, self.start_offset, self.stop_offset_high, self.stop_offset_low, self.subtree_start_index, self.subtree_stop_index)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.left_child is not None or self.payload is not None or self.right_child is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, None), state
 */
  /*else*/ {
    __pyx_t_12 = (((PyObject *)__pyx_v_self->left_child) != Py_None);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_11 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (__pyx_v_self->payload != Py_None);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_11 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (((PyObject *)__pyx_v_self->right_child) != Py_None);
    __pyx_t_11 = __pyx_t_12;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_11;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.left_child is not None or self.payload is not None or self.right_child is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, None), state
 *     else:
 */
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.left_child is not None or self.payload is not None or self.right_child is not None
 *     if use_setstate:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_pyx_unpickle__CNode); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))))) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_int_244102488);
    __Pyx_GIVEREF(__pyx_int_244102488);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_int_244102488)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 2, Py_None)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_10);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_10)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_v_state)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_9 = 0;
    __pyx_t_10 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.left_child is not None or self.payload is not None or self.right_child is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, None), state
 *     else:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__CNode__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_pyx_unpickle__CNode); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))))) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_int_244102488);
    __Pyx_GIVEREF(__pyx_int_244102488);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_int_244102488)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_v_state)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_10);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_10)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_10 = 0;
    __pyx_r = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__CNode__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_6_CNode_5__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_6_CNode_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_6_CNode_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 16, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(1, 16, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_6_CNode_4__setstate_cython__(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_6_CNode_4__setstate_cython__(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":17
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__CNode__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_7supriya_5utils_10_intervals___pyx_unpickle__CNode__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__CNode__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals._CNode.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":96
 *     ### INITIALIZER ###
 * 
 *     def __init__(self, intervals=None):             # <<<<<<<<<<<<<<
 *         self._root_node = None
 *         self.update(intervals or [])
 */

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_intervals = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_intervals,0};
    values[0] = __Pyx_Arg_NewRef_VARARGS(((PyObject *)Py_None));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_VARARGS(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_VARARGS(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_intervals);
          if (value) { values[0] = __Pyx_Arg_NewRef_VARARGS(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 96, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__init__") < 0)) __PYX_ERR(0, 96, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_VARARGS(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_intervals = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 96, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx___init__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_intervals);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_VARARGS(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx___init__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_intervals) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 1);

  /* "supriya/utils/_intervals.pyx":97
 * 
 *     def __init__(self, intervals=None):
 *         self._root_node = None             # <<<<<<<<<<<<<<
 *         self.update(intervals or [])
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_root_node);
  __Pyx_DECREF(__pyx_v_self->_root_node);
  __pyx_v_self->_root_node = Py_None;

  /* "supriya/utils/_intervals.pyx":98
 *     def __init__(self, intervals=None):
 *         self._root_node = None
 *         self.update(intervals or [])             # <<<<<<<<<<<<<<
 * 
 *     ### SPECIAL METHODS ###
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_update); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_intervals); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 98, __pyx_L1_error)
  if (!__pyx_t_4) {
  } else {
    __Pyx_INCREF(__pyx_v_intervals);
    __pyx_t_3 = __pyx_v_intervals;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_3 = __pyx_t_5;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":96
 *     ### INITIALIZER ###
 * 
 *     def __init__(self, intervals=None):             # <<<<<<<<<<<<<<
 *         self._root_node = None
 *         self.update(intervals or [])
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":102
 *     ### SPECIAL METHODS ###
 * 
 *     def __contains__(self, interval):             # <<<<<<<<<<<<<<
 *         if not self._is_interval(interval):
 *             raise ValueError(interval)
 */

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_3__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_interval); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_3__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_interval) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_2__contains__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), ((PyObject *)__pyx_v_interval));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_2__contains__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval) {
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_result = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__contains__", 1);

  /* "supriya/utils/_intervals.pyx":103
 * 
 *     def __contains__(self, interval):
 *         if not self._is_interval(interval):             # <<<<<<<<<<<<<<
 *             raise ValueError(interval)
 *         candidates = self.find_intervals_starting_at(interval.start_offset)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (unlikely(__pyx_t_6)) {

    /* "supriya/utils/_intervals.pyx":104
 *     def __contains__(self, interval):
 *         if not self._is_interval(interval):
 *             raise ValueError(interval)             # <<<<<<<<<<<<<<
 *         candidates = self.find_intervals_starting_at(interval.start_offset)
 *         result = interval in candidates
 */
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_interval); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 104, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":103
 * 
 *     def __contains__(self, interval):
 *         if not self._is_interval(interval):             # <<<<<<<<<<<<<<
 *             raise ValueError(interval)
 *         candidates = self.find_intervals_starting_at(interval.start_offset)
 */
  }

  /* "supriya/utils/_intervals.pyx":105
 *         if not self._is_interval(interval):
 *             raise ValueError(interval)
 *         candidates = self.find_intervals_starting_at(interval.start_offset)             # <<<<<<<<<<<<<<
 *         result = interval in candidates
 *         return result
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_find_intervals_starting_at); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_interval, __pyx_n_s_start_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_candidates = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":106
 *             raise ValueError(interval)
 *         candidates = self.find_intervals_starting_at(interval.start_offset)
 *         result = interval in candidates             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
  __pyx_t_6 = (__Pyx_PySequence_ContainsTF(__pyx_v_interval, __pyx_v_candidates, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 106, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":107
 *         candidates = self.find_intervals_starting_at(interval.start_offset)
 *         result = interval in candidates
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_result); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 107, __pyx_L1_error)
  __pyx_r = __pyx_t_8;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":102
 *     ### SPECIAL METHODS ###
 * 
 *     def __contains__(self, interval):             # <<<<<<<<<<<<<<
 *         if not self._is_interval(interval):
 *             raise ValueError(interval)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":109
 *         return result
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if isinstance(i, int):
 *             if self._root_node is None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_5__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_5__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_4__getitem__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), ((PyObject *)__pyx_v_i));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_4__getitem__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_i) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval = NULL;
  PyObject *__pyx_v_indices = NULL;
  PyObject *__pyx_v_start = NULL;
  PyObject *__pyx_v_stop = NULL;
  PyObject *__pyx_v_cintervals = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  unsigned int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);
  __Pyx_INCREF(__pyx_v_i);

  /* "supriya/utils/_intervals.pyx":110
 * 
 *     def __getitem__(self, i):
 *         if isinstance(i, int):             # <<<<<<<<<<<<<<
 *             if self._root_node is None:
 *                 raise IndexError
 */
  __pyx_t_1 = PyInt_Check(__pyx_v_i); 
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":111
 *     def __getitem__(self, i):
 *         if isinstance(i, int):
 *             if self._root_node is None:             # <<<<<<<<<<<<<<
 *                 raise IndexError
 *             if i < 0:
 */
    __pyx_t_1 = (__pyx_v_self->_root_node == Py_None);
    if (unlikely(__pyx_t_1)) {

      /* "supriya/utils/_intervals.pyx":112
 *         if isinstance(i, int):
 *             if self._root_node is None:
 *                 raise IndexError             # <<<<<<<<<<<<<<
 *             if i < 0:
 *                 i = self._root_node.subtree_stop_index + i
 */
      __Pyx_Raise(__pyx_builtin_IndexError, 0, 0, 0);
      __PYX_ERR(0, 112, __pyx_L1_error)

      /* "supriya/utils/_intervals.pyx":111
 *     def __getitem__(self, i):
 *         if isinstance(i, int):
 *             if self._root_node is None:             # <<<<<<<<<<<<<<
 *                 raise IndexError
 *             if i < 0:
 */
    }

    /* "supriya/utils/_intervals.pyx":113
 *             if self._root_node is None:
 *                 raise IndexError
 *             if i < 0:             # <<<<<<<<<<<<<<
 *                 i = self._root_node.subtree_stop_index + i
 *             if i < 0 or self._root_node.subtree_stop_index <= i:
 */
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_i, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 113, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":114
 *                 raise IndexError
 *             if i < 0:
 *                 i = self._root_node.subtree_stop_index + i             # <<<<<<<<<<<<<<
 *             if i < 0 or self._root_node.subtree_stop_index <= i:
 *                 raise IndexError
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_root_node, __pyx_n_s_subtree_stop_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_i, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "supriya/utils/_intervals.pyx":113
 *             if self._root_node is None:
 *                 raise IndexError
 *             if i < 0:             # <<<<<<<<<<<<<<
 *                 i = self._root_node.subtree_stop_index + i
 *             if i < 0 or self._root_node.subtree_stop_index <= i:
 */
    }

    /* "supriya/utils/_intervals.pyx":115
 *             if i < 0:
 *                 i = self._root_node.subtree_stop_index + i
 *             if i < 0 or self._root_node.subtree_stop_index <= i:             # <<<<<<<<<<<<<<
 *                 raise IndexError
 *             cinterval = self._recurse_getitem_by_index(self._root_node, i)
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_i, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 115, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_root_node, __pyx_n_s_subtree_stop_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_3, __pyx_v_i, Py_LE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 115, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __pyx_t_4;
    __pyx_L7_bool_binop_done:;
    if (unlikely(__pyx_t_1)) {

      /* "supriya/utils/_intervals.pyx":116
 *                 i = self._root_node.subtree_stop_index + i
 *             if i < 0 or self._root_node.subtree_stop_index <= i:
 *                 raise IndexError             # <<<<<<<<<<<<<<
 *             cinterval = self._recurse_getitem_by_index(self._root_node, i)
 *             return cinterval.original_interval
 */
      __Pyx_Raise(__pyx_builtin_IndexError, 0, 0, 0);
      __PYX_ERR(0, 116, __pyx_L1_error)

      /* "supriya/utils/_intervals.pyx":115
 *             if i < 0:
 *                 i = self._root_node.subtree_stop_index + i
 *             if i < 0 or self._root_node.subtree_stop_index <= i:             # <<<<<<<<<<<<<<
 *                 raise IndexError
 *             cinterval = self._recurse_getitem_by_index(self._root_node, i)
 */
    }

    /* "supriya/utils/_intervals.pyx":117
 *             if i < 0 or self._root_node.subtree_stop_index <= i:
 *                 raise IndexError
 *             cinterval = self._recurse_getitem_by_index(self._root_node, i)             # <<<<<<<<<<<<<<
 *             return cinterval.original_interval
 *         elif isinstance(i, slice):
 */
    if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 117, __pyx_L1_error)
    __pyx_t_2 = __pyx_v_self->_root_node;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_i); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_getitem_by_index(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_t_5)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_cinterval = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":118
 *                 raise IndexError
 *             cinterval = self._recurse_getitem_by_index(self._root_node, i)
 *             return cinterval.original_interval             # <<<<<<<<<<<<<<
 *         elif isinstance(i, slice):
 *             if self._root_node is None:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_cinterval->original_interval);
    __pyx_r = __pyx_v_cinterval->original_interval;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":110
 * 
 *     def __getitem__(self, i):
 *         if isinstance(i, int):             # <<<<<<<<<<<<<<
 *             if self._root_node is None:
 *                 raise IndexError
 */
  }

  /* "supriya/utils/_intervals.pyx":119
 *             cinterval = self._recurse_getitem_by_index(self._root_node, i)
 *             return cinterval.original_interval
 *         elif isinstance(i, slice):             # <<<<<<<<<<<<<<
 *             if self._root_node is None:
 *                 return []
 */
  __pyx_t_1 = PySlice_Check(__pyx_v_i); 
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":120
 *             return cinterval.original_interval
 *         elif isinstance(i, slice):
 *             if self._root_node is None:             # <<<<<<<<<<<<<<
 *                 return []
 *             indices = i.indices(self._root_node.subtree_stop_index)
 */
    __pyx_t_1 = (__pyx_v_self->_root_node == Py_None);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":121
 *         elif isinstance(i, slice):
 *             if self._root_node is None:
 *                 return []             # <<<<<<<<<<<<<<
 *             indices = i.indices(self._root_node.subtree_stop_index)
 *             start, stop = indices[0], indices[1]
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":120
 *             return cinterval.original_interval
 *         elif isinstance(i, slice):
 *             if self._root_node is None:             # <<<<<<<<<<<<<<
 *                 return []
 *             indices = i.indices(self._root_node.subtree_stop_index)
 */
    }

    /* "supriya/utils/_intervals.pyx":122
 *             if self._root_node is None:
 *                 return []
 *             indices = i.indices(self._root_node.subtree_stop_index)             # <<<<<<<<<<<<<<
 *             start, stop = indices[0], indices[1]
 *             cintervals = self._recurse_getitem_by_slice(self._root_node, start, stop)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_i, __pyx_n_s_indices); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 122, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_root_node, __pyx_n_s_subtree_stop_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 122, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_8 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_6};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_8, 1+__pyx_t_8);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_v_indices = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":123
 *                 return []
 *             indices = i.indices(self._root_node.subtree_stop_index)
 *             start, stop = indices[0], indices[1]             # <<<<<<<<<<<<<<
 *             cintervals = self._recurse_getitem_by_slice(self._root_node, start, stop)
 *             return self._unbox_cintervals(cintervals)
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_indices, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_indices, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_start = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v_stop = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":124
 *             indices = i.indices(self._root_node.subtree_stop_index)
 *             start, stop = indices[0], indices[1]
 *             cintervals = self._recurse_getitem_by_slice(self._root_node, start, stop)             # <<<<<<<<<<<<<<
 *             return self._unbox_cintervals(cintervals)
 *         raise TypeError('Indices must be integers or slices, got {}'.format(i))
 */
    if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 124, __pyx_L1_error)
    __pyx_t_2 = __pyx_v_self->_root_node;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_start); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 124, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_v_stop); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 124, __pyx_L1_error)
    __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_getitem_by_slice(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_t_5, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_cintervals = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":125
 *             start, stop = indices[0], indices[1]
 *             cintervals = self._recurse_getitem_by_slice(self._root_node, start, stop)
 *             return self._unbox_cintervals(cintervals)             # <<<<<<<<<<<<<<
 *         raise TypeError('Indices must be integers or slices, got {}'.format(i))
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_unbox_cintervals(__pyx_v_self, __pyx_v_cintervals); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":119
 *             cinterval = self._recurse_getitem_by_index(self._root_node, i)
 *             return cinterval.original_interval
 *         elif isinstance(i, slice):             # <<<<<<<<<<<<<<
 *             if self._root_node is None:
 *                 return []
 */
  }

  /* "supriya/utils/_intervals.pyx":126
 *             cintervals = self._recurse_getitem_by_slice(self._root_node, start, stop)
 *             return self._unbox_cintervals(cintervals)
 *         raise TypeError('Indices must be integers or slices, got {}'.format(i))             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Indices_must_be_integers_or_slic, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  __pyx_t_8 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_8 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_i};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_8, 1+__pyx_t_8);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 126, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":109
 *         return result
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if isinstance(i, int):
 *             if self._root_node is None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cinterval);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XDECREF(__pyx_v_start);
  __Pyx_XDECREF(__pyx_v_stop);
  __Pyx_XDECREF(__pyx_v_cintervals);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_8generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "supriya/utils/_intervals.pyx":128
 *         raise TypeError('Indices must be integers or slices, got {}'.format(i))
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef _CNode current
 *         cdef _CInterval cinterval
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_7__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_7__iter__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_6__iter__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_6__iter__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self) {
  struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);
  __pyx_cur_scope = (struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *)__pyx_tp_new_7supriya_5utils_10_intervals___pyx_scope_struct____iter__(__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 128, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_8generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_iter, __pyx_n_s_IntervalTreeDriverEx___iter, __pyx_n_s_supriya_utils__intervals); if (unlikely(!gen)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_8generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *__pyx_cur_scope = ((struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L11_resume_from_yield;
    case 2: goto __pyx_L18_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 128, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":131
 *         cdef _CNode current
 *         cdef _CInterval cinterval
 *         stack = []             # <<<<<<<<<<<<<<
 *         current = self._root_node
 *         while True:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_stack = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":132
 *         cdef _CInterval cinterval
 *         stack = []
 *         current = self._root_node             # <<<<<<<<<<<<<<
 *         while True:
 *             while current is not None:
 */
  if (!(likely(((__pyx_cur_scope->__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_cur_scope->__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_t_1 = __pyx_cur_scope->__pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_current = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":133
 *         stack = []
 *         current = self._root_node
 *         while True:             # <<<<<<<<<<<<<<
 *             while current is not None:
 *                 stack.append(current)
 */
  while (1) {

    /* "supriya/utils/_intervals.pyx":134
 *         current = self._root_node
 *         while True:
 *             while current is not None:             # <<<<<<<<<<<<<<
 *                 stack.append(current)
 *                 current = current.left_child
 */
    while (1) {
      __pyx_t_2 = (((PyObject *)__pyx_cur_scope->__pyx_v_current) != Py_None);
      if (!__pyx_t_2) break;

      /* "supriya/utils/_intervals.pyx":135
 *         while True:
 *             while current is not None:
 *                 stack.append(current)             # <<<<<<<<<<<<<<
 *                 current = current.left_child
 *             if not stack:
 */
      __pyx_t_3 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_stack, ((PyObject *)__pyx_cur_scope->__pyx_v_current)); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 135, __pyx_L1_error)

      /* "supriya/utils/_intervals.pyx":136
 *             while current is not None:
 *                 stack.append(current)
 *                 current = current.left_child             # <<<<<<<<<<<<<<
 *             if not stack:
 *                 return
 */
      __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_current->left_child);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_GOTREF((PyObject *)__pyx_cur_scope->__pyx_v_current);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_current, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1));
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;
    }

    /* "supriya/utils/_intervals.pyx":137
 *                 stack.append(current)
 *                 current = current.left_child
 *             if not stack:             # <<<<<<<<<<<<<<
 *                 return
 *             current = stack.pop()
 */
    __pyx_t_2 = (PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_stack) != 0);
    __pyx_t_4 = (!__pyx_t_2);
    if (__pyx_t_4) {

      /* "supriya/utils/_intervals.pyx":138
 *                 current = current.left_child
 *             if not stack:
 *                 return             # <<<<<<<<<<<<<<
 *             current = stack.pop()
 *             for i in range(len(current.payload)):
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = NULL;
      goto __pyx_L0;

      /* "supriya/utils/_intervals.pyx":137
 *                 stack.append(current)
 *                 current = current.left_child
 *             if not stack:             # <<<<<<<<<<<<<<
 *                 return
 *             current = stack.pop()
 */
    }

    /* "supriya/utils/_intervals.pyx":139
 *             if not stack:
 *                 return
 *             current = stack.pop()             # <<<<<<<<<<<<<<
 *             for i in range(len(current.payload)):
 *                 cinterval = current.payload[i]
 */
    __pyx_t_1 = __Pyx_PyList_Pop(__pyx_cur_scope->__pyx_v_stack); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope->__pyx_v_current);
    __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_current, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "supriya/utils/_intervals.pyx":140
 *                 return
 *             current = stack.pop()
 *             for i in range(len(current.payload)):             # <<<<<<<<<<<<<<
 *                 cinterval = current.payload[i]
 *                 yield cinterval.original_interval
 */
    __pyx_t_1 = __pyx_cur_scope->__pyx_v_current->payload;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_5 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_cur_scope->__pyx_v_i = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":141
 *             current = stack.pop()
 *             for i in range(len(current.payload)):
 *                 cinterval = current.payload[i]             # <<<<<<<<<<<<<<
 *                 yield cinterval.original_interval
 *             while current.right_child is None and stack:
 */
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_cur_scope->__pyx_v_current->payload, __pyx_cur_scope->__pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 141, __pyx_L1_error)
      __Pyx_XGOTREF((PyObject *)__pyx_cur_scope->__pyx_v_cinterval);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_cinterval, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_1));
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;

      /* "supriya/utils/_intervals.pyx":142
 *             for i in range(len(current.payload)):
 *                 cinterval = current.payload[i]
 *                 yield cinterval.original_interval             # <<<<<<<<<<<<<<
 *             while current.right_child is None and stack:
 *                 current = stack.pop()
 */
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_cinterval->original_interval);
      __pyx_r = __pyx_cur_scope->__pyx_v_cinterval->original_interval;
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
      __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L11_resume_from_yield:;
      __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
      __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
      __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 142, __pyx_L1_error)
    }

    /* "supriya/utils/_intervals.pyx":143
 *                 cinterval = current.payload[i]
 *                 yield cinterval.original_interval
 *             while current.right_child is None and stack:             # <<<<<<<<<<<<<<
 *                 current = stack.pop()
 *                 for i in range(len(current.payload)):
 */
    while (1) {
      __pyx_t_2 = (((PyObject *)__pyx_cur_scope->__pyx_v_current->right_child) == Py_None);
      if (__pyx_t_2) {
      } else {
        __pyx_t_4 = __pyx_t_2;
        goto __pyx_L14_bool_binop_done;
      }
      __pyx_t_2 = (PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_stack) != 0);
      __pyx_t_4 = __pyx_t_2;
      __pyx_L14_bool_binop_done:;
      if (!__pyx_t_4) break;

      /* "supriya/utils/_intervals.pyx":144
 *                 yield cinterval.original_interval
 *             while current.right_child is None and stack:
 *                 current = stack.pop()             # <<<<<<<<<<<<<<
 *                 for i in range(len(current.payload)):
 *                     cinterval = current.payload[i]
 */
      __pyx_t_1 = __Pyx_PyList_Pop(__pyx_cur_scope->__pyx_v_stack); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 144, __pyx_L1_error)
      __Pyx_GOTREF((PyObject *)__pyx_cur_scope->__pyx_v_current);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_current, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1));
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;

      /* "supriya/utils/_intervals.pyx":145
 *             while current.right_child is None and stack:
 *                 current = stack.pop()
 *                 for i in range(len(current.payload)):             # <<<<<<<<<<<<<<
 *                     cinterval = current.payload[i]
 *                     yield cinterval.original_interval
 */
      __pyx_t_1 = __pyx_cur_scope->__pyx_v_current->payload;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_5 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_6 = __pyx_t_5;
      for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
        __pyx_cur_scope->__pyx_v_i = __pyx_t_7;

        /* "supriya/utils/_intervals.pyx":146
 *                 current = stack.pop()
 *                 for i in range(len(current.payload)):
 *                     cinterval = current.payload[i]             # <<<<<<<<<<<<<<
 *                     yield cinterval.original_interval
 *             current = current.right_child
 */
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_cur_scope->__pyx_v_current->payload, __pyx_cur_scope->__pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 146, __pyx_L1_error)
        __Pyx_XGOTREF((PyObject *)__pyx_cur_scope->__pyx_v_cinterval);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_cinterval, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_1));
        __Pyx_GIVEREF(__pyx_t_1);
        __pyx_t_1 = 0;

        /* "supriya/utils/_intervals.pyx":147
 *                 for i in range(len(current.payload)):
 *                     cinterval = current.payload[i]
 *                     yield cinterval.original_interval             # <<<<<<<<<<<<<<
 *             current = current.right_child
 * 
 */
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_cinterval->original_interval);
        __pyx_r = __pyx_cur_scope->__pyx_v_cinterval->original_interval;
        __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
        __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
        __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
        __Pyx_XGIVEREF(__pyx_r);
        __Pyx_RefNannyFinishContext();
        __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
        /* return from generator, yielding value */
        __pyx_generator->resume_label = 2;
        return __pyx_r;
        __pyx_L18_resume_from_yield:;
        __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
        __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
        __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
        if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 147, __pyx_L1_error)
      }
    }

    /* "supriya/utils/_intervals.pyx":148
 *                     cinterval = current.payload[i]
 *                     yield cinterval.original_interval
 *             current = current.right_child             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
    __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_current->right_child);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope->__pyx_v_current);
    __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_current, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "supriya/utils/_intervals.pyx":128
 *         raise TypeError('Indices must be integers or slices, got {}'.format(i))
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         cdef _CNode current
 *         cdef _CInterval cinterval
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_Generator_Replace_StopIteration(0);
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":150
 *             current = current.right_child
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self._root_node is None:
 *             return 0
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10__len__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_9__len__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_9__len__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 1);

  /* "supriya/utils/_intervals.pyx":151
 * 
 *     def __len__(self):
 *         if self._root_node is None:             # <<<<<<<<<<<<<<
 *             return 0
 *         return self._root_node.subtree_stop_index
 */
  __pyx_t_1 = (__pyx_v_self->_root_node == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":152
 *     def __len__(self):
 *         if self._root_node is None:
 *             return 0             # <<<<<<<<<<<<<<
 *         return self._root_node.subtree_stop_index
 * 
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":151
 * 
 *     def __len__(self):
 *         if self._root_node is None:             # <<<<<<<<<<<<<<
 *             return 0
 *         return self._root_node.subtree_stop_index
 */
  }

  /* "supriya/utils/_intervals.pyx":153
 *         if self._root_node is None:
 *             return 0
 *         return self._root_node.subtree_stop_index             # <<<<<<<<<<<<<<
 * 
 *     ### PRIVATE METHODS ###
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_root_node, __pyx_n_s_subtree_stop_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyIndex_AsSsize_t(__pyx_t_2); if (unlikely((__pyx_t_3 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":150
 *             current = current.right_child
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self._root_node is None:
 *             return 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":157
 *     ### PRIVATE METHODS ###
 * 
 *     cdef _CNode _insert_node(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__insert_node(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_start_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_child_node = 0;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_insert_node", 1);

  /* "supriya/utils/_intervals.pyx":163
 *         ):
 *         cdef _CNode child_node
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return _CNode(start_offset)
 *         if start_offset < node.start_offset:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":164
 *         cdef _CNode child_node
 *         if node is None:
 *             return _CNode(start_offset)             # <<<<<<<<<<<<<<
 *         if start_offset < node.start_offset:
 *             child_node = self._insert_node(node.left_child, start_offset)
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_start_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CNode), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":163
 *         ):
 *         cdef _CNode child_node
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return _CNode(start_offset)
 *         if start_offset < node.start_offset:
 */
  }

  /* "supriya/utils/_intervals.pyx":165
 *         if node is None:
 *             return _CNode(start_offset)
 *         if start_offset < node.start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._insert_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 */
  __pyx_t_1 = (__pyx_v_start_offset < __pyx_v_node->start_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":166
 *             return _CNode(start_offset)
 *         if start_offset < node.start_offset:
 *             child_node = self._insert_node(node.left_child, start_offset)             # <<<<<<<<<<<<<<
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_insert_node(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_start_offset)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_child_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":167
 *         if start_offset < node.start_offset:
 *             child_node = self._insert_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)             # <<<<<<<<<<<<<<
 *         elif node.start_offset < start_offset:
 *             child_node = self._insert_node(node.right_child, start_offset)
 */
    ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_left_child(__pyx_v_self, __pyx_v_node, __pyx_v_child_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":165
 *         if node is None:
 *             return _CNode(start_offset)
 *         if start_offset < node.start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._insert_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":168
 *             child_node = self._insert_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._insert_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)
 */
  __pyx_t_1 = (__pyx_v_node->start_offset < __pyx_v_start_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":169
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:
 *             child_node = self._insert_node(node.right_child, start_offset)             # <<<<<<<<<<<<<<
 *             self._set_node_right_child(node, child_node)
 *         return self._rebalance(node)
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_insert_node(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_v_start_offset)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_child_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":170
 *         elif node.start_offset < start_offset:
 *             child_node = self._insert_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)             # <<<<<<<<<<<<<<
 *         return self._rebalance(node)
 * 
 */
    ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_right_child(__pyx_v_self, __pyx_v_node, __pyx_v_child_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 170, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":168
 *             child_node = self._insert_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._insert_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)
 */
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":171
 *             child_node = self._insert_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)
 *         return self._rebalance(node)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _insert_interval(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rebalance(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":157
 *     ### PRIVATE METHODS ###
 * 
 *     cdef _CNode _insert_node(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._insert_node", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child_node);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":184
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         node.payload.append(cinterval)
 *         node.payload.sort(key=lambda x: x.stop_offset)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16_insert_interval_lambda(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16_insert_interval_lambda = {"lambda", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16_insert_interval_lambda, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16_insert_interval_lambda(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_x = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_x)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 184, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "lambda") < 0)) __PYX_ERR(0, 184, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_x = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lambda", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 184, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._insert_interval.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_lambda_funcdef_lambda(__pyx_self, __pyx_v_x);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_stop_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._insert_interval.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":173
 *         return self._rebalance(node)
 * 
 *     cdef void _insert_interval(             # <<<<<<<<<<<<<<
 *         self,
 *         _CInterval cinterval,
 */

static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__insert_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_insert_interval", 1);

  /* "supriya/utils/_intervals.pyx":179
 *         cdef _CNode node
 *         self._root_node = self._insert_node(
 *             self._root_node,             # <<<<<<<<<<<<<<
 *             cinterval.start_offset,
 *             )
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 179, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);

  /* "supriya/utils/_intervals.pyx":178
 *         ):
 *         cdef _CNode node
 *         self._root_node = self._insert_node(             # <<<<<<<<<<<<<<
 *             self._root_node,
 *             cinterval.start_offset,
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_insert_node(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_v_cinterval->start_offset)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_root_node);
  __Pyx_DECREF(__pyx_v_self->_root_node);
  __pyx_v_self->_root_node = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":182
 *             cinterval.start_offset,
 *             )
 *         node = self._search(self._root_node, cinterval.start_offset)             # <<<<<<<<<<<<<<
 *         node.payload.append(cinterval)
 *         node.payload.sort(key=lambda x: x.stop_offset)
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 182, __pyx_L1_error)
  __pyx_t_2 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_search(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_v_cinterval->start_offset)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":183
 *             )
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         node.payload.append(cinterval)             # <<<<<<<<<<<<<<
 *         node.payload.sort(key=lambda x: x.stop_offset)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_Append(__pyx_v_node->payload, ((PyObject *)__pyx_v_cinterval)); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 183, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":184
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         node.payload.append(cinterval)
 *         node.payload.sort(key=lambda x: x.stop_offset)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node->payload, __pyx_n_s_sort); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16_insert_interval_lambda, 0, __pyx_n_s_IntervalTreeDriverEx__insert_int, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_t_4) < 0) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "supriya/utils/_intervals.pyx":173
 *         return self._rebalance(node)
 * 
 *     cdef void _insert_interval(             # <<<<<<<<<<<<<<
 *         self,
 *         _CInterval cinterval,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._insert_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_RefNannyFinishContext();
}

/* "supriya/utils/_intervals.pyx":186
 *         node.payload.sort(key=lambda x: x.stop_offset)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _is_interval(expr):
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_12_is_interval(CYTHON_UNUSED PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_12_is_interval = {"_is_interval", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_12_is_interval, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_12_is_interval(CYTHON_UNUSED PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_expr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_is_interval (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_expr,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_expr)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 186, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "_is_interval") < 0)) __PYX_ERR(0, 186, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_expr = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_is_interval", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 186, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._is_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_11_is_interval(__pyx_v_expr);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_11_is_interval(PyObject *__pyx_v_expr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_is_interval", 1);

  /* "supriya/utils/_intervals.pyx":188
 *     @staticmethod
 *     def _is_interval(expr):
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):             # <<<<<<<<<<<<<<
 *             return True
 *         return False
 */
  __pyx_t_2 = __Pyx_HasAttr(__pyx_v_expr, __pyx_n_u_start_offset); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 188, __pyx_L1_error)
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_HasAttr(__pyx_v_expr, __pyx_n_u_stop_offset); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 188, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":189
 *     def _is_interval(expr):
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
 *             return True             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":188
 *     @staticmethod
 *     def _is_interval(expr):
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):             # <<<<<<<<<<<<<<
 *             return True
 *         return False
 */
  }

  /* "supriya/utils/_intervals.pyx":190
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
 *             return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _rebalance(
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":186
 *         node.payload.sort(key=lambda x: x.stop_offset)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _is_interval(expr):
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._is_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":192
 *         return False
 * 
 *     cdef _CNode _rebalance(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rebalance(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_rebalance", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_node);

  /* "supriya/utils/_intervals.pyx":196
 *         _CNode node,
 *         ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         if 1 < node.balance:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":197
 *         ):
 *         if node is None:
 *             return None             # <<<<<<<<<<<<<<
 *         if 1 < node.balance:
 *             if 0 <= node.right_child.balance:
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":196
 *         _CNode node,
 *         ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         if 1 < node.balance:
 */
  }

  /* "supriya/utils/_intervals.pyx":198
 *         if node is None:
 *             return None
 *         if 1 < node.balance:             # <<<<<<<<<<<<<<
 *             if 0 <= node.right_child.balance:
 *                 node = self._rotate_right_right(node)
 */
  __pyx_t_1 = (1 < __pyx_v_node->balance);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":199
 *             return None
 *         if 1 < node.balance:
 *             if 0 <= node.right_child.balance:             # <<<<<<<<<<<<<<
 *                 node = self._rotate_right_right(node)
 *             else:
 */
    __pyx_t_1 = (0 <= __pyx_v_node->right_child->balance);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":200
 *         if 1 < node.balance:
 *             if 0 <= node.right_child.balance:
 *                 node = self._rotate_right_right(node)             # <<<<<<<<<<<<<<
 *             else:
 *                 node = self._rotate_right_left(node)
 */
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_right_right(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "supriya/utils/_intervals.pyx":199
 *             return None
 *         if 1 < node.balance:
 *             if 0 <= node.right_child.balance:             # <<<<<<<<<<<<<<
 *                 node = self._rotate_right_right(node)
 *             else:
 */
      goto __pyx_L5;
    }

    /* "supriya/utils/_intervals.pyx":202
 *                 node = self._rotate_right_right(node)
 *             else:
 *                 node = self._rotate_right_left(node)             # <<<<<<<<<<<<<<
 *         elif node.balance < -1:
 *             if node.left_child.balance <= 0:
 */
    /*else*/ {
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_right_left(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2));
      __pyx_t_2 = 0;
    }
    __pyx_L5:;

    /* "supriya/utils/_intervals.pyx":198
 *         if node is None:
 *             return None
 *         if 1 < node.balance:             # <<<<<<<<<<<<<<
 *             if 0 <= node.right_child.balance:
 *                 node = self._rotate_right_right(node)
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":203
 *             else:
 *                 node = self._rotate_right_left(node)
 *         elif node.balance < -1:             # <<<<<<<<<<<<<<
 *             if node.left_child.balance <= 0:
 *                 node = self._rotate_left_left(node)
 */
  __pyx_t_1 = (__pyx_v_node->balance < -1L);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":204
 *                 node = self._rotate_right_left(node)
 *         elif node.balance < -1:
 *             if node.left_child.balance <= 0:             # <<<<<<<<<<<<<<
 *                 node = self._rotate_left_left(node)
 *             else:
 */
    __pyx_t_1 = (__pyx_v_node->left_child->balance <= 0);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":205
 *         elif node.balance < -1:
 *             if node.left_child.balance <= 0:
 *                 node = self._rotate_left_left(node)             # <<<<<<<<<<<<<<
 *             else:
 *                 node = self._rotate_left_right(node)
 */
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_left_left(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "supriya/utils/_intervals.pyx":204
 *                 node = self._rotate_right_left(node)
 *         elif node.balance < -1:
 *             if node.left_child.balance <= 0:             # <<<<<<<<<<<<<<
 *                 node = self._rotate_left_left(node)
 *             else:
 */
      goto __pyx_L6;
    }

    /* "supriya/utils/_intervals.pyx":207
 *                 node = self._rotate_left_left(node)
 *             else:
 *                 node = self._rotate_left_right(node)             # <<<<<<<<<<<<<<
 *         return node
 * 
 */
    /*else*/ {
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_left_right(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2));
      __pyx_t_2 = 0;
    }
    __pyx_L6:;

    /* "supriya/utils/_intervals.pyx":203
 *             else:
 *                 node = self._rotate_right_left(node)
 *         elif node.balance < -1:             # <<<<<<<<<<<<<<
 *             if node.left_child.balance <= 0:
 *                 node = self._rotate_left_left(node)
 */
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":208
 *             else:
 *                 node = self._rotate_left_right(node)
 *         return node             # <<<<<<<<<<<<<<
 * 
 *     cdef object _recurse_find_intervals_intersecting_offset(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_node);
  __pyx_r = __pyx_v_node;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":192
 *         return False
 * 
 *     cdef _CNode _rebalance(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._rebalance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":210
 *         return node
 * 
 *     cdef object _recurse_find_intervals_intersecting_offset(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_intersecting_offset(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval = 0;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_subresult = NULL;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_find_intervals_intersecting_offset", 1);

  /* "supriya/utils/_intervals.pyx":216
 *         ):
 *         cdef _CInterval cinterval
 *         result = []             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return result
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":217
 *         cdef _CInterval cinterval
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.start_offset <= offset < node.stop_offset_high:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":218
 *         result = []
 *         if node is None:
 *             return result             # <<<<<<<<<<<<<<
 *         if node.start_offset <= offset < node.stop_offset_high:
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":217
 *         cdef _CInterval cinterval
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.start_offset <= offset < node.stop_offset_high:
 */
  }

  /* "supriya/utils/_intervals.pyx":219
 *         if node is None:
 *             return result
 *         if node.start_offset <= offset < node.stop_offset_high:             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)
 */
  __pyx_t_2 = (__pyx_v_node->start_offset <= __pyx_v_offset);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_offset < __pyx_v_node->stop_offset_high);
  }
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":221
 *         if node.start_offset <= offset < node.stop_offset_high:
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)             # <<<<<<<<<<<<<<
 *             result.extend(subresult)
 *             for i in range(len(node.payload)):
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_1);

    /* "supriya/utils/_intervals.pyx":220
 *             return result
 *         if node.start_offset <= offset < node.stop_offset_high:
 *             subresult = self._recurse_find_intervals_intersecting_offset(             # <<<<<<<<<<<<<<
 *                 node.left_child, offset)
 *             result.extend(subresult)
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_offset(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_v_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_subresult = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":222
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)
 *             result.extend(subresult)             # <<<<<<<<<<<<<<
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]
 */
    __pyx_t_4 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_v_subresult); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 222, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":223
 *                 node.left_child, offset)
 *             result.extend(subresult)
 *             for i in range(len(node.payload)):             # <<<<<<<<<<<<<<
 *                 cinterval = node.payload[i]
 *                 if offset < cinterval.stop_offset:
 */
    __pyx_t_3 = __pyx_v_node->payload;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 223, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_i = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":224
 *             result.extend(subresult)
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]             # <<<<<<<<<<<<<<
 *                 if offset < cinterval.stop_offset:
 *                     result.append(cinterval)
 */
      __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_node->payload, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 224, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_cinterval, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "supriya/utils/_intervals.pyx":225
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]
 *                 if offset < cinterval.stop_offset:             # <<<<<<<<<<<<<<
 *                     result.append(cinterval)
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 */
      __pyx_t_2 = (__pyx_v_offset < __pyx_v_cinterval->stop_offset);
      if (__pyx_t_2) {

        /* "supriya/utils/_intervals.pyx":226
 *                 cinterval = node.payload[i]
 *                 if offset < cinterval.stop_offset:
 *                     result.append(cinterval)             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.right_child, offset)
 */
        __pyx_t_4 = __Pyx_PyList_Append(__pyx_v_result, ((PyObject *)__pyx_v_cinterval)); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 226, __pyx_L1_error)

        /* "supriya/utils/_intervals.pyx":225
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]
 *                 if offset < cinterval.stop_offset:             # <<<<<<<<<<<<<<
 *                     result.append(cinterval)
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 */
      }
    }

    /* "supriya/utils/_intervals.pyx":228
 *                     result.append(cinterval)
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.right_child, offset)             # <<<<<<<<<<<<<<
 *             result.extend(subresult)
 *         elif offset <= node.start_offset:
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_3);

    /* "supriya/utils/_intervals.pyx":227
 *                 if offset < cinterval.stop_offset:
 *                     result.append(cinterval)
 *             subresult = self._recurse_find_intervals_intersecting_offset(             # <<<<<<<<<<<<<<
 *                 node.right_child, offset)
 *             result.extend(subresult)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_offset(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_subresult, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "supriya/utils/_intervals.pyx":229
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.right_child, offset)
 *             result.extend(subresult)             # <<<<<<<<<<<<<<
 *         elif offset <= node.start_offset:
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 */
    __pyx_t_4 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_v_subresult); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 229, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":219
 *         if node is None:
 *             return result
 *         if node.start_offset <= offset < node.stop_offset_high:             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":230
 *                 node.right_child, offset)
 *             result.extend(subresult)
 *         elif offset <= node.start_offset:             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)
 */
  __pyx_t_2 = (__pyx_v_offset <= __pyx_v_node->start_offset);
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":232
 *         elif offset <= node.start_offset:
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)             # <<<<<<<<<<<<<<
 *             result.extend(subresult)
 *         return result
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_1);

    /* "supriya/utils/_intervals.pyx":231
 *             result.extend(subresult)
 *         elif offset <= node.start_offset:
 *             subresult = self._recurse_find_intervals_intersecting_offset(             # <<<<<<<<<<<<<<
 *                 node.left_child, offset)
 *             result.extend(subresult)
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_offset(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_v_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_subresult = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":233
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)
 *             result.extend(subresult)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_4 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_v_subresult); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 233, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":230
 *                 node.right_child, offset)
 *             result.extend(subresult)
 *         elif offset <= node.start_offset:             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_offset(
 *                 node.left_child, offset)
 */
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":234
 *                 node.left_child, offset)
 *             result.extend(subresult)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef object _recurse_find_intervals_intersecting_interval(
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":210
 *         return node
 * 
 *     cdef object _recurse_find_intervals_intersecting_offset(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_find_intervals_intersecting_offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cinterval);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_subresult);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":236
 *         return result
 * 
 *     cdef object _recurse_find_intervals_intersecting_interval(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_intersecting_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_candidate_interval = 0;
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_node_interval = 0;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_subresult = NULL;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_find_intervals_intersecting_interval", 1);

  /* "supriya/utils/_intervals.pyx":242
 *         ):
 *         cdef _CInterval candidate_interval, node_interval
 *         result = []             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return result
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":243
 *         cdef _CInterval candidate_interval, node_interval
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         node_interval = _CInterval(
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":244
 *         result = []
 *         if node is None:
 *             return result             # <<<<<<<<<<<<<<
 *         node_interval = _CInterval(
 *             node.start_offset,
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":243
 *         cdef _CInterval candidate_interval, node_interval
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         node_interval = _CInterval(
 */
  }

  /* "supriya/utils/_intervals.pyx":246
 *             return result
 *         node_interval = _CInterval(
 *             node.start_offset,             # <<<<<<<<<<<<<<
 *             node.stop_offset_high,
 *             node,
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_node->start_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "supriya/utils/_intervals.pyx":247
 *         node_interval = _CInterval(
 *             node.start_offset,
 *             node.stop_offset_high,             # <<<<<<<<<<<<<<
 *             node,
 *             )
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_node->stop_offset_high); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "supriya/utils/_intervals.pyx":245
 *         if node is None:
 *             return result
 *         node_interval = _CInterval(             # <<<<<<<<<<<<<<
 *             node.start_offset,
 *             node.stop_offset_high,
 */
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1)) __PYX_ERR(0, 245, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_node);
  __Pyx_GIVEREF((PyObject *)__pyx_v_node);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_node))) __PYX_ERR(0, 245, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_node_interval = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":250
 *             node,
 *             )
 *         if cinterval.intersects_interval(node_interval):             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.left_child, cinterval)
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals__CInterval *)__pyx_v_cinterval->__pyx_vtab)->intersects_interval(__pyx_v_cinterval, __pyx_v_node_interval, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 250, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":252
 *         if cinterval.intersects_interval(node_interval):
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.left_child, cinterval)             # <<<<<<<<<<<<<<
 *             result.extend(subresult)
 *             for i in range(len(node.payload)):
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);

    /* "supriya/utils/_intervals.pyx":251
 *             )
 *         if cinterval.intersects_interval(node_interval):
 *             subresult = self._recurse_find_intervals_intersecting_interval(             # <<<<<<<<<<<<<<
 *                 node.left_child, cinterval)
 *             result.extend(subresult)
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_interval(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_cinterval); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 251, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_subresult = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":253
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.left_child, cinterval)
 *             result.extend(subresult)             # <<<<<<<<<<<<<<
 *             for i in range(len(node.payload)):
 *                 candidate_interval = node.payload[i]
 */
    __pyx_t_5 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_v_subresult); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 253, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":254
 *                 node.left_child, cinterval)
 *             result.extend(subresult)
 *             for i in range(len(node.payload)):             # <<<<<<<<<<<<<<
 *                 candidate_interval = node.payload[i]
 *                 if candidate_interval.intersects_interval(cinterval):
 */
    __pyx_t_4 = __pyx_v_node->payload;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_7 = __pyx_t_6;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_i = __pyx_t_8;

      /* "supriya/utils/_intervals.pyx":255
 *             result.extend(subresult)
 *             for i in range(len(node.payload)):
 *                 candidate_interval = node.payload[i]             # <<<<<<<<<<<<<<
 *                 if candidate_interval.intersects_interval(cinterval):
 *                     result.append(candidate_interval)
 */
      __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_node->payload, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 255, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 255, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_candidate_interval, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "supriya/utils/_intervals.pyx":256
 *             for i in range(len(node.payload)):
 *                 candidate_interval = node.payload[i]
 *                 if candidate_interval.intersects_interval(cinterval):             # <<<<<<<<<<<<<<
 *                     result.append(candidate_interval)
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals__CInterval *)__pyx_v_candidate_interval->__pyx_vtab)->intersects_interval(__pyx_v_candidate_interval, __pyx_v_cinterval, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 256, __pyx_L1_error)
      if (__pyx_t_2) {

        /* "supriya/utils/_intervals.pyx":257
 *                 candidate_interval = node.payload[i]
 *                 if candidate_interval.intersects_interval(cinterval):
 *                     result.append(candidate_interval)             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.right_child, cinterval)
 */
        __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_result, ((PyObject *)__pyx_v_candidate_interval)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 257, __pyx_L1_error)

        /* "supriya/utils/_intervals.pyx":256
 *             for i in range(len(node.payload)):
 *                 candidate_interval = node.payload[i]
 *                 if candidate_interval.intersects_interval(cinterval):             # <<<<<<<<<<<<<<
 *                     result.append(candidate_interval)
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 */
      }
    }

    /* "supriya/utils/_intervals.pyx":259
 *                     result.append(candidate_interval)
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.right_child, cinterval)             # <<<<<<<<<<<<<<
 *             result.extend(subresult)
 *         elif (cinterval.start_offset <= node.start_offset) or \
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_4);

    /* "supriya/utils/_intervals.pyx":258
 *                 if candidate_interval.intersects_interval(cinterval):
 *                     result.append(candidate_interval)
 *             subresult = self._recurse_find_intervals_intersecting_interval(             # <<<<<<<<<<<<<<
 *                 node.right_child, cinterval)
 *             result.extend(subresult)
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_interval(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4), __pyx_v_cinterval); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_subresult, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":260
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.right_child, cinterval)
 *             result.extend(subresult)             # <<<<<<<<<<<<<<
 *         elif (cinterval.start_offset <= node.start_offset) or \
 *             (cinterval.stop_offset <= node.start_offset):
 */
    __pyx_t_5 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_v_subresult); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 260, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":250
 *             node,
 *             )
 *         if cinterval.intersects_interval(node_interval):             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.left_child, cinterval)
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":261
 *                 node.right_child, cinterval)
 *             result.extend(subresult)
 *         elif (cinterval.start_offset <= node.start_offset) or \             # <<<<<<<<<<<<<<
 *             (cinterval.stop_offset <= node.start_offset):
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 */
  __pyx_t_9 = (__pyx_v_cinterval->start_offset <= __pyx_v_node->start_offset);
  if (!__pyx_t_9) {
  } else {
    __pyx_t_2 = __pyx_t_9;
    goto __pyx_L8_bool_binop_done;
  }

  /* "supriya/utils/_intervals.pyx":262
 *             result.extend(subresult)
 *         elif (cinterval.start_offset <= node.start_offset) or \
 *             (cinterval.stop_offset <= node.start_offset):             # <<<<<<<<<<<<<<
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.left_child, cinterval)
 */
  __pyx_t_9 = (__pyx_v_cinterval->stop_offset <= __pyx_v_node->start_offset);
  __pyx_t_2 = __pyx_t_9;
  __pyx_L8_bool_binop_done:;

  /* "supriya/utils/_intervals.pyx":261
 *                 node.right_child, cinterval)
 *             result.extend(subresult)
 *         elif (cinterval.start_offset <= node.start_offset) or \             # <<<<<<<<<<<<<<
 *             (cinterval.stop_offset <= node.start_offset):
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 */
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":264
 *             (cinterval.stop_offset <= node.start_offset):
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.left_child, cinterval)             # <<<<<<<<<<<<<<
 *             result.extend(subresult)
 *         return result
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);

    /* "supriya/utils/_intervals.pyx":263
 *         elif (cinterval.start_offset <= node.start_offset) or \
 *             (cinterval.stop_offset <= node.start_offset):
 *             subresult = self._recurse_find_intervals_intersecting_interval(             # <<<<<<<<<<<<<<
 *                 node.left_child, cinterval)
 *             result.extend(subresult)
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_interval(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_cinterval); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_subresult = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":265
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 *                 node.left_child, cinterval)
 *             result.extend(subresult)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_5 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_v_subresult); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 265, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":261
 *                 node.right_child, cinterval)
 *             result.extend(subresult)
 *         elif (cinterval.start_offset <= node.start_offset) or \             # <<<<<<<<<<<<<<
 *             (cinterval.stop_offset <= node.start_offset):
 *             subresult = self._recurse_find_intervals_intersecting_interval(
 */
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":266
 *                 node.left_child, cinterval)
 *             result.extend(subresult)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef object _recurse_find_intervals_stopping_at(
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":236
 *         return result
 * 
 *     cdef object _recurse_find_intervals_intersecting_interval(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_find_intervals_intersecting_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_candidate_interval);
  __Pyx_XDECREF((PyObject *)__pyx_v_node_interval);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_subresult);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":268
 *         return result
 * 
 *     cdef object _recurse_find_intervals_stopping_at(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_stopping_at(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval = 0;
  PyObject *__pyx_v_result = NULL;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_find_intervals_stopping_at", 1);

  /* "supriya/utils/_intervals.pyx":274
 *         ):
 *         cdef _CInterval cinterval
 *         result = []             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return result
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":275
 *         cdef _CInterval cinterval
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.stop_offset_low <= offset <= node.stop_offset_high:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":276
 *         result = []
 *         if node is None:
 *             return result             # <<<<<<<<<<<<<<
 *         if node.stop_offset_low <= offset <= node.stop_offset_high:
 *             for i in range(len(node.payload)):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":275
 *         cdef _CInterval cinterval
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.stop_offset_low <= offset <= node.stop_offset_high:
 */
  }

  /* "supriya/utils/_intervals.pyx":277
 *         if node is None:
 *             return result
 *         if node.stop_offset_low <= offset <= node.stop_offset_high:             # <<<<<<<<<<<<<<
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]
 */
  __pyx_t_2 = (__pyx_v_node->stop_offset_low <= __pyx_v_offset);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_offset <= __pyx_v_node->stop_offset_high);
  }
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":278
 *             return result
 *         if node.stop_offset_low <= offset <= node.stop_offset_high:
 *             for i in range(len(node.payload)):             # <<<<<<<<<<<<<<
 *                 cinterval = node.payload[i]
 *                 if cinterval.stop_offset == offset:
 */
    __pyx_t_1 = __pyx_v_node->payload;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 278, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "supriya/utils/_intervals.pyx":279
 *         if node.stop_offset_low <= offset <= node.stop_offset_high:
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]             # <<<<<<<<<<<<<<
 *                 if cinterval.stop_offset == offset:
 *                     result.append(cinterval)
 */
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_node->payload, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 279, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_cinterval, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "supriya/utils/_intervals.pyx":280
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]
 *                 if cinterval.stop_offset == offset:             # <<<<<<<<<<<<<<
 *                     result.append(cinterval)
 *             if node.left_child is not None:
 */
      __pyx_t_2 = (__pyx_v_cinterval->stop_offset == __pyx_v_offset);
      if (__pyx_t_2) {

        /* "supriya/utils/_intervals.pyx":281
 *                 cinterval = node.payload[i]
 *                 if cinterval.stop_offset == offset:
 *                     result.append(cinterval)             # <<<<<<<<<<<<<<
 *             if node.left_child is not None:
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 */
        __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_result, ((PyObject *)__pyx_v_cinterval)); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 281, __pyx_L1_error)

        /* "supriya/utils/_intervals.pyx":280
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]
 *                 if cinterval.stop_offset == offset:             # <<<<<<<<<<<<<<
 *                     result.append(cinterval)
 *             if node.left_child is not None:
 */
      }
    }

    /* "supriya/utils/_intervals.pyx":282
 *                 if cinterval.stop_offset == offset:
 *                     result.append(cinterval)
 *             if node.left_child is not None:             # <<<<<<<<<<<<<<
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.left_child, offset))
 */
    __pyx_t_2 = (((PyObject *)__pyx_v_node->left_child) != Py_None);
    if (__pyx_t_2) {

      /* "supriya/utils/_intervals.pyx":284
 *             if node.left_child is not None:
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.left_child, offset))             # <<<<<<<<<<<<<<
 *             if node.right_child is not None:
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 */
      __pyx_t_1 = ((PyObject *)__pyx_v_node->left_child);
      __Pyx_INCREF(__pyx_t_1);

      /* "supriya/utils/_intervals.pyx":283
 *                     result.append(cinterval)
 *             if node.left_child is not None:
 *                 result.extend(self._recurse_find_intervals_stopping_at(             # <<<<<<<<<<<<<<
 *                     node.left_child, offset))
 *             if node.right_child is not None:
 */
      __pyx_t_7 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_stopping_at(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_v_offset); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 283, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_6 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_7); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 283, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "supriya/utils/_intervals.pyx":282
 *                 if cinterval.stop_offset == offset:
 *                     result.append(cinterval)
 *             if node.left_child is not None:             # <<<<<<<<<<<<<<
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.left_child, offset))
 */
    }

    /* "supriya/utils/_intervals.pyx":285
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.left_child, offset))
 *             if node.right_child is not None:             # <<<<<<<<<<<<<<
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.right_child, offset))
 */
    __pyx_t_2 = (((PyObject *)__pyx_v_node->right_child) != Py_None);
    if (__pyx_t_2) {

      /* "supriya/utils/_intervals.pyx":287
 *             if node.right_child is not None:
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.right_child, offset))             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
      __pyx_t_7 = ((PyObject *)__pyx_v_node->right_child);
      __Pyx_INCREF(__pyx_t_7);

      /* "supriya/utils/_intervals.pyx":286
 *                     node.left_child, offset))
 *             if node.right_child is not None:
 *                 result.extend(self._recurse_find_intervals_stopping_at(             # <<<<<<<<<<<<<<
 *                     node.right_child, offset))
 *         return result
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_stopping_at(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_7), __pyx_v_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_6 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_1); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 286, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "supriya/utils/_intervals.pyx":285
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.left_child, offset))
 *             if node.right_child is not None:             # <<<<<<<<<<<<<<
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.right_child, offset))
 */
    }

    /* "supriya/utils/_intervals.pyx":277
 *         if node is None:
 *             return result
 *         if node.stop_offset_low <= offset <= node.stop_offset_high:             # <<<<<<<<<<<<<<
 *             for i in range(len(node.payload)):
 *                 cinterval = node.payload[i]
 */
  }

  /* "supriya/utils/_intervals.pyx":288
 *                 result.extend(self._recurse_find_intervals_stopping_at(
 *                     node.right_child, offset))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef float _recurse_get_offset_after(
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":268
 *         return result
 * 
 *     cdef object _recurse_find_intervals_stopping_at(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_find_intervals_stopping_at", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cinterval);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":290
 *         return result
 * 
 *     cdef float _recurse_get_offset_after(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static float __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset, struct __pyx_opt_args_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after *__pyx_optional_args) {
  int __pyx_v_depth = ((int)1);
  PyObject *__pyx_v_result = NULL;
  double __pyx_v_candidate;
  PyObject *__pyx_v_interval = NULL;
  float __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  float __pyx_t_5;
  struct __pyx_opt_args_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_get_offset_after", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_depth = __pyx_optional_args->depth;
    }
  }

  /* "supriya/utils/_intervals.pyx":296
 *         int depth=1,
 *     ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             # print(("    " * depth) + "None")
 *             return offset
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":298
 *         if node is None:
 *             # print(("    " * depth) + "None")
 *             return offset             # <<<<<<<<<<<<<<
 *         result = offset
 *         # print(("    " * depth) + "Node: {} {} {}".format(node.start_offset, node.stop_offset_low, node.stop_offset_high))
 */
    __pyx_r = __pyx_v_offset;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":296
 *         int depth=1,
 *     ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             # print(("    " * depth) + "None")
 *             return offset
 */
  }

  /* "supriya/utils/_intervals.pyx":299
 *             # print(("    " * depth) + "None")
 *             return offset
 *         result = offset             # <<<<<<<<<<<<<<
 *         # print(("    " * depth) + "Node: {} {} {}".format(node.start_offset, node.stop_offset_low, node.stop_offset_high))
 *         if node.start_offset > offset:
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":301
 *         result = offset
 *         # print(("    " * depth) + "Node: {} {} {}".format(node.start_offset, node.stop_offset_low, node.stop_offset_high))
 *         if node.start_offset > offset:             # <<<<<<<<<<<<<<
 *             # print(("    " * depth) + "After")
 *             result = node.start_offset
 */
  __pyx_t_1 = (__pyx_v_node->start_offset > __pyx_v_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":303
 *         if node.start_offset > offset:
 *             # print(("    " * depth) + "After")
 *             result = node.start_offset             # <<<<<<<<<<<<<<
 *             if offset < node.stop_offset_low < result:
 *                 result = node.stop_offset_low
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_node->start_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 303, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":304
 *             # print(("    " * depth) + "After")
 *             result = node.start_offset
 *             if offset < node.stop_offset_low < result:             # <<<<<<<<<<<<<<
 *                 result = node.stop_offset_low
 *             # print(("    " * depth) + "Checking Left")
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_node->stop_offset_low); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
    if (__Pyx_PyObject_IsTrue(__pyx_t_4)) {
      __Pyx_DECREF(__pyx_t_4);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_result, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":305
 *             result = node.start_offset
 *             if offset < node.stop_offset_low < result:
 *                 result = node.stop_offset_low             # <<<<<<<<<<<<<<
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 */
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_node->stop_offset_low); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 305, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "supriya/utils/_intervals.pyx":304
 *             # print(("    " * depth) + "After")
 *             result = node.start_offset
 *             if offset < node.stop_offset_low < result:             # <<<<<<<<<<<<<<
 *                 result = node.stop_offset_low
 *             # print(("    " * depth) + "Checking Left")
 */
    }

    /* "supriya/utils/_intervals.pyx":307
 *                 result = node.stop_offset_low
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)             # <<<<<<<<<<<<<<
 *             if offset != result:
 *                 if offset < candidate < result:
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.depth = (__pyx_v_depth + 1);
    __pyx_t_5 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_offset_after(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4), __pyx_v_offset, &__pyx_t_6); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_candidate = __pyx_t_5;

    /* "supriya/utils/_intervals.pyx":308
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:             # <<<<<<<<<<<<<<
 *                 if offset < candidate < result:
 *                     result = candidate
 */
    __pyx_t_4 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_4, __pyx_v_result, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":309
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:
 *                 if offset < candidate < result:             # <<<<<<<<<<<<<<
 *                     result = candidate
 *             elif offset < candidate:
 */
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = PyObject_RichCompare(__pyx_t_3, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
      if (__Pyx_PyObject_IsTrue(__pyx_t_2)) {
        __Pyx_DECREF(__pyx_t_2);
        __pyx_t_2 = PyObject_RichCompare(__pyx_t_4, __pyx_v_result, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_1) {

        /* "supriya/utils/_intervals.pyx":310
 *             if offset != result:
 *                 if offset < candidate < result:
 *                     result = candidate             # <<<<<<<<<<<<<<
 *             elif offset < candidate:
 *                 result = candidate
 */
        __pyx_t_2 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "supriya/utils/_intervals.pyx":309
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:
 *                 if offset < candidate < result:             # <<<<<<<<<<<<<<
 *                     result = candidate
 *             elif offset < candidate:
 */
      }

      /* "supriya/utils/_intervals.pyx":308
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:             # <<<<<<<<<<<<<<
 *                 if offset < candidate < result:
 *                     result = candidate
 */
      goto __pyx_L6;
    }

    /* "supriya/utils/_intervals.pyx":311
 *                 if offset < candidate < result:
 *                     result = candidate
 *             elif offset < candidate:             # <<<<<<<<<<<<<<
 *                 result = candidate
 *         elif node.start_offset <= offset and (
 */
    __pyx_t_1 = (__pyx_v_offset < __pyx_v_candidate);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":312
 *                     result = candidate
 *             elif offset < candidate:
 *                 result = candidate             # <<<<<<<<<<<<<<
 *         elif node.start_offset <= offset and (
 *             node.stop_offset_high > offset or node.stop_offset_high > offset
 */
      __pyx_t_2 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 312, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "supriya/utils/_intervals.pyx":311
 *                 if offset < candidate < result:
 *                     result = candidate
 *             elif offset < candidate:             # <<<<<<<<<<<<<<
 *                 result = candidate
 *         elif node.start_offset <= offset and (
 */
    }
    __pyx_L6:;

    /* "supriya/utils/_intervals.pyx":301
 *         result = offset
 *         # print(("    " * depth) + "Node: {} {} {}".format(node.start_offset, node.stop_offset_low, node.stop_offset_high))
 *         if node.start_offset > offset:             # <<<<<<<<<<<<<<
 *             # print(("    " * depth) + "After")
 *             result = node.start_offset
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":313
 *             elif offset < candidate:
 *                 result = candidate
 *         elif node.start_offset <= offset and (             # <<<<<<<<<<<<<<
 *             node.stop_offset_high > offset or node.stop_offset_high > offset
 *         ):
 */
  __pyx_t_7 = (__pyx_v_node->start_offset <= __pyx_v_offset);
  if (__pyx_t_7) {
  } else {
    __pyx_t_1 = __pyx_t_7;
    goto __pyx_L8_bool_binop_done;
  }

  /* "supriya/utils/_intervals.pyx":314
 *                 result = candidate
 *         elif node.start_offset <= offset and (
 *             node.stop_offset_high > offset or node.stop_offset_high > offset             # <<<<<<<<<<<<<<
 *         ):
 *             # print(("    " * depth) + "Before or Equal")
 */
  __pyx_t_7 = (__pyx_v_node->stop_offset_high > __pyx_v_offset);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_1 = __pyx_t_7;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_7 = (__pyx_v_node->stop_offset_high > __pyx_v_offset);
  __pyx_t_1 = __pyx_t_7;
  __pyx_L8_bool_binop_done:;

  /* "supriya/utils/_intervals.pyx":313
 *             elif offset < candidate:
 *                 result = candidate
 *         elif node.start_offset <= offset and (             # <<<<<<<<<<<<<<
 *             node.stop_offset_high > offset or node.stop_offset_high > offset
 *         ):
 */
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":317
 *         ):
 *             # print(("    " * depth) + "Before or Equal")
 *             for interval in node.payload:             # <<<<<<<<<<<<<<
 *                 if interval.stop_offset > offset:
 *                     result = interval.stop_offset
 */
    if (likely(PyList_CheckExact(__pyx_v_node->payload)) || PyTuple_CheckExact(__pyx_v_node->payload)) {
      __pyx_t_2 = __pyx_v_node->payload; __Pyx_INCREF(__pyx_t_2);
      __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_node->payload); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_9 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 317, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 317, __pyx_L1_error)
            #endif
            if (__pyx_t_8 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely((0 < 0))) __PYX_ERR(0, 317, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 317, __pyx_L1_error)
            #endif
            if (__pyx_t_8 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely((0 < 0))) __PYX_ERR(0, 317, __pyx_L1_error)
          #else
          __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 317, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_9(__pyx_t_2);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 317, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_XDECREF_SET(__pyx_v_interval, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "supriya/utils/_intervals.pyx":318
 *             # print(("    " * depth) + "Before or Equal")
 *             for interval in node.payload:
 *                 if interval.stop_offset > offset:             # <<<<<<<<<<<<<<
 *                     result = interval.stop_offset
 *                     # print(("    " * depth) + "Found: {}".format(result))
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_interval, __pyx_n_s_stop_offset); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_10 = PyObject_RichCompare(__pyx_t_4, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 318, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 318, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (__pyx_t_1) {

        /* "supriya/utils/_intervals.pyx":319
 *             for interval in node.payload:
 *                 if interval.stop_offset > offset:
 *                     result = interval.stop_offset             # <<<<<<<<<<<<<<
 *                     # print(("    " * depth) + "Found: {}".format(result))
 *                     break
 */
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_interval, __pyx_n_s_stop_offset); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 319, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "supriya/utils/_intervals.pyx":321
 *                     result = interval.stop_offset
 *                     # print(("    " * depth) + "Found: {}".format(result))
 *                     break             # <<<<<<<<<<<<<<
 *             # print(("    " * depth) + "Checking Right")
 *             candidate = self._recurse_get_offset_after(node.right_child, offset, depth + 1)
 */
        goto __pyx_L12_break;

        /* "supriya/utils/_intervals.pyx":318
 *             # print(("    " * depth) + "Before or Equal")
 *             for interval in node.payload:
 *                 if interval.stop_offset > offset:             # <<<<<<<<<<<<<<
 *                     result = interval.stop_offset
 *                     # print(("    " * depth) + "Found: {}".format(result))
 */
      }

      /* "supriya/utils/_intervals.pyx":317
 *         ):
 *             # print(("    " * depth) + "Before or Equal")
 *             for interval in node.payload:             # <<<<<<<<<<<<<<
 *                 if interval.stop_offset > offset:
 *                     result = interval.stop_offset
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L14_for_end;
    __pyx_L12_break:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L14_for_end;
    __pyx_L14_for_end:;

    /* "supriya/utils/_intervals.pyx":323
 *                     break
 *             # print(("    " * depth) + "Checking Right")
 *             candidate = self._recurse_get_offset_after(node.right_child, offset, depth + 1)             # <<<<<<<<<<<<<<
 *             if offset != result:
 *                 if offset < candidate < result:
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.depth = (__pyx_v_depth + 1);
    __pyx_t_5 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_offset_after(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_v_offset, &__pyx_t_6); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 323, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_candidate = __pyx_t_5;

    /* "supriya/utils/_intervals.pyx":324
 *             # print(("    " * depth) + "Checking Right")
 *             candidate = self._recurse_get_offset_after(node.right_child, offset, depth + 1)
 *             if offset != result:             # <<<<<<<<<<<<<<
 *                 if offset < candidate < result:
 *                     result = candidate
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = PyObject_RichCompare(__pyx_t_2, __pyx_v_result, Py_NE); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":325
 *             candidate = self._recurse_get_offset_after(node.right_child, offset, depth + 1)
 *             if offset != result:
 *                 if offset < candidate < result:             # <<<<<<<<<<<<<<
 *                     result = candidate
 *             elif offset < candidate:
 */
      __pyx_t_10 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_2 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyObject_RichCompare(__pyx_t_10, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
      if (__Pyx_PyObject_IsTrue(__pyx_t_3)) {
        __Pyx_DECREF(__pyx_t_3);
        __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_v_result, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 325, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_1) {

        /* "supriya/utils/_intervals.pyx":326
 *             if offset != result:
 *                 if offset < candidate < result:
 *                     result = candidate             # <<<<<<<<<<<<<<
 *             elif offset < candidate:
 *                 result = candidate
 */
        __pyx_t_3 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_3);
        __pyx_t_3 = 0;

        /* "supriya/utils/_intervals.pyx":325
 *             candidate = self._recurse_get_offset_after(node.right_child, offset, depth + 1)
 *             if offset != result:
 *                 if offset < candidate < result:             # <<<<<<<<<<<<<<
 *                     result = candidate
 *             elif offset < candidate:
 */
      }

      /* "supriya/utils/_intervals.pyx":324
 *             # print(("    " * depth) + "Checking Right")
 *             candidate = self._recurse_get_offset_after(node.right_child, offset, depth + 1)
 *             if offset != result:             # <<<<<<<<<<<<<<
 *                 if offset < candidate < result:
 *                     result = candidate
 */
      goto __pyx_L15;
    }

    /* "supriya/utils/_intervals.pyx":327
 *                 if offset < candidate < result:
 *                     result = candidate
 *             elif offset < candidate:             # <<<<<<<<<<<<<<
 *                 result = candidate
 *             # print(("    " * depth) + "Checking Left")
 */
    __pyx_t_1 = (__pyx_v_offset < __pyx_v_candidate);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":328
 *                     result = candidate
 *             elif offset < candidate:
 *                 result = candidate             # <<<<<<<<<<<<<<
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 */
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "supriya/utils/_intervals.pyx":327
 *                 if offset < candidate < result:
 *                     result = candidate
 *             elif offset < candidate:             # <<<<<<<<<<<<<<
 *                 result = candidate
 *             # print(("    " * depth) + "Checking Left")
 */
    }
    __pyx_L15:;

    /* "supriya/utils/_intervals.pyx":330
 *                 result = candidate
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)             # <<<<<<<<<<<<<<
 *             if offset != result:
 *                 if offset < candidate < result:
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.depth = (__pyx_v_depth + 1);
    __pyx_t_5 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_offset_after(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_offset, &__pyx_t_6); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_candidate = __pyx_t_5;

    /* "supriya/utils/_intervals.pyx":331
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:             # <<<<<<<<<<<<<<
 *                 if offset < candidate < result:
 *                     result = candidate
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_3, __pyx_v_result, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":332
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:
 *                 if offset < candidate < result:             # <<<<<<<<<<<<<<
 *                     result = candidate
 *             elif offset < candidate:
 */
      __pyx_t_2 = PyFloat_FromDouble(__pyx_v_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_10 = PyObject_RichCompare(__pyx_t_2, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 332, __pyx_L1_error)
      if (__Pyx_PyObject_IsTrue(__pyx_t_10)) {
        __Pyx_DECREF(__pyx_t_10);
        __pyx_t_10 = PyObject_RichCompare(__pyx_t_3, __pyx_v_result, Py_LT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 332, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 332, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (__pyx_t_1) {

        /* "supriya/utils/_intervals.pyx":333
 *             if offset != result:
 *                 if offset < candidate < result:
 *                     result = candidate             # <<<<<<<<<<<<<<
 *             elif offset < candidate:
 *                 result = candidate
 */
        __pyx_t_10 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 333, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "supriya/utils/_intervals.pyx":332
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:
 *                 if offset < candidate < result:             # <<<<<<<<<<<<<<
 *                     result = candidate
 *             elif offset < candidate:
 */
      }

      /* "supriya/utils/_intervals.pyx":331
 *             # print(("    " * depth) + "Checking Left")
 *             candidate = self._recurse_get_offset_after(node.left_child, offset, depth + 1)
 *             if offset != result:             # <<<<<<<<<<<<<<
 *                 if offset < candidate < result:
 *                     result = candidate
 */
      goto __pyx_L17;
    }

    /* "supriya/utils/_intervals.pyx":334
 *                 if offset < candidate < result:
 *                     result = candidate
 *             elif offset < candidate:             # <<<<<<<<<<<<<<
 *                 result = candidate
 *         else:
 */
    __pyx_t_1 = (__pyx_v_offset < __pyx_v_candidate);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":335
 *                     result = candidate
 *             elif offset < candidate:
 *                 result = candidate             # <<<<<<<<<<<<<<
 *         else:
 *             # print(("    " * depth) + "Nope")
 */
      __pyx_t_10 = PyFloat_FromDouble(__pyx_v_candidate); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 335, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "supriya/utils/_intervals.pyx":334
 *                 if offset < candidate < result:
 *                     result = candidate
 *             elif offset < candidate:             # <<<<<<<<<<<<<<
 *                 result = candidate
 *         else:
 */
    }
    __pyx_L17:;

    /* "supriya/utils/_intervals.pyx":313
 *             elif offset < candidate:
 *                 result = candidate
 *         elif node.start_offset <= offset and (             # <<<<<<<<<<<<<<
 *             node.stop_offset_high > offset or node.stop_offset_high > offset
 *         ):
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":338
 *         else:
 *             # print(("    " * depth) + "Nope")
 *             pass             # <<<<<<<<<<<<<<
 *         # print(("    " * depth) + "Result: {}".format(result))
 *         return result
 */
  /*else*/ {
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":340
 *             pass
 *         # print(("    " * depth) + "Result: {}".format(result))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _recurse_get_start_offset_after(
 */
  __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_v_result); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 340, __pyx_L1_error)
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":290
 *         return result
 * 
 *     cdef float _recurse_get_offset_after(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_get_offset_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_interval);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":342
 *         return result
 * 
 *     cdef _CNode _recurse_get_start_offset_after(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_start_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_result = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_get_start_offset_after", 1);

  /* "supriya/utils/_intervals.pyx":347
 *         float offset,
 *     ):
 *         result = None             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return result
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_result = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None);

  /* "supriya/utils/_intervals.pyx":348
 *     ):
 *         result = None
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.start_offset <= offset and node.right_child:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":349
 *         result = None
 *         if node is None:
 *             return result             # <<<<<<<<<<<<<<
 *         if node.start_offset <= offset and node.right_child:
 *             result = self._recurse_get_start_offset_after(
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":348
 *     ):
 *         result = None
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.start_offset <= offset and node.right_child:
 */
  }

  /* "supriya/utils/_intervals.pyx":350
 *         if node is None:
 *             return result
 *         if node.start_offset <= offset and node.right_child:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_after(
 *                 node.right_child, offset)
 */
  __pyx_t_2 = (__pyx_v_node->start_offset <= __pyx_v_offset);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->right_child)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 350, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":352
 *         if node.start_offset <= offset and node.right_child:
 *             result = self._recurse_get_start_offset_after(
 *                 node.right_child, offset)             # <<<<<<<<<<<<<<
 *         elif offset < node.start_offset:
 *             result = self._recurse_get_start_offset_after(
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_3);

    /* "supriya/utils/_intervals.pyx":351
 *             return result
 *         if node.start_offset <= offset and node.right_child:
 *             result = self._recurse_get_start_offset_after(             # <<<<<<<<<<<<<<
 *                 node.right_child, offset)
 *         elif offset < node.start_offset:
 */
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_start_offset_after(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_offset)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":350
 *         if node is None:
 *             return result
 *         if node.start_offset <= offset and node.right_child:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_after(
 *                 node.right_child, offset)
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":353
 *             result = self._recurse_get_start_offset_after(
 *                 node.right_child, offset)
 *         elif offset < node.start_offset:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_after(
 *                 node.left_child, offset) or node
 */
  __pyx_t_1 = (__pyx_v_offset < __pyx_v_node->start_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":355
 *         elif offset < node.start_offset:
 *             result = self._recurse_get_start_offset_after(
 *                 node.left_child, offset) or node             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);

    /* "supriya/utils/_intervals.pyx":354
 *                 node.right_child, offset)
 *         elif offset < node.start_offset:
 *             result = self._recurse_get_start_offset_after(             # <<<<<<<<<<<<<<
 *                 node.left_child, offset) or node
 *         return result
 */
    __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_start_offset_after(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_offset)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 354, __pyx_L1_error)
    if (!__pyx_t_1) {
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_4 = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L7_bool_binop_done;
    }

    /* "supriya/utils/_intervals.pyx":355
 *         elif offset < node.start_offset:
 *             result = self._recurse_get_start_offset_after(
 *                 node.left_child, offset) or node             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_t_4 = ((PyObject *)__pyx_v_node);
    __pyx_L7_bool_binop_done:;
    __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":353
 *             result = self._recurse_get_start_offset_after(
 *                 node.right_child, offset)
 *         elif offset < node.start_offset:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_after(
 *                 node.left_child, offset) or node
 */
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":356
 *             result = self._recurse_get_start_offset_after(
 *                 node.left_child, offset) or node
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _recurse_get_start_offset_before(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":342
 *         return result
 * 
 *     cdef _CNode _recurse_get_start_offset_after(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_get_start_offset_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":358
 *         return result
 * 
 *     cdef _CNode _recurse_get_start_offset_before(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_start_offset_before(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_result = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_get_start_offset_before", 1);

  /* "supriya/utils/_intervals.pyx":363
 *         float offset,
 *     ):
 *         result = None             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return result
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_result = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None);

  /* "supriya/utils/_intervals.pyx":364
 *     ):
 *         result = None
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.start_offset < offset:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":365
 *         result = None
 *         if node is None:
 *             return result             # <<<<<<<<<<<<<<
 *         if node.start_offset < offset:
 *             result = self._recurse_get_start_offset_before(
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":364
 *     ):
 *         result = None
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if node.start_offset < offset:
 */
  }

  /* "supriya/utils/_intervals.pyx":366
 *         if node is None:
 *             return result
 *         if node.start_offset < offset:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_before(
 *                 node.right_child, offset) or node
 */
  __pyx_t_1 = (__pyx_v_node->start_offset < __pyx_v_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":368
 *         if node.start_offset < offset:
 *             result = self._recurse_get_start_offset_before(
 *                 node.right_child, offset) or node             # <<<<<<<<<<<<<<
 *         elif offset <= node.start_offset and node.left_child:
 *             result = self._recurse_get_start_offset_before(
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_3);

    /* "supriya/utils/_intervals.pyx":367
 *             return result
 *         if node.start_offset < offset:
 *             result = self._recurse_get_start_offset_before(             # <<<<<<<<<<<<<<
 *                 node.right_child, offset) or node
 *         elif offset <= node.start_offset and node.left_child:
 */
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_start_offset_before(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_offset)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 367, __pyx_L1_error)
    if (!__pyx_t_1) {
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_2 = __pyx_t_4;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L5_bool_binop_done;
    }

    /* "supriya/utils/_intervals.pyx":368
 *         if node.start_offset < offset:
 *             result = self._recurse_get_start_offset_before(
 *                 node.right_child, offset) or node             # <<<<<<<<<<<<<<
 *         elif offset <= node.start_offset and node.left_child:
 *             result = self._recurse_get_start_offset_before(
 */
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_t_2 = ((PyObject *)__pyx_v_node);
    __pyx_L5_bool_binop_done:;
    __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":366
 *         if node is None:
 *             return result
 *         if node.start_offset < offset:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_before(
 *                 node.right_child, offset) or node
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":369
 *             result = self._recurse_get_start_offset_before(
 *                 node.right_child, offset) or node
 *         elif offset <= node.start_offset and node.left_child:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_before(
 *                 node.left_child, offset)
 */
  __pyx_t_5 = (__pyx_v_offset <= __pyx_v_node->start_offset);
  if (__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->left_child)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 369, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_5;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":371
 *         elif offset <= node.start_offset and node.left_child:
 *             result = self._recurse_get_start_offset_before(
 *                 node.left_child, offset)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_2);

    /* "supriya/utils/_intervals.pyx":370
 *                 node.right_child, offset) or node
 *         elif offset <= node.start_offset and node.left_child:
 *             result = self._recurse_get_start_offset_before(             # <<<<<<<<<<<<<<
 *                 node.left_child, offset)
 *         return result
 */
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_start_offset_before(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_v_offset)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 370, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":369
 *             result = self._recurse_get_start_offset_before(
 *                 node.right_child, offset) or node
 *         elif offset <= node.start_offset and node.left_child:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_start_offset_before(
 *                 node.left_child, offset)
 */
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":372
 *             result = self._recurse_get_start_offset_before(
 *                 node.left_child, offset)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef _CInterval _recurse_getitem_by_index(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":358
 *         return result
 * 
 *     cdef _CNode _recurse_get_start_offset_before(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_get_start_offset_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":374
 *         return result
 * 
 *     cdef _CInterval _recurse_getitem_by_index(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_getitem_by_index(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, int __pyx_v_index) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_getitem_by_index", 1);

  /* "supriya/utils/_intervals.pyx":379
 *         int index,
 *         ):
 *         if node.node_start_index <= index < node.node_stop_index:             # <<<<<<<<<<<<<<
 *             return node.payload[index - node.node_start_index]
 *         elif node.left_child and index < node.node_start_index:
 */
  __pyx_t_1 = (__pyx_v_node->node_start_index <= __pyx_v_index);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_index < __pyx_v_node->node_stop_index);
  }
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":380
 *         ):
 *         if node.node_start_index <= index < node.node_stop_index:
 *             return node.payload[index - node.node_start_index]             # <<<<<<<<<<<<<<
 *         elif node.left_child and index < node.node_start_index:
 *             return self._recurse_getitem_by_index(node.left_child, index)
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_2 = (__pyx_v_index - __pyx_v_node->node_start_index);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_node->payload, __pyx_t_2, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 380, __pyx_L1_error)
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":379
 *         int index,
 *         ):
 *         if node.node_start_index <= index < node.node_stop_index:             # <<<<<<<<<<<<<<
 *             return node.payload[index - node.node_start_index]
 *         elif node.left_child and index < node.node_start_index:
 */
  }

  /* "supriya/utils/_intervals.pyx":381
 *         if node.node_start_index <= index < node.node_stop_index:
 *             return node.payload[index - node.node_start_index]
 *         elif node.left_child and index < node.node_start_index:             # <<<<<<<<<<<<<<
 *             return self._recurse_getitem_by_index(node.left_child, index)
 *         elif node.right_child and node.node_stop_index <= index:
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->left_child)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 381, __pyx_L1_error)
  if (__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = (__pyx_v_index < __pyx_v_node->node_start_index);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":382
 *             return node.payload[index - node.node_start_index]
 *         elif node.left_child and index < node.node_start_index:
 *             return self._recurse_getitem_by_index(node.left_child, index)             # <<<<<<<<<<<<<<
 *         elif node.right_child and node.node_stop_index <= index:
 *             return self._recurse_getitem_by_index(node.right_child, index)
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_getitem_by_index(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_index)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_5);
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":381
 *         if node.node_start_index <= index < node.node_stop_index:
 *             return node.payload[index - node.node_start_index]
 *         elif node.left_child and index < node.node_start_index:             # <<<<<<<<<<<<<<
 *             return self._recurse_getitem_by_index(node.left_child, index)
 *         elif node.right_child and node.node_stop_index <= index:
 */
  }

  /* "supriya/utils/_intervals.pyx":383
 *         elif node.left_child and index < node.node_start_index:
 *             return self._recurse_getitem_by_index(node.left_child, index)
 *         elif node.right_child and node.node_stop_index <= index:             # <<<<<<<<<<<<<<
 *             return self._recurse_getitem_by_index(node.right_child, index)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->right_child)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 383, __pyx_L1_error)
  if (__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_4 = (__pyx_v_node->node_stop_index <= __pyx_v_index);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":384
 *             return self._recurse_getitem_by_index(node.left_child, index)
 *         elif node.right_child and node.node_stop_index <= index:
 *             return self._recurse_getitem_by_index(node.right_child, index)             # <<<<<<<<<<<<<<
 * 
 *     cdef object _recurse_getitem_by_slice(
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_5 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_getitem_by_index(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_5), __pyx_v_index)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":383
 *         elif node.left_child and index < node.node_start_index:
 *             return self._recurse_getitem_by_index(node.left_child, index)
 *         elif node.right_child and node.node_stop_index <= index:             # <<<<<<<<<<<<<<
 *             return self._recurse_getitem_by_index(node.right_child, index)
 * 
 */
  }

  /* "supriya/utils/_intervals.pyx":374
 *         return result
 * 
 *     cdef _CInterval _recurse_getitem_by_index(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_getitem_by_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":386
 *             return self._recurse_getitem_by_index(node.right_child, index)
 * 
 *     cdef object _recurse_getitem_by_slice(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_getitem_by_slice(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, int __pyx_v_start, int __pyx_v_stop) {
  int __pyx_v_node_start;
  int __pyx_v_node_stop;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_recurse_getitem_by_slice", 1);

  /* "supriya/utils/_intervals.pyx":393
 *         ):
 *         cdef int node_start, node_stop
 *         result = []             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return result
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":394
 *         cdef int node_start, node_stop
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if start < node.node_start_index and node.left_child:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":395
 *         result = []
 *         if node is None:
 *             return result             # <<<<<<<<<<<<<<
 *         if start < node.node_start_index and node.left_child:
 *             result.extend(self._recurse_getitem_by_slice(
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_result);
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":394
 *         cdef int node_start, node_stop
 *         result = []
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return result
 *         if start < node.node_start_index and node.left_child:
 */
  }

  /* "supriya/utils/_intervals.pyx":396
 *         if node is None:
 *             return result
 *         if start < node.node_start_index and node.left_child:             # <<<<<<<<<<<<<<
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.left_child, start, stop))
 */
  __pyx_t_3 = (__pyx_v_start < __pyx_v_node->node_start_index);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->left_child)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 396, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":398
 *         if start < node.node_start_index and node.left_child:
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.left_child, start, stop))             # <<<<<<<<<<<<<<
 *         if start < node.node_stop_index and node.node_start_index < stop:
 *             node_start = start - node.node_start_index
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_1);

    /* "supriya/utils/_intervals.pyx":397
 *             return result
 *         if start < node.node_start_index and node.left_child:
 *             result.extend(self._recurse_getitem_by_slice(             # <<<<<<<<<<<<<<
 *                 node.left_child, start, stop))
 *         if start < node.node_stop_index and node.node_start_index < stop:
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_getitem_by_slice(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_v_start, __pyx_v_stop); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 397, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_4); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 397, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":396
 *         if node is None:
 *             return result
 *         if start < node.node_start_index and node.left_child:             # <<<<<<<<<<<<<<
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.left_child, start, stop))
 */
  }

  /* "supriya/utils/_intervals.pyx":399
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.left_child, start, stop))
 *         if start < node.node_stop_index and node.node_start_index < stop:             # <<<<<<<<<<<<<<
 *             node_start = start - node.node_start_index
 *             if node_start < 0:
 */
  __pyx_t_3 = (__pyx_v_start < __pyx_v_node->node_stop_index);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_node->node_start_index < __pyx_v_stop);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":400
 *                 node.left_child, start, stop))
 *         if start < node.node_stop_index and node.node_start_index < stop:
 *             node_start = start - node.node_start_index             # <<<<<<<<<<<<<<
 *             if node_start < 0:
 *                 node_start = 0
 */
    __pyx_v_node_start = (__pyx_v_start - __pyx_v_node->node_start_index);

    /* "supriya/utils/_intervals.pyx":401
 *         if start < node.node_stop_index and node.node_start_index < stop:
 *             node_start = start - node.node_start_index
 *             if node_start < 0:             # <<<<<<<<<<<<<<
 *                 node_start = 0
 *             node_stop = stop - node.node_start_index
 */
    __pyx_t_2 = (__pyx_v_node_start < 0);
    if (__pyx_t_2) {

      /* "supriya/utils/_intervals.pyx":402
 *             node_start = start - node.node_start_index
 *             if node_start < 0:
 *                 node_start = 0             # <<<<<<<<<<<<<<
 *             node_stop = stop - node.node_start_index
 *             result.extend(node.payload[node_start:node_stop])
 */
      __pyx_v_node_start = 0;

      /* "supriya/utils/_intervals.pyx":401
 *         if start < node.node_stop_index and node.node_start_index < stop:
 *             node_start = start - node.node_start_index
 *             if node_start < 0:             # <<<<<<<<<<<<<<
 *                 node_start = 0
 *             node_stop = stop - node.node_start_index
 */
    }

    /* "supriya/utils/_intervals.pyx":403
 *             if node_start < 0:
 *                 node_start = 0
 *             node_stop = stop - node.node_start_index             # <<<<<<<<<<<<<<
 *             result.extend(node.payload[node_start:node_stop])
 *         if node.node_stop_index <= stop and node.right_child:
 */
    __pyx_v_node_stop = (__pyx_v_stop - __pyx_v_node->node_start_index);

    /* "supriya/utils/_intervals.pyx":404
 *                 node_start = 0
 *             node_stop = stop - node.node_start_index
 *             result.extend(node.payload[node_start:node_stop])             # <<<<<<<<<<<<<<
 *         if node.node_stop_index <= stop and node.right_child:
 *             result.extend(self._recurse_getitem_by_slice(
 */
    __pyx_t_4 = __Pyx_PyObject_GetSlice(__pyx_v_node->payload, __pyx_v_node_start, __pyx_v_node_stop, NULL, NULL, NULL, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_4); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":399
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.left_child, start, stop))
 *         if start < node.node_stop_index and node.node_start_index < stop:             # <<<<<<<<<<<<<<
 *             node_start = start - node.node_start_index
 *             if node_start < 0:
 */
  }

  /* "supriya/utils/_intervals.pyx":405
 *             node_stop = stop - node.node_start_index
 *             result.extend(node.payload[node_start:node_stop])
 *         if node.node_stop_index <= stop and node.right_child:             # <<<<<<<<<<<<<<
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.right_child, start, stop))
 */
  __pyx_t_3 = (__pyx_v_node->node_stop_index <= __pyx_v_stop);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->right_child)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 405, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_2) {

    /* "supriya/utils/_intervals.pyx":407
 *         if node.node_stop_index <= stop and node.right_child:
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.right_child, start, stop))             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_4 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_4);

    /* "supriya/utils/_intervals.pyx":406
 *             result.extend(node.payload[node_start:node_stop])
 *         if node.node_stop_index <= stop and node.right_child:
 *             result.extend(self._recurse_getitem_by_slice(             # <<<<<<<<<<<<<<
 *                 node.right_child, start, stop))
 *         return result
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_getitem_by_slice(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4), __pyx_v_start, __pyx_v_stop); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "supriya/utils/_intervals.pyx":405
 *             node_stop = stop - node.node_start_index
 *             result.extend(node.payload[node_start:node_stop])
 *         if node.node_stop_index <= stop and node.right_child:             # <<<<<<<<<<<<<<
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.right_child, start, stop))
 */
  }

  /* "supriya/utils/_intervals.pyx":408
 *             result.extend(self._recurse_getitem_by_slice(
 *                 node.right_child, start, stop))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _remove_node(
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":386
 *             return self._recurse_getitem_by_index(node.right_child, index)
 * 
 *     cdef object _recurse_getitem_by_slice(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._recurse_getitem_by_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":410
 *         return result
 * 
 *     cdef _CNode _remove_node(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__remove_node(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_start_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_next_node = 0;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_child_node = 0;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  float __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_remove_node", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_node);

  /* "supriya/utils/_intervals.pyx":416
 *         ):
 *         cdef _CNode next_node, child_node
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         if node.start_offset == start_offset:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":417
 *         cdef _CNode next_node, child_node
 *         if node is None:
 *             return             # <<<<<<<<<<<<<<
 *         if node.start_offset == start_offset:
 *             if node.left_child and node.right_child:
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":416
 *         ):
 *         cdef _CNode next_node, child_node
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         if node.start_offset == start_offset:
 */
  }

  /* "supriya/utils/_intervals.pyx":418
 *         if node is None:
 *             return
 *         if node.start_offset == start_offset:             # <<<<<<<<<<<<<<
 *             if node.left_child and node.right_child:
 *                 next_node = node.right_child
 */
  __pyx_t_1 = (__pyx_v_node->start_offset == __pyx_v_start_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":419
 *             return
 *         if node.start_offset == start_offset:
 *             if node.left_child and node.right_child:             # <<<<<<<<<<<<<<
 *                 next_node = node.right_child
 *                 while next_node.left_child is not None:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->left_child)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 419, __pyx_L1_error)
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->right_child)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 419, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_2;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":420
 *         if node.start_offset == start_offset:
 *             if node.left_child and node.right_child:
 *                 next_node = node.right_child             # <<<<<<<<<<<<<<
 *                 while next_node.left_child is not None:
 *                     next_node = next_node.left_child
 */
      __pyx_t_3 = ((PyObject *)__pyx_v_node->right_child);
      __Pyx_INCREF(__pyx_t_3);
      __pyx_v_next_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
      __pyx_t_3 = 0;

      /* "supriya/utils/_intervals.pyx":421
 *             if node.left_child and node.right_child:
 *                 next_node = node.right_child
 *                 while next_node.left_child is not None:             # <<<<<<<<<<<<<<
 *                     next_node = next_node.left_child
 *                 node.start_offset = next_node.start_offset
 */
      while (1) {
        __pyx_t_1 = (((PyObject *)__pyx_v_next_node->left_child) != Py_None);
        if (!__pyx_t_1) break;

        /* "supriya/utils/_intervals.pyx":422
 *                 next_node = node.right_child
 *                 while next_node.left_child is not None:
 *                     next_node = next_node.left_child             # <<<<<<<<<<<<<<
 *                 node.start_offset = next_node.start_offset
 *                 node.payload = next_node.payload
 */
        __pyx_t_3 = ((PyObject *)__pyx_v_next_node->left_child);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_DECREF_SET(__pyx_v_next_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3));
        __pyx_t_3 = 0;
      }

      /* "supriya/utils/_intervals.pyx":423
 *                 while next_node.left_child is not None:
 *                     next_node = next_node.left_child
 *                 node.start_offset = next_node.start_offset             # <<<<<<<<<<<<<<
 *                 node.payload = next_node.payload
 *                 self._set_node_right_child(node, self._remove_node(
 */
      __pyx_t_4 = __pyx_v_next_node->start_offset;
      __pyx_v_node->start_offset = __pyx_t_4;

      /* "supriya/utils/_intervals.pyx":424
 *                     next_node = next_node.left_child
 *                 node.start_offset = next_node.start_offset
 *                 node.payload = next_node.payload             # <<<<<<<<<<<<<<
 *                 self._set_node_right_child(node, self._remove_node(
 *                     node.right_child,
 */
      __pyx_t_3 = __pyx_v_next_node->payload;
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_v_node->payload);
      __Pyx_DECREF(__pyx_v_node->payload);
      __pyx_v_node->payload = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "supriya/utils/_intervals.pyx":426
 *                 node.payload = next_node.payload
 *                 self._set_node_right_child(node, self._remove_node(
 *                     node.right_child,             # <<<<<<<<<<<<<<
 *                     next_node.start_offset,
 *                     ))
 */
      __pyx_t_3 = ((PyObject *)__pyx_v_node->right_child);
      __Pyx_INCREF(__pyx_t_3);

      /* "supriya/utils/_intervals.pyx":425
 *                 node.start_offset = next_node.start_offset
 *                 node.payload = next_node.payload
 *                 self._set_node_right_child(node, self._remove_node(             # <<<<<<<<<<<<<<
 *                     node.right_child,
 *                     next_node.start_offset,
 */
      __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_remove_node(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_next_node->start_offset)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 425, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_right_child(__pyx_v_self, __pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_5)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 425, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "supriya/utils/_intervals.pyx":419
 *             return
 *         if node.start_offset == start_offset:
 *             if node.left_child and node.right_child:             # <<<<<<<<<<<<<<
 *                 next_node = node.right_child
 *                 while next_node.left_child is not None:
 */
      goto __pyx_L5;
    }

    /* "supriya/utils/_intervals.pyx":430
 *                     ))
 *             else:
 *                 node = node.left_child or node.right_child             # <<<<<<<<<<<<<<
 *         elif start_offset < node.start_offset:
 *             child_node = self._remove_node(node.left_child, start_offset)
 */
    /*else*/ {
      __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_node->left_child)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 430, __pyx_L1_error)
      if (!__pyx_t_1) {
      } else {
        __Pyx_INCREF((PyObject *)__pyx_v_node->left_child);
        __pyx_t_5 = ((PyObject *)__pyx_v_node->left_child);
        goto __pyx_L10_bool_binop_done;
      }
      __Pyx_INCREF((PyObject *)__pyx_v_node->right_child);
      __pyx_t_5 = ((PyObject *)__pyx_v_node->right_child);
      __pyx_L10_bool_binop_done:;
      __Pyx_DECREF_SET(__pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_5));
      __pyx_t_5 = 0;
    }
    __pyx_L5:;

    /* "supriya/utils/_intervals.pyx":418
 *         if node is None:
 *             return
 *         if node.start_offset == start_offset:             # <<<<<<<<<<<<<<
 *             if node.left_child and node.right_child:
 *                 next_node = node.right_child
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":431
 *             else:
 *                 node = node.left_child or node.right_child
 *         elif start_offset < node.start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._remove_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 */
  __pyx_t_1 = (__pyx_v_start_offset < __pyx_v_node->start_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":432
 *                 node = node.left_child or node.right_child
 *         elif start_offset < node.start_offset:
 *             child_node = self._remove_node(node.left_child, start_offset)             # <<<<<<<<<<<<<<
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:
 */
    __pyx_t_5 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_remove_node(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_5), __pyx_v_start_offset)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_child_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":433
 *         elif start_offset < node.start_offset:
 *             child_node = self._remove_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)             # <<<<<<<<<<<<<<
 *         elif node.start_offset < start_offset:
 *             child_node = self._remove_node(node.right_child, start_offset)
 */
    ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_left_child(__pyx_v_self, __pyx_v_node, __pyx_v_child_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 433, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":431
 *             else:
 *                 node = node.left_child or node.right_child
 *         elif start_offset < node.start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._remove_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":434
 *             child_node = self._remove_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._remove_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)
 */
  __pyx_t_1 = (__pyx_v_node->start_offset < __pyx_v_start_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":435
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:
 *             child_node = self._remove_node(node.right_child, start_offset)             # <<<<<<<<<<<<<<
 *             self._set_node_right_child(node, child_node)
 *         return self._rebalance(node)
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_remove_node(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_start_offset)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_child_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "supriya/utils/_intervals.pyx":436
 *         elif node.start_offset < start_offset:
 *             child_node = self._remove_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)             # <<<<<<<<<<<<<<
 *         return self._rebalance(node)
 * 
 */
    ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_right_child(__pyx_v_self, __pyx_v_node, __pyx_v_child_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 436, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":434
 *             child_node = self._remove_node(node.left_child, start_offset)
 *             self._set_node_left_child(node, child_node)
 *         elif node.start_offset < start_offset:             # <<<<<<<<<<<<<<
 *             child_node = self._remove_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)
 */
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":437
 *             child_node = self._remove_node(node.right_child, start_offset)
 *             self._set_node_right_child(node, child_node)
 *         return self._rebalance(node)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _remove_interval(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rebalance(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_5);
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":410
 *         return result
 * 
 *     cdef _CNode _remove_node(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._remove_node", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_next_node);
  __Pyx_XDECREF((PyObject *)__pyx_v_child_node);
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":439
 *         return self._rebalance(node)
 * 
 *     cdef void _remove_interval(             # <<<<<<<<<<<<<<
 *         self,
 *         _CInterval cinterval,
 */

static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__remove_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_remove_interval", 1);

  /* "supriya/utils/_intervals.pyx":444
 *         ):
 *         cdef _CNode node
 *         node = self._search(self._root_node, cinterval.start_offset)             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 444, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_search(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_v_cinterval->start_offset)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":445
 *         cdef _CNode node
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         if cinterval in node.payload:
 */
  __pyx_t_3 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_3) {

    /* "supriya/utils/_intervals.pyx":446
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         if node is None:
 *             return             # <<<<<<<<<<<<<<
 *         if cinterval in node.payload:
 *             node.payload.remove(cinterval)
 */
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":445
 *         cdef _CNode node
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         if cinterval in node.payload:
 */
  }

  /* "supriya/utils/_intervals.pyx":447
 *         if node is None:
 *             return
 *         if cinterval in node.payload:             # <<<<<<<<<<<<<<
 *             node.payload.remove(cinterval)
 *         if not node.payload:
 */
  __pyx_t_3 = (__Pyx_PySequence_ContainsTF(((PyObject *)__pyx_v_cinterval), __pyx_v_node->payload, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 447, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "supriya/utils/_intervals.pyx":448
 *             return
 *         if cinterval in node.payload:
 *             node.payload.remove(cinterval)             # <<<<<<<<<<<<<<
 *         if not node.payload:
 *             self._root_node = self._remove_node(
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node->payload, __pyx_n_s_remove); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 448, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_5 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, ((PyObject *)__pyx_v_cinterval)};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 448, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":447
 *         if node is None:
 *             return
 *         if cinterval in node.payload:             # <<<<<<<<<<<<<<
 *             node.payload.remove(cinterval)
 *         if not node.payload:
 */
  }

  /* "supriya/utils/_intervals.pyx":449
 *         if cinterval in node.payload:
 *             node.payload.remove(cinterval)
 *         if not node.payload:             # <<<<<<<<<<<<<<
 *             self._root_node = self._remove_node(
 *                 self._root_node,
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_node->payload); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 449, __pyx_L1_error)
  __pyx_t_6 = (!__pyx_t_3);
  if (__pyx_t_6) {

    /* "supriya/utils/_intervals.pyx":451
 *         if not node.payload:
 *             self._root_node = self._remove_node(
 *                 self._root_node,             # <<<<<<<<<<<<<<
 *                 cinterval.start_offset,
 *                 )
 */
    if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 451, __pyx_L1_error)
    __pyx_t_2 = __pyx_v_self->_root_node;
    __Pyx_INCREF(__pyx_t_2);

    /* "supriya/utils/_intervals.pyx":450
 *             node.payload.remove(cinterval)
 *         if not node.payload:
 *             self._root_node = self._remove_node(             # <<<<<<<<<<<<<<
 *                 self._root_node,
 *                 cinterval.start_offset,
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_remove_node(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_v_cinterval->start_offset)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_root_node);
    __Pyx_DECREF(__pyx_v_self->_root_node);
    __pyx_v_self->_root_node = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "supriya/utils/_intervals.pyx":449
 *         if cinterval in node.payload:
 *             node.payload.remove(cinterval)
 *         if not node.payload:             # <<<<<<<<<<<<<<
 *             self._root_node = self._remove_node(
 *                 self._root_node,
 */
  }

  /* "supriya/utils/_intervals.pyx":439
 *         return self._rebalance(node)
 * 
 *     cdef void _remove_interval(             # <<<<<<<<<<<<<<
 *         self,
 *         _CInterval cinterval,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._remove_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_RefNannyFinishContext();
}

/* "supriya/utils/_intervals.pyx":455
 *                 )
 * 
 *     cdef _CNode _rotate_left_left(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_left_left(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_next_node = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_rotate_left_left", 1);

  /* "supriya/utils/_intervals.pyx":459
 *         _CNode node
 *         ):
 *         next_node = node.left_child             # <<<<<<<<<<<<<<
 *         self._set_node_left_child(node, next_node.right_child)
 *         self._set_node_right_child(next_node, node)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_node->left_child);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_next_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":460
 *         ):
 *         next_node = node.left_child
 *         self._set_node_left_child(node, next_node.right_child)             # <<<<<<<<<<<<<<
 *         self._set_node_right_child(next_node, node)
 *         return next_node
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_next_node->right_child);
  __Pyx_INCREF(__pyx_t_1);
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_left_child(__pyx_v_self, __pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":461
 *         next_node = node.left_child
 *         self._set_node_left_child(node, next_node.right_child)
 *         self._set_node_right_child(next_node, node)             # <<<<<<<<<<<<<<
 *         return next_node
 * 
 */
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_right_child(__pyx_v_self, __pyx_v_next_node, __pyx_v_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 461, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":462
 *         self._set_node_left_child(node, next_node.right_child)
 *         self._set_node_right_child(next_node, node)
 *         return next_node             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _rotate_left_right(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_next_node);
  __pyx_r = __pyx_v_next_node;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":455
 *                 )
 * 
 *     cdef _CNode _rotate_left_left(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._rotate_left_left", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_next_node);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":464
 *         return next_node
 * 
 *     cdef _CNode _rotate_left_right(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_left_right(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_next_node = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_rotate_left_right", 1);

  /* "supriya/utils/_intervals.pyx":469
 *         ):
 *         self._set_node_left_child(
 *             node, self._rotate_right_right(node.left_child))             # <<<<<<<<<<<<<<
 *         next_node = self._rotate_left_left(node)
 *         return next_node
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_node->left_child);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_right_right(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":468
 *         _CNode node,
 *         ):
 *         self._set_node_left_child(             # <<<<<<<<<<<<<<
 *             node, self._rotate_right_right(node.left_child))
 *         next_node = self._rotate_left_left(node)
 */
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_left_child(__pyx_v_self, __pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":470
 *         self._set_node_left_child(
 *             node, self._rotate_right_right(node.left_child))
 *         next_node = self._rotate_left_left(node)             # <<<<<<<<<<<<<<
 *         return next_node
 * 
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_left_left(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_next_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":471
 *             node, self._rotate_right_right(node.left_child))
 *         next_node = self._rotate_left_left(node)
 *         return next_node             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _rotate_right_left(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_next_node);
  __pyx_r = __pyx_v_next_node;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":464
 *         return next_node
 * 
 *     cdef _CNode _rotate_left_right(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._rotate_left_right", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_next_node);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":473
 *         return next_node
 * 
 *     cdef _CNode _rotate_right_left(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_right_left(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_next_node = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_rotate_right_left", 1);

  /* "supriya/utils/_intervals.pyx":478
 *         ):
 *         self._set_node_right_child(
 *             node, self._rotate_left_left(node.right_child))             # <<<<<<<<<<<<<<
 *         next_node = self._rotate_right_right(node)
 *         return next_node
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_node->right_child);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_left_left(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":477
 *         _CNode node,
 *         ):
 *         self._set_node_right_child(             # <<<<<<<<<<<<<<
 *             node, self._rotate_left_left(node.right_child))
 *         next_node = self._rotate_right_right(node)
 */
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_right_child(__pyx_v_self, __pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":479
 *         self._set_node_right_child(
 *             node, self._rotate_left_left(node.right_child))
 *         next_node = self._rotate_right_right(node)             # <<<<<<<<<<<<<<
 *         return next_node
 * 
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_rotate_right_right(__pyx_v_self, __pyx_v_node)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_next_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":480
 *             node, self._rotate_left_left(node.right_child))
 *         next_node = self._rotate_right_right(node)
 *         return next_node             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _rotate_right_right(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_next_node);
  __pyx_r = __pyx_v_next_node;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":473
 *         return next_node
 * 
 *     cdef _CNode _rotate_right_left(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._rotate_right_left", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_next_node);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":482
 *         return next_node
 * 
 *     cdef _CNode _rotate_right_right(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_right_right(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_next_node = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_rotate_right_right", 1);

  /* "supriya/utils/_intervals.pyx":486
 *         _CNode node,
 *         ):
 *         next_node = node.right_child             # <<<<<<<<<<<<<<
 *         self._set_node_right_child(node, next_node.left_child)
 *         self._set_node_left_child(next_node, node)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_node->right_child);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_next_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":487
 *         ):
 *         next_node = node.right_child
 *         self._set_node_right_child(node, next_node.left_child)             # <<<<<<<<<<<<<<
 *         self._set_node_left_child(next_node, node)
 *         return next_node
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_next_node->left_child);
  __Pyx_INCREF(__pyx_t_1);
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_right_child(__pyx_v_self, __pyx_v_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":488
 *         next_node = node.right_child
 *         self._set_node_right_child(node, next_node.left_child)
 *         self._set_node_left_child(next_node, node)             # <<<<<<<<<<<<<<
 *         return next_node
 * 
 */
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_set_node_left_child(__pyx_v_self, __pyx_v_next_node, __pyx_v_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 488, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":489
 *         self._set_node_right_child(node, next_node.left_child)
 *         self._set_node_left_child(next_node, node)
 *         return next_node             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _search(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_next_node);
  __pyx_r = __pyx_v_next_node;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":482
 *         return next_node
 * 
 *     cdef _CNode _rotate_right_right(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._rotate_right_right", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_next_node);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":491
 *         return next_node
 * 
 *     cdef _CNode _search(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__search(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, float __pyx_v_start_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_search", 1);

  /* "supriya/utils/_intervals.pyx":496
 *         float start_offset,
 *         ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         if node.start_offset == start_offset:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":497
 *         ):
 *         if node is None:
 *             return None             # <<<<<<<<<<<<<<
 *         if node.start_offset == start_offset:
 *             return node
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":496
 *         float start_offset,
 *         ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         if node.start_offset == start_offset:
 */
  }

  /* "supriya/utils/_intervals.pyx":498
 *         if node is None:
 *             return None
 *         if node.start_offset == start_offset:             # <<<<<<<<<<<<<<
 *             return node
 *         elif (
 */
  __pyx_t_1 = (__pyx_v_node->start_offset == __pyx_v_start_offset);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":499
 *             return None
 *         if node.start_offset == start_offset:
 *             return node             # <<<<<<<<<<<<<<
 *         elif (
 *             node.left_child is not None and
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_node);
    __pyx_r = __pyx_v_node;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":498
 *         if node is None:
 *             return None
 *         if node.start_offset == start_offset:             # <<<<<<<<<<<<<<
 *             return node
 *         elif (
 */
  }

  /* "supriya/utils/_intervals.pyx":501
 *             return node
 *         elif (
 *             node.left_child is not None and             # <<<<<<<<<<<<<<
 *             start_offset < node.start_offset
 *             ):
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_node->left_child) != Py_None);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }

  /* "supriya/utils/_intervals.pyx":502
 *         elif (
 *             node.left_child is not None and
 *             start_offset < node.start_offset             # <<<<<<<<<<<<<<
 *             ):
 *             return self._search(node.left_child, start_offset)
 */
  __pyx_t_2 = (__pyx_v_start_offset < __pyx_v_node->start_offset);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;

  /* "supriya/utils/_intervals.pyx":500
 *         if node.start_offset == start_offset:
 *             return node
 *         elif (             # <<<<<<<<<<<<<<
 *             node.left_child is not None and
 *             start_offset < node.start_offset
 */
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":504
 *             start_offset < node.start_offset
 *             ):
 *             return self._search(node.left_child, start_offset)             # <<<<<<<<<<<<<<
 *         elif (
 *             node.right_child is not None and
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_search(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3), __pyx_v_start_offset)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 504, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":500
 *         if node.start_offset == start_offset:
 *             return node
 *         elif (             # <<<<<<<<<<<<<<
 *             node.left_child is not None and
 *             start_offset < node.start_offset
 */
  }

  /* "supriya/utils/_intervals.pyx":506
 *             return self._search(node.left_child, start_offset)
 *         elif (
 *             node.right_child is not None and             # <<<<<<<<<<<<<<
 *             node.start_offset < start_offset
 *             ):
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_node->right_child) != Py_None);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }

  /* "supriya/utils/_intervals.pyx":507
 *         elif (
 *             node.right_child is not None and
 *             node.start_offset < start_offset             # <<<<<<<<<<<<<<
 *             ):
 *             return self._search(node.right_child, start_offset)
 */
  __pyx_t_2 = (__pyx_v_node->start_offset < __pyx_v_start_offset);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;

  /* "supriya/utils/_intervals.pyx":505
 *             ):
 *             return self._search(node.left_child, start_offset)
 *         elif (             # <<<<<<<<<<<<<<
 *             node.right_child is not None and
 *             node.start_offset < start_offset
 */
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":509
 *             node.start_offset < start_offset
 *             ):
 *             return self._search(node.right_child, start_offset)             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_t_4 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_search(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_4), __pyx_v_start_offset)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":505
 *             ):
 *             return self._search(node.left_child, start_offset)
 *         elif (             # <<<<<<<<<<<<<<
 *             node.right_child is not None and
 *             node.start_offset < start_offset
 */
  }

  /* "supriya/utils/_intervals.pyx":510
 *             ):
 *             return self._search(node.right_child, start_offset)
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cdef void _set_node_left_child(
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":491
 *         return next_node
 * 
 *     cdef _CNode _search(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._search", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":512
 *         return None
 * 
 *     cdef void _set_node_left_child(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__set_node_left_child(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_left_child) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_node_left_child", 1);

  /* "supriya/utils/_intervals.pyx":517
 *         _CNode left_child,
 *         ):
 *         node.left_child = left_child             # <<<<<<<<<<<<<<
 *         self._update_node_height(node)
 * 
 */
  __Pyx_INCREF((PyObject *)__pyx_v_left_child);
  __Pyx_GIVEREF((PyObject *)__pyx_v_left_child);
  __Pyx_GOTREF((PyObject *)__pyx_v_node->left_child);
  __Pyx_DECREF((PyObject *)__pyx_v_node->left_child);
  __pyx_v_node->left_child = __pyx_v_left_child;

  /* "supriya/utils/_intervals.pyx":518
 *         ):
 *         node.left_child = left_child
 *         self._update_node_height(node)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _set_node_right_child(
 */
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_node_height(__pyx_v_self, __pyx_v_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 518, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":512
 *         return None
 * 
 *     cdef void _set_node_left_child(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._set_node_left_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "supriya/utils/_intervals.pyx":520
 *         self._update_node_height(node)
 * 
 *     cdef void _set_node_right_child(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__set_node_right_child(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_right_child) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_node_right_child", 1);

  /* "supriya/utils/_intervals.pyx":525
 *         _CNode right_child,
 *         ):
 *         node.right_child = right_child             # <<<<<<<<<<<<<<
 *         self._update_node_height(node)
 * 
 */
  __Pyx_INCREF((PyObject *)__pyx_v_right_child);
  __Pyx_GIVEREF((PyObject *)__pyx_v_right_child);
  __Pyx_GOTREF((PyObject *)__pyx_v_node->right_child);
  __Pyx_DECREF((PyObject *)__pyx_v_node->right_child);
  __pyx_v_node->right_child = __pyx_v_right_child;

  /* "supriya/utils/_intervals.pyx":526
 *         ):
 *         node.right_child = right_child
 *         self._update_node_height(node)             # <<<<<<<<<<<<<<
 * 
 *     cdef object _unbox_cintervals(
 */
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_node_height(__pyx_v_self, __pyx_v_node); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 526, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":520
 *         self._update_node_height(node)
 * 
 *     cdef void _set_node_right_child(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._set_node_right_child", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "supriya/utils/_intervals.pyx":533
 *         ):
 *         cdef _CInterval cinterval
 *         cintervals.sort(key=lambda x: (x.start_offset, x.stop_offset))             # <<<<<<<<<<<<<<
 *         return [cinterval.original_interval for cinterval in cintervals]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17_unbox_cintervals_lambda1(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17_unbox_cintervals_lambda1 = {"lambda1", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17_unbox_cintervals_lambda1, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17_unbox_cintervals_lambda1(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_x = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda1 (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_x)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 533, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "lambda1") < 0)) __PYX_ERR(0, 533, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_x = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lambda1", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 533, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._unbox_cintervals.lambda1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_lambda_funcdef_lambda1(__pyx_self, __pyx_v_x);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda1(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda1", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_start_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_stop_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._unbox_cintervals.lambda1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":528
 *         self._update_node_height(node)
 * 
 *     cdef object _unbox_cintervals(             # <<<<<<<<<<<<<<
 *         self,
 *         cintervals,
 */

static PyObject *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__unbox_cintervals(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_cintervals) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_7genexpr__pyx_v_cinterval = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unbox_cintervals", 1);

  /* "supriya/utils/_intervals.pyx":533
 *         ):
 *         cdef _CInterval cinterval
 *         cintervals.sort(key=lambda x: (x.start_offset, x.stop_offset))             # <<<<<<<<<<<<<<
 *         return [cinterval.original_interval for cinterval in cintervals]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_cintervals, __pyx_n_s_sort); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17_unbox_cintervals_lambda1, 0, __pyx_n_s_IntervalTreeDriverEx__unbox_cint, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_key, __pyx_t_3) < 0) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":534
 *         cdef _CInterval cinterval
 *         cintervals.sort(key=lambda x: (x.start_offset, x.stop_offset))
 *         return [cinterval.original_interval for cinterval in cintervals]             # <<<<<<<<<<<<<<
 * 
 *     cdef void _update_node_height(
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 534, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (likely(PyList_CheckExact(__pyx_v_cintervals)) || PyTuple_CheckExact(__pyx_v_cintervals)) {
      __pyx_t_2 = __pyx_v_cintervals; __Pyx_INCREF(__pyx_t_2);
      __pyx_t_4 = 0;
      __pyx_t_5 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_cintervals); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 534, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 534, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_5)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 534, __pyx_L5_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 534, __pyx_L5_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 534, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
            #if !CYTHON_ASSUME_SAFE_MACROS
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 534, __pyx_L5_error)
            #endif
            if (__pyx_t_4 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely((0 < 0))) __PYX_ERR(0, 534, __pyx_L5_error)
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 534, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_5(__pyx_t_2);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 534, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 534, __pyx_L5_error)
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_cinterval, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_1));
      __pyx_t_1 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_7genexpr__pyx_v_cinterval->original_interval))) __PYX_ERR(0, 534, __pyx_L5_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF((PyObject *)__pyx_7genexpr__pyx_v_cinterval); __pyx_7genexpr__pyx_v_cinterval = 0;
    goto __pyx_L9_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF((PyObject *)__pyx_7genexpr__pyx_v_cinterval); __pyx_7genexpr__pyx_v_cinterval = 0;
    goto __pyx_L1_error;
    __pyx_L9_exit_scope:;
  } /* exit inner scope */
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":528
 *         self._update_node_height(node)
 * 
 *     cdef object _unbox_cintervals(             # <<<<<<<<<<<<<<
 *         self,
 *         cintervals,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._unbox_cintervals", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_7genexpr__pyx_v_cinterval);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":536
 *         return [cinterval.original_interval for cinterval in cintervals]
 * 
 *     cdef void _update_node_height(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_node_height(CYTHON_UNUSED struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node) {
  int __pyx_v_left_height;
  int __pyx_v_right_height;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "supriya/utils/_intervals.pyx":541
 *         ):
 *         cdef int left_height, right_height
 *         left_height = -1             # <<<<<<<<<<<<<<
 *         right_height = -1
 *         if node.left_child is not None:
 */
  __pyx_v_left_height = -1;

  /* "supriya/utils/_intervals.pyx":542
 *         cdef int left_height, right_height
 *         left_height = -1
 *         right_height = -1             # <<<<<<<<<<<<<<
 *         if node.left_child is not None:
 *             left_height = node.left_child.height
 */
  __pyx_v_right_height = -1;

  /* "supriya/utils/_intervals.pyx":543
 *         left_height = -1
 *         right_height = -1
 *         if node.left_child is not None:             # <<<<<<<<<<<<<<
 *             left_height = node.left_child.height
 *         if node.right_child is not None:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node->left_child) != Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":544
 *         right_height = -1
 *         if node.left_child is not None:
 *             left_height = node.left_child.height             # <<<<<<<<<<<<<<
 *         if node.right_child is not None:
 *             right_height = node.right_child.height
 */
    __pyx_t_2 = __pyx_v_node->left_child->height;
    __pyx_v_left_height = __pyx_t_2;

    /* "supriya/utils/_intervals.pyx":543
 *         left_height = -1
 *         right_height = -1
 *         if node.left_child is not None:             # <<<<<<<<<<<<<<
 *             left_height = node.left_child.height
 *         if node.right_child is not None:
 */
  }

  /* "supriya/utils/_intervals.pyx":545
 *         if node.left_child is not None:
 *             left_height = node.left_child.height
 *         if node.right_child is not None:             # <<<<<<<<<<<<<<
 *             right_height = node.right_child.height
 *         node.height = max(left_height, right_height) + 1
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node->right_child) != Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":546
 *             left_height = node.left_child.height
 *         if node.right_child is not None:
 *             right_height = node.right_child.height             # <<<<<<<<<<<<<<
 *         node.height = max(left_height, right_height) + 1
 *         node.balance = right_height - left_height
 */
    __pyx_t_2 = __pyx_v_node->right_child->height;
    __pyx_v_right_height = __pyx_t_2;

    /* "supriya/utils/_intervals.pyx":545
 *         if node.left_child is not None:
 *             left_height = node.left_child.height
 *         if node.right_child is not None:             # <<<<<<<<<<<<<<
 *             right_height = node.right_child.height
 *         node.height = max(left_height, right_height) + 1
 */
  }

  /* "supriya/utils/_intervals.pyx":547
 *         if node.right_child is not None:
 *             right_height = node.right_child.height
 *         node.height = max(left_height, right_height) + 1             # <<<<<<<<<<<<<<
 *         node.balance = right_height - left_height
 * 
 */
  __pyx_t_2 = __pyx_v_right_height;
  __pyx_t_3 = __pyx_v_left_height;
  __pyx_t_1 = (__pyx_t_2 > __pyx_t_3);
  if (__pyx_t_1) {
    __pyx_t_4 = __pyx_t_2;
  } else {
    __pyx_t_4 = __pyx_t_3;
  }
  __pyx_v_node->height = (__pyx_t_4 + 1);

  /* "supriya/utils/_intervals.pyx":548
 *             right_height = node.right_child.height
 *         node.height = max(left_height, right_height) + 1
 *         node.balance = right_height - left_height             # <<<<<<<<<<<<<<
 * 
 *     cdef void _update_indices(
 */
  __pyx_v_node->balance = (__pyx_v_right_height - __pyx_v_left_height);

  /* "supriya/utils/_intervals.pyx":536
 *         return [cinterval.original_interval for cinterval in cintervals]
 * 
 *     cdef void _update_node_height(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
}

/* "supriya/utils/_intervals.pyx":550
 *         node.balance = right_height - left_height
 * 
 *     cdef void _update_indices(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static void __pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_indices(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node, int __pyx_v_parent_stop_index) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_update_indices", 1);

  /* "supriya/utils/_intervals.pyx":555
 *         int parent_stop_index,
 *         ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         if node.left_child is not None:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":556
 *         ):
 *         if node is None:
 *             return             # <<<<<<<<<<<<<<
 *         if node.left_child is not None:
 *             self._update_indices(
 */
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":555
 *         int parent_stop_index,
 *         ):
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         if node.left_child is not None:
 */
  }

  /* "supriya/utils/_intervals.pyx":557
 *         if node is None:
 *             return
 *         if node.left_child is not None:             # <<<<<<<<<<<<<<
 *             self._update_indices(
 *                 node.left_child,
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node->left_child) != Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":559
 *         if node.left_child is not None:
 *             self._update_indices(
 *                 node.left_child,             # <<<<<<<<<<<<<<
 *                 parent_stop_index,
 *                 )
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_2);

    /* "supriya/utils/_intervals.pyx":558
 *             return
 *         if node.left_child is not None:
 *             self._update_indices(             # <<<<<<<<<<<<<<
 *                 node.left_child,
 *                 parent_stop_index,
 */
    ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_indices(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_v_parent_stop_index); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 558, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":562
 *                 parent_stop_index,
 *                 )
 *             node.node_start_index = node.left_child.subtree_stop_index             # <<<<<<<<<<<<<<
 *             node.subtree_start_index = node.left_child.subtree_start_index
 *         elif parent_stop_index == -1:
 */
    __pyx_t_3 = __pyx_v_node->left_child->subtree_stop_index;
    __pyx_v_node->node_start_index = __pyx_t_3;

    /* "supriya/utils/_intervals.pyx":563
 *                 )
 *             node.node_start_index = node.left_child.subtree_stop_index
 *             node.subtree_start_index = node.left_child.subtree_start_index             # <<<<<<<<<<<<<<
 *         elif parent_stop_index == -1:
 *             node.node_start_index = 0
 */
    __pyx_t_3 = __pyx_v_node->left_child->subtree_start_index;
    __pyx_v_node->subtree_start_index = __pyx_t_3;

    /* "supriya/utils/_intervals.pyx":557
 *         if node is None:
 *             return
 *         if node.left_child is not None:             # <<<<<<<<<<<<<<
 *             self._update_indices(
 *                 node.left_child,
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":564
 *             node.node_start_index = node.left_child.subtree_stop_index
 *             node.subtree_start_index = node.left_child.subtree_start_index
 *         elif parent_stop_index == -1:             # <<<<<<<<<<<<<<
 *             node.node_start_index = 0
 *             node.subtree_start_index = 0
 */
  __pyx_t_1 = (__pyx_v_parent_stop_index == -1L);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":565
 *             node.subtree_start_index = node.left_child.subtree_start_index
 *         elif parent_stop_index == -1:
 *             node.node_start_index = 0             # <<<<<<<<<<<<<<
 *             node.subtree_start_index = 0
 *         else:
 */
    __pyx_v_node->node_start_index = 0;

    /* "supriya/utils/_intervals.pyx":566
 *         elif parent_stop_index == -1:
 *             node.node_start_index = 0
 *             node.subtree_start_index = 0             # <<<<<<<<<<<<<<
 *         else:
 *             node.node_start_index = parent_stop_index
 */
    __pyx_v_node->subtree_start_index = 0;

    /* "supriya/utils/_intervals.pyx":564
 *             node.node_start_index = node.left_child.subtree_stop_index
 *             node.subtree_start_index = node.left_child.subtree_start_index
 *         elif parent_stop_index == -1:             # <<<<<<<<<<<<<<
 *             node.node_start_index = 0
 *             node.subtree_start_index = 0
 */
    goto __pyx_L4;
  }

  /* "supriya/utils/_intervals.pyx":568
 *             node.subtree_start_index = 0
 *         else:
 *             node.node_start_index = parent_stop_index             # <<<<<<<<<<<<<<
 *             node.subtree_start_index = parent_stop_index
 *         node.node_stop_index = node.node_start_index + len(node.payload)
 */
  /*else*/ {
    __pyx_v_node->node_start_index = __pyx_v_parent_stop_index;

    /* "supriya/utils/_intervals.pyx":569
 *         else:
 *             node.node_start_index = parent_stop_index
 *             node.subtree_start_index = parent_stop_index             # <<<<<<<<<<<<<<
 *         node.node_stop_index = node.node_start_index + len(node.payload)
 *         node.subtree_stop_index = node.node_stop_index
 */
    __pyx_v_node->subtree_start_index = __pyx_v_parent_stop_index;
  }
  __pyx_L4:;

  /* "supriya/utils/_intervals.pyx":570
 *             node.node_start_index = parent_stop_index
 *             node.subtree_start_index = parent_stop_index
 *         node.node_stop_index = node.node_start_index + len(node.payload)             # <<<<<<<<<<<<<<
 *         node.subtree_stop_index = node.node_stop_index
 *         if node.right_child is not None:
 */
  __pyx_t_2 = __pyx_v_node->payload;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 570, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_node->node_stop_index = (__pyx_v_node->node_start_index + __pyx_t_4);

  /* "supriya/utils/_intervals.pyx":571
 *             node.subtree_start_index = parent_stop_index
 *         node.node_stop_index = node.node_start_index + len(node.payload)
 *         node.subtree_stop_index = node.node_stop_index             # <<<<<<<<<<<<<<
 *         if node.right_child is not None:
 *             self._update_indices(
 */
  __pyx_t_3 = __pyx_v_node->node_stop_index;
  __pyx_v_node->subtree_stop_index = __pyx_t_3;

  /* "supriya/utils/_intervals.pyx":572
 *         node.node_stop_index = node.node_start_index + len(node.payload)
 *         node.subtree_stop_index = node.node_stop_index
 *         if node.right_child is not None:             # <<<<<<<<<<<<<<
 *             self._update_indices(
 *                 node.right_child,
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node->right_child) != Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":574
 *         if node.right_child is not None:
 *             self._update_indices(
 *                 node.right_child,             # <<<<<<<<<<<<<<
 *                 node.node_stop_index,
 *                 )
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_2);

    /* "supriya/utils/_intervals.pyx":573
 *         node.subtree_stop_index = node.node_stop_index
 *         if node.right_child is not None:
 *             self._update_indices(             # <<<<<<<<<<<<<<
 *                 node.right_child,
 *                 node.node_stop_index,
 */
    ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_indices(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_v_node->node_stop_index); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 573, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":577
 *                 node.node_stop_index,
 *                 )
 *             node.subtree_stop_index = node.right_child.subtree_stop_index             # <<<<<<<<<<<<<<
 * 
 *     cdef _CNode _update_offsets(
 */
    __pyx_t_3 = __pyx_v_node->right_child->subtree_stop_index;
    __pyx_v_node->subtree_stop_index = __pyx_t_3;

    /* "supriya/utils/_intervals.pyx":572
 *         node.node_stop_index = node.node_start_index + len(node.payload)
 *         node.subtree_stop_index = node.node_stop_index
 *         if node.right_child is not None:             # <<<<<<<<<<<<<<
 *             self._update_indices(
 *                 node.right_child,
 */
  }

  /* "supriya/utils/_intervals.pyx":550
 *         node.balance = right_height - left_height
 * 
 *     cdef void _update_indices(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._update_indices", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "supriya/utils/_intervals.pyx":579
 *             node.subtree_stop_index = node.right_child.subtree_stop_index
 * 
 *     cdef _CNode _update_offsets(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

static struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_offsets(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_child_node = 0;
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *__pyx_v_cinterval = 0;
  float __pyx_v_stop_offset_low;
  double __pyx_v_stop_offset_high;
  Py_ssize_t __pyx_v_i;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  double __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  float __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_update_offsets", 1);

  /* "supriya/utils/_intervals.pyx":586
 *         cdef _CInterval cinterval
 *         cdef float stop_offset_low, stop_offset_hight
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         stop_offset_low = float('inf')
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":587
 *         cdef float stop_offset_low, stop_offset_hight
 *         if node is None:
 *             return             # <<<<<<<<<<<<<<
 *         stop_offset_low = float('inf')
 *         stop_offset_high = float('-inf')
 */
    __Pyx_XDECREF((PyObject *)__pyx_r);
    __pyx_r = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":586
 *         cdef _CInterval cinterval
 *         cdef float stop_offset_low, stop_offset_hight
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return
 *         stop_offset_low = float('inf')
 */
  }

  /* "supriya/utils/_intervals.pyx":588
 *         if node is None:
 *             return
 *         stop_offset_low = float('inf')             # <<<<<<<<<<<<<<
 *         stop_offset_high = float('-inf')
 *         for i in range(len(node.payload)):
 */
  __pyx_t_2 = __Pyx_PyUnicode_AsDouble(__pyx_n_u_inf); if (unlikely(__pyx_t_2 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 588, __pyx_L1_error)
  __pyx_v_stop_offset_low = __pyx_t_2;

  /* "supriya/utils/_intervals.pyx":589
 *             return
 *         stop_offset_low = float('inf')
 *         stop_offset_high = float('-inf')             # <<<<<<<<<<<<<<
 *         for i in range(len(node.payload)):
 *             cinterval = node.payload[i]
 */
  __pyx_t_2 = __Pyx_PyUnicode_AsDouble(__pyx_kp_u_inf_2); if (unlikely(__pyx_t_2 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(0, 589, __pyx_L1_error)
  __pyx_v_stop_offset_high = __pyx_t_2;

  /* "supriya/utils/_intervals.pyx":590
 *         stop_offset_low = float('inf')
 *         stop_offset_high = float('-inf')
 *         for i in range(len(node.payload)):             # <<<<<<<<<<<<<<
 *             cinterval = node.payload[i]
 *             if cinterval.stop_offset < stop_offset_low:
 */
  __pyx_t_3 = __pyx_v_node->payload;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __pyx_t_4;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "supriya/utils/_intervals.pyx":591
 *         stop_offset_high = float('-inf')
 *         for i in range(len(node.payload)):
 *             cinterval = node.payload[i]             # <<<<<<<<<<<<<<
 *             if cinterval.stop_offset < stop_offset_low:
 *                 stop_offset_low = cinterval.stop_offset
 */
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_node->payload, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_cinterval, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":592
 *         for i in range(len(node.payload)):
 *             cinterval = node.payload[i]
 *             if cinterval.stop_offset < stop_offset_low:             # <<<<<<<<<<<<<<
 *                 stop_offset_low = cinterval.stop_offset
 *             if cinterval.stop_offset > stop_offset_high:
 */
    __pyx_t_1 = (__pyx_v_cinterval->stop_offset < __pyx_v_stop_offset_low);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":593
 *             cinterval = node.payload[i]
 *             if cinterval.stop_offset < stop_offset_low:
 *                 stop_offset_low = cinterval.stop_offset             # <<<<<<<<<<<<<<
 *             if cinterval.stop_offset > stop_offset_high:
 *                 stop_offset_high = cinterval.stop_offset
 */
      __pyx_t_7 = __pyx_v_cinterval->stop_offset;
      __pyx_v_stop_offset_low = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":592
 *         for i in range(len(node.payload)):
 *             cinterval = node.payload[i]
 *             if cinterval.stop_offset < stop_offset_low:             # <<<<<<<<<<<<<<
 *                 stop_offset_low = cinterval.stop_offset
 *             if cinterval.stop_offset > stop_offset_high:
 */
    }

    /* "supriya/utils/_intervals.pyx":594
 *             if cinterval.stop_offset < stop_offset_low:
 *                 stop_offset_low = cinterval.stop_offset
 *             if cinterval.stop_offset > stop_offset_high:             # <<<<<<<<<<<<<<
 *                 stop_offset_high = cinterval.stop_offset
 *         if node.left_child is not None:
 */
    __pyx_t_1 = (__pyx_v_cinterval->stop_offset > __pyx_v_stop_offset_high);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":595
 *                 stop_offset_low = cinterval.stop_offset
 *             if cinterval.stop_offset > stop_offset_high:
 *                 stop_offset_high = cinterval.stop_offset             # <<<<<<<<<<<<<<
 *         if node.left_child is not None:
 *             child_node = self._update_offsets(
 */
      __pyx_t_7 = __pyx_v_cinterval->stop_offset;
      __pyx_v_stop_offset_high = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":594
 *             if cinterval.stop_offset < stop_offset_low:
 *                 stop_offset_low = cinterval.stop_offset
 *             if cinterval.stop_offset > stop_offset_high:             # <<<<<<<<<<<<<<
 *                 stop_offset_high = cinterval.stop_offset
 *         if node.left_child is not None:
 */
    }
  }

  /* "supriya/utils/_intervals.pyx":596
 *             if cinterval.stop_offset > stop_offset_high:
 *                 stop_offset_high = cinterval.stop_offset
 *         if node.left_child is not None:             # <<<<<<<<<<<<<<
 *             child_node = self._update_offsets(
 *                 node.left_child,
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node->left_child) != Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":598
 *         if node.left_child is not None:
 *             child_node = self._update_offsets(
 *                 node.left_child,             # <<<<<<<<<<<<<<
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:
 */
    __pyx_t_3 = ((PyObject *)__pyx_v_node->left_child);
    __Pyx_INCREF(__pyx_t_3);

    /* "supriya/utils/_intervals.pyx":597
 *                 stop_offset_high = cinterval.stop_offset
 *         if node.left_child is not None:
 *             child_node = self._update_offsets(             # <<<<<<<<<<<<<<
 *                 node.left_child,
 *                 )
 */
    __pyx_t_8 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_offsets(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 597, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_child_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "supriya/utils/_intervals.pyx":600
 *                 node.left_child,
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:             # <<<<<<<<<<<<<<
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:
 */
    __pyx_t_1 = (__pyx_v_child_node->stop_offset_low < __pyx_v_stop_offset_low);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":601
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:
 *                 stop_offset_low = child_node.stop_offset_low             # <<<<<<<<<<<<<<
 *             if stop_offset_high < child_node.stop_offset_high:
 *                 stop_offset_high = child_node.stop_offset_high
 */
      __pyx_t_7 = __pyx_v_child_node->stop_offset_low;
      __pyx_v_stop_offset_low = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":600
 *                 node.left_child,
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:             # <<<<<<<<<<<<<<
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:
 */
    }

    /* "supriya/utils/_intervals.pyx":602
 *             if child_node.stop_offset_low < stop_offset_low:
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:             # <<<<<<<<<<<<<<
 *                 stop_offset_high = child_node.stop_offset_high
 *         if node.right_child is not None:
 */
    __pyx_t_1 = (__pyx_v_stop_offset_high < __pyx_v_child_node->stop_offset_high);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":603
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:
 *                 stop_offset_high = child_node.stop_offset_high             # <<<<<<<<<<<<<<
 *         if node.right_child is not None:
 *             child_node = self._update_offsets(
 */
      __pyx_t_7 = __pyx_v_child_node->stop_offset_high;
      __pyx_v_stop_offset_high = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":602
 *             if child_node.stop_offset_low < stop_offset_low:
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:             # <<<<<<<<<<<<<<
 *                 stop_offset_high = child_node.stop_offset_high
 *         if node.right_child is not None:
 */
    }

    /* "supriya/utils/_intervals.pyx":596
 *             if cinterval.stop_offset > stop_offset_high:
 *                 stop_offset_high = cinterval.stop_offset
 *         if node.left_child is not None:             # <<<<<<<<<<<<<<
 *             child_node = self._update_offsets(
 *                 node.left_child,
 */
  }

  /* "supriya/utils/_intervals.pyx":604
 *             if stop_offset_high < child_node.stop_offset_high:
 *                 stop_offset_high = child_node.stop_offset_high
 *         if node.right_child is not None:             # <<<<<<<<<<<<<<
 *             child_node = self._update_offsets(
 *                 node.right_child,
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_node->right_child) != Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":606
 *         if node.right_child is not None:
 *             child_node = self._update_offsets(
 *                 node.right_child,             # <<<<<<<<<<<<<<
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:
 */
    __pyx_t_8 = ((PyObject *)__pyx_v_node->right_child);
    __Pyx_INCREF(__pyx_t_8);

    /* "supriya/utils/_intervals.pyx":605
 *                 stop_offset_high = child_node.stop_offset_high
 *         if node.right_child is not None:
 *             child_node = self._update_offsets(             # <<<<<<<<<<<<<<
 *                 node.right_child,
 *                 )
 */
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_offsets(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_8))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF_SET(__pyx_v_child_node, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "supriya/utils/_intervals.pyx":608
 *                 node.right_child,
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:             # <<<<<<<<<<<<<<
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:
 */
    __pyx_t_1 = (__pyx_v_child_node->stop_offset_low < __pyx_v_stop_offset_low);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":609
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:
 *                 stop_offset_low = child_node.stop_offset_low             # <<<<<<<<<<<<<<
 *             if stop_offset_high < child_node.stop_offset_high:
 *                 stop_offset_high = child_node.stop_offset_high
 */
      __pyx_t_7 = __pyx_v_child_node->stop_offset_low;
      __pyx_v_stop_offset_low = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":608
 *                 node.right_child,
 *                 )
 *             if child_node.stop_offset_low < stop_offset_low:             # <<<<<<<<<<<<<<
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:
 */
    }

    /* "supriya/utils/_intervals.pyx":610
 *             if child_node.stop_offset_low < stop_offset_low:
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:             # <<<<<<<<<<<<<<
 *                 stop_offset_high = child_node.stop_offset_high
 *         node.stop_offset_low = stop_offset_low
 */
    __pyx_t_1 = (__pyx_v_stop_offset_high < __pyx_v_child_node->stop_offset_high);
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":611
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:
 *                 stop_offset_high = child_node.stop_offset_high             # <<<<<<<<<<<<<<
 *         node.stop_offset_low = stop_offset_low
 *         node.stop_offset_high = stop_offset_high
 */
      __pyx_t_7 = __pyx_v_child_node->stop_offset_high;
      __pyx_v_stop_offset_high = __pyx_t_7;

      /* "supriya/utils/_intervals.pyx":610
 *             if child_node.stop_offset_low < stop_offset_low:
 *                 stop_offset_low = child_node.stop_offset_low
 *             if stop_offset_high < child_node.stop_offset_high:             # <<<<<<<<<<<<<<
 *                 stop_offset_high = child_node.stop_offset_high
 *         node.stop_offset_low = stop_offset_low
 */
    }

    /* "supriya/utils/_intervals.pyx":604
 *             if stop_offset_high < child_node.stop_offset_high:
 *                 stop_offset_high = child_node.stop_offset_high
 *         if node.right_child is not None:             # <<<<<<<<<<<<<<
 *             child_node = self._update_offsets(
 *                 node.right_child,
 */
  }

  /* "supriya/utils/_intervals.pyx":612
 *             if stop_offset_high < child_node.stop_offset_high:
 *                 stop_offset_high = child_node.stop_offset_high
 *         node.stop_offset_low = stop_offset_low             # <<<<<<<<<<<<<<
 *         node.stop_offset_high = stop_offset_high
 *         return node
 */
  __pyx_v_node->stop_offset_low = __pyx_v_stop_offset_low;

  /* "supriya/utils/_intervals.pyx":613
 *                 stop_offset_high = child_node.stop_offset_high
 *         node.stop_offset_low = stop_offset_low
 *         node.stop_offset_high = stop_offset_high             # <<<<<<<<<<<<<<
 *         return node
 * 
 */
  __pyx_v_node->stop_offset_high = __pyx_v_stop_offset_high;

  /* "supriya/utils/_intervals.pyx":614
 *         node.stop_offset_low = stop_offset_low
 *         node.stop_offset_high = stop_offset_high
 *         return node             # <<<<<<<<<<<<<<
 * 
 *     ### PUBLIC METHODS ###
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_node);
  __pyx_r = __pyx_v_node;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":579
 *             node.subtree_stop_index = node.right_child.subtree_stop_index
 * 
 *     cdef _CNode _update_offsets(             # <<<<<<<<<<<<<<
 *         self,
 *         _CNode node,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx._update_offsets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child_node);
  __Pyx_XDECREF((PyObject *)__pyx_v_cinterval);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":618
 *     ### PUBLIC METHODS ###
 * 
 *     def add(self, interval):             # <<<<<<<<<<<<<<
 *         if not self._is_interval(interval):
 *             # raise ValueError(interval)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_14add(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_14add = {"add", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_14add, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_14add(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_interval = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_interval,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_interval)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 618, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "add") < 0)) __PYX_ERR(0, 618, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_interval = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 618, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.add", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_13add(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_interval);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_13add(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval) {
  PyObject *__pyx_v_cinterval = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add", 1);

  /* "supriya/utils/_intervals.pyx":619
 * 
 *     def add(self, interval):
 *         if not self._is_interval(interval):             # <<<<<<<<<<<<<<
 *             # raise ValueError(interval)
 *             return
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (__pyx_t_6) {

    /* "supriya/utils/_intervals.pyx":621
 *         if not self._is_interval(interval):
 *             # raise ValueError(interval)
 *             return             # <<<<<<<<<<<<<<
 *         cinterval = _CInterval.from_interval(interval)
 *         self._insert_interval(cinterval)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":619
 * 
 *     def add(self, interval):
 *         if not self._is_interval(interval):             # <<<<<<<<<<<<<<
 *             # raise ValueError(interval)
 *             return
 */
  }

  /* "supriya/utils/_intervals.pyx":622
 *             # raise ValueError(interval)
 *             return
 *         cinterval = _CInterval.from_interval(interval)             # <<<<<<<<<<<<<<
 *         self._insert_interval(cinterval)
 *         self._update_indices(self._root_node, -1)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval), __pyx_n_s_from_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 622, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 622, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_cinterval = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":623
 *             return
 *         cinterval = _CInterval.from_interval(interval)
 *         self._insert_interval(cinterval)             # <<<<<<<<<<<<<<
 *         self._update_indices(self._root_node, -1)
 *         self._update_offsets(self._root_node)
 */
  if (!(likely(((__pyx_v_cinterval) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_cinterval, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 623, __pyx_L1_error)
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_insert_interval(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_cinterval)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 623, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":624
 *         cinterval = _CInterval.from_interval(interval)
 *         self._insert_interval(cinterval)
 *         self._update_indices(self._root_node, -1)             # <<<<<<<<<<<<<<
 *         self._update_offsets(self._root_node)
 * 
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 624, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_indices(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), -1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":625
 *         self._insert_interval(cinterval)
 *         self._update_indices(self._root_node, -1)
 *         self._update_offsets(self._root_node)             # <<<<<<<<<<<<<<
 * 
 *     def find_intervals_starting_at(self, offset):
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 625, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_offsets(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":618
 *     ### PUBLIC METHODS ###
 * 
 *     def add(self, interval):             # <<<<<<<<<<<<<<
 *         if not self._is_interval(interval):
 *             # raise ValueError(interval)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.add", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cinterval);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":627
 *         self._update_offsets(self._root_node)
 * 
 *     def find_intervals_starting_at(self, offset):             # <<<<<<<<<<<<<<
 *         results = []
 *         node = self._search(self._root_node, offset)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16find_intervals_starting_at(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16find_intervals_starting_at = {"find_intervals_starting_at", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16find_intervals_starting_at, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16find_intervals_starting_at(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_offset = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("find_intervals_starting_at (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_offset,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 627, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "find_intervals_starting_at") < 0)) __PYX_ERR(0, 627, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_offset = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("find_intervals_starting_at", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 627, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_starting_at", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_15find_intervals_starting_at(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_offset);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_15find_intervals_starting_at(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset) {
  PyObject *__pyx_v_results = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_intervals_starting_at", 1);

  /* "supriya/utils/_intervals.pyx":628
 * 
 *     def find_intervals_starting_at(self, offset):
 *         results = []             # <<<<<<<<<<<<<<
 *         node = self._search(self._root_node, offset)
 *         if node is None:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_results = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":629
 *     def find_intervals_starting_at(self, offset):
 *         results = []
 *         node = self._search(self._root_node, offset)             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return results
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 629, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 629, __pyx_L1_error)
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_search(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_t_2)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":630
 *         results = []
 *         node = self._search(self._root_node, offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return results
 *         return self._unbox_cintervals(node.payload)
 */
  __pyx_t_4 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_4) {

    /* "supriya/utils/_intervals.pyx":631
 *         node = self._search(self._root_node, offset)
 *         if node is None:
 *             return results             # <<<<<<<<<<<<<<
 *         return self._unbox_cintervals(node.payload)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_results);
    __pyx_r = __pyx_v_results;
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":630
 *         results = []
 *         node = self._search(self._root_node, offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return results
 *         return self._unbox_cintervals(node.payload)
 */
  }

  /* "supriya/utils/_intervals.pyx":632
 *         if node is None:
 *             return results
 *         return self._unbox_cintervals(node.payload)             # <<<<<<<<<<<<<<
 * 
 *     def find_intervals_stopping_at(self, offset):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_v_node->payload;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_1 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_unbox_cintervals(__pyx_v_self, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 632, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":627
 *         self._update_offsets(self._root_node)
 * 
 *     def find_intervals_starting_at(self, offset):             # <<<<<<<<<<<<<<
 *         results = []
 *         node = self._search(self._root_node, offset)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_starting_at", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":634
 *         return self._unbox_cintervals(node.payload)
 * 
 *     def find_intervals_stopping_at(self, offset):             # <<<<<<<<<<<<<<
 *         cintervals = self._recurse_find_intervals_stopping_at(
 *             self._root_node, offset)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_18find_intervals_stopping_at(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_18find_intervals_stopping_at = {"find_intervals_stopping_at", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_18find_intervals_stopping_at, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_18find_intervals_stopping_at(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_offset = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("find_intervals_stopping_at (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_offset,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 634, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "find_intervals_stopping_at") < 0)) __PYX_ERR(0, 634, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_offset = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("find_intervals_stopping_at", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 634, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_stopping_at", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17find_intervals_stopping_at(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_offset);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_17find_intervals_stopping_at(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset) {
  PyObject *__pyx_v_cintervals = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_intervals_stopping_at", 1);

  /* "supriya/utils/_intervals.pyx":636
 *     def find_intervals_stopping_at(self, offset):
 *         cintervals = self._recurse_find_intervals_stopping_at(
 *             self._root_node, offset)             # <<<<<<<<<<<<<<
 *         return self._unbox_cintervals(cintervals)
 * 
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 636, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 636, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":635
 * 
 *     def find_intervals_stopping_at(self, offset):
 *         cintervals = self._recurse_find_intervals_stopping_at(             # <<<<<<<<<<<<<<
 *             self._root_node, offset)
 *         return self._unbox_cintervals(cintervals)
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_stopping_at(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_cintervals = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":637
 *         cintervals = self._recurse_find_intervals_stopping_at(
 *             self._root_node, offset)
 *         return self._unbox_cintervals(cintervals)             # <<<<<<<<<<<<<<
 * 
 *     def find_intervals_intersecting_offset(self, offset):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_unbox_cintervals(__pyx_v_self, __pyx_v_cintervals); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":634
 *         return self._unbox_cintervals(node.payload)
 * 
 *     def find_intervals_stopping_at(self, offset):             # <<<<<<<<<<<<<<
 *         cintervals = self._recurse_find_intervals_stopping_at(
 *             self._root_node, offset)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_stopping_at", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cintervals);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":639
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_offset(self, offset):             # <<<<<<<<<<<<<<
 *         offset = float(offset)
 *         cintervals = self._recurse_find_intervals_intersecting_offset(
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_20find_intervals_intersecting_offset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_20find_intervals_intersecting_offset = {"find_intervals_intersecting_offset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_20find_intervals_intersecting_offset, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_20find_intervals_intersecting_offset(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_offset = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("find_intervals_intersecting_offset (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_offset,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 639, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "find_intervals_intersecting_offset") < 0)) __PYX_ERR(0, 639, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_offset = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("find_intervals_intersecting_offset", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 639, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_intersecting_offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_19find_intervals_intersecting_offset(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_offset);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_19find_intervals_intersecting_offset(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset) {
  PyObject *__pyx_v_cintervals = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_intervals_intersecting_offset", 0);
  __Pyx_INCREF(__pyx_v_offset);

  /* "supriya/utils/_intervals.pyx":640
 * 
 *     def find_intervals_intersecting_offset(self, offset):
 *         offset = float(offset)             # <<<<<<<<<<<<<<
 *         cintervals = self._recurse_find_intervals_intersecting_offset(
 *             self._root_node, offset)
 */
  __pyx_t_1 = __Pyx_PyNumber_Float(__pyx_v_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_offset, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":642
 *         offset = float(offset)
 *         cintervals = self._recurse_find_intervals_intersecting_offset(
 *             self._root_node, offset)             # <<<<<<<<<<<<<<
 *         return self._unbox_cintervals(cintervals)
 * 
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 642, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 642, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":641
 *     def find_intervals_intersecting_offset(self, offset):
 *         offset = float(offset)
 *         cintervals = self._recurse_find_intervals_intersecting_offset(             # <<<<<<<<<<<<<<
 *             self._root_node, offset)
 *         return self._unbox_cintervals(cintervals)
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_offset(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_cintervals = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":643
 *         cintervals = self._recurse_find_intervals_intersecting_offset(
 *             self._root_node, offset)
 *         return self._unbox_cintervals(cintervals)             # <<<<<<<<<<<<<<
 * 
 *     def find_intervals_intersecting_interval(self, interval):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_unbox_cintervals(__pyx_v_self, __pyx_v_cintervals); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":639
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_offset(self, offset):             # <<<<<<<<<<<<<<
 *         offset = float(offset)
 *         cintervals = self._recurse_find_intervals_intersecting_offset(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_intersecting_offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cintervals);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":645
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_interval(self, interval):             # <<<<<<<<<<<<<<
 *         cinterval = _CInterval.from_interval(interval)
 *         assert isinstance(cinterval, _CInterval)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_22find_intervals_intersecting_interval(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_22find_intervals_intersecting_interval = {"find_intervals_intersecting_interval", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_22find_intervals_intersecting_interval, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_22find_intervals_intersecting_interval(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_interval = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("find_intervals_intersecting_interval (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_interval,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_interval)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 645, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "find_intervals_intersecting_interval") < 0)) __PYX_ERR(0, 645, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_interval = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("find_intervals_intersecting_interval", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 645, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_intersecting_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_21find_intervals_intersecting_interval(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_interval);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_21find_intervals_intersecting_interval(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval) {
  PyObject *__pyx_v_cinterval = NULL;
  PyObject *__pyx_v_cintervals = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("find_intervals_intersecting_interval", 1);

  /* "supriya/utils/_intervals.pyx":646
 * 
 *     def find_intervals_intersecting_interval(self, interval):
 *         cinterval = _CInterval.from_interval(interval)             # <<<<<<<<<<<<<<
 *         assert isinstance(cinterval, _CInterval)
 *         cintervals = self._recurse_find_intervals_intersecting_interval(
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval), __pyx_n_s_from_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 646, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_cinterval = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":647
 *     def find_intervals_intersecting_interval(self, interval):
 *         cinterval = _CInterval.from_interval(interval)
 *         assert isinstance(cinterval, _CInterval)             # <<<<<<<<<<<<<<
 *         cintervals = self._recurse_find_intervals_intersecting_interval(
 *             self._root_node, cinterval)
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_cinterval, __pyx_ptype_7supriya_5utils_10_intervals__CInterval); 
    if (unlikely(!__pyx_t_5)) {
      __Pyx_Raise(__pyx_builtin_AssertionError, 0, 0, 0);
      __PYX_ERR(0, 647, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 647, __pyx_L1_error)
  #endif

  /* "supriya/utils/_intervals.pyx":649
 *         assert isinstance(cinterval, _CInterval)
 *         cintervals = self._recurse_find_intervals_intersecting_interval(
 *             self._root_node, cinterval)             # <<<<<<<<<<<<<<
 *         return self._unbox_cintervals(cintervals)
 * 
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 649, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_v_cinterval) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_cinterval, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 649, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":648
 *         cinterval = _CInterval.from_interval(interval)
 *         assert isinstance(cinterval, _CInterval)
 *         cintervals = self._recurse_find_intervals_intersecting_interval(             # <<<<<<<<<<<<<<
 *             self._root_node, cinterval)
 *         return self._unbox_cintervals(cintervals)
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_find_intervals_intersecting_interval(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_cinterval)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_cintervals = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":650
 *         cintervals = self._recurse_find_intervals_intersecting_interval(
 *             self._root_node, cinterval)
 *         return self._unbox_cintervals(cintervals)             # <<<<<<<<<<<<<<
 * 
 *     def get_offset_after(self, offset):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_unbox_cintervals(__pyx_v_self, __pyx_v_cintervals); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 650, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":645
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_interval(self, interval):             # <<<<<<<<<<<<<<
 *         cinterval = _CInterval.from_interval(interval)
 *         assert isinstance(cinterval, _CInterval)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.find_intervals_intersecting_interval", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cinterval);
  __Pyx_XDECREF(__pyx_v_cintervals);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":652
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def get_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         # print("Searching for: {}".format(offset))
 *         result = None
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_24get_offset_after(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_24get_offset_after = {"get_offset_after", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_24get_offset_after, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_24get_offset_after(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_offset = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_offset_after (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_offset,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 652, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "get_offset_after") < 0)) __PYX_ERR(0, 652, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_offset = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_offset_after", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 652, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.get_offset_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_23get_offset_after(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_offset);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_23get_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  float __pyx_t_3;
  float __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_offset_after", 1);

  /* "supriya/utils/_intervals.pyx":654
 *     def get_offset_after(self, offset):
 *         # print("Searching for: {}".format(offset))
 *         result = None             # <<<<<<<<<<<<<<
 *         if self._root_node is not None:
 *             result = self._recurse_get_offset_after(self._root_node, offset)
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_result = Py_None;

  /* "supriya/utils/_intervals.pyx":655
 *         # print("Searching for: {}".format(offset))
 *         result = None
 *         if self._root_node is not None:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_offset_after(self._root_node, offset)
 *             if result == offset:
 */
  __pyx_t_1 = (__pyx_v_self->_root_node != Py_None);
  if (__pyx_t_1) {

    /* "supriya/utils/_intervals.pyx":656
 *         result = None
 *         if self._root_node is not None:
 *             result = self._recurse_get_offset_after(self._root_node, offset)             # <<<<<<<<<<<<<<
 *             if result == offset:
 *                 result = None
 */
    if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 656, __pyx_L1_error)
    __pyx_t_2 = __pyx_v_self->_root_node;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_offset); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 656, __pyx_L1_error)
    __pyx_t_4 = ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_offset_after(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), __pyx_t_3, NULL); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 656, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyFloat_FromDouble(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "supriya/utils/_intervals.pyx":657
 *         if self._root_node is not None:
 *             result = self._recurse_get_offset_after(self._root_node, offset)
 *             if result == offset:             # <<<<<<<<<<<<<<
 *                 result = None
 *         return result
 */
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_result, __pyx_v_offset, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 657, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 657, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "supriya/utils/_intervals.pyx":658
 *             result = self._recurse_get_offset_after(self._root_node, offset)
 *             if result == offset:
 *                 result = None             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
      __Pyx_INCREF(Py_None);
      __Pyx_DECREF_SET(__pyx_v_result, Py_None);

      /* "supriya/utils/_intervals.pyx":657
 *         if self._root_node is not None:
 *             result = self._recurse_get_offset_after(self._root_node, offset)
 *             if result == offset:             # <<<<<<<<<<<<<<
 *                 result = None
 *         return result
 */
    }

    /* "supriya/utils/_intervals.pyx":655
 *         # print("Searching for: {}".format(offset))
 *         result = None
 *         if self._root_node is not None:             # <<<<<<<<<<<<<<
 *             result = self._recurse_get_offset_after(self._root_node, offset)
 *             if result == offset:
 */
  }

  /* "supriya/utils/_intervals.pyx":659
 *             if result == offset:
 *                 result = None
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def get_start_offset_after(self, offset):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":652
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def get_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         # print("Searching for: {}".format(offset))
 *         result = None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.get_offset_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":661
 *         return result
 * 
 *     def get_start_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)
 *         if node is None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_26get_start_offset_after(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_26get_start_offset_after = {"get_start_offset_after", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_26get_start_offset_after, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_26get_start_offset_after(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_offset = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_start_offset_after (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_offset,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 661, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "get_start_offset_after") < 0)) __PYX_ERR(0, 661, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_offset = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_start_offset_after", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 661, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.get_start_offset_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_25get_start_offset_after(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_offset);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_25get_start_offset_after(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_start_offset_after", 1);

  /* "supriya/utils/_intervals.pyx":662
 * 
 *     def get_start_offset_after(self, offset):
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return None
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 662, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 662, __pyx_L1_error)
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_start_offset_after(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_t_2)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 662, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":663
 *     def get_start_offset_after(self, offset):
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         return node.start_offset
 */
  __pyx_t_4 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_4) {

    /* "supriya/utils/_intervals.pyx":664
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)
 *         if node is None:
 *             return None             # <<<<<<<<<<<<<<
 *         return node.start_offset
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":663
 *     def get_start_offset_after(self, offset):
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         return node.start_offset
 */
  }

  /* "supriya/utils/_intervals.pyx":665
 *         if node is None:
 *             return None
 *         return node.start_offset             # <<<<<<<<<<<<<<
 * 
 *     def get_start_offset_before(self, offset):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_node->start_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":661
 *         return result
 * 
 *     def get_start_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)
 *         if node is None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.get_start_offset_after", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":667
 *         return node.start_offset
 * 
 *     def get_start_offset_before(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)
 *         if node is None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_28get_start_offset_before(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_28get_start_offset_before = {"get_start_offset_before", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_28get_start_offset_before, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_28get_start_offset_before(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_offset = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_start_offset_before (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_offset,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_offset)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 667, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "get_start_offset_before") < 0)) __PYX_ERR(0, 667, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_offset = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_start_offset_before", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 667, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.get_start_offset_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_27get_start_offset_before(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_offset);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_27get_start_offset_before(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_offset) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_start_offset_before", 1);

  /* "supriya/utils/_intervals.pyx":668
 * 
 *     def get_start_offset_before(self, offset):
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)             # <<<<<<<<<<<<<<
 *         if node is None:
 *             return None
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 668, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_offset); if (unlikely((__pyx_t_2 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 668, __pyx_L1_error)
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_recurse_get_start_offset_before(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_t_2)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":669
 *     def get_start_offset_before(self, offset):
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         return node.start_offset
 */
  __pyx_t_4 = (((PyObject *)__pyx_v_node) == Py_None);
  if (__pyx_t_4) {

    /* "supriya/utils/_intervals.pyx":670
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)
 *         if node is None:
 *             return None             # <<<<<<<<<<<<<<
 *         return node.start_offset
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "supriya/utils/_intervals.pyx":669
 *     def get_start_offset_before(self, offset):
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             return None
 *         return node.start_offset
 */
  }

  /* "supriya/utils/_intervals.pyx":671
 *         if node is None:
 *             return None
 *         return node.start_offset             # <<<<<<<<<<<<<<
 * 
 *     def index(self, interval):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_node->start_offset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":667
 *         return node.start_offset
 * 
 *     def get_start_offset_before(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)
 *         if node is None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.get_start_offset_before", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":673
 *         return node.start_offset
 * 
 *     def index(self, interval):             # <<<<<<<<<<<<<<
 *         assert self._is_interval(interval)
 *         cinterval = _CInterval.from_interval(interval)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_30index(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_30index = {"index", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_30index, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_30index(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_interval = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("index (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_interval,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_interval)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 673, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "index") < 0)) __PYX_ERR(0, 673, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_interval = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("index", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 673, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_29index(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_interval);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_29index(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval) {
  PyObject *__pyx_v_cinterval = NULL;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v_node = NULL;
  PyObject *__pyx_v_index = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_t_5;
  float __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("index", 1);

  /* "supriya/utils/_intervals.pyx":674
 * 
 *     def index(self, interval):
 *         assert self._is_interval(interval)             # <<<<<<<<<<<<<<
 *         cinterval = _CInterval.from_interval(interval)
 *         node = self._search(self._root_node, cinterval.start_offset)
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    __pyx_t_4 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_4 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) {
      __Pyx_Raise(__pyx_builtin_AssertionError, 0, 0, 0);
      __PYX_ERR(0, 674, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(0, 674, __pyx_L1_error)
  #endif

  /* "supriya/utils/_intervals.pyx":675
 *     def index(self, interval):
 *         assert self._is_interval(interval)
 *         cinterval = _CInterval.from_interval(interval)             # <<<<<<<<<<<<<<
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         if node is None:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval), __pyx_n_s_from_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 675, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_v_cinterval = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":676
 *         assert self._is_interval(interval)
 *         cinterval = _CInterval.from_interval(interval)
 *         node = self._search(self._root_node, cinterval.start_offset)             # <<<<<<<<<<<<<<
 *         if node is None:
 *             raise ValueError('{} not in interval tree.'.format(interval))
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 676, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_cinterval, __pyx_n_s_start_offset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 676, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 676, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_search(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), __pyx_t_6)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 676, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_node = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":677
 *         cinterval = _CInterval.from_interval(interval)
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         if cinterval not in node.payload:
 */
  __pyx_t_5 = (((PyObject *)__pyx_v_node) == Py_None);
  if (unlikely(__pyx_t_5)) {

    /* "supriya/utils/_intervals.pyx":678
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         if node is None:
 *             raise ValueError('{} not in interval tree.'.format(interval))             # <<<<<<<<<<<<<<
 *         if cinterval not in node.payload:
 *             raise ValueError('{} not in interval tree.'.format(interval))
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_not_in_interval_tree, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_4 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
      __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 678, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 678, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 678, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":677
 *         cinterval = _CInterval.from_interval(interval)
 *         node = self._search(self._root_node, cinterval.start_offset)
 *         if node is None:             # <<<<<<<<<<<<<<
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         if cinterval not in node.payload:
 */
  }

  /* "supriya/utils/_intervals.pyx":679
 *         if node is None:
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         if cinterval not in node.payload:             # <<<<<<<<<<<<<<
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         index = node.payload.index(cinterval) + node.node_start_index
 */
  __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_v_cinterval, __pyx_v_node->payload, Py_NE)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 679, __pyx_L1_error)
  if (unlikely(__pyx_t_5)) {

    /* "supriya/utils/_intervals.pyx":680
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         if cinterval not in node.payload:
 *             raise ValueError('{} not in interval tree.'.format(interval))             # <<<<<<<<<<<<<<
 *         index = node.payload.index(cinterval) + node.node_start_index
 *         return index
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_not_in_interval_tree, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 680, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    __pyx_t_4 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_4 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_interval};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 680, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 680, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 680, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":679
 *         if node is None:
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         if cinterval not in node.payload:             # <<<<<<<<<<<<<<
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         index = node.payload.index(cinterval) + node.node_start_index
 */
  }

  /* "supriya/utils/_intervals.pyx":681
 *         if cinterval not in node.payload:
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         index = node.payload.index(cinterval) + node.node_start_index             # <<<<<<<<<<<<<<
 *         return index
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node->payload, __pyx_n_s_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_cinterval};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_node->node_start_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_index = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":682
 *             raise ValueError('{} not in interval tree.'.format(interval))
 *         index = node.payload.index(cinterval) + node.node_start_index
 *         return index             # <<<<<<<<<<<<<<
 * 
 *     def remove(self, interval):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_index);
  __pyx_r = __pyx_v_index;
  goto __pyx_L0;

  /* "supriya/utils/_intervals.pyx":673
 *         return node.start_offset
 * 
 *     def index(self, interval):             # <<<<<<<<<<<<<<
 *         assert self._is_interval(interval)
 *         cinterval = _CInterval.from_interval(interval)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cinterval);
  __Pyx_XDECREF((PyObject *)__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":684
 *         return index
 * 
 *     def remove(self, interval):             # <<<<<<<<<<<<<<
 *         if interval not in self:
 *             raise ValueError(interval)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_32remove(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_32remove = {"remove", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_32remove, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_32remove(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_interval = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("remove (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_interval,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_interval)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 684, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "remove") < 0)) __PYX_ERR(0, 684, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_interval = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("remove", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 684, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.remove", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_31remove(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_interval);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_31remove(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_interval) {
  PyObject *__pyx_v_cinterval = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("remove", 1);

  /* "supriya/utils/_intervals.pyx":685
 * 
 *     def remove(self, interval):
 *         if interval not in self:             # <<<<<<<<<<<<<<
 *             raise ValueError(interval)
 *         cinterval = _CInterval.from_interval(interval)
 */
  __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_v_interval, ((PyObject *)__pyx_v_self), Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 685, __pyx_L1_error)
  if (unlikely(__pyx_t_1)) {

    /* "supriya/utils/_intervals.pyx":686
 *     def remove(self, interval):
 *         if interval not in self:
 *             raise ValueError(interval)             # <<<<<<<<<<<<<<
 *         cinterval = _CInterval.from_interval(interval)
 *         self._remove_interval(cinterval)
 */
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 686, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":685
 * 
 *     def remove(self, interval):
 *         if interval not in self:             # <<<<<<<<<<<<<<
 *             raise ValueError(interval)
 *         cinterval = _CInterval.from_interval(interval)
 */
  }

  /* "supriya/utils/_intervals.pyx":687
 *         if interval not in self:
 *             raise ValueError(interval)
 *         cinterval = _CInterval.from_interval(interval)             # <<<<<<<<<<<<<<
 *         self._remove_interval(cinterval)
 *         self._update_indices(self._root_node, -1)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval), __pyx_n_s_from_interval); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_interval};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_v_cinterval = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":688
 *             raise ValueError(interval)
 *         cinterval = _CInterval.from_interval(interval)
 *         self._remove_interval(cinterval)             # <<<<<<<<<<<<<<
 *         self._update_indices(self._root_node, -1)
 *         self._update_offsets(self._root_node)
 */
  if (!(likely(((__pyx_v_cinterval) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_cinterval, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 688, __pyx_L1_error)
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_remove_interval(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_cinterval)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 688, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":689
 *         cinterval = _CInterval.from_interval(interval)
 *         self._remove_interval(cinterval)
 *         self._update_indices(self._root_node, -1)             # <<<<<<<<<<<<<<
 *         self._update_offsets(self._root_node)
 * 
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 689, __pyx_L1_error)
  __pyx_t_2 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_indices(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2), -1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":690
 *         self._remove_interval(cinterval)
 *         self._update_indices(self._root_node, -1)
 *         self._update_offsets(self._root_node)             # <<<<<<<<<<<<<<
 * 
 *     def update(self, intervals):
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 690, __pyx_L1_error)
  __pyx_t_2 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_offsets(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_2))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 690, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "supriya/utils/_intervals.pyx":684
 *         return index
 * 
 *     def remove(self, interval):             # <<<<<<<<<<<<<<
 *         if interval not in self:
 *             raise ValueError(interval)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.remove", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cinterval);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":692
 *         self._update_offsets(self._root_node)
 * 
 *     def update(self, intervals):             # <<<<<<<<<<<<<<
 *         # for interval in intervals:
 *         #     if not self._is_interval(interval):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_34update(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_34update = {"update", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_34update, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_34update(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_intervals = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("update (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_intervals,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_intervals)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 692, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "update") < 0)) __PYX_ERR(0, 692, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_intervals = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("update", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 692, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_33update(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v_intervals);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_33update(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_intervals) {
  PyObject *__pyx_v_interval = NULL;
  PyObject *__pyx_v_cinterval = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("update", 1);

  /* "supriya/utils/_intervals.pyx":696
 *         #     if not self._is_interval(interval):
 *         #         raise ValueError(interval)
 *         for interval in intervals:             # <<<<<<<<<<<<<<
 *             if not self._is_interval(interval):
 *                 # raise ValueError(interval)
 */
  if (likely(PyList_CheckExact(__pyx_v_intervals)) || PyTuple_CheckExact(__pyx_v_intervals)) {
    __pyx_t_1 = __pyx_v_intervals; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_intervals); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 696, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 696, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 696, __pyx_L1_error)
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 696, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 696, __pyx_L1_error)
          #endif
          if (__pyx_t_2 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(0, 696, __pyx_L1_error)
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 696, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 696, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_interval, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":697
 *         #         raise ValueError(interval)
 *         for interval in intervals:
 *             if not self._is_interval(interval):             # <<<<<<<<<<<<<<
 *                 # raise ValueError(interval)
 *                 continue
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_interval); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_interval};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 697, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = (!__pyx_t_8);
    if (__pyx_t_9) {

      /* "supriya/utils/_intervals.pyx":699
 *             if not self._is_interval(interval):
 *                 # raise ValueError(interval)
 *                 continue             # <<<<<<<<<<<<<<
 *             cinterval = _CInterval.from_interval(interval)
 *             self._insert_interval(cinterval)
 */
      goto __pyx_L3_continue;

      /* "supriya/utils/_intervals.pyx":697
 *         #         raise ValueError(interval)
 *         for interval in intervals:
 *             if not self._is_interval(interval):             # <<<<<<<<<<<<<<
 *                 # raise ValueError(interval)
 *                 continue
 */
    }

    /* "supriya/utils/_intervals.pyx":700
 *                 # raise ValueError(interval)
 *                 continue
 *             cinterval = _CInterval.from_interval(interval)             # <<<<<<<<<<<<<<
 *             self._insert_interval(cinterval)
 *         self._update_indices(self._root_node, -1)
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval), __pyx_n_s_from_interval); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_interval};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_cinterval, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "supriya/utils/_intervals.pyx":701
 *                 continue
 *             cinterval = _CInterval.from_interval(interval)
 *             self._insert_interval(cinterval)             # <<<<<<<<<<<<<<
 *         self._update_indices(self._root_node, -1)
 *         self._update_offsets(self._root_node)
 */
    if (!(likely(((__pyx_v_cinterval) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_cinterval, __pyx_ptype_7supriya_5utils_10_intervals__CInterval))))) __PYX_ERR(0, 701, __pyx_L1_error)
    ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_insert_interval(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)__pyx_v_cinterval)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 701, __pyx_L1_error)

    /* "supriya/utils/_intervals.pyx":696
 *         #     if not self._is_interval(interval):
 *         #         raise ValueError(interval)
 *         for interval in intervals:             # <<<<<<<<<<<<<<
 *             if not self._is_interval(interval):
 *                 # raise ValueError(interval)
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":702
 *             cinterval = _CInterval.from_interval(interval)
 *             self._insert_interval(cinterval)
 *         self._update_indices(self._root_node, -1)             # <<<<<<<<<<<<<<
 *         self._update_offsets(self._root_node)
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 702, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  ((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_indices(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1), -1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 702, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "supriya/utils/_intervals.pyx":703
 *             self._insert_interval(cinterval)
 *         self._update_indices(self._root_node, -1)
 *         self._update_offsets(self._root_node)             # <<<<<<<<<<<<<<
 */
  if (!(likely(((__pyx_v_self->_root_node) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_self->_root_node, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(0, 703, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_self->_root_node;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self->__pyx_vtab)->_update_offsets(__pyx_v_self, ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "supriya/utils/_intervals.pyx":692
 *         self._update_offsets(self._root_node)
 * 
 *     def update(self, intervals):             # <<<<<<<<<<<<<<
 *         # for interval in intervals:
 *         #     if not self._is_interval(interval):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.update", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_interval);
  __Pyx_XDECREF(__pyx_v_cinterval);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "supriya/utils/_intervals.pyx":92
 *     ### CLASS VARIABLES ###
 * 
 *     cdef public object _root_node             # <<<<<<<<<<<<<<
 * 
 *     ### INITIALIZER ###
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node___get__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node___get__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 1);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_root_node);
  __pyx_r = __pyx_v_self->_root_node;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_2__set__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_2__set__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 1);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_root_node);
  __Pyx_DECREF(__pyx_v_self->_root_node);
  __pyx_v_self->_root_node = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_4__del__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_4__del__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_root_node);
  __Pyx_DECREF(__pyx_v_self->_root_node);
  __pyx_v_self->_root_node = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_36__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_36__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_36__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_36__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) {
    __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL;}
  if (unlikely(__pyx_kwds) && __Pyx_NumKwargs_FASTCALL(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__reduce_cython__", 0))) return NULL;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_35__reduce_cython__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_35__reduce_cython__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 1);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._root_node,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->_root_node);
  __Pyx_GIVEREF(__pyx_v_self->_root_node);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->_root_node)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._root_node,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self._root_node,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._root_node is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._root_node,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._root_node is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, None), state
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->_root_node != Py_None);
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._root_node is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, None), state
 *     else:
 */
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._root_node is not None
 *     if use_setstate:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_IntervalTreeDrive); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))))) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_int_8917134);
    __Pyx_GIVEREF(__pyx_int_8917134);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_8917134)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._root_node is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, None), state
 *     else:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IntervalTreeDriverEx__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_IntervalTreeDrive); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))))) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_int_8917134);
    __Pyx_GIVEREF(__pyx_int_8917134);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_8917134)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IntervalTreeDriverEx__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_38__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_38__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_38__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_38__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 16, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__setstate_cython__") < 0)) __PYX_ERR(1, 16, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_37__setstate_cython__(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_37__setstate_cython__(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 1);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_IntervalTreeDriverEx__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_7supriya_5utils_10_intervals___pyx_unpickle_IntervalTreeDriverEx__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IntervalTreeDriverEx__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("supriya.utils._intervals.IntervalTreeDriverEx.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle__CNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_1__pyx_unpickle__CNode(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_1__pyx_unpickle__CNode = {"__pyx_unpickle__CNode", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_1__pyx_unpickle__CNode, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_1__pyx_unpickle__CNode(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle__CNode (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_type)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_checksum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__CNode", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__CNode", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__pyx_unpickle__CNode") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__CNode", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.__pyx_unpickle__CNode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals___pyx_unpickle__CNode(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals___pyx_unpickle__CNode(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__CNode", 1);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe8cb558, 0x8efd0e5, 0x9b91c78):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple_, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe8cb558, 0x8efd0e5, 0x9b91c78):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum
 *     __pyx_result = _CNode.__new__(__pyx_type)
 */
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError)) __PYX_ERR(1, 5, __pyx_L1_error);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xe8cb558, 0x8efd0e5, 0x9b91c78):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = _CNode.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_1, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe8cb558, 0x8efd0e5, 0x9b91c78):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum
 *     __pyx_result = _CNode.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CNode), __pyx_n_s_new); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v___pyx_type};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_v___pyx_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum
 *     __pyx_result = _CNode.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_2 = (__pyx_v___pyx_state != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = _CNode.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_7supriya_5utils_10_intervals___pyx_unpickle__CNode__set_state(((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum
 *     __pyx_result = _CNode.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle__CNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.__pyx_unpickle__CNode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_7supriya_5utils_10_intervals___pyx_unpickle__CNode__set_state(struct __pyx_obj_7supriya_5utils_10_intervals__CNode *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  float __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__CNode__set_state", 1);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[12])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->balance = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->height = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->left_child);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->left_child);
  __pyx_v___pyx_result->left_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->node_start_index = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->node_stop_index = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->payload);
  __Pyx_DECREF(__pyx_v___pyx_result->payload);
  __pyx_v___pyx_result->payload = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7supriya_5utils_10_intervals__CNode))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->right_child);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->right_child);
  __pyx_v___pyx_result->right_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->start_offset = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->stop_offset_high = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->stop_offset_low = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->subtree_start_index = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->subtree_stop_index = __pyx_t_2;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[12])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 > 12);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":14
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[12])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[12])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("supriya.utils._intervals.__pyx_unpickle__CNode__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_IntervalTreeDriverEx(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_3__pyx_unpickle_IntervalTreeDriverEx(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7supriya_5utils_10_intervals_3__pyx_unpickle_IntervalTreeDriverEx = {"__pyx_unpickle_IntervalTreeDriverEx", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_3__pyx_unpickle_IntervalTreeDriverEx, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7supriya_5utils_10_intervals_3__pyx_unpickle_IntervalTreeDriverEx(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_IntervalTreeDriverEx (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_type)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_checksum)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IntervalTreeDriverEx", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pyx_state)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IntervalTreeDriverEx", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "__pyx_unpickle_IntervalTreeDriverEx") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
      values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_IntervalTreeDriverEx", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("supriya.utils._intervals.__pyx_unpickle_IntervalTreeDriverEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7supriya_5utils_10_intervals_2__pyx_unpickle_IntervalTreeDriverEx(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7supriya_5utils_10_intervals_2__pyx_unpickle_IntervalTreeDriverEx(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  unsigned int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IntervalTreeDriverEx", 1);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x088108e, 0xeddf68f, 0xa0b8d2d):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))" % __pyx_checksum
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__3, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x088108e, 0xeddf68f, 0xa0b8d2d):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))" % __pyx_checksum
 *     __pyx_result = IntervalTreeDriverEx.__new__(__pyx_type)
 */
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError)) __PYX_ERR(1, 5, __pyx_L1_error);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0x088108e, 0xeddf68f, 0xa0b8d2d):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))" % __pyx_checksum             # <<<<<<<<<<<<<<
 *     __pyx_result = IntervalTreeDriverEx.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_v___pyx_PickleError, __pyx_t_1, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x088108e, 0xeddf68f, 0xa0b8d2d):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))" % __pyx_checksum
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))" % __pyx_checksum
 *     __pyx_result = IntervalTreeDriverEx.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IntervalTreeDriverEx__set_state(<IntervalTreeDriverEx> __pyx_result, __pyx_state)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx), __pyx_n_s_new); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v___pyx_type};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_5, 1+__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_v___pyx_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))" % __pyx_checksum
 *     __pyx_result = IntervalTreeDriverEx.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IntervalTreeDriverEx__set_state(<IntervalTreeDriverEx> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_2 = (__pyx_v___pyx_state != Py_None);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = IntervalTreeDriverEx.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IntervalTreeDriverEx__set_state(<IntervalTreeDriverEx> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_IntervalTreeDriverEx__set_state(IntervalTreeDriverEx __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_v___pyx_state))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_7supriya_5utils_10_intervals___pyx_unpickle_IntervalTreeDriverEx__set_state(((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0x088108e, 0xeddf68f, 0xa0b8d2d) = (_root_node))" % __pyx_checksum
 *     __pyx_result = IntervalTreeDriverEx.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_IntervalTreeDriverEx__set_state(<IntervalTreeDriverEx> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_IntervalTreeDriverEx__set_state(<IntervalTreeDriverEx> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_IntervalTreeDriverEx__set_state(IntervalTreeDriverEx __pyx_result, tuple __pyx_state):
 *     __pyx_result._root_node = __pyx_state[0]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_IntervalTreeDriverEx(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("supriya.utils._intervals.__pyx_unpickle_IntervalTreeDriverEx", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_IntervalTreeDriverEx__set_state(<IntervalTreeDriverEx> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IntervalTreeDriverEx__set_state(IntervalTreeDriverEx __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._root_node = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_7supriya_5utils_10_intervals___pyx_unpickle_IntervalTreeDriverEx__set_state(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  unsigned int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_IntervalTreeDriverEx__set_state", 1);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_IntervalTreeDriverEx__set_state(IntervalTreeDriverEx __pyx_result, tuple __pyx_state):
 *     __pyx_result._root_node = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_root_node);
  __Pyx_DECREF(__pyx_v___pyx_result->_root_node);
  __pyx_v___pyx_result->_root_node = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_IntervalTreeDriverEx__set_state(IntervalTreeDriverEx __pyx_result, tuple __pyx_state):
 *     __pyx_result._root_node = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 > 1);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._root_node = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_update); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_8 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_8, 1+__pyx_t_8);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_IntervalTreeDriverEx__set_state(IntervalTreeDriverEx __pyx_result, tuple __pyx_state):
 *     __pyx_result._root_node = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_IntervalTreeDriverEx__set_state(<IntervalTreeDriverEx> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_IntervalTreeDriverEx__set_state(IntervalTreeDriverEx __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._root_node = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("supriya.utils._intervals.__pyx_unpickle_IntervalTreeDriverEx__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_7supriya_5utils_10_intervals__CInterval __pyx_vtable_7supriya_5utils_10_intervals__CInterval;

static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals__CInterval(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)o);
  p->__pyx_vtab = __pyx_vtabptr_7supriya_5utils_10_intervals__CInterval;
  p->original_interval = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7supriya_5utils_10_intervals__CInterval(PyObject *o) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *p = (struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7supriya_5utils_10_intervals__CInterval) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->original_interval);
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_7supriya_5utils_10_intervals__CInterval(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *p = (struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)o;
  if (p->original_interval) {
    e = (*v)(p->original_interval, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7supriya_5utils_10_intervals__CInterval(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *p = (struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *)o;
  tmp = ((PyObject*)p->original_interval);
  p->original_interval = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_10_CInterval_start_offset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_10_CInterval_12start_offset_1__get__(o);
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_10_CInterval_stop_offset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_10_CInterval_11stop_offset_1__get__(o);
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_10_CInterval_original_interval(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_10_CInterval_17original_interval_1__get__(o);
}

static PyMethodDef __pyx_methods_7supriya_5utils_10_intervals__CInterval[] = {
  {"from_interval", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_5from_interval, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_9__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_11__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7supriya_5utils_10_intervals__CInterval[] = {
  {(char *)"start_offset", __pyx_getprop_7supriya_5utils_10_intervals_10_CInterval_start_offset, 0, (char *)0, 0},
  {(char *)"stop_offset", __pyx_getprop_7supriya_5utils_10_intervals_10_CInterval_stop_offset, 0, (char *)0, 0},
  {(char *)"original_interval", __pyx_getprop_7supriya_5utils_10_intervals_10_CInterval_original_interval, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7supriya_5utils_10_intervals__CInterval_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7supriya_5utils_10_intervals__CInterval},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7supriya_5utils_10_intervals__CInterval},
  {Py_tp_clear, (void *)__pyx_tp_clear_7supriya_5utils_10_intervals__CInterval},
  {Py_tp_richcompare, (void *)__pyx_pw_7supriya_5utils_10_intervals_10_CInterval_3__richcmp__},
  {Py_tp_methods, (void *)__pyx_methods_7supriya_5utils_10_intervals__CInterval},
  {Py_tp_getset, (void *)__pyx_getsets_7supriya_5utils_10_intervals__CInterval},
  {Py_tp_new, (void *)__pyx_tp_new_7supriya_5utils_10_intervals__CInterval},
  {0, 0},
};
static PyType_Spec __pyx_type_7supriya_5utils_10_intervals__CInterval_spec = {
  "supriya.utils._intervals._CInterval",
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_7supriya_5utils_10_intervals__CInterval_slots,
};
#else

static PyTypeObject __pyx_type_7supriya_5utils_10_intervals__CInterval = {
  PyVarObject_HEAD_INIT(0, 0)
  "supriya.utils._intervals.""_CInterval", /*tp_name*/
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7supriya_5utils_10_intervals__CInterval, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7supriya_5utils_10_intervals__CInterval, /*tp_traverse*/
  __pyx_tp_clear_7supriya_5utils_10_intervals__CInterval, /*tp_clear*/
  __pyx_pw_7supriya_5utils_10_intervals_10_CInterval_3__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7supriya_5utils_10_intervals__CInterval, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7supriya_5utils_10_intervals__CInterval, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7supriya_5utils_10_intervals__CInterval, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals__CNode(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)o);
  p->left_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); Py_INCREF(Py_None);
  p->payload = Py_None; Py_INCREF(Py_None);
  p->right_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_7supriya_5utils_10_intervals__CNode(PyObject *o) {
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *p = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7supriya_5utils_10_intervals__CNode) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->left_child);
  Py_CLEAR(p->payload);
  Py_CLEAR(p->right_child);
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_7supriya_5utils_10_intervals__CNode(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *p = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)o;
  if (p->left_child) {
    e = (*v)(((PyObject *)p->left_child), a); if (e) return e;
  }
  if (p->payload) {
    e = (*v)(p->payload, a); if (e) return e;
  }
  if (p->right_child) {
    e = (*v)(((PyObject *)p->right_child), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7supriya_5utils_10_intervals__CNode(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7supriya_5utils_10_intervals__CNode *p = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)o;
  tmp = ((PyObject*)p->left_child);
  p->left_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->payload);
  p->payload = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->right_child);
  p->right_child = ((struct __pyx_obj_7supriya_5utils_10_intervals__CNode *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_balance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7balance_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_balance(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7balance_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_height(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_6height_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_height(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_6height_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_left_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_left_child(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_3__set__(o, v);
  }
  else {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_10left_child_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_node_start_index(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16node_start_index_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_node_start_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16node_start_index_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_node_stop_index(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_node_stop_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15node_stop_index_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_payload(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_payload(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_3__set__(o, v);
  }
  else {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_7payload_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_right_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_right_child(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_3__set__(o, v);
  }
  else {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_11right_child_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_start_offset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_12start_offset_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_start_offset(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_12start_offset_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_high(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_high(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_16stop_offset_high_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_low(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_low(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_15stop_offset_low_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_subtree_start_index(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_subtree_start_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_19subtree_start_index_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_6_CNode_subtree_stop_index(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_subtree_stop_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_6_CNode_18subtree_stop_index_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7supriya_5utils_10_intervals__CNode[] = {
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_6_CNode_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_6_CNode_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7supriya_5utils_10_intervals__CNode[] = {
  {(char *)"balance", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_balance, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_balance, (char *)0, 0},
  {(char *)"height", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_height, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_height, (char *)0, 0},
  {(char *)"left_child", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_left_child, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_left_child, (char *)0, 0},
  {(char *)"node_start_index", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_node_start_index, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_node_start_index, (char *)0, 0},
  {(char *)"node_stop_index", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_node_stop_index, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_node_stop_index, (char *)0, 0},
  {(char *)"payload", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_payload, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_payload, (char *)0, 0},
  {(char *)"right_child", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_right_child, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_right_child, (char *)0, 0},
  {(char *)"start_offset", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_start_offset, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_start_offset, (char *)0, 0},
  {(char *)"stop_offset_high", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_high, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_high, (char *)0, 0},
  {(char *)"stop_offset_low", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_low, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_stop_offset_low, (char *)0, 0},
  {(char *)"subtree_start_index", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_subtree_start_index, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_subtree_start_index, (char *)0, 0},
  {(char *)"subtree_stop_index", __pyx_getprop_7supriya_5utils_10_intervals_6_CNode_subtree_stop_index, __pyx_setprop_7supriya_5utils_10_intervals_6_CNode_subtree_stop_index, (char *)0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7supriya_5utils_10_intervals__CNode_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7supriya_5utils_10_intervals__CNode},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7supriya_5utils_10_intervals__CNode},
  {Py_tp_clear, (void *)__pyx_tp_clear_7supriya_5utils_10_intervals__CNode},
  {Py_tp_methods, (void *)__pyx_methods_7supriya_5utils_10_intervals__CNode},
  {Py_tp_getset, (void *)__pyx_getsets_7supriya_5utils_10_intervals__CNode},
  {Py_tp_init, (void *)__pyx_pw_7supriya_5utils_10_intervals_6_CNode_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_7supriya_5utils_10_intervals__CNode},
  {0, 0},
};
static PyType_Spec __pyx_type_7supriya_5utils_10_intervals__CNode_spec = {
  "supriya.utils._intervals._CNode",
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals__CNode),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_7supriya_5utils_10_intervals__CNode_slots,
};
#else

static PyTypeObject __pyx_type_7supriya_5utils_10_intervals__CNode = {
  PyVarObject_HEAD_INIT(0, 0)
  "supriya.utils._intervals.""_CNode", /*tp_name*/
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals__CNode), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7supriya_5utils_10_intervals__CNode, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7supriya_5utils_10_intervals__CNode, /*tp_traverse*/
  __pyx_tp_clear_7supriya_5utils_10_intervals__CNode, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7supriya_5utils_10_intervals__CNode, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7supriya_5utils_10_intervals__CNode, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_7supriya_5utils_10_intervals_6_CNode_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7supriya_5utils_10_intervals__CNode, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_7supriya_5utils_10_intervals_IntervalTreeDriverEx __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx;

static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals_IntervalTreeDriverEx(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *p;
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  if (likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  #endif
  p = ((struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)o);
  p->__pyx_vtab = __pyx_vtabptr_7supriya_5utils_10_intervals_IntervalTreeDriverEx;
  p->_root_node = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_7supriya_5utils_10_intervals_IntervalTreeDriverEx(PyObject *o) {
  struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *p = (struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7supriya_5utils_10_intervals_IntervalTreeDriverEx) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_root_node);
  #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
  (*Py_TYPE(o)->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
}

static int __pyx_tp_traverse_7supriya_5utils_10_intervals_IntervalTreeDriverEx(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *p = (struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)o;
  if (p->_root_node) {
    e = (*v)(p->_root_node, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7supriya_5utils_10_intervals_IntervalTreeDriverEx(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *p = (struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *)o;
  tmp = ((PyObject*)p->_root_node);
  p->_root_node = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_7supriya_5utils_10_intervals_IntervalTreeDriverEx(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_getprop_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__root_node(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_1__get__(o);
}

static int __pyx_setprop_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__root_node(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_3__set__(o, v);
  }
  else {
    return __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10_root_node_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_7supriya_5utils_10_intervals_IntervalTreeDriverEx[] = {
  {"_is_interval", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_12_is_interval, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"add", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_14add, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"find_intervals_starting_at", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16find_intervals_starting_at, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"find_intervals_stopping_at", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_18find_intervals_stopping_at, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"find_intervals_intersecting_offset", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_20find_intervals_intersecting_offset, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"find_intervals_intersecting_interval", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_22find_intervals_intersecting_interval, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"get_offset_after", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_24get_offset_after, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"get_start_offset_after", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_26get_start_offset_after, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"get_start_offset_before", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_28get_start_offset_before, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"index", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_30index, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"remove", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_32remove, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"update", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_34update, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_36__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_38__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7supriya_5utils_10_intervals_IntervalTreeDriverEx[] = {
  {(char *)"_root_node", __pyx_getprop_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__root_node, __pyx_setprop_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__root_node, (char *)0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7supriya_5utils_10_intervals_IntervalTreeDriverEx},
  {Py_sq_length, (void *)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10__len__},
  {Py_sq_item, (void *)__pyx_sq_item_7supriya_5utils_10_intervals_IntervalTreeDriverEx},
  {Py_sq_contains, (void *)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_3__contains__},
  {Py_mp_length, (void *)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10__len__},
  {Py_mp_subscript, (void *)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_5__getitem__},
  {Py_tp_doc, (void *)PyDoc_STR("\n    A mutable always-sorted collection of intervals.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7supriya_5utils_10_intervals_IntervalTreeDriverEx},
  {Py_tp_clear, (void *)__pyx_tp_clear_7supriya_5utils_10_intervals_IntervalTreeDriverEx},
  {Py_tp_iter, (void *)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_7__iter__},
  {Py_tp_methods, (void *)__pyx_methods_7supriya_5utils_10_intervals_IntervalTreeDriverEx},
  {Py_tp_getset, (void *)__pyx_getsets_7supriya_5utils_10_intervals_IntervalTreeDriverEx},
  {Py_tp_init, (void *)__pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_7supriya_5utils_10_intervals_IntervalTreeDriverEx},
  {0, 0},
};
static PyType_Spec __pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx_spec = {
  "supriya.utils._intervals.IntervalTreeDriverEx",
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx_slots,
};
#else

static PySequenceMethods __pyx_tp_as_sequence_IntervalTreeDriverEx = {
  __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_7supriya_5utils_10_intervals_IntervalTreeDriverEx, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_3__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_IntervalTreeDriverEx = {
  __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_10__len__, /*mp_length*/
  __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_5__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx = {
  PyVarObject_HEAD_INIT(0, 0)
  "supriya.utils._intervals.""IntervalTreeDriverEx", /*tp_name*/
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7supriya_5utils_10_intervals_IntervalTreeDriverEx, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_IntervalTreeDriverEx, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_IntervalTreeDriverEx, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    A mutable always-sorted collection of intervals.\n    "), /*tp_doc*/
  __pyx_tp_traverse_7supriya_5utils_10_intervals_IntervalTreeDriverEx, /*tp_traverse*/
  __pyx_tp_clear_7supriya_5utils_10_intervals_IntervalTreeDriverEx, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_7__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7supriya_5utils_10_intervals_IntervalTreeDriverEx, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7supriya_5utils_10_intervals_IntervalTreeDriverEx, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7supriya_5utils_10_intervals_IntervalTreeDriverEx, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

#if CYTHON_USE_FREELISTS
static struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *__pyx_freelist_7supriya_5utils_10_intervals___pyx_scope_struct____iter__[8];
static int __pyx_freecount_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ = 0;
#endif

static PyObject *__pyx_tp_new_7supriya_5utils_10_intervals___pyx_scope_struct____iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_COMPILING_IN_LIMITED_API
  allocfunc alloc_func = (allocfunc)PyType_GetSlot(t, Py_tp_alloc);
  o = alloc_func(t, 0);
  #else
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_freecount_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ > 0) & (int)(t->tp_basicsize == sizeof(struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__)))) {
    o = (PyObject*)__pyx_freelist_7supriya_5utils_10_intervals___pyx_scope_struct____iter__[--__pyx_freecount_7supriya_5utils_10_intervals___pyx_scope_struct____iter__];
    memset(o, 0, sizeof(struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  #endif
  return o;
}

static void __pyx_tp_dealloc_7supriya_5utils_10_intervals___pyx_scope_struct____iter__(PyObject *o) {
  struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *p = (struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely((PY_VERSION_HEX >= 0x03080000 || __Pyx_PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE)) && __Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7supriya_5utils_10_intervals___pyx_scope_struct____iter__) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_cinterval);
  Py_CLEAR(p->__pyx_v_current);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_stack);
  #if CYTHON_USE_FREELISTS
  if (((int)(__pyx_freecount_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ < 8) & (int)(Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__)))) {
    __pyx_freelist_7supriya_5utils_10_intervals___pyx_scope_struct____iter__[__pyx_freecount_7supriya_5utils_10_intervals___pyx_scope_struct____iter__++] = ((struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *)o);
  } else
  #endif
  {
    #if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
    (*Py_TYPE(o)->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(Py_TYPE(o), Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
  }
}

static int __pyx_tp_traverse_7supriya_5utils_10_intervals___pyx_scope_struct____iter__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *p = (struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ *)o;
  if (p->__pyx_v_cinterval) {
    e = (*v)(((PyObject *)p->__pyx_v_cinterval), a); if (e) return e;
  }
  if (p->__pyx_v_current) {
    e = (*v)(((PyObject *)p->__pyx_v_current), a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_stack) {
    e = (*v)(p->__pyx_v_stack, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter___slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7supriya_5utils_10_intervals___pyx_scope_struct____iter__},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7supriya_5utils_10_intervals___pyx_scope_struct____iter__},
  {Py_tp_new, (void *)__pyx_tp_new_7supriya_5utils_10_intervals___pyx_scope_struct____iter__},
  {0, 0},
};
static PyType_Spec __pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter___spec = {
  "supriya.utils._intervals.__pyx_scope_struct____iter__",
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE,
  __pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter___slots,
};
#else

static PyTypeObject __pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "supriya.utils._intervals.""__pyx_scope_struct____iter__", /*tp_name*/
  sizeof(struct __pyx_obj_7supriya_5utils_10_intervals___pyx_scope_struct____iter__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7supriya_5utils_10_intervals___pyx_scope_struct____iter__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7supriya_5utils_10_intervals___pyx_scope_struct____iter__, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7supriya_5utils_10_intervals___pyx_scope_struct____iter__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static int __Pyx_CreateStringTabAndInitStrings(void) {
  __Pyx_StringTabEntry __pyx_string_tab[] = {
    {&__pyx_n_s_AssertionError, __pyx_k_AssertionError, sizeof(__pyx_k_AssertionError), 0, 0, 1, 1},
    {&__pyx_n_s_CInterval, __pyx_k_CInterval, sizeof(__pyx_k_CInterval), 0, 0, 1, 1},
    {&__pyx_n_s_CInterval___reduce_cython, __pyx_k_CInterval___reduce_cython, sizeof(__pyx_k_CInterval___reduce_cython), 0, 0, 1, 1},
    {&__pyx_n_s_CInterval___setstate_cython, __pyx_k_CInterval___setstate_cython, sizeof(__pyx_k_CInterval___setstate_cython), 0, 0, 1, 1},
    {&__pyx_n_s_CInterval_from_interval, __pyx_k_CInterval_from_interval, sizeof(__pyx_k_CInterval_from_interval), 0, 0, 1, 1},
    {&__pyx_n_s_CInterval_intersects_interval, __pyx_k_CInterval_intersects_interval, sizeof(__pyx_k_CInterval_intersects_interval), 0, 0, 1, 1},
    {&__pyx_n_s_CNode, __pyx_k_CNode, sizeof(__pyx_k_CNode), 0, 0, 1, 1},
    {&__pyx_n_s_CNode___reduce_cython, __pyx_k_CNode___reduce_cython, sizeof(__pyx_k_CNode___reduce_cython), 0, 0, 1, 1},
    {&__pyx_n_s_CNode___setstate_cython, __pyx_k_CNode___setstate_cython, sizeof(__pyx_k_CNode___setstate_cython), 0, 0, 1, 1},
    {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 0, 1, 0},
    {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2, __pyx_k_Incompatible_checksums_0x_x_vs_0_2, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_2), 0, 0, 1, 0},
    {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
    {&__pyx_kp_u_Indices_must_be_integers_or_slic, __pyx_k_Indices_must_be_integers_or_slic, sizeof(__pyx_k_Indices_must_be_integers_or_slic), 0, 1, 0, 0},
    {&__pyx_n_s_IntervalTreeDriverEx, __pyx_k_IntervalTreeDriverEx, sizeof(__pyx_k_IntervalTreeDriverEx), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx___iter, __pyx_k_IntervalTreeDriverEx___iter, sizeof(__pyx_k_IntervalTreeDriverEx___iter), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx___reduce_cy, __pyx_k_IntervalTreeDriverEx___reduce_cy, sizeof(__pyx_k_IntervalTreeDriverEx___reduce_cy), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx___setstate, __pyx_k_IntervalTreeDriverEx___setstate, sizeof(__pyx_k_IntervalTreeDriverEx___setstate), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx__insert_int, __pyx_k_IntervalTreeDriverEx__insert_int, sizeof(__pyx_k_IntervalTreeDriverEx__insert_int), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx__is_interva, __pyx_k_IntervalTreeDriverEx__is_interva, sizeof(__pyx_k_IntervalTreeDriverEx__is_interva), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx__unbox_cint, __pyx_k_IntervalTreeDriverEx__unbox_cint, sizeof(__pyx_k_IntervalTreeDriverEx__unbox_cint), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_add, __pyx_k_IntervalTreeDriverEx_add, sizeof(__pyx_k_IntervalTreeDriverEx_add), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_find_interv, __pyx_k_IntervalTreeDriverEx_find_interv, sizeof(__pyx_k_IntervalTreeDriverEx_find_interv), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_find_interv_2, __pyx_k_IntervalTreeDriverEx_find_interv_2, sizeof(__pyx_k_IntervalTreeDriverEx_find_interv_2), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_find_interv_3, __pyx_k_IntervalTreeDriverEx_find_interv_3, sizeof(__pyx_k_IntervalTreeDriverEx_find_interv_3), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_find_interv_4, __pyx_k_IntervalTreeDriverEx_find_interv_4, sizeof(__pyx_k_IntervalTreeDriverEx_find_interv_4), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_get_offset, __pyx_k_IntervalTreeDriverEx_get_offset, sizeof(__pyx_k_IntervalTreeDriverEx_get_offset), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_get_start_o, __pyx_k_IntervalTreeDriverEx_get_start_o, sizeof(__pyx_k_IntervalTreeDriverEx_get_start_o), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_get_start_o_2, __pyx_k_IntervalTreeDriverEx_get_start_o_2, sizeof(__pyx_k_IntervalTreeDriverEx_get_start_o_2), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_index, __pyx_k_IntervalTreeDriverEx_index, sizeof(__pyx_k_IntervalTreeDriverEx_index), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_remove, __pyx_k_IntervalTreeDriverEx_remove, sizeof(__pyx_k_IntervalTreeDriverEx_remove), 0, 0, 1, 1},
    {&__pyx_n_s_IntervalTreeDriverEx_update, __pyx_k_IntervalTreeDriverEx_update, sizeof(__pyx_k_IntervalTreeDriverEx_update), 0, 0, 1, 1},
    {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
    {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
    {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
    {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
    {&__pyx_n_s__41, __pyx_k__41, sizeof(__pyx_k__41), 0, 0, 1, 1},
    {&__pyx_n_s_add, __pyx_k_add, sizeof(__pyx_k_add), 0, 0, 1, 1},
    {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
    {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
    {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
    {&__pyx_n_s_cinterval, __pyx_k_cinterval, sizeof(__pyx_k_cinterval), 0, 0, 1, 1},
    {&__pyx_n_s_cintervals, __pyx_k_cintervals, sizeof(__pyx_k_cintervals), 0, 0, 1, 1},
    {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
    {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
    {&__pyx_n_s_cls, __pyx_k_cls, sizeof(__pyx_k_cls), 0, 0, 1, 1},
    {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
    {&__pyx_n_s_dict_2, __pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 0, 1, 1},
    {&__pyx_kp_u_disable, __pyx_k_disable, sizeof(__pyx_k_disable), 0, 1, 0, 0},
    {&__pyx_kp_u_enable, __pyx_k_enable, sizeof(__pyx_k_enable), 0, 1, 0, 0},
    {&__pyx_n_s_expr, __pyx_k_expr, sizeof(__pyx_k_expr), 0, 0, 1, 1},
    {&__pyx_n_s_find_intervals_intersecting_inte, __pyx_k_find_intervals_intersecting_inte, sizeof(__pyx_k_find_intervals_intersecting_inte), 0, 0, 1, 1},
    {&__pyx_n_s_find_intervals_intersecting_offs, __pyx_k_find_intervals_intersecting_offs, sizeof(__pyx_k_find_intervals_intersecting_offs), 0, 0, 1, 1},
    {&__pyx_n_s_find_intervals_starting_at, __pyx_k_find_intervals_starting_at, sizeof(__pyx_k_find_intervals_starting_at), 0, 0, 1, 1},
    {&__pyx_n_s_find_intervals_stopping_at, __pyx_k_find_intervals_stopping_at, sizeof(__pyx_k_find_intervals_stopping_at), 0, 0, 1, 1},
    {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
    {&__pyx_n_s_from_interval, __pyx_k_from_interval, sizeof(__pyx_k_from_interval), 0, 0, 1, 1},
    {&__pyx_kp_u_gc, __pyx_k_gc, sizeof(__pyx_k_gc), 0, 1, 0, 0},
    {&__pyx_n_s_get_offset_after, __pyx_k_get_offset_after, sizeof(__pyx_k_get_offset_after), 0, 0, 1, 1},
    {&__pyx_n_s_get_start_offset_after, __pyx_k_get_start_offset_after, sizeof(__pyx_k_get_start_offset_after), 0, 0, 1, 1},
    {&__pyx_n_s_get_start_offset_before, __pyx_k_get_start_offset_before, sizeof(__pyx_k_get_start_offset_before), 0, 0, 1, 1},
    {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
    {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
    {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
    {&__pyx_n_s_indices, __pyx_k_indices, sizeof(__pyx_k_indices), 0, 0, 1, 1},
    {&__pyx_n_u_inf, __pyx_k_inf, sizeof(__pyx_k_inf), 0, 1, 0, 1},
    {&__pyx_kp_u_inf_2, __pyx_k_inf_2, sizeof(__pyx_k_inf_2), 0, 1, 0, 0},
    {&__pyx_n_s_intersects_interval, __pyx_k_intersects_interval, sizeof(__pyx_k_intersects_interval), 0, 0, 1, 1},
    {&__pyx_n_s_interval, __pyx_k_interval, sizeof(__pyx_k_interval), 0, 0, 1, 1},
    {&__pyx_n_s_intervals, __pyx_k_intervals, sizeof(__pyx_k_intervals), 0, 0, 1, 1},
    {&__pyx_n_s_is_coroutine, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
    {&__pyx_n_s_is_interval, __pyx_k_is_interval, sizeof(__pyx_k_is_interval), 0, 0, 1, 1},
    {&__pyx_kp_u_isenabled, __pyx_k_isenabled, sizeof(__pyx_k_isenabled), 0, 1, 0, 0},
    {&__pyx_n_s_iter, __pyx_k_iter, sizeof(__pyx_k_iter), 0, 0, 1, 1},
    {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
    {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
    {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
    {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
    {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
    {&__pyx_n_s_node, __pyx_k_node, sizeof(__pyx_k_node), 0, 0, 1, 1},
    {&__pyx_kp_u_not_in_interval_tree, __pyx_k_not_in_interval_tree, sizeof(__pyx_k_not_in_interval_tree), 0, 1, 0, 0},
    {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
    {&__pyx_n_s_original_interval, __pyx_k_original_interval, sizeof(__pyx_k_original_interval), 0, 0, 1, 1},
    {&__pyx_n_s_other, __pyx_k_other, sizeof(__pyx_k_other), 0, 0, 1, 1},
    {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
    {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_unpickle_IntervalTreeDrive, __pyx_k_pyx_unpickle_IntervalTreeDrive, sizeof(__pyx_k_pyx_unpickle_IntervalTreeDrive), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_unpickle__CNode, __pyx_k_pyx_unpickle__CNode, sizeof(__pyx_k_pyx_unpickle__CNode), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
    {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
    {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
    {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
    {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
    {&__pyx_n_s_remove, __pyx_k_remove, sizeof(__pyx_k_remove), 0, 0, 1, 1},
    {&__pyx_n_s_result, __pyx_k_result, sizeof(__pyx_k_result), 0, 0, 1, 1},
    {&__pyx_n_s_results, __pyx_k_results, sizeof(__pyx_k_results), 0, 0, 1, 1},
    {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
    {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
    {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
    {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
    {&__pyx_n_s_sort, __pyx_k_sort, sizeof(__pyx_k_sort), 0, 0, 1, 1},
    {&__pyx_n_s_start_offset, __pyx_k_start_offset, sizeof(__pyx_k_start_offset), 0, 0, 1, 1},
    {&__pyx_n_u_start_offset, __pyx_k_start_offset, sizeof(__pyx_k_start_offset), 0, 1, 0, 1},
    {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
    {&__pyx_n_s_staticmethod, __pyx_k_staticmethod, sizeof(__pyx_k_staticmethod), 0, 0, 1, 1},
    {&__pyx_n_s_stop_offset, __pyx_k_stop_offset, sizeof(__pyx_k_stop_offset), 0, 0, 1, 1},
    {&__pyx_n_u_stop_offset, __pyx_k_stop_offset, sizeof(__pyx_k_stop_offset), 0, 1, 0, 1},
    {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
    {&__pyx_n_s_subtree_stop_index, __pyx_k_subtree_stop_index, sizeof(__pyx_k_subtree_stop_index), 0, 0, 1, 1},
    {&__pyx_n_s_supriya_utils__intervals, __pyx_k_supriya_utils__intervals, sizeof(__pyx_k_supriya_utils__intervals), 0, 0, 1, 1},
    {&__pyx_kp_s_supriya_utils__intervals_pyx, __pyx_k_supriya_utils__intervals_pyx, sizeof(__pyx_k_supriya_utils__intervals_pyx), 0, 0, 1, 0},
    {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
    {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
    {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
    {&__pyx_n_s_use_setstate, __pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 0, 1, 1},
    {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0}
  };
  return __Pyx_InitStrings(__pyx_string_tab);
}
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_staticmethod = __Pyx_GetBuiltinName(__pyx_n_s_staticmethod); if (!__pyx_builtin_staticmethod) __PYX_ERR(0, 186, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 104, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 112, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 140, __pyx_L1_error)
  __pyx_builtin_AssertionError = __Pyx_GetBuiltinName(__pyx_n_s_AssertionError); if (!__pyx_builtin_AssertionError) __PYX_ERR(0, 647, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xe8cb558, 0x8efd0e5, 0x9b91c78):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError, "Incompatible checksums (0x%x vs (0xe8cb558, 0x8efd0e5, 0x9b91c78) = (balance, height, left_child, node_start_index, node_stop_index, payload, right_child, start_offset, stop_offset_high, stop_offset_low, subtree_start_index, subtree_stop_index))" % __pyx_checksum
 */
  __pyx_tuple_ = PyTuple_Pack(3, __pyx_int_244102488, __pyx_int_149934309, __pyx_int_163126392); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);
  __pyx_tuple__3 = PyTuple_Pack(3, __pyx_int_8917134, __pyx_int_249427599, __pyx_int_168529197); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "supriya/utils/_intervals.pyx":33
 *             return True
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_interval(cls, interval):
 *         start_offset = float(interval.start_offset)
 */
  __pyx_tuple__4 = PyTuple_Pack(4, __pyx_n_s_cls, __pyx_n_s_interval, __pyx_n_s_start_offset, __pyx_n_s_stop_offset); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_from_interval, 33, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 33, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":43
 *             )
 * 
 *     cpdef bint intersects_interval(_CInterval self, _CInterval other):             # <<<<<<<<<<<<<<
 *         return (
 *             (
 */
  __pyx_tuple__6 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_other); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_intersects_interval, 43, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 43, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(1, 1, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */
  __pyx_tuple__10 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 3, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(1, 3, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_tuple__12 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_state, __pyx_n_s_dict_2, __pyx_n_s_use_setstate); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) __PYX_ERR(1, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__CNode__set_state(self, __pyx_state)
 */
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(1, 16, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":186
 *         node.payload.sort(key=lambda x: x.stop_offset)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _is_interval(expr):
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_n_s_expr); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_is_interval, 186, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 186, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":618
 *     ### PUBLIC METHODS ###
 * 
 *     def add(self, interval):             # <<<<<<<<<<<<<<
 *         if not self._is_interval(interval):
 *             # raise ValueError(interval)
 */
  __pyx_tuple__17 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_interval, __pyx_n_s_cinterval); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_add, 618, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 618, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":627
 *         self._update_offsets(self._root_node)
 * 
 *     def find_intervals_starting_at(self, offset):             # <<<<<<<<<<<<<<
 *         results = []
 *         node = self._search(self._root_node, offset)
 */
  __pyx_tuple__19 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_offset, __pyx_n_s_results, __pyx_n_s_node); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_find_intervals_starting_at, 627, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 627, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":634
 *         return self._unbox_cintervals(node.payload)
 * 
 *     def find_intervals_stopping_at(self, offset):             # <<<<<<<<<<<<<<
 *         cintervals = self._recurse_find_intervals_stopping_at(
 *             self._root_node, offset)
 */
  __pyx_tuple__21 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_offset, __pyx_n_s_cintervals); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_find_intervals_stopping_at, 634, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 634, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":639
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_offset(self, offset):             # <<<<<<<<<<<<<<
 *         offset = float(offset)
 *         cintervals = self._recurse_find_intervals_intersecting_offset(
 */
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_find_intervals_intersecting_offs, 639, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) __PYX_ERR(0, 639, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":645
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_interval(self, interval):             # <<<<<<<<<<<<<<
 *         cinterval = _CInterval.from_interval(interval)
 *         assert isinstance(cinterval, _CInterval)
 */
  __pyx_tuple__24 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_interval, __pyx_n_s_cinterval, __pyx_n_s_cintervals); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_find_intervals_intersecting_inte, 645, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 645, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":652
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def get_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         # print("Searching for: {}".format(offset))
 *         result = None
 */
  __pyx_tuple__26 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_offset, __pyx_n_s_result); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);
  __pyx_codeobj__27 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__26, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_get_offset_after, 652, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__27)) __PYX_ERR(0, 652, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":661
 *         return result
 * 
 *     def get_start_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)
 *         if node is None:
 */
  __pyx_tuple__28 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_offset, __pyx_n_s_node); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_get_start_offset_after, 661, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(0, 661, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":667
 *         return node.start_offset
 * 
 *     def get_start_offset_before(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)
 *         if node is None:
 */
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_get_start_offset_before, 667, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(0, 667, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":673
 *         return node.start_offset
 * 
 *     def index(self, interval):             # <<<<<<<<<<<<<<
 *         assert self._is_interval(interval)
 *         cinterval = _CInterval.from_interval(interval)
 */
  __pyx_tuple__31 = PyTuple_Pack(5, __pyx_n_s_self, __pyx_n_s_interval, __pyx_n_s_cinterval, __pyx_n_s_node, __pyx_n_s_index); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_index, 673, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 673, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":684
 *         return index
 * 
 *     def remove(self, interval):             # <<<<<<<<<<<<<<
 *         if interval not in self:
 *             raise ValueError(interval)
 */
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_remove, 684, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(0, 684, __pyx_L1_error)

  /* "supriya/utils/_intervals.pyx":692
 *         self._update_offsets(self._root_node)
 * 
 *     def update(self, intervals):             # <<<<<<<<<<<<<<
 *         # for interval in intervals:
 *         #     if not self._is_interval(interval):
 */
  __pyx_tuple__34 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_intervals, __pyx_n_s_interval, __pyx_n_s_cinterval); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_supriya_utils__intervals_pyx, __pyx_n_s_update, 692, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(0, 692, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(1, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IntervalTreeDriverEx__set_state(self, __pyx_state)
 */
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(1, 16, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle__CNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__38 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle__CNode, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(3, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_IntervalTreeDrive, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitConstants(void) {
  __pyx_umethod_PyList_Type_pop.type = (PyObject*)&PyList_Type;
  __pyx_umethod_PyList_Type_pop.method_name = &__pyx_n_s_pop;
  if (__Pyx_CreateStringTabAndInitStrings() < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_8917134 = PyInt_FromLong(8917134L); if (unlikely(!__pyx_int_8917134)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_149934309 = PyInt_FromLong(149934309L); if (unlikely(!__pyx_int_149934309)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_163126392 = PyInt_FromLong(163126392L); if (unlikely(!__pyx_int_163126392)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_168529197 = PyInt_FromLong(168529197L); if (unlikely(!__pyx_int_168529197)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_244102488 = PyInt_FromLong(244102488L); if (unlikely(!__pyx_int_244102488)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_249427599 = PyInt_FromLong(249427599L); if (unlikely(!__pyx_int_249427599)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else

if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_7supriya_5utils_10_intervals__CInterval = &__pyx_vtable_7supriya_5utils_10_intervals__CInterval;
  __pyx_vtable_7supriya_5utils_10_intervals__CInterval.intersects_interval = (int (*)(struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *, int __pyx_skip_dispatch))__pyx_f_7supriya_5utils_10_intervals_10_CInterval_intersects_interval;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7supriya_5utils_10_intervals__CInterval = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7supriya_5utils_10_intervals__CInterval_spec, NULL); if (unlikely(!__pyx_ptype_7supriya_5utils_10_intervals__CInterval)) __PYX_ERR(0, 4, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7supriya_5utils_10_intervals__CInterval_spec, __pyx_ptype_7supriya_5utils_10_intervals__CInterval) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  #else
  __pyx_ptype_7supriya_5utils_10_intervals__CInterval = &__pyx_type_7supriya_5utils_10_intervals__CInterval;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7supriya_5utils_10_intervals__CInterval) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7supriya_5utils_10_intervals__CInterval->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7supriya_5utils_10_intervals__CInterval->tp_dictoffset && __pyx_ptype_7supriya_5utils_10_intervals__CInterval->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7supriya_5utils_10_intervals__CInterval->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_7supriya_5utils_10_intervals__CInterval, __pyx_vtabptr_7supriya_5utils_10_intervals__CInterval) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_7supriya_5utils_10_intervals__CInterval) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CInterval, (PyObject *) __pyx_ptype_7supriya_5utils_10_intervals__CInterval) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_7supriya_5utils_10_intervals__CInterval) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7supriya_5utils_10_intervals__CNode = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7supriya_5utils_10_intervals__CNode_spec, NULL); if (unlikely(!__pyx_ptype_7supriya_5utils_10_intervals__CNode)) __PYX_ERR(0, 55, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7supriya_5utils_10_intervals__CNode_spec, __pyx_ptype_7supriya_5utils_10_intervals__CNode) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  #else
  __pyx_ptype_7supriya_5utils_10_intervals__CNode = &__pyx_type_7supriya_5utils_10_intervals__CNode;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7supriya_5utils_10_intervals__CNode) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7supriya_5utils_10_intervals__CNode->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7supriya_5utils_10_intervals__CNode->tp_dictoffset && __pyx_ptype_7supriya_5utils_10_intervals__CNode->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7supriya_5utils_10_intervals__CNode->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CNode, (PyObject *) __pyx_ptype_7supriya_5utils_10_intervals__CNode) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_7supriya_5utils_10_intervals__CNode) < 0) __PYX_ERR(0, 55, __pyx_L1_error)
  #endif
  __pyx_vtabptr_7supriya_5utils_10_intervals_IntervalTreeDriverEx = &__pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._insert_node = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__insert_node;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._insert_interval = (void (*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__insert_interval;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._rebalance = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rebalance;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_find_intervals_intersecting_offset = (PyObject *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_intersecting_offset;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_find_intervals_intersecting_interval = (PyObject *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_intersecting_interval;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_find_intervals_stopping_at = (PyObject *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_find_intervals_stopping_at;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_get_offset_after = (float (*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float, struct __pyx_opt_args_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after *__pyx_optional_args))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_offset_after;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_get_start_offset_after = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_start_offset_after;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_get_start_offset_before = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_get_start_offset_before;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_getitem_by_index = (struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, int))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_getitem_by_index;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._recurse_getitem_by_slice = (PyObject *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, int, int))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__recurse_getitem_by_slice;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._remove_node = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__remove_node;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._remove_interval = (void (*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CInterval *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__remove_interval;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._rotate_left_left = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_left_left;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._rotate_left_right = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_left_right;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._rotate_right_left = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_right_left;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._rotate_right_right = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__rotate_right_right;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._search = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, float))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__search;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._set_node_left_child = (void (*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__set_node_left_child;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._set_node_right_child = (void (*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__set_node_right_child;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._unbox_cintervals = (PyObject *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, PyObject *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__unbox_cintervals;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._update_node_height = (void (*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_node_height;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._update_indices = (void (*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *, int))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_indices;
  __pyx_vtable_7supriya_5utils_10_intervals_IntervalTreeDriverEx._update_offsets = (struct __pyx_obj_7supriya_5utils_10_intervals__CNode *(*)(struct __pyx_obj_7supriya_5utils_10_intervals_IntervalTreeDriverEx *, struct __pyx_obj_7supriya_5utils_10_intervals__CNode *))__pyx_f_7supriya_5utils_10_intervals_20IntervalTreeDriverEx__update_offsets;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx_spec, NULL); if (unlikely(!__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx)) __PYX_ERR(0, 85, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx_spec, __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  #else
  __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx = &__pyx_type_7supriya_5utils_10_intervals_IntervalTreeDriverEx;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx->tp_dictoffset && __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx->tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_vtabptr_7supriya_5utils_10_intervals_IntervalTreeDriverEx) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_MergeVtables(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_IntervalTreeDriverEx, (PyObject *) __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if (__Pyx_setup_reduce((PyObject *) __pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter___spec, NULL); if (unlikely(!__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter___spec, __pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__) < 0) __PYX_ERR(0, 128, __pyx_L1_error)
  #else
  __pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__ = &__pyx_type_7supriya_5utils_10_intervals___pyx_scope_struct____iter__;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__) < 0) __PYX_ERR(0, 128, __pyx_L1_error)
  #endif
  #if PY_MAJOR_VERSION < 3
  __pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__->tp_print = 0;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__->tp_dictoffset && __pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_ptype_7supriya_5utils_10_intervals___pyx_scope_struct____iter__->tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__intervals(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__intervals},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "_intervals",
      0, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_intervals(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_intervals(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__intervals(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__intervals(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__intervals(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_intervals' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_intervals", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #elif CYTHON_USE_MODULE_STATE
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  {
    int add_module_result = PyState_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "_intervals" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #endif
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = __Pyx_PyImport_AddModuleRef((const char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__intervals(void)", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_supriya__utils___intervals) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "supriya.utils._intervals")) {
      if (unlikely((PyDict_SetItemString(modules, "supriya.utils._intervals", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely((__Pyx_modinit_type_init_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "supriya/utils/_intervals.pyx":33
 *             return True
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def from_interval(cls, interval):
 *         start_offset = float(interval.start_offset)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_5from_interval, __Pyx_CYFUNCTION_CLASSMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_CInterval_from_interval, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__5)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval, __pyx_n_s_from_interval, __pyx_t_2) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals__CInterval);
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval, __pyx_n_s_from_interval); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Method_ClassMethod(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval, __pyx_n_s_from_interval, __pyx_t_3) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals__CInterval);

  /* "supriya/utils/_intervals.pyx":43
 *             )
 * 
 *     cpdef bint intersects_interval(_CInterval self, _CInterval other):             # <<<<<<<<<<<<<<
 *         return (
 *             (
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_7intersects_interval, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_CInterval_intersects_interval, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__7)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CInterval, __pyx_n_s_intersects_interval, __pyx_t_3) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals__CInterval);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_9__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_CInterval___reduce_cython, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__9)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_reduce_cython, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_10_CInterval_11__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_CInterval___setstate_cython, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__11)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_setstate_cython, __pyx_t_3) < 0) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_6_CNode_3__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_CNode___reduce_cython, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__13)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CNode, __pyx_n_s_reduce_cython, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals__CNode);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__CNode, (type(self), 0xe8cb558, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__CNode__set_state(self, __pyx_state)
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_6_CNode_5__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_CNode___setstate_cython, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals__CNode, __pyx_n_s_setstate_cython, __pyx_t_3) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals__CNode);

  /* "supriya/utils/_intervals.pyx":186
 *         node.payload.sort(key=lambda x: x.stop_offset)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _is_interval(expr):
 *         if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_12_is_interval, __Pyx_CYFUNCTION_STATICMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx__is_interva, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_is_interval, __pyx_t_3) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);
  __Pyx_GetNameInClass(__pyx_t_3, (PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_is_interval); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_is_interval, __pyx_t_2) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":618
 *     ### PUBLIC METHODS ###
 * 
 *     def add(self, interval):             # <<<<<<<<<<<<<<
 *         if not self._is_interval(interval):
 *             # raise ValueError(interval)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_14add, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_add, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_add, __pyx_t_2) < 0) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":627
 *         self._update_offsets(self._root_node)
 * 
 *     def find_intervals_starting_at(self, offset):             # <<<<<<<<<<<<<<
 *         results = []
 *         node = self._search(self._root_node, offset)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_16find_intervals_starting_at, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_find_interv, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_find_intervals_starting_at, __pyx_t_2) < 0) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":634
 *         return self._unbox_cintervals(node.payload)
 * 
 *     def find_intervals_stopping_at(self, offset):             # <<<<<<<<<<<<<<
 *         cintervals = self._recurse_find_intervals_stopping_at(
 *             self._root_node, offset)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_18find_intervals_stopping_at, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_find_interv_2, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__22)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_find_intervals_stopping_at, __pyx_t_2) < 0) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":639
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_offset(self, offset):             # <<<<<<<<<<<<<<
 *         offset = float(offset)
 *         cintervals = self._recurse_find_intervals_intersecting_offset(
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_20find_intervals_intersecting_offset, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_find_interv_3, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__23)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_find_intervals_intersecting_offs, __pyx_t_2) < 0) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":645
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def find_intervals_intersecting_interval(self, interval):             # <<<<<<<<<<<<<<
 *         cinterval = _CInterval.from_interval(interval)
 *         assert isinstance(cinterval, _CInterval)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_22find_intervals_intersecting_interval, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_find_interv_4, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__25)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_find_intervals_intersecting_inte, __pyx_t_2) < 0) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":652
 *         return self._unbox_cintervals(cintervals)
 * 
 *     def get_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         # print("Searching for: {}".format(offset))
 *         result = None
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_24get_offset_after, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_get_offset, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__27)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_get_offset_after, __pyx_t_2) < 0) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":661
 *         return result
 * 
 *     def get_start_offset_after(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_after(self._root_node, offset)
 *         if node is None:
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_26get_start_offset_after, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_get_start_o, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__29)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_get_start_offset_after, __pyx_t_2) < 0) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":667
 *         return node.start_offset
 * 
 *     def get_start_offset_before(self, offset):             # <<<<<<<<<<<<<<
 *         node = self._recurse_get_start_offset_before(self._root_node, offset)
 *         if node is None:
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_28get_start_offset_before, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_get_start_o_2, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__30)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_get_start_offset_before, __pyx_t_2) < 0) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":673
 *         return node.start_offset
 * 
 *     def index(self, interval):             # <<<<<<<<<<<<<<
 *         assert self._is_interval(interval)
 *         cinterval = _CInterval.from_interval(interval)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_30index, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_index, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__32)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_index, __pyx_t_2) < 0) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":684
 *         return index
 * 
 *     def remove(self, interval):             # <<<<<<<<<<<<<<
 *         if interval not in self:
 *             raise ValueError(interval)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_32remove, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_remove, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__33)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_remove, __pyx_t_2) < 0) __PYX_ERR(0, 684, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "supriya/utils/_intervals.pyx":692
 *         self._update_offsets(self._root_node)
 * 
 *     def update(self, intervals):             # <<<<<<<<<<<<<<
 *         # for interval in intervals:
 *         #     if not self._is_interval(interval):
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_34update, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx_update, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__35)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_update, __pyx_t_2) < 0) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_36__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx___reduce_cy, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_reduce_cython, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_IntervalTreeDriverEx, (type(self), 0x088108e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_IntervalTreeDriverEx__set_state(self, __pyx_state)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_20IntervalTreeDriverEx_38__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_IntervalTreeDriverEx___setstate, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__37)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetItemOnTypeDict((PyObject *)__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx, __pyx_n_s_setstate_cython, __pyx_t_2) < 0) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7supriya_5utils_10_intervals_IntervalTreeDriverEx);

  /* "(tree fragment)":1
 * def __pyx_unpickle__CNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_1__pyx_unpickle__CNode, 0, __pyx_n_s_pyx_unpickle__CNode, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__39)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle__CNode, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle__CNode__set_state(<_CNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__CNode__set_state(_CNode __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.balance = __pyx_state[0]; __pyx_result.height = __pyx_state[1]; __pyx_result.left_child = __pyx_state[2]; __pyx_result.node_start_index = __pyx_state[3]; __pyx_result.node_stop_index = __pyx_state[4]; __pyx_result.payload = __pyx_state[5]; __pyx_result.right_child = __pyx_state[6]; __pyx_result.start_offset = __pyx_state[7]; __pyx_result.stop_offset_high = __pyx_state[8]; __pyx_result.stop_offset_low = __pyx_state[9]; __pyx_result.subtree_start_index = __pyx_state[10]; __pyx_result.subtree_stop_index = __pyx_state[11]
 *     if len(__pyx_state) > 12 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7supriya_5utils_10_intervals_3__pyx_unpickle_IntervalTreeDriverEx, 0, __pyx_n_s_pyx_unpickle_IntervalTreeDrive, NULL, __pyx_n_s_supriya_utils__intervals, __pyx_d, ((PyObject *)__pyx_codeobj__40)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_IntervalTreeDrive, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "supriya/utils/_intervals.pyx":1
 * # cython: language_level=3             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init supriya.utils._intervals", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init supriya.utils._intervals");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs = PyTuple_GET_SIZE(kwnames);
    PyObject *dict;
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        Py_XDECREF(key); key = NULL;
        Py_XDECREF(value); value = NULL;
        if (kwds_is_tuple) {
            Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(kwds);
#else
            size = PyTuple_Size(kwds);
            if (size < 0) goto bad;
#endif
            if (pos >= size) break;
#if CYTHON_AVOID_BORROWED_REFS
            key = __Pyx_PySequence_ITEM(kwds, pos);
            if (!key) goto bad;
#elif CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kwds, pos);
#else
            key = PyTuple_GetItem(kwds, pos);
            if (!key) goto bad;
#endif
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
#endif
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(value);
            Py_DECREF(key);
#endif
            key = NULL;
            value = NULL;
            continue;
        }
#if !CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(key);
#endif
        Py_INCREF(value);
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    Py_XDECREF(key);
    Py_XDECREF(value);
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    Py_XDECREF(key);
    Py_XDECREF(value);
    return -1;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    type_name = __Pyx_PyType_GetName(type);
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")", name, type_name, obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    #endif
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) < 0) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
        #if PY_VERSION_HEX < 0x03090000
        vectorcallfunc f = _PyVectorcall_Function(func);
        #else
        vectorcallfunc f = PyVectorcall_Function(func);
        #endif
        if (f) {
            return f(func, args, (size_t)nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, (size_t)nargs, NULL);
        }
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* pybytes_as_double */
static double __Pyx_SlowPyString_AsDouble(PyObject *obj) {
    PyObject *float_value;
#if PY_MAJOR_VERSION >= 3
    float_value = PyFloat_FromString(obj);
#else
    float_value = PyFloat_FromString(obj, 0);
#endif
    if (likely(float_value)) {
#if CYTHON_ASSUME_SAFE_MACROS
        double value = PyFloat_AS_DOUBLE(float_value);
#else
        double value = PyFloat_AsDouble(float_value);
#endif
        Py_DECREF(float_value);
        return value;
    }
    return (double)-1;
}
static const char* __Pyx__PyBytes_AsDouble_Copy(const char* start, char* buffer, Py_ssize_t length) {
    int last_was_punctuation = 1;
    Py_ssize_t i;
    for (i=0; i < length; i++) {
        char chr = start[i];
        int is_punctuation = (chr == '_') | (chr == '.') | (chr == 'e') | (chr == 'E');
        *buffer = chr;
        buffer += (chr != '_');
        if (unlikely(last_was_punctuation & is_punctuation)) goto parse_failure;
        last_was_punctuation = is_punctuation;
    }
    if (unlikely(last_was_punctuation)) goto parse_failure;
    *buffer = '\0';
    return buffer;
parse_failure:
    return NULL;
}
static double __Pyx__PyBytes_AsDouble_inf_nan(const char* start, Py_ssize_t length) {
    int matches = 1;
    char sign = start[0];
    int is_signed = (sign == '+') | (sign == '-');
    start += is_signed;
    length -= is_signed;
    switch (start[0]) {
        #ifdef Py_NAN
        case 'n':
        case 'N':
            if (unlikely(length != 3)) goto parse_failure;
            matches &= (start[1] == 'a' || start[1] == 'A');
            matches &= (start[2] == 'n' || start[2] == 'N');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_NAN : Py_NAN;
        #endif
        case 'i':
        case 'I':
            if (unlikely(length < 3)) goto parse_failure;
            matches &= (start[1] == 'n' || start[1] == 'N');
            matches &= (start[2] == 'f' || start[2] == 'F');
            if (likely(length == 3 && matches))
                return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
            if (unlikely(length != 8)) goto parse_failure;
            matches &= (start[3] == 'i' || start[3] == 'I');
            matches &= (start[4] == 'n' || start[4] == 'N');
            matches &= (start[5] == 'i' || start[5] == 'I');
            matches &= (start[6] == 't' || start[6] == 'T');
            matches &= (start[7] == 'y' || start[7] == 'Y');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
        case '.': case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
            break;
        default:
            goto parse_failure;
    }
    return 0.0;
parse_failure:
    return -1.0;
}
static CYTHON_INLINE int __Pyx__PyBytes_AsDouble_IsSpace(char ch) {
    return (ch == 0x20) | !((ch < 0x9) | (ch > 0xd));
}
CYTHON_UNUSED static double __Pyx__PyBytes_AsDouble(PyObject *obj, const char* start, Py_ssize_t length) {
    double value;
    Py_ssize_t i, digits;
    const char *last = start + length;
    char *end;
    while (__Pyx__PyBytes_AsDouble_IsSpace(*start))
        start++;
    while (start < last - 1 && __Pyx__PyBytes_AsDouble_IsSpace(last[-1]))
        last--;
    length = last - start;
    if (unlikely(length <= 0)) goto fallback;
    value = __Pyx__PyBytes_AsDouble_inf_nan(start, length);
    if (unlikely(value == -1.0)) goto fallback;
    if (value != 0.0) return value;
    digits = 0;
    for (i=0; i < length; digits += start[i++] != '_');
    if (likely(digits == length)) {
        value = PyOS_string_to_double(start, &end, NULL);
    } else if (digits < 40) {
        char number[40];
        last = __Pyx__PyBytes_AsDouble_Copy(start, number, length);
        if (unlikely(!last)) goto fallback;
        value = PyOS_string_to_double(number, &end, NULL);
    } else {
        char *number = (char*) PyMem_Malloc((digits + 1) * sizeof(char));
        if (unlikely(!number)) goto fallback;
        last = __Pyx__PyBytes_AsDouble_Copy(start, number, length);
        if (unlikely(!last)) {
            PyMem_Free(number);
            goto fallback;
        }
        value = PyOS_string_to_double(number, &end, NULL);
        PyMem_Free(number);
    }
    if (likely(end == last) || (value == (double)-1 && PyErr_Occurred())) {
        return value;
    }
fallback:
    return __Pyx_SlowPyString_AsDouble(obj);
}

/* pyobject_as_double */
static double __Pyx__PyObject_AsDouble(PyObject* obj) {
    if (PyUnicode_CheckExact(obj)) {
        return __Pyx_PyUnicode_AsDouble(obj);
    } else if (PyBytes_CheckExact(obj)) {
        return __Pyx_PyBytes_AsDouble(obj);
    } else if (PyByteArray_CheckExact(obj)) {
        return __Pyx_PyByteArray_AsDouble(obj);
    } else {
        PyObject* float_value;
#if !CYTHON_USE_TYPE_SLOTS
        float_value = PyNumber_Float(obj);  if ((0)) goto bad;
        (void)__Pyx_PyObject_CallOneArg;
#else
        PyNumberMethods *nb = Py_TYPE(obj)->tp_as_number;
        if (likely(nb) && likely(nb->nb_float)) {
            float_value = nb->nb_float(obj);
            if (likely(float_value) && unlikely(!PyFloat_Check(float_value))) {
                __Pyx_TypeName float_value_type_name = __Pyx_PyType_GetName(Py_TYPE(float_value));
                PyErr_Format(PyExc_TypeError,
                    "__float__ returned non-float (type " __Pyx_FMT_TYPENAME ")",
                    float_value_type_name);
                __Pyx_DECREF_TypeName(float_value_type_name);
                Py_DECREF(float_value);
                goto bad;
            }
        } else {
            float_value = __Pyx_PyObject_CallOneArg((PyObject*)&PyFloat_Type, obj);
        }
#endif
        if (likely(float_value)) {
            double value = PyFloat_AS_DOUBLE(float_value);
            Py_DECREF(float_value);
            return value;
        }
    }
bad:
    return (double)-1;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kw,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kw, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kw))) {
        Py_ssize_t kwsize;
#if CYTHON_ASSUME_SAFE_MACROS
        kwsize = PyTuple_GET_SIZE(kw);
#else
        kwsize = PyTuple_Size(kw);
        if (kwsize < 0) return 0;
#endif
        if (unlikely(kwsize == 0))
            return 1;
        if (!kw_allowed) {
#if CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kw, 0);
#else
            key = PyTuple_GetItem(kw, pos);
            if (!key) return 0;
#endif
            goto invalid_keyword;
        }
#if PY_VERSION_HEX < 0x03090000
        for (pos = 0; pos < kwsize; pos++) {
#if CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kw, pos);
#else
            key = PyTuple_GetItem(kw, pos);
            if (!key) return 0;
#endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
        }
#endif
        return 1;
    }
    while (PyDict_Next(kw, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if (!kw_allowed && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
      #if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
      #elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyString_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#elif PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* UnpackUnboundCMethod */
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
     METH_VARARGS | METH_KEYWORDS,
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #else
    if (likely(!__Pyx_CyOrPyCFunction_Check(method)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            target->method = unbound_method;
        }
    }
    return 0;
}

/* CallUnboundCMethod0 */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* pop */
static CYTHON_INLINE PyObject* __Pyx__PyObject_Pop(PyObject* L) {
    if (__Pyx_IS_TYPE(L, &PySet_Type)) {
        return PySet_Pop(L);
    }
    return __Pyx_PyObject_CallMethod0(L, __pyx_n_s_pop);
}
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE PyObject* __Pyx_PyList_Pop(PyObject* L) {
    if (likely(PyList_GET_SIZE(L) > (((PyListObject*)L)->allocated >> 1))) {
        __Pyx_SET_SIZE(L, Py_SIZE(L) - 1);
        return PyList_GET_ITEM(L, PyList_GET_SIZE(L));
    }
    return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyList_Type_pop, L);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C00A6
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE && PY_VERSION_HEX >= 0x030C00A6
    if (unlikely(tstate->current_exception))
#elif CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc;
    __Pyx_PyThreadState_declare
    #ifdef __Pyx_StopAsyncIteration_USED
    int is_async_stopiteration = 0;
    #endif
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    cur_exc = PyErr_Occurred();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        #ifdef __Pyx_StopAsyncIteration_USED
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, __Pyx_PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else
        #endif
            return;
    }
    __Pyx_PyThreadState_assign
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(val);
    Py_XDECREF(tb);
    PyErr_SetString(PyExc_RuntimeError,
        #ifdef __Pyx_StopAsyncIteration_USED
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        #endif
        "generator raised StopIteration");
}

/* PyObjectCall2Args */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallMethod1 */
#if !(CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C00A2)
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && __PYX_LIMITED_VERSION_HEX >= 0x030C00A2
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_GetMethod;
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* FixUpExtensionType */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* FetchSharedCythonModule */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef((char*) __PYX_ABI_MODULE_NAME);
}

/* FetchCommonType */
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t basicsize,
                               Py_ssize_t expected_basicsize) {
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* abi_module;
    const char* object_name;
    PyTypeObject *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    object_name = strrchr(type->tp_name, '.');
    object_name = object_name ? object_name+1 : type->tp_name;
    cached_type = (PyTypeObject*) PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        if (__Pyx_VerifyCachedType(
              (PyObject *)cached_type,
              object_name,
              cached_type->tp_basicsize,
              type->tp_basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    if (PyType_Ready(type) < 0) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, (PyObject *)type) < 0)
        goto bad;
    Py_INCREF(type);
    cached_type = type;
done:
    Py_DECREF(abi_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#else
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module, *cached_type = NULL;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
        PyObject *py_basicsize;
        py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
        if (unlikely(!py_basicsize)) goto bad;
        basicsize = PyLong_AsSsize_t(py_basicsize);
        Py_DECREF(py_basicsize);
        py_basicsize = 0;
        if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
#else
        basicsize = likely(PyType_Check(cached_type)) ? ((PyTypeObject*) cached_type)->tp_basicsize : -1;
#endif
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              basicsize,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, cached_type) < 0) goto bad;
done:
    Py_DECREF(abi_module);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &= Py_TYPE(key)->tp_flags;
        Py_INCREF(key);
        Py_INCREF(value);
        PyTuple_SET_ITEM(kwnames, i, key);
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    if (likely(kw == NULL) || PyDict_GET_SIZE(kw) == 0) {
        return vc(func, args, nargs, NULL);
    }
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure)
{
    CYTHON_UNUSED_VAR(closure);
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#elif PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_tuple;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_kwdict;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->func_annotations;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    int is_coroutine;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
#if PY_VERSION_HEX >= 0x03050000
    if (is_coroutine) {
        PyObject *module, *fromlist, *marker = __pyx_n_s_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_n_s_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        op->func_is_coroutine = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(op->func_is_coroutine)) {
            return __Pyx_NewRef(op->func_is_coroutine);
        }
ignore:
        PyErr_Clear();
    }
#endif
    op->func_is_coroutine = __Pyx_PyBool_FromLong(is_coroutine);
    return __Pyx_NewRef(op->func_is_coroutine);
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {(char *) "_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#endif
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    CYTHON_UNUSED_VAR(args);
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(((PyCFunctionObject*)m)->m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyObject *py_name = NULL;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
    if (!py_name) return NULL;
    PyErr_Format(PyExc_TypeError, "%.200S() takes no keyword arguments",
                 py_name);
    Py_DECREF(py_name);
#else
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
#endif
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_MACROS
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(!argc) < 0) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            PyErr_Format(PyExc_TypeError, "%.200s() needs an argument",
                         ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(PyTuple_GET_SIZE(kwnames))) {
        PyErr_Format(PyExc_TypeError,
                     "%.200s() takes no keyword arguments", ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))def->ml_meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))def->ml_meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if (defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
#else
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
#if !CYTHON_METH_FASTCALL
    0,
#elif CYTHON_BACKPORT_VECTORCALL
    (printfunc)offsetof(__pyx_CyFunctionObject, func_vectorcall),
#else
    offsetof(PyCFunctionObject, vectorcall),
#endif
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_PyMethod_New,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if PY_VERSION_HEX >= 0x030d00A4
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_CyFunction_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
#endif
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (!r) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#else
    CYTHON_UNUSED_VAR(wraparound);
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#else
    CYTHON_UNUSED_VAR(wraparound);
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* pynumber_float */
static CYTHON_INLINE PyObject* __Pyx__PyNumber_Float(PyObject* obj) {
    double val;
    if (PyLong_CheckExact(obj)) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (likely(__Pyx_PyLong_IsCompact(obj))) {
            val = (double) __Pyx_PyLong_CompactValue(obj);
            goto no_error;
        }
#endif
        val = PyLong_AsDouble(obj);
    } else if (PyUnicode_CheckExact(obj)) {
        val = __Pyx_PyUnicode_AsDouble(obj);
    } else if (PyBytes_CheckExact(obj)) {
        val = __Pyx_PyBytes_AsDouble(obj);
    } else if (PyByteArray_CheckExact(obj)) {
        val = __Pyx_PyByteArray_AsDouble(obj);
    } else {
        return PyNumber_Float(obj);
    }
    if (unlikely(val == -1 && PyErr_Occurred())) {
        return NULL;
    }
#if CYTHON_USE_PYLONG_INTERNALS
no_error:
#endif
    return PyFloat_FromDouble(val);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.') != NULL) {
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_kp_u__2);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* ValidateBasesTuple */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_MACROS
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (n < 0) return -1;
#endif
    for (i = 1; i < n; i++)
    {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        PyTypeObject *b;
#if PY_MAJOR_VERSION < 3
        if (PyClass_Check(b0))
        {
            PyErr_Format(PyExc_TypeError, "base class '%.200s' is an old-style class",
                         PyString_AS_STRING(((PyClassObject*)b0)->cl_name));
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !(CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY)
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !(CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API) || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if PY_VERSION_HEX >= 0x030700a1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400)
        gc = PyImport_GetModule(__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if PY_VERSION_HEX >= 0x03050000 && !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    __Pyx_TypeName type_name = __Pyx_PyType_GetName(tp);
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, attr_name);
#else
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%.400s'",
                 type_name, PyString_AS_STRING(attr_name));
#endif
    __Pyx_DECREF_TypeName(type_name);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_n_s_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_n_s_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_n_s_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_n_s_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i;
    void** base_vtables;
    __Pyx_TypeName tp_base_name;
    __Pyx_TypeName base_name;
    void* unknown = (void*)-1;
    PyObject* bases = type->tp_bases;
    int base_depth = 0;
    {
        PyTypeObject* base = type->tp_base;
        while (base) {
            base_depth += 1;
            base = base->tp_base;
        }
    }
    base_vtables = (void**) malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
    for (i = 1; i < PyTuple_GET_SIZE(bases); i++) {
        void* base_vtable = __Pyx_GetVtable(((PyTypeObject*)PyTuple_GET_ITEM(bases, i)));
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = type->tp_base;
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = base->tp_base;
            }
        }
    }
    PyErr_Clear();
    free(base_vtables);
    return 0;
bad:
    tp_base_name = __Pyx_PyType_GetName(type->tp_base);
    base_name = __Pyx_PyType_GetName((PyTypeObject*)PyTuple_GET_ITEM(bases, i));
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
    free(base_vtables);
    return -1;
}
#endif

/* SetupReduce */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}
#endif

/* ClassMethod */
static PyObject* __Pyx_Method_ClassMethod(PyObject *method) {
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM <= 0x05080000
    if (PyObject_TypeCheck(method, &PyWrapperDescr_Type)) {
        return PyClassMethod_New(method);
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (PyMethodDescr_Check(method))
#else
    #if PY_MAJOR_VERSION == 2
    static PyTypeObject *methoddescr_type = NULL;
    if (unlikely(methoddescr_type == NULL)) {
       PyObject *meth = PyObject_GetAttrString((PyObject*)&PyList_Type, "append");
       if (unlikely(!meth)) return NULL;
       methoddescr_type = Py_TYPE(meth);
       Py_DECREF(meth);
    }
    #else
    PyTypeObject *methoddescr_type = &PyMethodDescr_Type;
    #endif
    if (__Pyx_TypeCheck(method, methoddescr_type))
#endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject *)method;
        #if PY_VERSION_HEX < 0x03020000
        PyTypeObject *d_type = descr->d_type;
        #else
        PyTypeObject *d_type = descr->d_common.d_type;
        #endif
        return PyDescr_NewClassMethod(d_type, descr->d_method);
    }
#endif
    else if (PyMethod_Check(method)) {
        return PyClassMethod_New(PyMethod_GET_FUNCTION(method));
    }
    else {
        return PyClassMethod_New(method);
    }
}

/* GetNameInClass */
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    PyObject *dict;
    assert(PyType_Check(nmspace));
#if CYTHON_USE_TYPE_SLOTS
    dict = ((PyTypeObject*)nmspace)->tp_dict;
    Py_XINCREF(dict);
#else
    dict = PyObject_GetAttr(nmspace, __pyx_n_s_dict);
#endif
    if (likely(dict)) {
        result = PyObject_GetItem(dict, name);
        Py_DECREF(dict);
        if (result) {
            return result;
        }
    }
    PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result;
        result = PyObject_Call(replace, __pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    #if __PYX_LIMITED_VERSION_HEX < 0x030780000
    {
        PyObject *compiled = NULL, *result = NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "code", code))) return NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "type", (PyObject*)(&PyType_Type)))) return NULL;
        compiled = Py_CompileString(
            "out = type(code)(\n"
            "  code.co_argcount, code.co_kwonlyargcount, code.co_nlocals, code.co_stacksize,\n"
            "  code.co_flags, code.co_code, code.co_consts, code.co_names,\n"
            "  code.co_varnames, code.co_filename, co_name, co_firstlineno,\n"
            "  code.co_lnotab)\n", "<dummy>", Py_file_input);
        if (!compiled) return NULL;
        result = PyEval_EvalCode(compiled, scratch_dict, scratch_dict);
        Py_DECREF(compiled);
        if (!result) PyErr_Print();
        Py_DECREF(result);
        result = PyDict_GetItemString(scratch_dict, "out");
        if (result) Py_INCREF(result);
        return result;
    }
    #else
    return NULL;
    #endif
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
    if (unlikely(!code_object)) goto bad;
    py_py_line = PyLong_FromLong(py_line);
    if (unlikely(!py_py_line)) goto bad;
    py_funcname = PyUnicode_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    dict = PyDict_New();
    if (unlikely(!dict)) goto bad;
    {
        PyObject *old_code_object = code_object;
        code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
        Py_DECREF(old_code_object);
    }
    if (unlikely(!code_object)) goto bad;
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XDECREF(name);
        name = __Pyx_NewRef(__pyx_n_s__41);
    }
    return name;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* CoroutineBase */
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    CYTHON_UNUSED_VAR(__pyx_tstate);
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_CLEAR(exc_state->exc_value);
#else
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
#endif
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(PyObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    CYTHON_MAYBE_UNUSED_VAR(closing);
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_value) {
        #if CYTHON_COMPILING_IN_PYPY
        #else
        PyObject *exc_tb;
        #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
        exc_tb = PyException_GetTraceback(exc_state->exc_value);
        #elif PY_VERSION_HEX >= 0x030B00a4
        exc_tb = ((PyBaseExceptionObject*) exc_state->exc_value)->traceback;
        #else
        exc_tb = exc_state->exc_traceback;
        #endif
        if (exc_tb) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
            #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
            Py_DECREF(exc_tb);
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body(self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
#if CYTHON_COMPILING_IN_PYPY
    CYTHON_UNUSED_VAR(exc_state);
#else
    PyObject *exc_tb;
    #if PY_VERSION_HEX >= 0x030B00a4
    if (!exc_state->exc_value) return;
    exc_tb = PyException_GetTraceback(exc_state->exc_value);
    #else
    exc_tb = exc_state->exc_traceback;
    #endif
    if (likely(exc_tb)) {
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
        #if PY_VERSION_HEX >= 0x030B00a4
        Py_DECREF(exc_tb);
        #endif
    }
#endif
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(PyObject* gen, PyObject *retval) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
#if PY_VERSION_HEX < 0x030d00A1
            _PyGen_SetStopIterationValue(result);
#else
            if (!PyTuple_Check(result) && !PyExceptionInstance_Check(result)) {
                PyErr_SetObject(PyExc_StopIteration, result);
            } else {
                PyObject *exc = __Pyx_PyObject_CallOneArg(PyExc_StopIteration, result);
                if (likely(exc != NULL)) {
                    PyErr_SetObject(PyExc_StopIteration, exc);
                    Py_DECREF(exc);
                }
            }
#endif
        }
        Py_DECREF(result);
        result = NULL;
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = __Pyx_PyObject_GetIterNextFunc(yf)(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (unlikely(PyErr_Occurred())) {
                PyErr_WriteUnraisable(yf);
            }
        } else {
            retval = __Pyx_PyObject_CallNoArg(meth);
            Py_DECREF(meth);
            if (unlikely(!retval))
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = __Pyx_PyObject_GetIterNextFunc(yf)(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
    CYTHON_UNUSED_VAR(arg);
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (unlikely(PyErr_Occurred())) {
                    gen->is_running = 0;
                    return NULL;
                }
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = __Pyx_PyObject_Call(meth, args, NULL);
            } else {
                PyObject *cargs[4] = {NULL, typ, val, tb};
                ret = __Pyx_PyObject_FastCall(meth, cargs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (unlikely(!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb)))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_VISIT(exc_state->exc_value);
#else
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
#endif
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
#else
        Py_TYPE(gen)->tp_del(self);
        if (unlikely(Py_REFCNT(self) > 0))
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    __Pyx_PyHeapTypeObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (likely(--self->ob_refcnt == 0)) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_name;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_name, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_qualname;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context)
{
    PyObject *frame = self->gi_frame;
    CYTHON_UNUSED_VAR(context);
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    #if PY_VERSION_HEX >= 0x030B00a4
    gen->gi_exc_state.exc_value = NULL;
    #else
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
    #endif
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (unlikely(!module)) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_GeneratorType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
    {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
    {Py_tp_iter, (void *)PyObject_SelfIter},
    {Py_tp_iternext, (void *)__Pyx_Generator_Next},
    {Py_tp_methods, (void *)__pyx_Generator_methods},
    {Py_tp_members, (void *)__pyx_Generator_memberlist},
    {Py_tp_getset, (void *)__pyx_Generator_getsets},
    {Py_tp_getattro, (void *) __Pyx_PyObject_GenericGetAttrNoDict},
#if CYTHON_USE_TP_FINALIZE
    {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
#endif
    {0, 0},
};
static PyType_Spec __pyx_GeneratorType_spec = {
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    __pyx_GeneratorType_slots
};
#else
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if PY_VERSION_HEX >= 0x030d00A4
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_Generator_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_GeneratorType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
#endif
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030B00A4
    return Py_Version & ~0xFFUL;
#else
    const char* rt_version = Py_GetVersion();
    unsigned long version = 0;
    unsigned long factor = 0x01000000UL;
    unsigned int digit = 0;
    int i = 0;
    while (factor) {
        while ('0' <= rt_version[i] && rt_version[i] <= '9') {
            digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
            ++i;
        }
        version += factor * digit;
        if (rt_version[i] != '.')
            break;
        digit = 0;
        factor >>= 8;
        ++i;
    }
    return version;
#endif
}
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* InitStrings */
#if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
